@@ -12,15 +12,16 @@
 namespace switches {
 
 extern const char kDisableBreakpad[];
-extern const char kEnableCrashReporter[];
-extern const char kFullMemoryCrashReport[];
-extern const char kEnableLowEndDeviceMode[];
 extern const char kDisableLowEndDeviceMode[];
+extern const char kEnableCrashReporter[];
+extern const char kEnableHeapProfiling[];
+extern const char kEnableLowEndDeviceMode[];
+extern const char kForceFieldTrials[];
+extern const char kFullMemoryCrashReport[];
 extern const char kNoErrorDialogs[];
 extern const char kProfilerTiming[];
 extern const char kProfilerTimingDisabledValue[];
 extern const char kTestChildProcess[];
-extern const char kTraceToConsole[];
 extern const char kTraceToFile[];
 extern const char kTraceToFileName[];
 extern const char kV[];
