@@ -5,6 +5,8 @@
 #ifndef CC_ANIMATION_TIMING_FUNCTION_H_
 #define CC_ANIMATION_TIMING_FUNCTION_H_
 
+#include "base/macros.h"
+#include "base/memory/scoped_ptr.h"
 #include "cc/base/cc_export.h"
 #include "ui/gfx/geometry/cubic_bezier.h"
 
@@ -81,6 +83,14 @@ class CC_EXPORT EaseInOutTimingFunction {
   DISALLOW_IMPLICIT_CONSTRUCTORS(EaseInOutTimingFunction);
 };
 
+class CC_EXPORT EaseOutNaturalTimingFunction {
+ public:
+  static scoped_ptr<TimingFunction> Create();
+
+ private:
+  DISALLOW_IMPLICIT_CONSTRUCTORS(EaseOutNaturalTimingFunction);
+};
+
 class CC_EXPORT StepsTimingFunction : public TimingFunction {
  public:
   static scoped_ptr<StepsTimingFunction> Create(int steps,
