@@ -5,6 +5,7 @@
 #ifndef CC_BLINK_WEB_SCROLLBAR_LAYER_IMPL_H_
 #define CC_BLINK_WEB_SCROLLBAR_LAYER_IMPL_H_
 
+#include "base/macros.h"
 #include "base/memory/scoped_ptr.h"
 #include "cc/blink/cc_blink_export.h"
 #include "third_party/WebKit/public/platform/WebScrollbar.h"
@@ -30,12 +31,11 @@ class WebScrollbarLayerImpl : public blink::WebScrollbarLayer {
       int thumb_thickness,
       int track_start,
       bool is_left_side_vertical_scrollbar);
-  virtual ~WebScrollbarLayerImpl();
+  ~WebScrollbarLayerImpl() override;
 
   // blink::WebScrollbarLayer implementation.
-  virtual blink::WebLayer* layer();
-  virtual void setScrollLayer(blink::WebLayer* layer);
-  virtual void setClipLayer(blink::WebLayer* layer);
+  blink::WebLayer* layer() override;
+  void setScrollLayer(blink::WebLayer* layer) override;
 
  private:
   scoped_ptr<WebLayerImpl> layer_;
