@@ -5,11 +5,14 @@
 #ifndef CC_DEBUG_LAYER_TREE_DEBUG_STATE_H_
 #define CC_DEBUG_LAYER_TREE_DEBUG_STATE_H_
 
-#include "base/basictypes.h"
 #include "cc/base/cc_export.h"
 
 namespace cc {
 
+namespace proto {
+class LayerTreeDebugState;
+}  // namespace proto
+
 class CC_EXPORT LayerTreeDebugState {
  public:
   LayerTreeDebugState();
@@ -17,7 +20,6 @@ class CC_EXPORT LayerTreeDebugState {
 
   bool show_fps_counter;
   bool show_debug_borders;
-  bool continuous_painting;
 
   bool show_paint_rects;
   bool show_property_changed_rects;
@@ -41,6 +43,9 @@ class CC_EXPORT LayerTreeDebugState {
   bool ShowHudRects() const;
   bool ShowMemoryStats() const;
 
+  void ToProtobuf(proto::LayerTreeDebugState* proto) const;
+  void FromProtobuf(const proto::LayerTreeDebugState& proto);
+
   static bool Equal(const LayerTreeDebugState& a, const LayerTreeDebugState& b);
   static LayerTreeDebugState Unite(const LayerTreeDebugState& a,
                                    const LayerTreeDebugState& b);
