@@ -4,11 +4,12 @@
 
 #include "cc/debug/rasterize_and_record_benchmark.h"
 
+#include <stddef.h>
+
 #include <algorithm>
 #include <limits>
 #include <string>
 
-#include "base/basictypes.h"
 #include "base/strings/stringprintf.h"
 #include "base/values.h"
 #include "cc/debug/lap_timer.h"
@@ -17,9 +18,10 @@
 #include "cc/layers/layer.h"
 #include "cc/layers/picture_layer.h"
 #include "cc/playback/display_item_list.h"
-#include "cc/playback/picture_pile.h"
+#include "cc/playback/display_list_recording_source.h"
 #include "cc/trees/layer_tree_host.h"
 #include "cc/trees/layer_tree_host_common.h"
+#include "skia/ext/analysis_canvas.h"
 #include "third_party/skia/include/utils/SkPictureUtils.h"
 #include "ui/gfx/geometry/rect.h"
 
@@ -34,12 +36,13 @@ const int kTimeLimitMillis = 1;
 const int kWarmupRuns = 0;
 const int kTimeCheckInterval = 1;
 
-const char* kModeSuffixes[RecordingSource::RECORDING_MODE_COUNT] = {
+const char* kModeSuffixes[DisplayListRecordingSource::RECORDING_MODE_COUNT] = {
     "",
     "_sk_null_canvas",
     "_painting_disabled",
     "_caching_disabled",
-    "_construction_disabled"};
+    "_construction_disabled",
+    "_subsequence_caching_disabled"};
 
 }  // namespace
 
@@ -48,7 +51,7 @@ RasterizeAndRecordBenchmark::RasterizeAndRecordBenchmark(
     const MicroBenchmark::DoneCallback& callback)
     : MicroBenchmark(callback),
       record_repeat_count_(kDefaultRecordRepeatCount),
-      settings_(value.Pass()),
+      settings_(std::move(value)),
       main_thread_benchmark_done_(false),
       host_(nullptr),
       weak_ptr_factory_(this) {
@@ -77,7 +80,7 @@ void RasterizeAndRecordBenchmark::DidUpdateLayers(LayerTreeHost* host) {
   results_->SetInteger("picture_memory_usage",
                        static_cast<int>(record_results_.bytes_used));
 
-  for (int i = 0; i < RecordingSource::RECORDING_MODE_COUNT; i++) {
+  for (int i = 0; i < DisplayListRecordingSource::RECORDING_MODE_COUNT; i++) {
     std::string name = base::StringPrintf("record_time%s_ms", kModeSuffixes[i]);
     results_->SetDouble(name,
                         record_results_.total_best_time[i].InMillisecondsF());
@@ -95,7 +98,7 @@ void RasterizeAndRecordBenchmark::RecordRasterResults(
 
   results_->MergeDictionary(results);
 
-  NotifyDone(results_.Pass());
+  NotifyDone(std::move(results_));
 }
 
 scoped_ptr<MicroBenchmarkImpl> RasterizeAndRecordBenchmark::CreateBenchmarkImpl(
@@ -109,100 +112,41 @@ scoped_ptr<MicroBenchmarkImpl> RasterizeAndRecordBenchmark::CreateBenchmarkImpl(
 void RasterizeAndRecordBenchmark::RunOnLayer(PictureLayer* layer) {
   DCHECK(host_);
 
+  if (!layer->DrawsContent())
+    return;
+
   gfx::Rect visible_layer_rect = layer->visible_layer_rect();
   if (visible_layer_rect.IsEmpty())
     return;
 
-  if (host_->settings().use_display_lists) {
-    RunOnDisplayListLayer(layer, visible_layer_rect);
-  } else {
-    RunOnPictureLayer(layer, visible_layer_rect);
-  }
-}
-
-void RasterizeAndRecordBenchmark::RunOnPictureLayer(
-    PictureLayer* layer,
-    const gfx::Rect& visible_layer_rect) {
   ContentLayerClient* painter = layer->client();
 
-  DCHECK(host_ && !host_->settings().use_display_lists);
-
-  gfx::Size tile_grid_size = host_->settings().default_tile_size;
-
-  for (int mode_index = 0; mode_index < RecordingSource::RECORDING_MODE_COUNT;
-       mode_index++) {
-    RecordingSource::RecordingMode mode =
-        static_cast<RecordingSource::RecordingMode>(mode_index);
-
-    // Not supported for SkPicture recording.
-    if (mode == RecordingSource::RECORD_WITH_CONSTRUCTION_DISABLED)
-      continue;
-
-    base::TimeDelta min_time = base::TimeDelta::Max();
-    size_t memory_used = 0;
-
-    for (int i = 0; i < record_repeat_count_; ++i) {
-      // Run for a minimum amount of time to avoid problems with timer
-      // quantization when the layer is very small.
-      LapTimer timer(kWarmupRuns,
-                     base::TimeDelta::FromMilliseconds(kTimeLimitMillis),
-                     kTimeCheckInterval);
-      scoped_refptr<Picture> picture;
-      do {
-        picture = Picture::Create(visible_layer_rect, painter, tile_grid_size,
-                                  false, mode);
-        if (memory_used) {
-          // Verify we are recording the same thing each time.
-          DCHECK(memory_used == picture->ApproximateMemoryUsage());
-        } else {
-          memory_used = picture->ApproximateMemoryUsage();
-        }
-
-        timer.NextLap();
-      } while (!timer.HasTimeLimitExpired());
-      base::TimeDelta duration =
-          base::TimeDelta::FromMillisecondsD(timer.MsPerLap());
-      if (duration < min_time)
-        min_time = duration;
-    }
-
-    if (mode == RecordingSource::RECORD_NORMALLY) {
-      record_results_.bytes_used += memory_used;
-      record_results_.pixels_recorded +=
-          visible_layer_rect.width() * visible_layer_rect.height();
-    }
-    record_results_.total_best_time[mode_index] += min_time;
-  }
-}
-
-void RasterizeAndRecordBenchmark::RunOnDisplayListLayer(
-    PictureLayer* layer,
-    const gfx::Rect& visible_layer_rect) {
-  ContentLayerClient* painter = layer->client();
-
-  DCHECK(host_ && host_->settings().use_display_lists);
-
-  for (int mode_index = 0; mode_index < RecordingSource::RECORDING_MODE_COUNT;
+  for (int mode_index = 0;
+       mode_index < DisplayListRecordingSource::RECORDING_MODE_COUNT;
        mode_index++) {
     ContentLayerClient::PaintingControlSetting painting_control =
-        ContentLayerClient::PAINTING_BEHAVIOR_NORMAL;
-    switch (static_cast<RecordingSource::RecordingMode>(mode_index)) {
-      case RecordingSource::RECORD_NORMALLY:
+        ContentLayerClient::PAINTING_BEHAVIOR_NORMAL_FOR_TEST;
+    switch (
+        static_cast<DisplayListRecordingSource::RecordingMode>(mode_index)) {
+      case DisplayListRecordingSource::RECORD_NORMALLY:
         // Already setup for normal recording.
         break;
-      case RecordingSource::RECORD_WITH_SK_NULL_CANVAS:
+      case DisplayListRecordingSource::RECORD_WITH_SK_NULL_CANVAS:
         // Not supported for Display List recording.
         continue;
-      case RecordingSource::RECORD_WITH_PAINTING_DISABLED:
+      case DisplayListRecordingSource::RECORD_WITH_PAINTING_DISABLED:
         painting_control = ContentLayerClient::DISPLAY_LIST_PAINTING_DISABLED;
         break;
-      case RecordingSource::RECORD_WITH_CACHING_DISABLED:
+      case DisplayListRecordingSource::RECORD_WITH_CACHING_DISABLED:
         painting_control = ContentLayerClient::DISPLAY_LIST_CACHING_DISABLED;
         break;
-      case RecordingSource::RECORD_WITH_CONSTRUCTION_DISABLED:
+      case DisplayListRecordingSource::RECORD_WITH_CONSTRUCTION_DISABLED:
         painting_control =
             ContentLayerClient::DISPLAY_LIST_CONSTRUCTION_DISABLED;
         break;
+      case DisplayListRecordingSource::RECORD_WITH_SUBSEQUENCE_CACHING_DISABLED:
+        painting_control = ContentLayerClient::SUBSEQUENCE_CACHING_DISABLED;
+        break;
       default:
         NOTREACHED();
     }
@@ -218,8 +162,12 @@ void RasterizeAndRecordBenchmark::RunOnDisplayListLayer(
                      kTimeCheckInterval);
 
       do {
-        display_list = painter->PaintContentsToDisplayList(visible_layer_rect,
-                                                           painting_control);
+        display_list = painter->PaintContentsToDisplayList(painting_control);
+        if (display_list->ShouldBeAnalyzedForSolidColor()) {
+          gfx::Size layer_size = layer->paint_properties().bounds;
+          skia::AnalysisCanvas canvas(layer_size.width(), layer_size.height());
+          display_list->Raster(&canvas, nullptr, gfx::Rect(), 1.f);
+        }
 
         if (memory_used) {
           // Verify we are recording the same thing each time.
@@ -236,10 +184,11 @@ void RasterizeAndRecordBenchmark::RunOnDisplayListLayer(
         min_time = duration;
     }
 
-    if (mode_index == RecordingSource::RECORD_NORMALLY) {
-      record_results_.bytes_used += memory_used;
-      record_results_.pixels_recorded +=
-          visible_layer_rect.width() * visible_layer_rect.height();
+    if (mode_index == DisplayListRecordingSource::RECORD_NORMALLY) {
+      record_results_.bytes_used +=
+          memory_used + painter->GetApproximateUnsharedMemoryUsage();
+      record_results_.pixels_recorded += painter->PaintableRegion().width() *
+                                         painter->PaintableRegion().height();
     }
     record_results_.total_best_time[mode_index] += min_time;
   }
