@@ -7,9 +7,11 @@
 
 #include <string>
 
+#include "base/macros.h"
 #include "base/memory/scoped_ptr.h"
 #include "cc/base/cc_export.h"
 #include "cc/layers/layer.h"
+#include "third_party/skia/include/core/SkTypeface.h"
 
 namespace cc {
 
@@ -23,6 +25,9 @@ class CC_EXPORT HeadsUpDisplayLayer : public Layer {
 
   scoped_ptr<LayerImpl> CreateLayerImpl(LayerTreeImpl* tree_impl) override;
 
+  // Layer overrides.
+  void PushPropertiesTo(LayerImpl* layer) override;
+
  protected:
   explicit HeadsUpDisplayLayer(const LayerSettings& settings);
   bool HasDrawableContent() const override;
@@ -30,6 +35,8 @@ class CC_EXPORT HeadsUpDisplayLayer : public Layer {
  private:
   ~HeadsUpDisplayLayer() override;
 
+  skia::RefPtr<SkTypeface> typeface_;
+
   DISALLOW_COPY_AND_ASSIGN(HeadsUpDisplayLayer);
 };
 
