@@ -5,8 +5,6 @@
 #ifndef CC_LAYERS_LAYER_CLIENT_H_
 #define CC_LAYERS_LAYER_CLIENT_H_
 
-#include <string>
-
 #include "base/memory/ref_counted.h"
 #include "cc/base/cc_export.h"
 
@@ -18,14 +16,19 @@ class ConvertableToTraceFormat;
 
 namespace cc {
 
+class Layer;
+
 class CC_EXPORT LayerClient {
  public:
   // Returns a pointer to a debug info object, if one has been computed.
   // If not, returns nullptr.
   // If the returned pointer is non-nullptr, the caller takes
   // ownership of the pointer.
+  //
+  // A pointer to the layer is provided for the convenience of layer clients
+  // which service multiple layers.
   virtual scoped_refptr<base::trace_event::ConvertableToTraceFormat>
-      TakeDebugInfo() = 0;
+  TakeDebugInfo(Layer* layer) = 0;
 
  protected:
   virtual ~LayerClient() {}
