@@ -2,10 +2,13 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+#include <stddef.h>
+
 #include "cc/output/filter_operations.h"
 #include "skia/ext/refptr.h"
 #include "testing/gtest/include/gtest/gtest.h"
 #include "third_party/skia/include/effects/SkBlurImageFilter.h"
+#include "third_party/skia/include/effects/SkDropShadowImageFilter.h"
 #include "ui/gfx/geometry/point.h"
 
 namespace cc {
@@ -23,6 +26,39 @@ TEST(FilterOperationsTest, GetOutsetsBlur) {
   EXPECT_EQ(57, left);
 }
 
+TEST(FilterOperationsTest, GetOutsetsDropShadowReferenceFilter) {
+  // TODO(hendrikw): We need to make outsets for reference filters be in line
+  // with non-reference filters. See crbug.com/523534
+  skia::RefPtr<SkImageFilter> filter =
+      skia::AdoptRef(SkDropShadowImageFilter::Create(
+          SkIntToScalar(3), SkIntToScalar(8), SkIntToScalar(4),
+          SkIntToScalar(9), SK_ColorBLACK,
+          SkDropShadowImageFilter::kDrawShadowAndForeground_ShadowMode));
+  FilterOperations ops;
+  ops.Append(FilterOperation::CreateReferenceFilter(filter));
+
+  int top, right, bottom, left;
+  top = right = bottom = left = 0;
+  ops.GetOutsets(&top, &right, &bottom, &left);
+  EXPECT_EQ(35, top);
+  EXPECT_EQ(9, right);
+  EXPECT_EQ(19, bottom);
+  EXPECT_EQ(15, left);
+}
+
+TEST(FilterOperationsTest, GetOutsetsNullReferenceFilter) {
+  FilterOperations ops;
+  ops.Append(FilterOperation::CreateReferenceFilter(nullptr));
+
+  int top, right, bottom, left;
+  top = right = bottom = left = 0;
+  ops.GetOutsets(&top, &right, &bottom, &left);
+  EXPECT_EQ(0, top);
+  EXPECT_EQ(0, right);
+  EXPECT_EQ(0, bottom);
+  EXPECT_EQ(0, left);
+}
+
 TEST(FilterOperationsTest, GetOutsetsDropShadow) {
   FilterOperations ops;
   ops.Append(FilterOperation::CreateDropShadowFilter(gfx::Point(3, 8), 20, 0));
