@@ -5,7 +5,10 @@
 #ifndef CC_QUADS_RENDER_PASS_ID_H_
 #define CC_QUADS_RENDER_PASS_ID_H_
 
-#include "base/basictypes.h"
+#include <stddef.h>
+
+#include <tuple>
+
 #include "base/containers/hash_tables.h"
 #include "cc/base/cc_export.h"
 
@@ -14,18 +17,20 @@ namespace cc {
 class CC_EXPORT RenderPassId {
  public:
   int layer_id;
-  int index;
+  size_t index;
 
-  RenderPassId(int layer_id, int index) : layer_id(layer_id), index(index) {}
+  RenderPassId() : layer_id(-1), index(0) {}
+  RenderPassId(int layer_id, size_t index) : layer_id(layer_id), index(index) {}
   void* AsTracingId() const;
 
+  bool IsValid() const { return layer_id >= 0; }
+
   bool operator==(const RenderPassId& other) const {
     return layer_id == other.layer_id && index == other.index;
   }
   bool operator!=(const RenderPassId& other) const { return !(*this == other); }
   bool operator<(const RenderPassId& other) const {
-    return layer_id < other.layer_id ||
-           (layer_id == other.layer_id && index < other.index);
+    return std::tie(layer_id, index) < std::tie(other.layer_id, other.index);
   }
 };
 
