@@ -5,7 +5,7 @@
 #ifndef CC_RESOURCES_SHARED_BITMAP_MANAGER_H_
 #define CC_RESOURCES_SHARED_BITMAP_MANAGER_H_
 
-#include "base/basictypes.h"
+#include "base/macros.h"
 #include "base/memory/scoped_ptr.h"
 #include "cc/base/cc_export.h"
 #include "cc/resources/shared_bitmap.h"
