@@ -4,11 +4,13 @@
 
 #include "cc/tiles/picture_layer_tiling_set.h"
 
+#include <stddef.h>
+
 #include <limits>
 #include <set>
 #include <vector>
 
-#include "cc/playback/raster_source.h"
+#include "cc/playback/display_list_raster_source.h"
 
 namespace cc {
 
@@ -16,7 +18,8 @@ namespace {
 
 class LargestToSmallestScaleFunctor {
  public:
-  bool operator() (PictureLayerTiling* left, PictureLayerTiling* right) {
+  bool operator()(const scoped_ptr<PictureLayerTiling>& left,
+                  const scoped_ptr<PictureLayerTiling>& right) {
     return left->contents_scale() > right->contents_scale();
   }
 };
@@ -33,11 +36,11 @@ inline float LargerRatio(float float1, float float2) {
 scoped_ptr<PictureLayerTilingSet> PictureLayerTilingSet::Create(
     WhichTree tree,
     PictureLayerTilingClient* client,
-    size_t max_tiles_for_interest_area,
+    size_t tiling_interest_area_padding,
     float skewport_target_time_in_seconds,
     int skewport_extrapolation_limit_in_content_pixels) {
   return make_scoped_ptr(new PictureLayerTilingSet(
-      tree, client, max_tiles_for_interest_area,
+      tree, client, tiling_interest_area_padding,
       skewport_target_time_in_seconds,
       skewport_extrapolation_limit_in_content_pixels));
 }
@@ -45,23 +48,22 @@ scoped_ptr<PictureLayerTilingSet> PictureLayerTilingSet::Create(
 PictureLayerTilingSet::PictureLayerTilingSet(
     WhichTree tree,
     PictureLayerTilingClient* client,
-    size_t max_tiles_for_interest_area,
+    size_t tiling_interest_area_padding,
     float skewport_target_time_in_seconds,
     int skewport_extrapolation_limit_in_content_pixels)
-    : max_tiles_for_interest_area_(max_tiles_for_interest_area),
+    : tiling_interest_area_padding_(tiling_interest_area_padding),
       skewport_target_time_in_seconds_(skewport_target_time_in_seconds),
       skewport_extrapolation_limit_in_content_pixels_(
           skewport_extrapolation_limit_in_content_pixels),
       tree_(tree),
-      client_(client) {
-}
+      client_(client) {}
 
 PictureLayerTilingSet::~PictureLayerTilingSet() {
 }
 
 void PictureLayerTilingSet::CopyTilingsAndPropertiesFromPendingTwin(
     const PictureLayerTilingSet* pending_twin_set,
-    const scoped_refptr<RasterSource>& raster_source,
+    const scoped_refptr<DisplayListRasterSource>& raster_source,
     const Region& layer_invalidation) {
   if (pending_twin_set->tilings_.empty()) {
     // If the twin (pending) tiling set is empty, it was not updated for the
@@ -72,28 +74,30 @@ void PictureLayerTilingSet::CopyTilingsAndPropertiesFromPendingTwin(
   }
 
   bool tiling_sort_required = false;
-  for (PictureLayerTiling* pending_twin_tiling : pending_twin_set->tilings_) {
+  for (const auto& pending_twin_tiling : pending_twin_set->tilings_) {
     float contents_scale = pending_twin_tiling->contents_scale();
     PictureLayerTiling* this_tiling = FindTilingWithScale(contents_scale);
     if (!this_tiling) {
       scoped_ptr<PictureLayerTiling> new_tiling = PictureLayerTiling::Create(
           tree_, contents_scale, raster_source, client_,
-          max_tiles_for_interest_area_, skewport_target_time_in_seconds_,
+          tiling_interest_area_padding_, skewport_target_time_in_seconds_,
           skewport_extrapolation_limit_in_content_pixels_);
-      tilings_.push_back(new_tiling.Pass());
-      this_tiling = tilings_.back();
+      tilings_.push_back(std::move(new_tiling));
+      this_tiling = tilings_.back().get();
       tiling_sort_required = true;
     }
-    this_tiling->TakeTilesAndPropertiesFrom(pending_twin_tiling,
+    this_tiling->TakeTilesAndPropertiesFrom(pending_twin_tiling.get(),
                                             layer_invalidation);
   }
 
-  if (tiling_sort_required)
-    tilings_.sort(LargestToSmallestScaleFunctor());
+  if (tiling_sort_required) {
+    std::sort(tilings_.begin(), tilings_.end(),
+              LargestToSmallestScaleFunctor());
+  }
 }
 
 void PictureLayerTilingSet::UpdateTilingsToCurrentRasterSourceForActivation(
-    scoped_refptr<RasterSource> raster_source,
+    scoped_refptr<DisplayListRasterSource> raster_source,
     const PictureLayerTilingSet* pending_twin_set,
     const Region& layer_invalidation,
     float minimum_contents_scale,
@@ -108,7 +112,7 @@ void PictureLayerTilingSet::UpdateTilingsToCurrentRasterSourceForActivation(
 
   // If the tiling is not shared (FindTilingWithScale returns nullptr), then
   // invalidate tiles and update them to the new raster source.
-  for (PictureLayerTiling* tiling : tilings_) {
+  for (const auto& tiling : tilings_) {
     if (pending_twin_set->FindTilingWithScale(tiling->contents_scale()))
       continue;
 
@@ -120,15 +124,19 @@ void PictureLayerTilingSet::UpdateTilingsToCurrentRasterSourceForActivation(
     tiling->CreateMissingTilesInLiveTilesRect();
 
     // |this| is active set and |tiling| is not in the pending set, which means
-    // it is now NON_IDEAL_RESOLUTION.
-    tiling->set_resolution(NON_IDEAL_RESOLUTION);
+    // it is now NON_IDEAL_RESOLUTION. The exception is for LOW_RESOLUTION
+    // tilings, which are computed and created entirely on the active tree.
+    // Since the pending tree does not have them, we should just leave them as
+    // low resolution to not lose them.
+    if (tiling->resolution() != LOW_RESOLUTION)
+      tiling->set_resolution(NON_IDEAL_RESOLUTION);
   }
 
   VerifyTilings(pending_twin_set);
 }
 
 void PictureLayerTilingSet::UpdateTilingsToCurrentRasterSourceForCommit(
-    scoped_refptr<RasterSource> raster_source,
+    scoped_refptr<DisplayListRasterSource> raster_source,
     const Region& layer_invalidation,
     float minimum_contents_scale,
     float maximum_contents_scale) {
@@ -136,8 +144,8 @@ void PictureLayerTilingSet::UpdateTilingsToCurrentRasterSourceForCommit(
   RemoveTilingsAboveScale(maximum_contents_scale);
 
   // Invalidate tiles and update them to the new raster source.
-  for (PictureLayerTiling* tiling : tilings_) {
-    DCHECK_IMPLIES(tree_ == PENDING_TREE, !tiling->has_tiles());
+  for (const scoped_ptr<PictureLayerTiling>& tiling : tilings_) {
+    DCHECK(tree_ != PENDING_TREE || !tiling->has_tiles());
     tiling->SetRasterSourceAndResize(raster_source);
 
     // We can commit on either active or pending trees, but only active one can
@@ -154,9 +162,9 @@ void PictureLayerTilingSet::UpdateTilingsToCurrentRasterSourceForCommit(
 }
 
 void PictureLayerTilingSet::UpdateRasterSourceDueToLCDChange(
-    const scoped_refptr<RasterSource>& raster_source,
+    const scoped_refptr<DisplayListRasterSource>& raster_source,
     const Region& layer_invalidation) {
-  for (PictureLayerTiling* tiling : tilings_) {
+  for (const auto& tiling : tilings_) {
     tiling->SetRasterSourceAndResize(raster_source);
     tiling->Invalidate(layer_invalidation);
     // Since the invalidation changed, we need to create any missing tiles in
@@ -168,7 +176,7 @@ void PictureLayerTilingSet::UpdateRasterSourceDueToLCDChange(
 void PictureLayerTilingSet::VerifyTilings(
     const PictureLayerTilingSet* pending_twin_set) const {
 #if DCHECK_IS_ON()
-  for (PictureLayerTiling* tiling : tilings_) {
+  for (const auto& tiling : tilings_) {
     DCHECK(tiling->tile_size() ==
            client_->CalculateTileSize(tiling->tiling_size()))
         << "tile_size: " << tiling->tile_size().ToString()
@@ -197,38 +205,26 @@ void PictureLayerTilingSet::CleanUpTilings(
     float min_acceptable_high_res_scale,
     float max_acceptable_high_res_scale,
     const std::vector<PictureLayerTiling*>& needed_tilings,
-    bool should_have_low_res,
     PictureLayerTilingSet* twin_set) {
-  float twin_low_res_scale = 0.f;
-  if (twin_set) {
-    PictureLayerTiling* tiling =
-        twin_set->FindTilingWithResolution(LOW_RESOLUTION);
-    if (tiling)
-      twin_low_res_scale = tiling->contents_scale();
-  }
-
   std::vector<PictureLayerTiling*> to_remove;
-  for (auto* tiling : tilings_) {
+  for (const auto& tiling : tilings_) {
     // Keep all tilings within the min/max scales.
     if (tiling->contents_scale() >= min_acceptable_high_res_scale &&
         tiling->contents_scale() <= max_acceptable_high_res_scale) {
       continue;
     }
 
-    // Keep low resolution tilings, if the tiling set should have them.
-    if (should_have_low_res &&
-        (tiling->resolution() == LOW_RESOLUTION ||
-         tiling->contents_scale() == twin_low_res_scale)) {
+    // Keep low resolution tilings.
+    if (tiling->resolution() == LOW_RESOLUTION)
       continue;
-    }
 
     // Don't remove tilings that are required.
-    if (std::find(needed_tilings.begin(), needed_tilings.end(), tiling) !=
+    if (std::find(needed_tilings.begin(), needed_tilings.end(), tiling.get()) !=
         needed_tilings.end()) {
       continue;
     }
 
-    to_remove.push_back(tiling);
+    to_remove.push_back(tiling.get());
   }
 
   for (auto* tiling : to_remove) {
@@ -238,20 +234,22 @@ void PictureLayerTilingSet::CleanUpTilings(
 }
 
 void PictureLayerTilingSet::RemoveNonIdealTilings() {
-  auto to_remove = tilings_.remove_if([](PictureLayerTiling* t) {
-    return t->resolution() == NON_IDEAL_RESOLUTION;
-  });
+  auto to_remove =
+      std::remove_if(tilings_.begin(), tilings_.end(),
+                     [](const scoped_ptr<PictureLayerTiling>& t) {
+                       return t->resolution() == NON_IDEAL_RESOLUTION;
+                     });
   tilings_.erase(to_remove, tilings_.end());
 }
 
 void PictureLayerTilingSet::MarkAllTilingsNonIdeal() {
-  for (auto* tiling : tilings_)
+  for (const auto& tiling : tilings_)
     tiling->set_resolution(NON_IDEAL_RESOLUTION);
 }
 
 PictureLayerTiling* PictureLayerTilingSet::AddTiling(
     float contents_scale,
-    scoped_refptr<RasterSource> raster_source) {
+    scoped_refptr<DisplayListRasterSource> raster_source) {
   for (size_t i = 0; i < tilings_.size(); ++i) {
     DCHECK_NE(tilings_[i]->contents_scale(), contents_scale);
     DCHECK_EQ(tilings_[i]->raster_source(), raster_source.get());
@@ -259,52 +257,55 @@ PictureLayerTiling* PictureLayerTilingSet::AddTiling(
 
   tilings_.push_back(PictureLayerTiling::Create(
       tree_, contents_scale, raster_source, client_,
-      max_tiles_for_interest_area_, skewport_target_time_in_seconds_,
+      tiling_interest_area_padding_, skewport_target_time_in_seconds_,
       skewport_extrapolation_limit_in_content_pixels_));
-  PictureLayerTiling* appended = tilings_.back();
+  PictureLayerTiling* appended = tilings_.back().get();
 
-  tilings_.sort(LargestToSmallestScaleFunctor());
+  std::sort(tilings_.begin(), tilings_.end(), LargestToSmallestScaleFunctor());
   return appended;
 }
 
 int PictureLayerTilingSet::NumHighResTilings() const {
   return std::count_if(tilings_.begin(), tilings_.end(),
-                       [](PictureLayerTiling* tiling) {
-    return tiling->resolution() == HIGH_RESOLUTION;
-  });
+                       [](const scoped_ptr<PictureLayerTiling>& tiling) {
+                         return tiling->resolution() == HIGH_RESOLUTION;
+                       });
 }
 
 PictureLayerTiling* PictureLayerTilingSet::FindTilingWithScale(
     float scale) const {
   for (size_t i = 0; i < tilings_.size(); ++i) {
     if (tilings_[i]->contents_scale() == scale)
-      return tilings_[i];
+      return tilings_[i].get();
   }
-  return NULL;
+  return nullptr;
 }
 
 PictureLayerTiling* PictureLayerTilingSet::FindTilingWithResolution(
     TileResolution resolution) const {
-  auto iter = std::find_if(tilings_.begin(), tilings_.end(),
-                           [resolution](const PictureLayerTiling* tiling) {
-    return tiling->resolution() == resolution;
-  });
+  auto iter =
+      std::find_if(tilings_.begin(), tilings_.end(),
+                   [resolution](const scoped_ptr<PictureLayerTiling>& tiling) {
+                     return tiling->resolution() == resolution;
+                   });
   if (iter == tilings_.end())
-    return NULL;
-  return *iter;
+    return nullptr;
+  return iter->get();
 }
 
 void PictureLayerTilingSet::RemoveTilingsBelowScale(float minimum_scale) {
-  auto to_remove =
-      tilings_.remove_if([minimum_scale](PictureLayerTiling* tiling) {
+  auto to_remove = std::remove_if(
+      tilings_.begin(), tilings_.end(),
+      [minimum_scale](const scoped_ptr<PictureLayerTiling>& tiling) {
         return tiling->contents_scale() < minimum_scale;
       });
   tilings_.erase(to_remove, tilings_.end());
 }
 
 void PictureLayerTilingSet::RemoveTilingsAboveScale(float maximum_scale) {
-  auto to_remove =
-      tilings_.remove_if([maximum_scale](PictureLayerTiling* tiling) {
+  auto to_remove = std::remove_if(
+      tilings_.begin(), tilings_.end(),
+      [maximum_scale](const scoped_ptr<PictureLayerTiling>& tiling) {
         return tiling->contents_scale() > maximum_scale;
       });
   tilings_.erase(to_remove, tilings_.end());
@@ -315,8 +316,11 @@ void PictureLayerTilingSet::RemoveAllTilings() {
 }
 
 void PictureLayerTilingSet::Remove(PictureLayerTiling* tiling) {
-  ScopedPtrVector<PictureLayerTiling>::iterator iter =
-    std::find(tilings_.begin(), tilings_.end(), tiling);
+  auto iter =
+      std::find_if(tilings_.begin(), tilings_.end(),
+                   [tiling](const scoped_ptr<PictureLayerTiling>& candidate) {
+                     return candidate.get() == tiling;
+                   });
   if (iter == tilings_.end())
     return;
   tilings_.erase(iter);
@@ -333,7 +337,7 @@ float PictureLayerTilingSet::GetSnappedContentsScale(
   // If a tiling exists within the max snapping ratio, snap to its scale.
   float snapped_contents_scale = start_scale;
   float snapped_ratio = snap_to_existing_tiling_ratio;
-  for (const auto* tiling : tilings_) {
+  for (const auto& tiling : tilings_) {
     float tiling_contents_scale = tiling->contents_scale();
     float ratio = LargerRatio(tiling_contents_scale, start_scale);
     if (ratio < snapped_ratio) {
@@ -358,7 +362,7 @@ bool PictureLayerTilingSet::UpdateTilePriorities(
     const Occlusion& occlusion_in_layer_space,
     bool can_require_tiles_for_activation) {
   bool updated = false;
-  for (auto* tiling : tilings_) {
+  for (const auto& tiling : tilings_) {
     tiling->set_can_require_tiles_for_activation(
         can_require_tiles_for_activation);
     updated |= tiling->ComputeTilePriorityRects(
@@ -370,7 +374,7 @@ bool PictureLayerTilingSet::UpdateTilePriorities(
 
 void PictureLayerTilingSet::GetAllPrioritizedTilesForTracing(
     std::vector<PrioritizedTile>* prioritized_tiles) const {
-  for (auto* tiling : tilings_)
+  for (const auto& tiling : tilings_)
     tiling->GetAllPrioritizedTilesForTracing(prioritized_tiles);
 }
 
@@ -387,7 +391,7 @@ PictureLayerTilingSet::CoverageIterator::CoverageIterator(
 
   size_t tilings_size = set_->tilings_.size();
   for (ideal_tiling_ = 0; ideal_tiling_ < tilings_size; ++ideal_tiling_) {
-    PictureLayerTiling* tiling = set_->tilings_[ideal_tiling_];
+    PictureLayerTiling* tiling = set_->tilings_[ideal_tiling_].get();
     if (tiling->contents_scale() < ideal_contents_scale_) {
       if (ideal_tiling_ > 0)
         ideal_tiling_--;
@@ -421,13 +425,13 @@ gfx::RectF PictureLayerTilingSet::CoverageIterator::texture_rect() const {
 
 Tile* PictureLayerTilingSet::CoverageIterator::operator->() const {
   if (!tiling_iter_)
-    return NULL;
+    return nullptr;
   return *tiling_iter_;
 }
 
 Tile* PictureLayerTilingSet::CoverageIterator::operator*() const {
   if (!tiling_iter_)
-    return NULL;
+    return nullptr;
   return *tiling_iter_;
 }
 
@@ -440,10 +444,10 @@ TileResolution PictureLayerTilingSet::CoverageIterator::resolution() const {
 PictureLayerTiling* PictureLayerTilingSet::CoverageIterator::CurrentTiling()
     const {
   if (current_tiling_ == std::numeric_limits<size_t>::max())
-    return NULL;
+    return nullptr;
   if (current_tiling_ >= set_->tilings_.size())
-    return NULL;
-  return set_->tilings_[current_tiling_];
+    return nullptr;
+  return set_->tilings_[current_tiling_].get();
 }
 
 size_t PictureLayerTilingSet::CoverageIterator::NextTiling() const {
@@ -514,9 +518,7 @@ PictureLayerTilingSet::CoverageIterator::operator++() {
     // Construct a new iterator for the next tiling, but we need to loop
     // again until we get to a valid one.
     tiling_iter_ = PictureLayerTiling::CoverageIterator(
-        set_->tilings_[current_tiling_],
-        contents_scale_,
-        last_rect);
+        set_->tilings_[current_tiling_].get(), contents_scale_, last_rect);
   }
 
   return *this;
@@ -551,7 +553,7 @@ PictureLayerTilingSet::TilingRange PictureLayerTilingSet::GetTilingRange(
   TilingRange high_res_range(0, 0);
   TilingRange low_res_range(tilings_.size(), tilings_.size());
   for (size_t i = 0; i < tilings_size; ++i) {
-    const PictureLayerTiling* tiling = tilings_[i];
+    const PictureLayerTiling* tiling = tilings_[i].get();
     if (tiling->resolution() == HIGH_RESOLUTION)
       high_res_range = TilingRange(i, i + 1);
     if (tiling->resolution() == LOW_RESOLUTION)
