@@ -10,8 +10,8 @@
 #include "cc/tiles/tile_priority.h"
 
 namespace cc {
+class DisplayListRasterSource;
 class PictureLayerTiling;
-class RasterSource;
 
 class CC_EXPORT PrioritizedTile {
  public:
@@ -21,7 +21,7 @@ class CC_EXPORT PrioritizedTile {
   ~PrioritizedTile();
 
   Tile* tile() const { return tile_; }
-  RasterSource* raster_source() const { return raster_source_; }
+  DisplayListRasterSource* raster_source() const { return raster_source_; }
   const TilePriority& priority() const { return priority_; }
   bool is_occluded() const { return is_occluded_; }
 
@@ -31,12 +31,12 @@ class CC_EXPORT PrioritizedTile {
   friend class PictureLayerTiling;
 
   PrioritizedTile(Tile* tile,
-                  RasterSource* raster_source,
+                  DisplayListRasterSource* raster_source,
                   const TilePriority priority,
                   bool is_occluded);
 
   Tile* tile_;
-  RasterSource* raster_source_;
+  DisplayListRasterSource* raster_source_;
   TilePriority priority_;
   bool is_occluded_;
 };
