@@ -253,9 +253,6 @@ class CC_EXPORT LayerTreeHost : public MutatorHostClient {
   void SetDeviceScaleFactor(float device_scale_factor);
   void SetPaintedDeviceScaleFactor(float painted_device_scale_factor);
 
-  void SetContentSourceId(uint32_t);
-  uint32_t content_source_id() const { return content_source_id_; }
-
   float device_scale_factor() const { return device_scale_factor_; }
 
   void UpdateTopControlsState(TopControlsState constraints,
@@ -495,8 +492,6 @@ class CC_EXPORT LayerTreeHost : public MutatorHostClient {
 
   bool visible_;
 
-  uint32_t content_source_id_;
-
   float page_scale_factor_;
   float min_page_scale_factor_;
   float max_page_scale_factor_;
