@@ -83,14 +83,14 @@ class LayerTreeHostReadbackPixelTest
 
     if (!copy_subrect_.IsEmpty())
       request->set_area(copy_subrect_);
-    return request.Pass();
+    return request;
   }
 
   void BeginTest() override {
     if (insert_copy_request_after_frame_count_ == 0) {
       Layer* const target =
           readback_target_ ? readback_target_ : layer_tree_host()->root_layer();
-      target->RequestCopyOfOutput(CreateCopyOutputRequest().Pass());
+      target->RequestCopyOfOutput(CreateCopyOutputRequest());
     }
     PostSetNeedsCommitToMainThread();
   }
@@ -100,19 +100,19 @@ class LayerTreeHostReadbackPixelTest
         layer_tree_host()->source_frame_number()) {
       Layer* const target =
           readback_target_ ? readback_target_ : layer_tree_host()->root_layer();
-      target->RequestCopyOfOutput(CreateCopyOutputRequest().Pass());
+      target->RequestCopyOfOutput(CreateCopyOutputRequest());
     }
   }
 
   void ReadbackResultAsBitmap(scoped_ptr<CopyOutputResult> result) {
-    EXPECT_TRUE(proxy()->IsMainThread());
+    EXPECT_TRUE(task_runner_provider()->IsMainThread());
     EXPECT_TRUE(result->HasBitmap());
-    result_bitmap_ = result->TakeBitmap().Pass();
+    result_bitmap_ = result->TakeBitmap();
     EndTest();
   }
 
   void ReadbackResultAsTexture(scoped_ptr<CopyOutputResult> result) {
-    EXPECT_TRUE(proxy()->IsMainThread());
+    EXPECT_TRUE(task_runner_provider()->IsMainThread());
     EXPECT_TRUE(result->HasTexture());
 
     TextureMailbox texture_mailbox;
@@ -123,9 +123,10 @@ class LayerTreeHostReadbackPixelTest
 
     scoped_ptr<SkBitmap> bitmap =
         CopyTextureMailboxToBitmap(result->size(), texture_mailbox);
-    release_callback->Run(0, false);
+    release_callback->Run(gpu::SyncToken(), false);
 
-    ReadbackResultAsBitmap(CopyOutputResult::CreateBitmapResult(bitmap.Pass()));
+    ReadbackResultAsBitmap(
+        CopyOutputResult::CreateBitmapResult(std::move(bitmap)));
   }
 
   ReadbackType readback_type_;
@@ -391,7 +392,7 @@ TEST_P(LayerTreeHostReadbackPixelTest, ReadbackNonRootLayerOutsideViewport) {
       CreateSolidColorLayer(gfx::Rect(200, 200), SK_ColorGREEN);
   // Only the top left quarter of the layer is inside the viewport, so the
   // blue layer is entirely outside.
-  green->SetPosition(gfx::Point(100, 100));
+  green->SetPosition(gfx::PointF(100.f, 100.f));
   background->AddChild(green);
 
   scoped_refptr<SolidColorLayer> blue =
@@ -458,9 +459,9 @@ class LayerTreeHostReadbackDeviceScalePixelTest
  protected:
   LayerTreeHostReadbackDeviceScalePixelTest()
       : device_scale_factor_(1.f),
-        white_client_(SK_ColorWHITE),
-        green_client_(SK_ColorGREEN),
-        blue_client_(SK_ColorBLUE) {}
+        white_client_(SK_ColorWHITE, gfx::Size(200, 200)),
+        green_client_(SK_ColorGREEN, gfx::Size(200, 200)),
+        blue_client_(SK_ColorBLUE, gfx::Size(200, 200)) {}
 
   void InitializeSettings(LayerTreeSettings* settings) override {
     // Cause the device scale factor to be inherited by contents scales.
@@ -497,7 +498,7 @@ TEST_P(LayerTreeHostReadbackDeviceScalePixelTest, ReadbackSubrect) {
 
   scoped_refptr<FakePictureLayer> blue =
       FakePictureLayer::Create(layer_settings(), &blue_client_);
-  blue->SetPosition(gfx::Point(50, 50));
+  blue->SetPosition(gfx::PointF(50.f, 50.f));
   blue->SetBounds(gfx::Size(25, 25));
   blue->SetIsDrawable(true);
   green->AddChild(blue);
@@ -518,14 +519,14 @@ TEST_P(LayerTreeHostReadbackDeviceScalePixelTest, ReadbackNonRootLayerSubrect) {
 
   scoped_refptr<FakePictureLayer> green =
       FakePictureLayer::Create(layer_settings(), &green_client_);
-  green->SetPosition(gfx::Point(10, 20));
+  green->SetPosition(gfx::PointF(10.f, 20.f));
   green->SetBounds(gfx::Size(90, 80));
   green->SetIsDrawable(true);
   background->AddChild(green);
 
   scoped_refptr<FakePictureLayer> blue =
       FakePictureLayer::Create(layer_settings(), &blue_client_);
-  blue->SetPosition(gfx::Point(50, 50));
+  blue->SetPosition(gfx::PointF(50.f, 50.f));
   blue->SetBounds(gfx::Size(25, 25));
   blue->SetIsDrawable(true);
   green->AddChild(blue);
