@@ -68,7 +68,6 @@ cr.define('cr.login', function() {
     'gaiaUrl',       // Gaia url to use.
     'gaiaPath',      // Gaia path to use without a leading slash.
     'hl',            // Language code for the user interface.
-    'email',         // Pre-fill the email field in Gaia UI.
     'service',       // Name of Gaia service.
     'continueUrl',   // Continue url to use.
     'frameUrl',      // Initial frame URL to use. If empty defaults to
@@ -85,11 +84,30 @@ cr.define('cr.login', function() {
     'enterpriseDomain',    // Domain in which hosting device is (or should be)
                            // enrolled.
     'emailDomain',         // Value used to prefill domain for email.
+    'chromeType',          // Type of Chrome OS device, e.g. "chromebox".
     'clientVersion',       // Version of the Chrome build.
     'platformVersion',     // Version of the OS build.
     'releaseChannel',      // Installation channel.
     'endpointGen',         // Current endpoint generation.
     'gapsCookie',          // GAPS cookie
+
+    // The email fields allow for the following possibilities:
+    //
+    // 1/ If 'email' is not supplied, then the email text field is blank and the
+    // user must type an email to proceed.
+    //
+    // 2/ If 'email' is supplied, and 'readOnlyEmail' is truthy, then the email
+    // is hardcoded and the user cannot change it.  The user is asked for
+    // password.  This is useful for re-auth scenarios, where chrome needs the
+    // user to authenticate for a specific account and only that account.
+    //
+    // 3/ If 'email' is supplied, and 'readOnlyEmail' is falsy, gaia will
+    // prefill the email text field using the given email address, but the user
+    // can still change it and then proceed.  This is used on desktop when the
+    // user disconnects their profile then reconnects, to encourage them to use
+    // the same account.
+    'email',
+    'readOnlyEmail',
   ];
 
   /**
@@ -110,7 +128,6 @@ cr.define('cr.login', function() {
     this.skipForNow_ = false;
     this.authFlow = AuthFlow.DEFAULT;
     this.authDomain = '';
-    this.loaded_ = false;
     this.idpOrigin_ = null;
     this.continueUrl_ = null;
     this.continueUrlWithoutParams_ = null;
@@ -196,7 +213,6 @@ cr.define('cr.login', function() {
   Authenticator.prototype.load = function(authMode, data) {
     this.authMode = authMode;
     this.clearCredentials_();
-    this.loaded_ = false;
     // gaiaUrl parameter is used for testing. Once defined, it is never changed.
     this.idpOrigin_ = data.gaiaUrl || IDP_ORIGIN;
     this.continueUrl_ = data.continueUrl || CONTINUE_URL;
@@ -204,12 +220,13 @@ cr.define('cr.login', function() {
         this.continueUrl_.substring(0, this.continueUrl_.indexOf('?')) ||
         this.continueUrl_;
     this.isConstrainedWindow_ = data.constrained == '1';
-    this.isNewGaiaFlowChromeOS = data.isNewGaiaFlowChromeOS;
+    this.isNewGaiaFlow = data.isNewGaiaFlow;
     this.useEafe_ = data.useEafe || false;
     this.clientId_ = data.clientId;
     this.gapsCookie_ = data.gapsCookie;
     this.gapsCookieSent_ = false;
     this.newGapsCookie_ = null;
+    this.dontResizeNonEmbeddedPages = data.dontResizeNonEmbeddedPages;
 
     this.initialFrameUrl_ = this.constructInitialFrameUrl_(data);
     this.reloadUrl_ = data.frameUrl || this.initialFrameUrl_;
@@ -219,7 +236,7 @@ cr.define('cr.login', function() {
         this.idpOrigin_.indexOf('https://') == 0;
     this.needPassword = !('needPassword' in data) || data.needPassword;
 
-    if (this.isNewGaiaFlowChromeOS) {
+    if (this.isNewGaiaFlow) {
       this.webview_.contextMenus.onShow.addListener(function(e) {
         e.preventDefault();
       });
@@ -243,19 +260,18 @@ cr.define('cr.login', function() {
    */
   Authenticator.prototype.reload = function() {
     this.clearCredentials_();
-    this.loaded_ = false;
     this.webview_.src = this.reloadUrl_;
   };
 
   Authenticator.prototype.constructInitialFrameUrl_ = function(data) {
     var path = data.gaiaPath;
-    if (!path && this.isNewGaiaFlowChromeOS)
+    if (!path && this.isNewGaiaFlow)
       path = EMBEDDED_SETUP_CHROMEOS_ENDPOINT;
     if (!path)
       path = IDP_PATH;
     var url = this.idpOrigin_ + path;
 
-    if (this.isNewGaiaFlowChromeOS) {
+    if (this.isNewGaiaFlow) {
       if (data.chromeType)
         url = appendParam(url, 'chrometype', data.chromeType);
       if (data.clientId)
@@ -278,8 +294,13 @@ cr.define('cr.login', function() {
       url = appendParam(url, 'hl', data.hl);
     if (data.gaiaId)
       url = appendParam(url, 'user_id', data.gaiaId);
-    if (data.email)
-      url = appendParam(url, 'Email', data.email);
+    if (data.email) {
+      if (data.readOnlyEmail) {
+        url = appendParam(url, 'Email', data.email);
+      } else {
+        url = appendParam(url, 'email_hint', data.email);
+      }
+    }
     if (this.isConstrainedWindow_)
       url = appendParam(url, 'source', CONSTRAINED_FLOW_SOURCE);
     if (data.flow)
@@ -296,7 +317,8 @@ cr.define('cr.login', function() {
   Authenticator.prototype.onRequestCompleted_ = function(details) {
     var currentUrl = details.url;
 
-    if (currentUrl.lastIndexOf(this.continueUrlWithoutParams_, 0) == 0) {
+    if (!this.isNewGaiaFlow &&
+        currentUrl.lastIndexOf(this.continueUrlWithoutParams_, 0) == 0) {
       if (currentUrl.indexOf('ntp=1') >= 0)
         this.skipForNow_ = true;
 
@@ -318,7 +340,12 @@ cr.define('cr.login', function() {
           }
         }
       }
-      if (!isEmbeddedPage) {
+
+      // In some cases, non-embedded pages should not be resized.  For
+      // example, on desktop when reauthenticating for purposes of unlocking
+      // a profile, resizing would cause a browser window to open in the
+      // system profile, which is not allowed.
+      if (!isEmbeddedPage && !this.dontResizeNonEmbeddedPages) {
         this.dispatchEvent(new CustomEvent('resize', {detail: currentUrl}));
         return;
       }
@@ -394,7 +421,7 @@ cr.define('cr.login', function() {
         var location = decodeURIComponent(header.value);
         this.chooseWhatToSync_ = !!location.match(/(\?|&)source=3($|&)/);
       } else if (
-          this.isNewGaiaFlowChromeOS && headerName == SET_COOKIE_HEADER) {
+          this.isNewGaiaFlow && headerName == SET_COOKIE_HEADER) {
         var headerValue = header.value;
         if (headerValue.indexOf(OAUTH_CODE_COOKIE + '=', 0) == 0) {
           this.oauthCode_ =
@@ -441,7 +468,7 @@ cr.define('cr.login', function() {
   Authenticator.prototype.onBeforeSendHeaders_ = function(details) {
     // We should re-send cookie if first request was unsuccessful (i.e. no new
     // GAPS cookie was received).
-    if (this.isNewGaiaFlowChromeOS && this.gapsCookie_ &&
+    if (this.isNewGaiaFlow && this.gapsCookie_ &&
         (!this.gapsCookieSent_ || !this.newGapsCookie_)) {
       var headers = details.requestHeaders;
       var found = false;
@@ -515,13 +542,13 @@ cr.define('cr.login', function() {
     var msg = e.data;
     if (msg.method == 'attemptLogin') {
       this.email_ = msg.email;
-      this.password_ = msg.password;
+      if (this.authMode == AuthMode.DESKTOP)
+        this.password_ = msg.password;
+
       this.chooseWhatToSync_ = msg.chooseWhatToSync;
       // We need to dispatch only first event, before user enters password.
-      if (!msg.password) {
-        this.dispatchEvent(
-            new CustomEvent('attemptLogin', {detail: msg.email}));
-      }
+      this.dispatchEvent(
+          new CustomEvent('attemptLogin', {detail: msg.email}));
     } else if (msg.method == 'dialogShown') {
       this.dispatchEvent(new Event('dialogShown'));
     } else if (msg.method == 'dialogHidden') {
@@ -585,13 +612,6 @@ cr.define('cr.login', function() {
       return;
     }
 
-    // TODO(achuith): Eliminate this branch when credential passing api is
-    // stable on prod. crbug.com/467778.
-    if (this.authFlow != AuthFlow.SAML) {
-      this.onAuthCompleted_();
-      return;
-    }
-
     if (this.samlHandler_.scrapedPasswordCount == 0) {
       if (this.noPasswordCallback) {
         this.noPasswordCallback(this.email_);
@@ -703,6 +723,10 @@ cr.define('cr.login', function() {
       };
 
       this.webview_.contentWindow.postMessage(msg, currentUrl);
+
+      this.dispatchEvent(new Event('ready'));
+      // Focus webview after dispatching event when webview is already visible.
+      this.webview_.focus();
     }
   };
 
@@ -721,13 +745,6 @@ cr.define('cr.login', function() {
    * @private
    */
   Authenticator.prototype.onLoadStop_ = function(e) {
-    if (!this.loaded_) {
-      this.loaded_ = true;
-      this.dispatchEvent(new Event('ready'));
-      // Focus webview after dispatching event when webview is already visible.
-      this.webview_.focus();
-    }
-
     // Sends client id to EAFE on every loadstop after a small timeout. This is
     // needed because EAFE sits behind SSO and initialize asynchrounouly
     // and we don't know for sure when it is loaded and ready to listen
@@ -749,10 +766,8 @@ cr.define('cr.login', function() {
    * @private
    */
   Authenticator.prototype.onLoadCommit_ = function(e) {
-    if (this.oauthCode_) {
-      this.skipForNow_ = true;
+    if (this.oauthCode_)
       this.maybeCompleteAuth_();
-    }
   };
 
   /**
