@@ -24,7 +24,8 @@
   COLLAPSE_SESSION: 6,
   EXPAND_SESSION: 7,
   OPEN_ALL: 8,
-  LIMIT: 9  // Should always be the last one.
+  HAS_FOREIGN_DATA: 9,
+  LIMIT: 10  // Should always be the last one.
 };
 
 /**
@@ -282,7 +283,9 @@ Device.prototype.createSessionContents_ = function(maxNumTabs) {
         a.addEventListener('click', makeClickHandler(sessionTag,
                                                      String(win.sessionId),
                                                      String(tab.sessionId)));
-        contents.appendChild(a);
+        var wrapper = createElementWithClassName('div', 'device-tab-wrapper');
+        wrapper.appendChild(a);
+        contents.appendChild(wrapper);
       } else {
         numTabsHidden++;
       }
@@ -295,7 +298,9 @@ Device.prototype.createSessionContents_ = function(maxNumTabs) {
     moreLink.addEventListener('click', this.view_.increaseRowHeight.bind(
         this.view_, this.row_, numTabsHidden));
     moreLink.textContent = loadTimeData.getStringF('xMore', numTabsHidden);
-    contents.appendChild(moreLink);
+    var moreWrapper = createElementWithClassName('div', 'more-wrapper');
+    moreWrapper.appendChild(moreLink);
+    contents.appendChild(moreWrapper);
   }
 
   return contents;
@@ -349,6 +354,7 @@ function DevicesView() {
   this.rowHeights_ = [NB_ENTRIES_FIRST_ROW_COLUMN];
   this.focusGrids_ = [];
   this.updateSignInState(loadTimeData.getBoolean('isUserSignedIn'));
+  this.hasSeenForeignData_ = false;
   recordUmaEvent_(HISTOGRAM_EVENT.INITIALIZED);
 }
 
@@ -375,6 +381,14 @@ DevicesView.prototype.setSessionList = function(sessionList) {
   for (var i = 0; i < sessionList.length; i++)
     this.devices_.push(new Device(sessionList[i], this));
   this.displayResults_();
+
+  // This metric should only be emitted if we see foreign data, and it should
+  // only be emitted once per page refresh. Flip flag to remember because this
+  // method is called upon any update.
+  if (!this.hasSeenForeignData_ && sessionList.length > 0) {
+    this.hasSeenForeignData_ = true;
+    recordUmaEvent_(HISTOGRAM_EVENT.HAS_FOREIGN_DATA);
+  }
 };
 
 
@@ -422,31 +436,19 @@ DevicesView.prototype.increaseRowHeight = function(row, height) {
 // DevicesView, Private -------------------------------------------------------
 
 /**
- * Provides an implementation for a single column grid.
+ * @param {!Element} root
+ * @param {?Node} boundary
  * @constructor
  * @extends {cr.ui.FocusRow}
  */
-function DevicesViewFocusRow() {}
+function DevicesViewFocusRow(root, boundary) {
+  cr.ui.FocusRow.call(this, root, boundary);
+  assert(this.addItem('menu-button', 'button.drop-down') ||
+         this.addItem('device-tab', '.device-tab-entry') ||
+         this.addItem('more-tabs', '.device-show-more-tabs'));
+}
 
-/**
- * Decorates |rowElement| so that it can be treated as a DevicesViewFocusRow.
- * @param {Element} rowElement The element representing this row.
- * @param {Node} boundary Focus events are ignored outside of this node.
- */
-DevicesViewFocusRow.decorate = function(rowElement, boundary) {
-  rowElement.__proto__ = DevicesViewFocusRow.prototype;
-  rowElement.decorate(boundary);
-  rowElement.addFocusableElement(rowElement);
-};
-
-DevicesViewFocusRow.prototype = {
-  __proto__: cr.ui.FocusRow.prototype,
-
-  /** @override */
-  getEquivalentElement: function(element) {
-    return this;
-  },
-};
+DevicesViewFocusRow.prototype = {__proto__: cr.ui.FocusRow.prototype};
 
 /**
  * Update the page with results.
@@ -503,14 +505,14 @@ DevicesView.prototype.displayResults_ = function() {
 
   var devices = this.resultDiv_.querySelectorAll('.device-contents');
   for (var i = 0; i < devices.length; ++i) {
-    var rows = devices[i].querySelectorAll('.device-tab-entry, button');
+    var rows = devices[i].querySelectorAll(
+        'h3, .device-tab-wrapper, .more-wrapper');
     if (!rows.length)
       continue;
 
     var grid = new cr.ui.FocusGrid();
     for (var j = 0; j < rows.length; ++j) {
-      DevicesViewFocusRow.decorate(rows[j], devices[i]);
-      grid.addRow(rows[j]);
+      grid.addRow(new DevicesViewFocusRow(rows[j], devices[i]));
     }
     grid.ensureRowActive();
     this.focusGrids_.push(grid);
