@@ -26,13 +26,13 @@ Polymer({
 
   /**
    * Event triggered when an item is removed.
+   * @param {!{model: !{index: number}}} event
    * @private
    */
-  removeItem_: function(event) {
+  onRemoveTap_: function(event) {
     var index = event.model.index;
     this.splice('exclusions', index, 1);
-    console.debug('network-proxy-exclusions: removed: ' + index);
-    this.fire('changed');
+    this.fire('proxy-change');
   }
 });
 })();
