@@ -6,6 +6,7 @@
 
 #include "base/base_switches.h"
 #include "base/command_line.h"
+#include "build/build_config.h"
 
 namespace switches {
 
@@ -49,12 +50,13 @@ const char kAllowNaClFileHandleAPI[]        = "allow-nacl-file-handle-api";
 // Specifies comma-separated list of extension ids or hosts to grant
 // access to TCP/UDP socket APIs.
 const char kAllowNaClSocketAPI[]            = "allow-nacl-socket-api";
+#endif
 
-// Disables forcing on the experimental Plugin Power Saver feature.
-const char kDisablePluginPowerSaver[] = "disable-plugin-power-saver";
-
-// Enables forcing on the experimental Plugin Power Saver feature.
-const char kEnablePluginPowerSaver[] = "enable-plugin-power-saver";
+#if defined(FULL_SAFE_BROWSING)
+const char kAllowUncheckedDangerousDownloads[] =
+    "allow-unchecked-dangerous-downloads";
+const char kDisallowUncheckedDangerousDownloads[] =
+    "disallow-unchecked-dangerous-downloads";
 #endif
 
 // Don't block outdated plugins.
@@ -82,10 +84,6 @@ const char kAppId[]                         = "app-id";
 // mode.
 const char kApp[]                           = "app";
 
-// Overrides the apps checkout URL, which is used to determine when to expose
-// some private APIs.
-const char kAppsCheckoutURL[]               = "apps-checkout-url";
-
 // The URL that the webstore APIs download extensions from.
 // Note: the URL must contain one '%s' for the extension ID.
 const char kAppsGalleryDownloadURL[]        = "apps-gallery-download-url";
@@ -105,9 +103,8 @@ const char kAppModeOAuth2Token[]            = "app-mode-oauth-token";
 // Enables overriding the path for the default authentication extension.
 const char kAuthExtensionPath[]             = "auth-ext-path";
 
-// A flag that is used to tell Chrome that it was launched automatically at
-// computer startup and not by some user action.
-const char kAutoLaunchAtStartup[]           = "auto-launch-at-startup";
+// Whitelist for Negotiate Auth servers
+const char kAuthServerWhitelist[]           = "auth-server-whitelist";
 
 // This flag makes Chrome auto-select the provided choice when an extension asks
 // permission to start desktop capture. Should only be used for tests. For
@@ -226,6 +223,10 @@ const char kDisableCastStreamingHWEncoding[] =
 const char kDisableChildAccountDetection[] =
     "disable-child-account-detection";
 
+// Disables data volume counters in the Clear Browsing Data dialog.
+const char kDisableClearBrowsingDataCounters[] =
+    "disable-clear-browsing-data-counters";
+
 // Disables the client-side phishing detection feature. Note that even if
 // client-side phishing detection is enabled, it will only be active if the
 // user has opted in to UMA stats and SafeBrowsing is enabled in the
@@ -248,12 +249,12 @@ const char kDisableDefaultApps[]            = "disable-default-apps";
 const char kDisableDeviceDiscoveryNotifications[] =
     "disable-device-discovery-notifications";
 
-// Disables the dinosaur easter egg on the offline interstitial.
-const char kDisableDinosaurEasterEgg[] = "disable-dinosaur-easter-egg";
-
 // Disables Domain Reliability Monitoring.
 const char kDisableDomainReliability[]      = "disable-domain-reliability";
 
+// Disable download notification.
+const char kDisableDownloadNotification[] = "disable-download-notification";
+
 // Disable extensions.
 const char kDisableExtensions[]             = "disable-extensions";
 
@@ -267,6 +268,12 @@ const char kDisableExtensionsFileAccessCheck[] =
 const char kDisableExtensionsHttpThrottling[] =
     "disable-extensions-http-throttling";
 
+// Disable field trial tests configured in fieldtrial_testing_config.json.
+const char kDisableFieldTrialTestingConfig[] = "disable-field-trial-config";
+
+// Disables the Material Design version of chrome://downloads.
+const char kDisableMaterialDesignDownloads[] = "disable-md-downloads";
+
 // Disable the behavior that the second click on a launcher item (the click when
 // the item is already active) minimizes the item.
 const char kDisableMinimizeOnSecondLauncherItemClick[] =
@@ -275,9 +282,8 @@ const char kDisableMinimizeOnSecondLauncherItemClick[] =
 // Disables the new bookmark app system.
 const char kDisableNewBookmarkApps[]        = "disable-new-bookmark-apps";
 
-// Disables the new offline error page generated by NetErrorHelper for ChromeOS
-// and instead uses the old error page generated by OfflineResourceThrottle.
-const char kDisableNewOfflineErrorPage[] = "disable-new-offline-error-page";
+// Disables showing popular sites on the NTP.
+const char kDisableNTPPopularSites[]        = "disable-ntp-popular-sites";
 
 // Disable auto-reload of error pages if offline.
 const char kDisableOfflineAutoReload[]       = "disable-offline-auto-reload";
@@ -294,11 +300,9 @@ const char kDisableOutOfProcessPac[] = "disable-out-of-process-pac";
 const char kDisablePasswordManagerReauthentication[] =
     "disable-password-manager-reauthentication";
 
-// Disable the new material UI - requires out of process PDF plugin.
-const char kDisablePdfMaterialUI[]          = "disable-pdf-material-ui";
-
-// Don't use bubbles for content permissions requests instead of infobars.
-const char kDisablePermissionsBubbles[]      = "disable-permissions-bubbles";
+// Disables the Permissions Blacklist, which blocks access to permissions
+// for blacklisted sites.
+const char kDisablePermissionsBlacklist[] = "disable-permissions-blacklist";
 
 // Disable pop-up blocking.
 const char kDisablePopupBlocking[]          = "disable-popup-blocking";
@@ -321,12 +325,6 @@ const char kDisableQuic[]                   = "disable-quic";
 // This only has an effect if QUIC protocol is enabled.
 const char kDisableQuicPortSelection[]      = "disable-quic-port-selection";
 
-// Prevents the save password bubble from being enabled.
-const char kDisableSavePasswordBubble[] = "disable-save-password-bubble";
-
-// Prevents SDCH persistence from being used.
-const char kDisableSdchPersistence[] = "disable-sdch-persistence";
-
 // Disables using bubbles for session restore request.
 const char kDisableSessionCrashedBubble[] = "disable-session-crashed-bubble";
 
@@ -334,21 +332,9 @@ const char kDisableSessionCrashedBubble[] = "disable-session-crashed-bubble";
 // and allocates certain resources accordingly.
 const char kDisableSiteEngagementService[] = "disable-site-engagement-service";
 
-// Disables the suggestions service.
-const char kDisableSuggestionsService[]     = "disable-suggestions-service";
-
-// Disables syncing browser data to a Google Account.
-const char kDisableSync[]                   = "disable-sync";
-
-// Disables syncing one or more sync data types that are on by default.
-// See sync/internal_api/public/base/model_type.h for possible types. Types
-// should be comma separated, and follow the naming convention for string
-// representation of model types, e.g.:
-// --disable-synctypes='Typed URLs, Bookmarks, Autofill Profiles'
-const char kDisableSyncTypes[]              = "disable-sync-types";
-
-// Disables the backend service for web resources.
-const char kDisableWebResources[]           = "disable-web-resources";
+// Disables Web Notification custom layouts.
+const char kDisableWebNotificationCustomLayouts[] =
+    "disable-web-notification-custom-layouts";
 
 // Some tests seem to require the application to close when the last
 // browser window is closed. Thus, we need a switch to force this behavior
@@ -384,6 +370,10 @@ const char kEnableAddToShelf[] = "enable-add-to-shelf";
 // Enable OS integration for Chrome app file associations.
 const char kEnableAppsFileAssociations[]    = "enable-apps-file-associations";
 
+// If the WebRTC logging private API is active, enables audio debug recordings.
+const char kEnableAudioDebugRecordingsFromExtension[] =
+    "enable-audio-debug-recordings-from-extension";
+
 // Enables the benchmarking extensions.
 const char kEnableBenchmarking[]            = "enable-benchmarking";
 
@@ -394,6 +384,10 @@ const char kEnableBookmarkUndo[]            = "enable-bookmark-undo";
 const char kEnableChildAccountDetection[] =
     "enable-child-account-detection";
 
+// Enables data volume counters in the Clear Browsing Data dialog.
+const char kEnableClearBrowsingDataCounters[] =
+    "enable-clear-browsing-data-counters";
+
 // This applies only when the process type is "service". Enables the Cloud
 // Print Proxy component within the service process.
 const char kEnableCloudPrintProxy[]         = "enable-cloud-print-proxy";
@@ -405,22 +399,9 @@ const char kEnableDevToolsExperiments[]     = "enable-devtools-experiments";
 const char kEnableDeviceDiscoveryNotifications[] =
     "enable-device-discovery-notifications";
 
-// Enables the DOM distiller.
-const char kEnableDomDistiller[] = "enable-dom-distiller";
-
 // Enables Domain Reliability Monitoring.
 const char kEnableDomainReliability[] = "enable-domain-reliability";
 
-// Enables Download Notification.
-const char kEnableDownloadNotification[] = "enable-download-notification";
-
-// Enable Enhanced Bookmarks.
-const char kEnhancedBookmarksExperiment[] = "enhanced-bookmarks-experiment";
-
-// Enables experimentation with ephemeral apps to be launched from the webstore.
-const char kEnableEphemeralAppsInWebstore[] =
-    "enable-ephemeral-apps-in-webstore";
-
 // Enables experimental hotword features specific to always-on.
 const char kEnableExperimentalHotwordHardware[] = "enable-hotword-hardware";
 
@@ -434,26 +415,28 @@ const char kEnableExtensionActivityLogTesting[] =
 // Enable the fast unload controller, which speeds up tab/window close by
 // running a tab's onunload js handler independently of the GUI -
 // crbug.com/142458 .
-const char kEnableFastUnload[]         = "enable-fast-unload";
+const char kEnableFastUnload[] = "enable-fast-unload";
 
-// Enables experimentation with launching ephemeral apps via hyperlinks.
-const char kEnableLinkableEphemeralApps[]   = "enable-linkable-ephemeral-apps";
-
-// Enables an experimental Material Design version of chrome://downloads.
+// Enables the Material Design version of chrome://downloads.
 const char kEnableMaterialDesignDownloads[] = "enable-md-downloads";
 
-// Enables the material design Settings feature.
-const char kEnableMaterialDesignSettings[]  = "enable-md-settings";
+// Enables the Material Design version of chrome://extensions.
+const char kEnableMaterialDesignExtensions[] = "enable-md-extensions";
 
-#if defined(ENABLE_MEDIA_ROUTER)
-// Enables Media Router.
-const char kEnableMediaRouter[]             = "enable-media-router";
-#endif
+// Enables the Material Design version of chrome://history.
+const char kEnableMaterialDesignHistory[] = "enable-md-history";
+
+// Enables the Material Design policy page at chrome://md-policy.
+const char kEnableMaterialDesignPolicyPage[]  = "enable-md-policy-page";
 
 // Runs the Native Client inside the renderer process and enables GPU plugin
 // (internally adds lEnableGpuPlugin to the command line).
 const char kEnableNaCl[]                    = "enable-nacl";
 
+// Enables the use of native notifications instead of using the Chrome based
+// ones.
+const char kEnableNativeNotifications[] = "enable-native-notifications";
+
 // Enables tracing for each navigation. It will attempt to trace each navigation
 // for 10s, until the buffer is full, or until the next navigation.
 // It only works if a URL was provided by --trace-upload-url.
@@ -465,6 +448,14 @@ const char kEnableNetBenchmarking[]         = "enable-net-benchmarking";
 // Enables the new bookmark app system.
 const char kEnableNewBookmarkApps[]         = "enable-new-bookmark-apps";
 
+// Enables showing popular sites on the NTP.
+const char kEnableNTPPopularSites[]         = "enable-ntp-popular-sites";
+
+// Enables using the default search engine country to show country specific
+// popular sites on the NTP.
+const char kEnableNTPSearchEngineCountryDetection[] =
+    "enable-ntp-search-engine-country-detection";
+
 // Enable auto-reload of error pages if offline.
 const char kEnableOfflineAutoReload[]       = "enable-offline-auto-reload";
 
@@ -472,15 +463,13 @@ const char kEnableOfflineAutoReload[]       = "enable-offline-auto-reload";
 const char kEnableOfflineAutoReloadVisibleOnly[] =
     "enable-offline-auto-reload-visible-only";
 
-// Enables panels (always on-top docked pop-up windows).
+// Enables or disables panels (always on-top docked pop-up windows).
 const char kEnablePanels[]                  = "enable-panels";
+const char kDisablePanels[]                 = "disable-panels";
 
-// Enable the new material UI - requires out of process PDF plugin.
-const char kEnablePdfMaterialUI[]           = "enable-pdf-material-ui";
-
-// Enables presenting plugin placeholder content as shadow DOM.
-const char kEnablePluginPlaceholderShadowDom[] =
-    "enable-plugin-placeholder-shadow-dom";
+// Enables the Permissions Blacklist, which blocks access to permissions
+// for blacklisted sites.
+const char kEnablePermissionsBlacklist[] = "enable-permissions-blacklist";
 
 // Enables a number of potentially annoying security features (strict mixed
 // content mode, powerful feature restrictions, etc.)
@@ -501,8 +490,9 @@ const char kEnablePrintPreviewRegisterPromos[] =
 // during chrome_browser_main.
 const char kEnableProfiling[]               = "enable-profiling";
 
-// Enables query in the omnibox.
-const char kEnableQueryExtraction[]         = "enable-query-extraction";
+// Enable or disable background mode for the Push API.
+const char kEnablePushApiBackgroundMode[] = "enable-push-api-background-mode";
+const char kDisablePushApiBackgroundMode[] = "disable-push-api-background-mode";
 
 // Enables support for the QUIC protocol.  This is a temporary testing flag.
 const char kEnableQuic[]                    = "enable-quic";
@@ -511,11 +501,8 @@ const char kEnableQuic[]                    = "enable-quic";
 // This only has an effect if QUIC protocol is enabled.
 const char kEnableQuicPortSelection[]       = "enable-quic-port-selection";
 
-// Enables save password prompt bubble.
-const char kEnableSavePasswordBubble[]      = "enable-save-password-bubble";
-
-// Enables SDCH persistence.
-const char kEnableSdchPersistence[] = "enable-sdch-persistence";
+// Enables support for HTTP alternative services.
+const char kEnableAlternativeServices[] = "enable-alternative-services";
 
 // Enables using bubbles for session restore request instead of infobars.
 const char kEnableSessionCrashedBubble[] = "enable-session-crashed-bubble";
@@ -525,20 +512,32 @@ const char kEnableSessionCrashedBubble[] = "enable-session-crashed-bubble";
 const char kEnableSettingsWindow[]           = "enable-settings-window";
 const char kDisableSettingsWindow[]          = "disable-settings-window";
 
+// A new user experience for transitioning into fullscreen and mouse pointer
+// lock states.
+const char kEnableSimplifiedFullscreenUI[]  = "enable-simplified-fullscreen-ui";
+const char kDisableSimplifiedFullscreenUI[] =
+    "disable-simplified-fullscreen-ui";
+
+// Enable the Site Engagement App Banner which triggers app install banners
+// using the site engagement service rather than a navigation-based heuristic.
+// Implicitly enables the site engagement service.
+const char kEnableSiteEngagementAppBanner[] =
+    "enable-site-engagement-app-banner";
+
+// Enable the Site Engagement Eviction Policy which evicts temporary storage
+// using the site engagement service. Implicitly enables the site engagement
+// service.
+const char kEnableSiteEngagementEvictionPolicy[] =
+    "enable-site-engagement-eviction-policy";
+
 // Enable the Site Engagement service, which records interaction with sites and
 // allocates certain resources accordingly.
 const char kEnableSiteEngagementService[]   = "enable-site-engagement-service";
 
-// Enables the suggestions service.
-const char kEnableSuggestionsService[]      = "enable-suggestions-service";
-
 // Enables the supervised user managed bookmarks folder.
 const char kEnableSupervisedUserManagedBookmarksFolder[] =
     "enable-supervised-user-managed-bookmarks-folder";
 
-// Enables synced articles.
-const char kEnableSyncArticles[]            = "enable-sync-articles";
-
 // Enables user control over muting tab audio from the tab strip.
 const char kEnableTabAudioMuting[]  = "enable-tab-audio-muting";
 
@@ -552,10 +551,11 @@ const char kEnableUserAlternateProtocolPorts[] =
 
 // Enables a new "web app" style frame for hosted apps (including bookmark
 // apps).
-extern const char kEnableWebAppFrame[] = "enable-web-app-frame";
+const char kEnableWebAppFrame[] = "enable-web-app-frame";
 
-// Enables synchronizing WiFi credentials across devices, using Chrome Sync.
-const char kEnableWifiCredentialSync[]      = "enable-wifi-credential-sync";
+// Enables Web Notification custom layouts.
+const char kEnableWebNotificationCustomLayouts[] =
+    "enable-web-notification-custom-layouts";
 
 // Values for the kExtensionContentVerification flag.
 // See ContentVerifierDelegate::Mode for more explanation.
@@ -578,23 +578,11 @@ const char kExtensionsNotWebstore[] = "extensions-not-webstore";
 // Frequency in seconds for Extensions auto-update.
 const char kExtensionsUpdateFrequency[]     = "extensions-update-frequency";
 
-// Fakes the channel of the browser for purposes of Variations filtering. This
-// is to be used for testing only. Possible values are "stable", "beta", "dev"
-// and "canary". Note that this only applies if the browser's reported channel
-// is UNKNOWN.
-const char kFakeVariationsChannel[]         = "fake-variations-channel";
-
 // If this flag is present then this command line is being delegated to an
 // already running chrome process via the fast path, ie: before chrome.dll is
 // loaded. It is useful to tell the difference for tracking purposes.
 const char kFastStart[]            = "fast-start";
 
-// These two flags are added around the switches about:flags adds to the
-// command line. This is useful to see which switches were added by about:flags
-// on about:version. They don't have any effect.
-const char kFlagSwitchesBegin[]             = "flag-switches-begin";
-const char kFlagSwitchesEnd[]               = "flag-switches-end";
-
 // Forces application mode. This hides certain system UI elements and forces
 // the app to be installed if it hasn't been already.
 const char kForceAppMode[]                  = "force-app-mode";
@@ -618,6 +606,9 @@ const char kForceFirstRun[]                 = "force-first-run";
 // prefixed with the character "t" will be treated as Trigger Variation Ids.
 const char kForceVariationIds[]             = "force-variation-ids";
 
+// Forces Chrome to use localNTP instead of server (GWS) NTP.
+const char kForceLocalNtp[]                 = "force-local-ntp";
+
 // Enables grouping websites by domain and filtering them by period.
 const char kHistoryEnableGroupByDomain[]    = "enable-grouped-history";
 
@@ -660,12 +651,6 @@ const char kIncognito[]                     = "incognito";
 // Causes Chrome to initiate an installation flow for the given app.
 const char kInstallChromeApp[]              = "install-chrome-app";
 
-// Causes Chrome to attempt to get metadata from the webstore for the
-// app/extension ID given, and then prompt the user to download and install it.
-// This is allowed *only* for ephemeral apps. All other ids will be ignored.
-const char kInstallEphemeralAppFromWebstore[] =
-    "install-ephemeral-app-from-webstore";
-
 // A list of whitelists to install for a supervised user, for testing.
 // The list is of the following form: <id>[:<name>],[<id>[:<name>],...]
 const char kInstallSupervisedUserWhitelists[] =
@@ -674,9 +659,8 @@ const char kInstallSupervisedUserWhitelists[] =
 // Marks a renderer as an Instant process.
 const char kInstantProcess[]                = "instant-process";
 
-// Invalidation service should use GCM network channel even if experiment is not
-// enabled.
-const char kInvalidationUseGCMChannel[]     = "invalidation-use-gcm-channel";
+// The URL for the interests API.
+const char kInterestsURL[]                  = "interests-url";
 
 // Disable latest shipping ECMAScript 6 features.
 const char kDisableJavaScriptHarmonyShipping[] =
@@ -705,18 +689,21 @@ const char kLoadComponentExtension[]        = "load-component-extension";
 // Loads an extension from the specified directory.
 const char kLoadExtension[]                 = "load-extension";
 
+#if !defined(GOOGLE_CHROME_BUILD)
+// Enables a live-reload for local NTP resources. This only works when Chrome
+// is running from a Chrome source directory.
+const char kLocalNtpReload[]                = "local-ntp-reload";
+#endif
+
 // Makes Chrome default browser
 const char kMakeDefaultBrowser[]            = "make-default-browser";
 
-// Use to opt-in to marking HTTP as non-secure.
-const char kMarkNonSecureAs[]               = "mark-non-secure-as";
-const char kMarkNonSecureAsNeutral[]        = "neutral";
-const char kMarkNonSecureAsDubious[]        = "dubious";
-const char kMarkNonSecureAsNonSecure[]      = "non-secure";
-
 // Forces the maximum disk space to be used by the media cache, in bytes.
 const char kMediaCacheSize[]                = "media-cache-size";
 
+// Enables Media Router.
+const char kMediaRouter[]                   = "media-router";
+
 // Enables histograming of tasks served by MessageLoop. See
 // about:histograms/Loop for results, which show frequency of messages on each
 // thread, including APC count, object signalling count, etc.
@@ -898,6 +885,9 @@ const char kProxyServer[]                   = "proxy-server";
 // the server.
 const char kQuicConnectionOptions[]         = "quic-connection-options";
 
+// Specifies a comma separated list of hosts to whitelist QUIC for.
+const char kQuicHostWhitelist[]             = "quic-host-whitelist";
+
 // Specifies the maximum length for a QUIC packet.
 const char kQuicMaxPacketLength[]           = "quic-max-packet-length";
 
@@ -941,21 +931,26 @@ const char kSbDisableDownloadProtection[] =
 const char kSbDisableExtensionBlacklist[] =
     "safebrowsing-disable-extension-blacklist";
 
+// List of comma-separated sha256 hashes of executable files which the
+// download-protection service should treat as "dangerous."  For a file to
+// show a warning, it also must be considered a dangerous filetype and not
+// be whitelisted otherwise (by signature or URL) and must be on a supported
+// OS. Hashes are in hex. This is used for manual testing when looking
+// for ways to by-pass download protection.
+const char kSbManualDownloadBlacklist[] =
+    "safebrowsing-manual-download-blacklist";
+
 // Causes the process to run as a service process.
 const char kServiceProcess[]                = "service";
 
 // If true the app list will be shown.
 const char kShowAppList[]                   = "show-app-list";
 
-// Command line flag offering a "Show saved copy" option to the user if offline.
-// The various modes are disabled, primary, or secondary. Primary/secondary
-// refers to button placement (for experiment).
-const char kShowSavedCopy[]                 = "show-saved-copy";
-
-// Values for the kShowSavedCopy flag.
-const char kEnableShowSavedCopyPrimary[]    = "primary";
-const char kEnableShowSavedCopySecondary[]  = "secondary";
-const char kDisableShowSavedCopy[]          = "disable";
+// Causes SSL key material to be logged to the specified file for debugging
+// purposes. See
+// https://developer.mozilla.org/en-US/docs/Mozilla/Projects/NSS/Key_Log_Format
+// for the format.
+const char kSSLKeyLogFile[]               = "ssl-key-log-file";
 
 // Does not show an infobar when an extension attaches to a page using
 // chrome.debugger page. Required to attach to extension background pages.
@@ -1000,13 +995,14 @@ const char kSpeculativeResourcePrefetchingEnabled[] = "enabled";
 const char kEnableAndroidSpellChecker[] = "enable-android-spellchecker";
 #endif
 
+// Disables the multilingual spellchecker.
+const char kDisableMultilingualSpellChecker[] =
+    "disable-multilingual-spellchecker";
+
 // Enables the multilingual spellchecker.
 const char kEnableMultilingualSpellChecker[] =
     "enable-multilingual-spellchecker";
 
-// Enables auto correction for misspelled words.
-const char kEnableSpellingAutoCorrect[]     = "enable-spelling-auto-correct";
-
 // Enables participation in the field trial for user feedback to spelling
 // service.
 const char kEnableSpellingFeedbackFieldTrial[] =
@@ -1027,22 +1023,6 @@ const char kSpellingServiceFeedbackIntervalSeconds[] =
     "spelling-service-feedback-interval-seconds";
 #endif
 
-// Specifies the maximum SSL/TLS version ("tls1", "tls1.1", or "tls1.2").
-const char kSSLVersionMax[]                 = "ssl-version-max";
-
-// Specifies the minimum SSL/TLS version ("tls1", "tls1.1", or "tls1.2").
-const char kSSLVersionMin[]                 = "ssl-version-min";
-
-// Specifies the minimum SSL/TLS version ("tls1", "tls1.1", or "tls1.2") that
-// TLS fallback will accept.
-const char kSSLVersionFallbackMin[]         = "ssl-version-fallback-min";
-
-// These values aren't switches, but rather the values that kSSLVersionMax,
-// kSSLVersionMin and kSSLVersionFallbackMin can have.
-const char kSSLVersionTLSv1[]               = "tls1";
-const char kSSLVersionTLSv11[]              = "tls1.1";
-const char kSSLVersionTLSv12[]              = "tls1.2";
-
 // Starts the browser maximized, regardless of any previous settings.
 const char kStartMaximized[]                = "start-maximized";
 
@@ -1059,35 +1039,9 @@ const char kSupervisedUserSafeSites[]       = "supervised-user-safesites";
 // Setting this switch also causes Sync to be set up for a supervised user.
 const char kSupervisedUserSyncToken[]       = "managed-user-sync-token";
 
-// This flag causes sync to retry very quickly (see polling_constants.h) the
-// when it encounters an error, as the first step towards exponential backoff.
-const char kSyncShortInitialRetryOverride[] =
-    "sync-short-initial-retry-override";
-
-// Overrides the default server used for profile sync.
-const char kSyncServiceURL[]                = "sync-url";
-
-// Enables deferring sync backend initialization until user initiated changes
-// occur.
-const char kSyncDisableDeferredStartup[]     = "sync-disable-deferred-startup";
-
-// Allows overriding the deferred init fallback timeout.
-const char kSyncDeferredStartupTimeoutSeconds[] =
-    "sync-deferred-startup-timeout-seconds";
-
-// Enables feature to avoid unnecessary GetUpdate requests.
-const char kSyncEnableGetUpdateAvoidance[]   =
-    "sync-enable-get-update-avoidance";
-
-// Enables clearing of sync data when a user enables passphrase encryption.
-const char kSyncEnableClearDataOnPassphraseEncryption[] =
-    "enable-clear-sync-data-on-passphrase-encryption";
-
-// Disable data backup when user's not signed in.
-const char kSyncDisableBackup[] = "disable-sync-backup";
-
-// Disable sync rollback.
-const char kSyncDisableRollback[] = "disable-sync-rollback";
+// Frequency in Milliseconds for system log uploads. Should only be used for
+// testing purposes.
+const char kSystemLogUploadFrequency[] = "system-log-upload-frequency";
 
 // Passes the name of the current running automated test to Chrome.
 const char kTestName[]                      = "test-name";
@@ -1140,11 +1094,6 @@ const char kV8PacMojoInProcess[] = "v8-pac-mojo-in-process";
 // Enables using an out-of-process Mojo service for the v8 proxy resolver.
 const char kV8PacMojoOutOfProcess[] = "v8-pac-mojo-out-of-process";
 
-// Specifies a custom URL for the server which reports variation data to the
-// client. Specifying this switch enables the Variations service on
-// unofficial builds. See variations_service.cc.
-const char kVariationsServerURL[]           = "variations-server-url";
-
 // Prints version information and quits.
 const char kVersion[]                       = "version";
 
@@ -1163,15 +1112,12 @@ const char kWinHttpProxyResolver[]          = "winhttp-proxy-resolver";
 const char kWinJumplistAction[]             = "win-jumplist-action";
 
 #if defined(OS_ANDROID)
-// Disables support for playing videos on Chromecast devices.
-const char kDisableCast[]                    = "disable-cast";
+// Android authentication account type for SPNEGO authentication
+const char kAuthAndroidNegotiateAccountType[] = "auth-spnego-account-type";
 
 // Disables Contextual Search.
 const char kDisableContextualSearch[]        = "disable-contextual-search";
 
-// Disables zero suggest experiment on Dev channel.
-const char kDisableZeroSuggest[] = "disable-zero-suggest";
-
 // Enable the accessibility tab switcher.
 const char kEnableAccessibilityTabSwitcher[] =
     "enable-accessibility-tab-switcher";
@@ -1179,33 +1125,22 @@ const char kEnableAccessibilityTabSwitcher[] =
 // Enables Contextual Search.
 const char kEnableContextualSearch[]        = "enable-contextual-search";
 
-// Enables the DOM distiller animated button UI for Android.
-const char kEnableDomDistillerButtonAnimation[] =
-    "enable-dom-distiller-button-animation";
-
 // Enables chrome hosted mode for Android.
 const char kEnableHostedMode[] = "enable-hosted-mode";
 
-// Enables instant search clicks feature.
-const char kEnableInstantSearchClicks[] = "enable-instant-search-clicks";
+// Enables a hung renderer InfoBar allowing the user to close or wait on
+// unresponsive web content.
+const char kEnableHungRendererInfoBar[] = "enable-hung-renderer-infobar";
 
-// Enables context-sensitive reader mode button in the toolbar.
-const char kEnableReaderModeToolbarIcon[] =
-    "enable-reader-mode-toolbar-icon";
+// Specifies Android phone page loading progress bar animation.
+const char kProgressBarAnimation[]          = "progress-bar-animation";
 
-// Enables zero suggest functionality on Dev channel, showing most visited
-// sites as default suggestions.
-const char kEnableZeroSuggestMostVisited[] =
-    "enable-zero-suggest-most-visited";
+// Enabled tab switcher in document mode.
+const char kEnableTabSwitcherInDocumentMode[] =
+    "enable-tab-switcher-in-document-mode";
 
-// Enable zero suggest functionality on Dev channel, showing most visited
-// sites on non-search-result pages as default suggestions.
-const char kEnableZeroSuggestMostVisitedWithoutSerp[] =
-    "enable-zero-suggest-most-visited-without-serp";
-
-// Triggers prerendering of search base page to prefetch results for the typed
-// omnibox query. Only has an effect when prerender is enabled.
-const char kPrefetchSearchResults[]         = "prefetch-search-results";
+// Switch to an existing tab for a suggestion opened from the New Tab Page.
+const char kNtpSwitchToExistingTab[] = "ntp-switch-to-existing-tab";
 #endif  // defined(OS_ANDROID)
 
 #if defined(USE_ASH)
@@ -1247,10 +1182,19 @@ const char kDisableAppWindowCycling[] = "disable-app-window-cycling";
 // Disables app shim creation for hosted apps on Mac.
 const char kDisableHostedAppShimCreation[] = "disable-hosted-app-shim-creation";
 
+// Prevents hosted apps from being opened in windows on Mac.
+const char kDisableHostedAppsInWindows[] = "disable-hosted-apps-in-windows";
+
 // Disables use of toolkit-views based native app windows.
 const char kDisableMacViewsNativeAppWindows[] =
     "disable-mac-views-native-app-windows";
 
+// Prevents the save password bubble from being enabled.
+const char kDisableSavePasswordBubble[] = "disable-save-password-bubble";
+
+// Disables Translate experimental new UX which replaces the infobar.
+const char kDisableTranslateNewUX[] = "disable-translate-new-ux";
+
 // Enable the toolkit-views App Info dialog for Mac. This is accessible from
 // chrome://apps and chrome://extensions and is already enabled on non-mac.
 const char kEnableAppInfoDialogMac[] = "enable-app-info-dialog-mac";
@@ -1258,12 +1202,22 @@ const char kEnableAppInfoDialogMac[] = "enable-app-info-dialog-mac";
 // Enables custom Cmd+` window cycling for platform apps and hosted apps.
 const char kEnableAppWindowCycling[] = "enable-app-window-cycling";
 
+// Allows hosted apps to be opened in windows on Mac.
+const char kEnableHostedAppsInWindows[] = "enable-hosted-apps-in-windows";
+
 // Enables use of toolkit-views based native app windows.
 const char kEnableMacViewsNativeAppWindows[] =
     "enable-mac-views-native-app-windows";
 
+// Causes Chrome to use an equivalent toolkit-views version of a browser dialog
+// when available, rather than a Cocoa one.
+const char kEnableMacViewsDialogs[] = "enable-mac-views-dialogs";
+
+// Enables save password prompt bubble.
+const char kEnableSavePasswordBubble[] = "enable-save-password-bubble";
+
 // Enables Translate experimental new UX which replaces the infobar.
-const char kEnableTranslateNewUX[]         = "enable-translate-new-ux";
+const char kEnableTranslateNewUX[] = "enable-translate-new-ux";
 
 // This is how the metrics client ID is passed from the browser process to its
 // children. With Crashpad, the metrics client ID is distinct from the crash
@@ -1273,12 +1227,23 @@ const char kMetricsClientID[]               = "metrics-client-id";
 // A process type (switches::kProcessType) that relaunches the browser. See
 // chrome/browser/mac/relauncher.h.
 const char kRelauncherProcess[]             = "relauncher";
+
+// When switches::kProcessType is switches::kRelauncherProcess, if this switch
+// is also present, the relauncher process will unmount and eject a mounted disk
+// image and move its disk image file to the trash.  The argument's value must
+// be a BSD device name of the form "diskN" or "diskNsM".
+const char kRelauncherProcessDMGDevice[]    = "dmg-device";
 #endif  // defined(OS_MACOSX)
 
-// Use bubbles for content permissions requests instead of infobars.
-const char kEnablePermissionsBubbles[]      = "enable-permissions-bubbles";
-
 #if defined(OS_WIN)
+// A process type (switches::kProcessType) that indicates chrome.exe is being
+// launched as crashpad_handler. This is only used on Windows. We bundle the
+// handler into chrome.exe on Windows because there is high probability of a
+// "new" .exe being blocked or interfered with by application firewalls, AV
+// software, etc. On other platforms, crashpad_handler is a standalone
+// executable.
+const char kCrashpadHandler[]               = "crashpad-handler";
+
 // Fallback to XPS. By default connector uses CDD.
 const char kEnableCloudPrintXps[]           = "enable-cloud-print-xps";
 
@@ -1294,6 +1259,14 @@ const char kForceImmersive[]                = "force-immersive";
 // This flag is only relevant for Windows currently.
 const char kNoNetworkProfileWarning[]       = "no-network-profile-warning";
 
+// /prefetch:# arguments for the browser process launched in background mode and
+// for the watcher process. Use profiles 5, 6 and 7 as documented on
+// kPrefetchArgument* in content_switches.cc.
+const char kPrefetchArgumentBrowserBackground[] = "/prefetch:5";
+const char kPrefetchArgumentWatcher[] = "/prefetch:6";
+// /prefetch:7 is used by crashpad, which can't depend on constants defined
+// here. See crashpad_win.cc for more details.
+
 // For the DelegateExecute verb handler to launch Chrome in desktop mode on
 // Windows 8 and higher.  Used when relaunching metro Chrome.
 const char kForceDesktop[]                  = "force-desktop";
@@ -1322,6 +1295,9 @@ const char kViewerLaunchViaAppId[]          = "viewer-launch-via-appid";
 // Windows 8 and higher.
 const char kWaitForMutex[]                  = "wait-for-mutex";
 
+// Causes the process to run as a watcher process.
+const char kWatcherProcess[]                = "watcher";
+
 // Indicates that chrome was launched to service a search request in Windows 8.
 const char kWindows8Search[]                = "windows8-search";
 #endif  // defined(OS_WIN)
@@ -1336,43 +1312,68 @@ const char kIpcFuzzerTestcase[]             = "ipc-fuzzer-testcase";
 const char kDebugPrint[] = "debug-print";
 #endif
 
+#if defined(ENABLE_TASK_MANAGER)
+// Disables the new implementation of the task manager.
+const char kDisableNewTaskManager[]   = "disable-new-task-manager";
+#endif  // defined(ENABLE_TASK_MANAGER)
+
+#if defined(ENABLE_WAYLAND_SERVER)
+// Enables Wayland display server support.
+const char kEnableWaylandServer[] = "enable-wayland-server";
+#endif
+
+#if defined(OS_ANDROID)
+// Sets the threshold for when to disable auto-hiding the toolbar. If the
+// device's width and height are above the threshold, the toolbar will never
+// be hidden due to scrolling. It will still be hidden if page contents go
+// fullscreen.
+const char kDisableAutoHidingToolbarThreshold[] =
+    "disable-auto-hiding-toolbar-threshold";
+
+// Forces the update menu item to show.
+const char kForceShowUpdateMenuItem[] = "force-show-update-menu-item";
+
+// Forces a summary to be displayed below the update menu item.
+const char kForceShowUpdateMenuItemSummary[] = "show_summary";
+
+// Forces the new features summary to be displayed below the update menu item.
+const char kForceShowUpdateMenuItemNewFeaturesSummary[] =
+    "use_new_features_summary";
+
+// Forces a custom summary to be displayed below the update menu item.
+const char kForceShowUpdateMenuItemCustomSummary[] = "custom_summary";
+
+// Forces the update menu badge to show.
+const char kForceShowUpdateMenuBadge[] = "force-show-update-menu-badge";
+
+// Sets the market URL for Chrome for use in testing.
+const char kMarketUrlForTesting[] = "market-url-for-testing";
+#endif // defined(OS_ANDROID)
+
+#if defined(OS_WIN) || defined(OS_LINUX)
+extern const char kEnableInputImeAPI[] = "enable-input-ime-api";
+extern const char kDisableInputImeAPI[] = "disable-input-ime-api";
+#endif
+
 bool AboutInSettingsEnabled() {
   return SettingsWindowEnabled() &&
          !base::CommandLine::ForCurrentProcess()->HasSwitch(
              ::switches::kDisableAboutInSettings);
 }
 
-bool MdDownloadsEnabled() {
+bool MdExtensionsEnabled() {
   return base::CommandLine::ForCurrentProcess()->HasSwitch(
-      ::switches::kEnableMaterialDesignDownloads);
+      ::switches::kEnableMaterialDesignExtensions);
 }
 
-bool MdSettingsEnabled() {
+bool MdHistoryEnabled() {
   return base::CommandLine::ForCurrentProcess()->HasSwitch(
-      ::switches::kEnableMaterialDesignSettings);
+      ::switches::kEnableMaterialDesignHistory);
 }
 
-bool MediaRouterEnabled() {
+bool MdPolicyPageEnabled() {
   return base::CommandLine::ForCurrentProcess()->HasSwitch(
-      ::switches::kEnableMediaRouter);
-}
-
-// Will return true as a default value if the "disable-new-offline-error-page"
-// command-line switch is not available.
-bool NewOfflineErrorPageEnabled() {
-  return !base::CommandLine::ForCurrentProcess()->HasSwitch(
-      ::switches::kDisableNewOfflineErrorPage);
-}
-
-bool PdfMaterialUIEnabled() {
-  if (base::CommandLine::ForCurrentProcess()->HasSwitch(kEnablePdfMaterialUI))
-    return true;
-
-  if (base::CommandLine::ForCurrentProcess()->HasSwitch(kDisablePdfMaterialUI))
-    return false;
-
-  // Default.
-  return false;
+      ::switches::kEnableMaterialDesignPolicyPage);
 }
 
 bool SettingsWindowEnabled() {
@@ -1392,6 +1393,17 @@ bool PowerOverlayEnabled() {
 }
 #endif
 
+#if defined(ENABLE_TASK_MANAGER)
+bool NewTaskManagerEnabled() {
+#if defined(OS_MACOSX)
+  return false;  // The new task manager hasn't been adopted on Mac yet.
+#else
+  return !base::CommandLine::ForCurrentProcess()->HasSwitch(
+      kDisableNewTaskManager);
+#endif  // defined(OS_MACOSX)
+}
+#endif  // defined(ENABLE_TASK_MANAGER)
+
 // -----------------------------------------------------------------------------
 // DO NOT ADD YOUR CRAP TO THE BOTTOM OF THIS FILE.
 //
