@@ -2,21 +2,17 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
-// Defines all the command-line switches used by Chrome.
+// Defines the shared command-line switches used by code in the Chrome
+// directory that don't have anywhere more specific to go.
 
 #ifndef CHROME_COMMON_CHROME_SWITCHES_H_
 #define CHROME_COMMON_CHROME_SWITCHES_H_
 
 #include "build/build_config.h"
 
-#include "base/base_switches.h"
-#ifndef TOOLKIT_QT
-#include "components/autofill/core/common/autofill_switches.h"
-#include "components/data_reduction_proxy/core/common/data_reduction_proxy_switches.h"
-#include "components/password_manager/core/common/password_manager_switches.h"
-#include "components/signin/core/common/signin_switches.h"
-#endif
-#include "content/public/common/content_switches.h"
+// Don't add more switch files here. This is linked into some places like the
+// installer where dependencies should be limited. Instead, have files
+// directly include your switch file.
 
 namespace switches {
 
@@ -39,14 +35,13 @@ extern const char kAlternativeServiceProbabilityThreshold[];
 extern const char kAlwaysAuthorizePlugins[];
 extern const char kAppId[];
 extern const char kApp[];
-extern const char kAppsCheckoutURL[];
 extern const char kAppsGalleryDownloadURL[];
 extern const char kAppsGalleryURL[];
 extern const char kAppsGalleryUpdateURL[];
 extern const char kAppModeAuthCode[];
 extern const char kAppModeOAuth2Token[];
 extern const char kAuthExtensionPath[];
-extern const char kAutoLaunchAtStartup[];
+extern const char kAuthServerWhitelist[];
 extern const char kAutoSelectDesktopCaptureSource[];
 extern const char kBypassAppBannerEngagementChecks[];
 extern const char kCertificateTransparencyLog[];
@@ -71,40 +66,38 @@ extern const char kDisableBackgroundNetworking[];
 extern const char kDisableBundledPpapiFlash[];
 extern const char kDisableCastStreamingHWEncoding[];
 extern const char kDisableChildAccountDetection[];
+extern const char kDisableClearBrowsingDataCounters[];
 extern const char kDisableClientSidePhishingDetection[];
 extern const char kDisableComponentExtensionsWithBackgroundPages[];
 extern const char kDisableComponentUpdate[];
 extern const char kDisableDefaultApps[];
 extern const char kDisableDeviceDiscoveryNotifications[];
-extern const char kDisableDinosaurEasterEgg[];
 extern const char kDisableDomainReliability[];
+extern const char kDisableDownloadNotification[];
 extern const char kDisableExtensionsFileAccessCheck[];
 extern const char kDisableExtensionsHttpThrottling[];
 extern const char kDisableExtensions[];
+extern const char kDisableFieldTrialTestingConfig[];
 extern const char kDisableJavaScriptHarmonyShipping[];
+extern const char kDisableMaterialDesignDownloads[];
 extern const char kDisableMinimizeOnSecondLauncherItemClick[];
 extern const char kDisableNewBookmarkApps[];
-extern const char kDisableNewOfflineErrorPage[];
+extern const char kDisableNTPPopularSites[];
 extern const char kDisableOfflineAutoReload[];
 extern const char kDisableOfflineAutoReloadVisibleOnly[];
 extern const char kDisableOutOfProcessPac[];
 extern const char kDisablePasswordManagerReauthentication[];
-extern const char kDisablePdfMaterialUI[];
-extern const char kDisablePermissionsBubbles[];
+extern const char kDisablePermissionsBlacklist[];
 extern const char kDisablePopupBlocking[];
 extern const char kDisablePreconnect[];
 extern const char kDisablePrintPreview[];
 extern const char kDisablePromptOnRepost[];
+extern const char kDisablePushApiBackgroundMode[];
 extern const char kDisableQuic[];
 extern const char kDisableQuicPortSelection[];
-extern const char kDisableSavePasswordBubble[];
-extern const char kDisableSdchPersistence[];
 extern const char kDisableSessionCrashedBubble[];
 extern const char kDisableSiteEngagementService[];
-extern const char kDisableSuggestionsService[];
-extern const char kDisableSync[];
-extern const char kDisableSyncTypes[];
-extern const char kDisableWebResources[];
+extern const char kDisableWebNotificationCustomLayouts[];
 extern const char kDisableZeroBrowsersOpenForTests[];
 extern const char kDiskCacheDir[];
 extern const char kDiskCacheSize[];
@@ -114,57 +107,57 @@ extern const char kDumpBrowserHistograms[];
 extern const char kEasyUnlockAppPath[];
 extern const char kEnableAddToShelf[];
 extern const char kEnableAppsFileAssociations[];
+extern const char kEnableAudioDebugRecordingsFromExtension[];
 extern const char kEnableBenchmarking[];
 extern const char kEnableBookmarkUndo[];
 extern const char kEnableChildAccountDetection[];
+extern const char kEnableClearBrowsingDataCounters[];
 extern const char kEnableCloudPrintProxy[];
 extern const char kEnableDevToolsExperiments[];
 extern const char kEnableDeviceDiscoveryNotifications[];
-extern const char kEnableDomDistiller[];
-extern const char kEnhancedBookmarksExperiment[];
 extern const char kEnableDomainReliability[];
-extern const char kEnableDownloadNotification[];
-extern const char kEnableEphemeralAppsInWebstore[];
 extern const char kEnableExperimentalHotwordHardware[];
 extern const char kEnableExtensionActivityLogging[];
 extern const char kEnableExtensionActivityLogTesting[];
 extern const char kEnableFastUnload[];
-extern const char kEnableLinkableEphemeralApps[];
 extern const char kEnableMaterialDesignDownloads[];
-extern const char kEnableMaterialDesignSettings[];
-extern const char kEnableMediaRouter[];
+extern const char kEnableMaterialDesignExtensions[];
+extern const char kEnableMaterialDesignHistory[];
+extern const char kEnableMaterialDesignPolicyPage[];
 extern const char kEnableNaCl[];
+extern const char kEnableNativeNotifications[];
 extern const char kEnableNavigationTracing[];
 extern const char kEnableNetBenchmarking[];
 extern const char kEnableNewBookmarkApps[];
+extern const char kEnableNTPPopularSites[];
+extern const char kEnableNTPSearchEngineCountryDetection[];
 extern const char kEnableOfflineAutoReload[];
 extern const char kEnableOfflineAutoReloadVisibleOnly[];
 extern const char kEnablePanels[];
-extern const char kEnablePdfMaterialUI[];
-extern const char kEnablePermissionsBubbles[];
-extern const char kEnablePluginPlaceholderShadowDom[];
+extern const char kDisablePanels[];
+extern const char kEnablePermissionsBlacklist[];
 extern const char kEnablePotentiallyAnnoyingSecurityFeatures[];
 extern const char kEnablePowerOverlay[];
 extern const char kEnablePrintPreviewRegisterPromos[];
 extern const char kEnableProfiling[];
-extern const char kEnableQueryExtraction[];
+extern const char kEnablePushApiBackgroundMode[];
 extern const char kEnableQuic[];
 extern const char kEnableQuicPortSelection[];
-extern const char kEnableSavePasswordBubble[];
-extern const char kEnableSdchPersistence[];
+extern const char kEnableAlternativeServices[];
 extern const char kEnableSessionCrashedBubble[];
 extern const char kEnableSettingsWindow[];
 extern const char kDisableSettingsWindow[];
+extern const char kEnableSimplifiedFullscreenUI[];
+extern const char kDisableSimplifiedFullscreenUI[];
+extern const char kEnableSiteEngagementAppBanner[];
+extern const char kEnableSiteEngagementEvictionPolicy[];
 extern const char kEnableSiteEngagementService[];
-extern const char kEnableSuggestionsService[];
 extern const char kEnableSupervisedUserManagedBookmarksFolder[];
-extern const char kEnableSyncArticles[];
 extern const char kEnableTabAudioMuting[];
 extern const char kEnableThumbnailRetargeting[];
 extern const char kEnableUserAlternateProtocolPorts[];
 extern const char kEnableWebAppFrame[];
-extern const char kEnableWifiCredentialSync[];
-extern const char kEnhancedBookmarksExperiment[];
+extern const char kEnableWebNotificationCustomLayouts[];
 extern const char kExtensionContentVerificationBootstrap[];
 extern const char kExtensionContentVerificationEnforceStrict[];
 extern const char kExtensionContentVerificationEnforce[];
@@ -172,14 +165,12 @@ extern const char kExtensionContentVerification[];
 extern const char kExtensionsInstallVerification[];
 extern const char kExtensionsNotWebstore[];
 extern const char kExtensionsUpdateFrequency[];
-extern const char kFakeVariationsChannel[];
 extern const char kFastStart[];
-extern const char kFlagSwitchesBegin[];
-extern const char kFlagSwitchesEnd[];
 extern const char kForceAppMode[];
 extern const char kForceFieldTrialParams[];
 extern const char kForceFirstRun[];
 extern const char kForceVariationIds[];
+extern const char kForceLocalNtp[];
 extern const char kHistoryEnableGroupByDomain[];
 extern const char kHomePage[];
 extern const char kHostRules[];
@@ -187,10 +178,9 @@ extern const char kHostResolverRetryAttempts[];
 extern const char kIgnoreUrlFetcherCertRequests[];
 extern const char kIncognito[];
 extern const char kInstallChromeApp[];
-extern const char kInstallEphemeralAppFromWebstore[];
 extern const char kInstallSupervisedUserWhitelists[];
 extern const char kInstantProcess[];
-extern const char kInvalidationUseGCMChannel[];
+extern const char kInterestsURL[];
 extern const char kIpcDumpDirectory[];
 extern const char kJavaScriptHarmony[];
 extern const char kKeepAliveForTest[];
@@ -198,14 +188,14 @@ extern const char kKioskMode[];
 extern const char kKioskModePrinting[];
 extern const char kLoadComponentExtension[];
 extern const char kLoadExtension[];
+#if !defined(GOOGLE_CHROME_BUILD)
+extern const char kLocalNtpReload[];
+#endif
 extern const char kMakeDefaultBrowser[];
 extern const char kManualEnhancedBookmarks[];
 extern const char kManualEnhancedBookmarksOptout[];
-extern const char kMarkNonSecureAs[];
-extern const char kMarkNonSecureAsNeutral[];
-extern const char kMarkNonSecureAsDubious[];
-extern const char kMarkNonSecureAsNonSecure[];
 extern const char kMediaCacheSize[];
+extern const char kMediaRouter[];
 extern const char kMessageLoopHistogrammer[];
 extern const char kMetricsRecordingOnly[];
 extern const char kMonitoringDestinationID[];
@@ -248,6 +238,7 @@ extern const char kProxyBypassList[];
 extern const char kProxyPacUrl[];
 extern const char kProxyServer[];
 extern const char kQuicConnectionOptions[];
+extern const char kQuicHostWhitelist[];
 extern const char kQuicMaxPacketLength[];
 extern const char kQuicVersion[];
 extern const char kRemoteDebuggingTargets[];
@@ -257,14 +248,12 @@ extern const char kSavePageAsMHTML[];
 extern const char kSbDisableAutoUpdate[];
 extern const char kSbDisableDownloadProtection[];
 extern const char kSbDisableExtensionBlacklist[];
+extern const char kSbManualDownloadBlacklist[];
 extern const char kServiceProcess[];
 extern const char kSilentDebuggerExtensionAPI[];
 extern const char kSilentLaunch[];
 extern const char kShowAppList[];
-extern const char kShowSavedCopy[];
-extern const char kEnableShowSavedCopyPrimary[];
-extern const char kEnableShowSavedCopySecondary[];
-extern const char kDisableShowSavedCopy[];
+extern const char kSSLKeyLogFile[];
 extern const char kSimulateElevatedRecovery[];
 extern const char kSimulateUpgrade[];
 extern const char kSimulateCriticalUpdate[];
@@ -278,30 +267,17 @@ extern const char kSpeculativeResourcePrefetchingLearning[];
 #if defined(OS_ANDROID)
 extern const char kEnableAndroidSpellChecker[];
 #endif
+extern const char kDisableMultilingualSpellChecker[];
 extern const char kEnableMultilingualSpellChecker[];
-extern const char kEnableSpellingAutoCorrect[];
 extern const char kEnableSpellingFeedbackFieldTrial[];
 extern const char kSpellingServiceFeedbackUrl[];
 extern const char kSpellingServiceFeedbackIntervalSeconds[];
 #endif
-extern const char kSSLVersionMax[];
-extern const char kSSLVersionMin[];
-extern const char kSSLVersionFallbackMin[];
-extern const char kSSLVersionTLSv1[];
-extern const char kSSLVersionTLSv11[];
-extern const char kSSLVersionTLSv12[];
 extern const char kStartMaximized[];
 extern const char kSupervisedUserId[];
 extern const char kSupervisedUserSafeSites[];
 extern const char kSupervisedUserSyncToken[];
-extern const char kSyncShortInitialRetryOverride[];
-extern const char kSyncServiceURL[];
-extern const char kSyncDisableDeferredStartup[];
-extern const char kSyncDeferredStartupTimeoutSeconds[];
-extern const char kSyncEnableGetUpdateAvoidance[];
-extern const char kSyncEnableClearDataOnPassphraseEncryption[];
-extern const char kSyncDisableBackup[];
-extern const char kSyncDisableRollback[];
+extern const char kSystemLogUploadFrequency[];
 extern const char kTestName[];
 extern const char kTrustedSpdyProxy[];
 extern const char kTryChromeAgain[];
@@ -315,7 +291,6 @@ extern const char kUserDataDir[];
 extern const char kV8PacMojoInProcess[];
 extern const char kV8PacMojoOutOfProcess[];
 extern const char kValidateCrx[];
-extern const char kVariationsServerURL[];
 extern const char kVersion[];
 extern const char kWindowPosition[];
 extern const char kWindowSize[];
@@ -323,18 +298,23 @@ extern const char kWinHttpProxyResolver[];
 extern const char kWinJumplistAction[];
 
 #if defined(OS_ANDROID)
-extern const char kDisableCast[];
+extern const char kAuthAndroidNegotiateAccountType[];
 extern const char kDisableContextualSearch[];
-extern const char kDisableZeroSuggest[];
 extern const char kEnableAccessibilityTabSwitcher[];
 extern const char kEnableContextualSearch[];
-extern const char kEnableDomDistillerButtonAnimation[];
 extern const char kEnableHostedMode[];
-extern const char kEnableInstantSearchClicks[];
-extern const char kEnableReaderModeToolbarIcon[];
-extern const char kEnableZeroSuggestMostVisited[];
-extern const char kEnableZeroSuggestMostVisitedWithoutSerp[];
-extern const char kPrefetchSearchResults[];
+extern const char kEnableHungRendererInfoBar[];
+extern const char kProgressBarAnimation[];
+extern const char kEnableThemeColorInTabbedMode[];
+extern const char kDisableAutoHidingToolbarThreshold[];
+extern const char kEnableTabSwitcherInDocumentMode[];
+extern const char kNtpSwitchToExistingTab[];
+extern const char kForceShowUpdateMenuItem[];
+extern const char kForceShowUpdateMenuItemSummary[];
+extern const char kForceShowUpdateMenuItemNewFeaturesSummary[];
+extern const char kForceShowUpdateMenuItemCustomSummary[];
+extern const char kForceShowUpdateMenuBadge[];
+extern const char kMarketUrlForTesting[];
 #endif  // defined(OS_ANDROID)
 
 #if defined(USE_ASH)
@@ -357,27 +337,38 @@ extern const char kHostedAppQuitNotification[];
 extern const char kDisableAppInfoDialogMac[];
 extern const char kDisableAppWindowCycling[];
 extern const char kDisableHostedAppShimCreation[];
+extern const char kDisableHostedAppsInWindows[];
 extern const char kDisableMacViewsNativeAppWindows[];
+extern const char kDisableSavePasswordBubble[];
+extern const char kDisableTranslateNewUX[];
 extern const char kEnableAppInfoDialogMac[];
 extern const char kEnableAppWindowCycling[];
+extern const char kEnableHostedAppsInWindows[];
 extern const char kEnableMacViewsNativeAppWindows[];
+extern const char kEnableMacViewsDialogs[];
+extern const char kEnableSavePasswordBubble[];
 extern const char kEnableTranslateNewUX[];
 extern const char kMetricsClientID[];
 extern const char kRelauncherProcess[];
+extern const char kRelauncherProcessDMGDevice[];
 #endif  // defined(OS_MACOSX)
 
 #if defined(OS_WIN)
+extern const char kCrashpadHandler[];
 extern const char kEnableCloudPrintXps[];
 extern const char kEnableProfileShortcutManager[];
 extern const char kForceDesktop[];
 extern const char kForceImmersive[];
 extern const char kHideIcons[];
 extern const char kNoNetworkProfileWarning[];
+extern const char kPrefetchArgumentBrowserBackground[];
+extern const char kPrefetchArgumentWatcher[];
 extern const char kRelaunchShortcut[];
 extern const char kShowIcons[];
 extern const char kUninstall[];
 extern const char kViewerLaunchViaAppId[];
 extern const char kWaitForMutex[];
+extern const char kWatcherProcess[];
 extern const char kWindows8Search[];
 #endif  // defined(OS_WIN)
 
@@ -393,15 +384,30 @@ extern const char kDebugPrint[];
 extern const char kAllowNaClCrxFsAPI[];
 extern const char kAllowNaClFileHandleAPI[];
 extern const char kAllowNaClSocketAPI[];
-extern const char kDisablePluginPowerSaver[];
-extern const char kEnablePluginPowerSaver[];
+#endif
+
+#if defined(FULL_SAFE_BROWSING)
+extern const char kAllowUncheckedDangerousDownloads[];
+extern const char kDisallowUncheckedDangerousDownloads[];
+#endif
+
+#if defined(ENABLE_TASK_MANAGER)
+extern const char kDisableNewTaskManager[];
+#endif  // defined(ENABLE_TASK_MANAGER)
+
+#if defined(ENABLE_WAYLAND_SERVER)
+extern const char kEnableWaylandServer[];
+#endif
+
+#if defined(OS_WIN) || defined(OS_LINUX)
+extern const char kEnableInputImeAPI[];
+extern const char kDisableInputImeAPI[];
 #endif
 
 bool AboutInSettingsEnabled();
-bool MdDownloadsEnabled();
-bool MdSettingsEnabled();
-bool MediaRouterEnabled();
-bool NewOfflineErrorPageEnabled();
+bool MdExtensionsEnabled();
+bool MdHistoryEnabled();
+bool MdPolicyPageEnabled();
 bool PdfMaterialUIEnabled();
 bool SettingsWindowEnabled();
 
@@ -409,6 +415,10 @@ bool SettingsWindowEnabled();
 bool PowerOverlayEnabled();
 #endif
 
+#if defined(ENABLE_TASK_MANAGER)
+bool NewTaskManagerEnabled();
+#endif  // defined(ENABLE_TASK_MANAGER)
+
 // DON'T ADD RANDOM STUFF HERE. Put it in the main section above in
 // alphabetical order, or in one of the ifdefs (also in order in each section).
 
