@@ -9,13 +9,6 @@
 
 #include <windows.h>
 
-// directory names
-extern const wchar_t kAppDataDirName[];
-extern const wchar_t kCanaryAppDataDirName[];
-extern const wchar_t kLocalStateFilename[];
-extern const wchar_t kPreferencesFilename[];
-extern const wchar_t kUserDataDirName[];
-
 namespace blacklist {
 
 // The registry path of the blacklist beacon.
