@@ -409,7 +409,7 @@ bool IsIDNComponentSafe(const base::char16* str,
       icu::UnicodeString(
           L"[[\\ \u00ad\u00bc\u00bd\u01c3\u0337\u0338"
           L"\u05c3\u05f4\u06d4\u0702\u115f\u1160][\u2000-\u200b]"
-          L"[\u2010\u2024\u2027\u2028\u2029\u2039\u203a\u2044\u205f]"
+          L"[\u2024\u2027\u2028\u2029\u2039\u203a\u2044\u205f]"
           L"[\u2154-\u2156][\u2159-\u215b][\u215f\u2215\u23ae"
           L"\u29f6\u29f8\u2afb\u2afd][\u2ff0-\u2ffb][\u3014"
           L"\u3015\u3033\u3164\u321d\u321e\u33ae\u33af\u33c6\u33df\ufe14"
@@ -421,8 +421,6 @@ bool IsIDNComponentSafe(const base::char16* str,
       icu::UnicodeString(
           // Lone katakana no, so, or n
           L"[^\\p{Katakana}][\u30ce\u30f3\u30bd][^\\p{Katakana}]"
-          // - Disalow mixing of Latin and Canadian Syllabary.
-          L"|[\\p{sc=cans}].*[a-z]|[a-z].*[\\p{sc=cans}]"
           // Repeating Japanese accent characters
           L"|[\u3099\u309a\u309b\u309c][\u3099\u309a\u309b\u309c]"),
       0, status);
@@ -431,12 +429,9 @@ bool IsIDNComponentSafe(const base::char16* str,
       icu::UnicodeString(
           "[[\\u0020\\u00ad\\u00bc\\u00bd\\u01c3\\u0337\\u0338"
           "\\u05c3\\u05f4\\u06d4\\u0702\\u115f\\u1160][\\u2000-\\u200b]"
-          "[\\u2010\\u2024\\u2027\\u2028\\u2029\\u2039\\u203a\\u2044\\u205f]"
+          "[\\u2024\\u2027\\u2028\\u2029\\u2039\\u203a\\u2044\\u205f]"
           "[\\u2154-\\u2156][\\u2159-\\u215b][\\u215f\\u2215\\u23ae"
           "\\u29f6\\u29f8\\u2afb\\u2afd][\\u2ff0-\\u2ffb][\\u3014"
-#if defined(OS_MACOSX)
-          "\\u0f8cu\\u0f8du\\u0f8eu\\u0f8fu"
-#endif
           "\\u3015\\u3033\\u3164\\u321d\\u321e\\u33ae\\u33af\\u33c6\\u33df\\ufe"
           "14"
           "\\ufe15\\ufe3f\\ufe5d\\ufe5e\\ufeff\\uff0e\\uff06\\uff61\\uffa0\\uff"
@@ -449,8 +444,6 @@ bool IsIDNComponentSafe(const base::char16* str,
       icu::UnicodeString(
           // Lone katakana no, so, or n
           "[^\\p{Katakana}][\\u30ce\\u30f3\\u30bd][^\\p{Katakana}]"
-          // - Disalow mixing of Latin and Canadian Syllabary.
-          "|[\\p{sc=cans}].*[a-z]|[a-z].*[\\p{sc=cans}]"
           // Repeating Japanese accent characters
           "|[\\u3099\\u309a\\u309b\\u309c][\\u3099\\u309a\\u309b\\u309c]"),
       0, status);
