@@ -2,11 +2,15 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+#include "content/browser/compositor/buffer_queue.h"
+
+#include <stddef.h>
+#include <stdint.h>
 #include <set>
+#include <utility>
 
 #include "cc/test/test_context_provider.h"
 #include "cc/test/test_web_graphics_context_3d.h"
-#include "content/browser/compositor/buffer_queue.h"
 #include "content/browser/compositor/gpu_surfaceless_browser_compositor_output_surface.h"
 #include "content/browser/gpu/browser_gpu_memory_buffer_manager.h"
 #include "content/common/gpu/client/gl_helper.h"
@@ -27,11 +31,17 @@ class StubGpuMemoryBufferImpl : public gfx::GpuMemoryBuffer {
   StubGpuMemoryBufferImpl() {}
 
   // Overridden from gfx::GpuMemoryBuffer:
-  bool Map(void** data) override { return false; }
+  bool Map() override { return false; }
+  void* memory(size_t plane) override { return nullptr; }
   void Unmap() override {}
-  bool IsMapped() const override { return false; }
-  Format GetFormat() const override { return gfx::GpuMemoryBuffer::RGBX_8888; }
-  void GetStride(int* stride) const override {}
+  gfx::Size GetSize() const override { return gfx::Size(); }
+  gfx::BufferFormat GetFormat() const override {
+    return gfx::BufferFormat::BGRX_8888;
+  }
+  int stride(size_t plane) const override { return 0; }
+  gfx::GpuMemoryBufferId GetId() const override {
+    return gfx::GpuMemoryBufferId(0);
+  }
   gfx::GpuMemoryBufferHandle GetHandle() const override {
     return gfx::GpuMemoryBufferHandle();
   }
@@ -42,22 +52,32 @@ class StubGpuMemoryBufferImpl : public gfx::GpuMemoryBuffer {
 
 class StubBrowserGpuMemoryBufferManager : public BrowserGpuMemoryBufferManager {
  public:
-  StubBrowserGpuMemoryBufferManager() : BrowserGpuMemoryBufferManager(1) {}
+  StubBrowserGpuMemoryBufferManager()
+      : BrowserGpuMemoryBufferManager(1, 1), allocate_succeeds_(true) {}
+
+  void set_allocate_succeeds(bool value) { allocate_succeeds_ = value; }
 
   scoped_ptr<gfx::GpuMemoryBuffer> AllocateGpuMemoryBufferForScanout(
       const gfx::Size& size,
-      gfx::GpuMemoryBuffer::Format format,
-      int32 surface_id) override {
-    return make_scoped_ptr<gfx::GpuMemoryBuffer>(new StubGpuMemoryBufferImpl);
+      gfx::BufferFormat format,
+      int32_t surface_id) override {
+    if (allocate_succeeds_)
+      return make_scoped_ptr<gfx::GpuMemoryBuffer>(new StubGpuMemoryBufferImpl);
+    return nullptr;
   }
+
+ private:
+  bool allocate_succeeds_;
 };
 
 class MockBufferQueue : public BufferQueue {
  public:
   MockBufferQueue(scoped_refptr<cc::ContextProvider> context_provider,
                   BrowserGpuMemoryBufferManager* gpu_memory_buffer_manager,
+                  unsigned int target,
                   unsigned int internalformat)
       : BufferQueue(context_provider,
+                    target,
                     internalformat,
                     nullptr,
                     gpu_memory_buffer_manager,
@@ -76,40 +96,43 @@ class BufferQueueTest : public ::testing::Test {
 
   void InitWithContext(scoped_ptr<cc::TestWebGraphicsContext3D> context) {
     scoped_refptr<cc::TestContextProvider> context_provider =
-        cc::TestContextProvider::Create(context.Pass());
+        cc::TestContextProvider::Create(std::move(context));
     context_provider->BindToCurrentThread();
     gpu_memory_buffer_manager_.reset(new StubBrowserGpuMemoryBufferManager);
-    mock_output_surface_ = new MockBufferQueue(
-        context_provider, gpu_memory_buffer_manager_.get(), GL_RGBA);
+    mock_output_surface_ =
+        new MockBufferQueue(context_provider, gpu_memory_buffer_manager_.get(),
+                            GL_TEXTURE_2D, GL_RGBA);
     output_surface_.reset(mock_output_surface_);
     output_surface_->Initialize();
   }
 
-  unsigned current_surface() { return output_surface_->current_surface_.image; }
-  const std::vector<BufferQueue::AllocatedSurface>& available_surfaces() {
+  unsigned current_surface() {
+    return output_surface_->current_surface_
+               ? output_surface_->current_surface_->image
+               : 0;
+  }
+  const std::vector<scoped_ptr<BufferQueue::AllocatedSurface>>&
+  available_surfaces() {
     return output_surface_->available_surfaces_;
   }
-  const std::deque<BufferQueue::AllocatedSurface>& in_flight_surfaces() {
+  std::deque<scoped_ptr<BufferQueue::AllocatedSurface>>& in_flight_surfaces() {
     return output_surface_->in_flight_surfaces_;
   }
 
-  const BufferQueue::AllocatedSurface& displayed_frame() {
-    return output_surface_->displayed_surface_;
+  const BufferQueue::AllocatedSurface* displayed_frame() {
+    return output_surface_->displayed_surface_.get();
   }
-  const BufferQueue::AllocatedSurface& current_frame() {
-    return output_surface_->current_surface_;
+  const BufferQueue::AllocatedSurface* current_frame() {
+    return output_surface_->current_surface_.get();
   }
-  const BufferQueue::AllocatedSurface& last_frame() {
-    return output_surface_->in_flight_surfaces_.back();
-  }
-  const BufferQueue::AllocatedSurface& next_frame() {
-    return output_surface_->available_surfaces_.back();
+  const BufferQueue::AllocatedSurface* next_frame() {
+    return output_surface_->available_surfaces_.back().get();
   }
   const gfx::Size size() { return output_surface_->size_; }
 
   int CountBuffers() {
     int n = available_surfaces().size() + in_flight_surfaces().size() +
-            (displayed_frame().texture ? 1 : 0);
+            (displayed_frame() ? 1 : 0);
     if (current_surface())
       n++;
     return n;
@@ -119,14 +142,14 @@ class BufferQueueTest : public ::testing::Test {
   void CheckUnique() {
     std::set<unsigned> buffers;
     EXPECT_TRUE(InsertUnique(&buffers, current_surface()));
-    EXPECT_TRUE(InsertUnique(&buffers, displayed_frame().image));
-    for (size_t i = 0; i < available_surfaces().size(); i++)
-      EXPECT_TRUE(InsertUnique(&buffers, available_surfaces()[i].image));
-    for (std::deque<BufferQueue::AllocatedSurface>::const_iterator it =
-             in_flight_surfaces().begin();
-         it != in_flight_surfaces().end();
-         ++it)
-      EXPECT_TRUE(InsertUnique(&buffers, it->image));
+    if (displayed_frame())
+      EXPECT_TRUE(InsertUnique(&buffers, displayed_frame()->image));
+    for (auto& surface : available_surfaces())
+      EXPECT_TRUE(InsertUnique(&buffers, surface->image));
+    for (auto& surface : in_flight_surfaces()) {
+      if (surface)
+        EXPECT_TRUE(InsertUnique(&buffers, surface->image));
+    }
   }
 
   void SwapBuffers() {
@@ -155,7 +178,7 @@ class BufferQueueTest : public ::testing::Test {
     return true;
   }
 
-  scoped_ptr<BrowserGpuMemoryBufferManager> gpu_memory_buffer_manager_;
+  scoped_ptr<StubBrowserGpuMemoryBufferManager> gpu_memory_buffer_manager_;
   scoped_ptr<BufferQueue> output_surface_;
   MockBufferQueue* mock_output_surface_;
   bool doublebuffering_;
@@ -202,6 +225,7 @@ class BufferQueueMockedContextTest : public BufferQueueTest {
 };
 
 scoped_ptr<BufferQueue> CreateOutputSurfaceWithMock(
+    unsigned int target,
     MockedContext** context,
     BrowserGpuMemoryBufferManager* gpu_memory_buffer_manager) {
   *context = new MockedContext();
@@ -209,18 +233,19 @@ scoped_ptr<BufferQueue> CreateOutputSurfaceWithMock(
       cc::TestContextProvider::Create(
           scoped_ptr<cc::TestWebGraphicsContext3D>(*context));
   context_provider->BindToCurrentThread();
-  scoped_ptr<BufferQueue> buffer_queue(new BufferQueue(
-      context_provider, GL_RGBA, nullptr, gpu_memory_buffer_manager, 1));
+  scoped_ptr<BufferQueue> buffer_queue(
+      new BufferQueue(context_provider, target, GL_RGBA, nullptr,
+                      gpu_memory_buffer_manager, 1));
   buffer_queue->Initialize();
-  return buffer_queue.Pass();
+  return buffer_queue;
 }
 
 TEST(BufferQueueStandaloneTest, FboInitialization) {
   MockedContext* context;
   scoped_ptr<BrowserGpuMemoryBufferManager> gpu_memory_buffer_manager(
       new StubBrowserGpuMemoryBufferManager);
-  scoped_ptr<BufferQueue> output_surface =
-      CreateOutputSurfaceWithMock(&context, gpu_memory_buffer_manager.get());
+  scoped_ptr<BufferQueue> output_surface = CreateOutputSurfaceWithMock(
+      GL_TEXTURE_2D, &context, gpu_memory_buffer_manager.get());
 
   EXPECT_CALL(*context, bindFramebuffer(GL_FRAMEBUFFER, Ne(0U)));
   ON_CALL(*context, framebufferTexture2D(_, _, _, _, _))
@@ -230,29 +255,33 @@ TEST(BufferQueueStandaloneTest, FboInitialization) {
 }
 
 TEST(BufferQueueStandaloneTest, FboBinding) {
-  MockedContext* context;
-  scoped_ptr<BrowserGpuMemoryBufferManager> gpu_memory_buffer_manager(
-      new StubBrowserGpuMemoryBufferManager);
-  scoped_ptr<BufferQueue> output_surface =
-      CreateOutputSurfaceWithMock(&context, gpu_memory_buffer_manager.get());
-  EXPECT_CALL(*context, bindTexture(GL_TEXTURE_2D, Ne(0U)));
-  EXPECT_CALL(*context, destroyImageCHROMIUM(1));
-  Expectation image =
-      EXPECT_CALL(*context, createImageCHROMIUM(_, 0, 0, GL_RGBA))
-          .WillOnce(Return(1));
-  Expectation fb =
-      EXPECT_CALL(*context, bindFramebuffer(GL_FRAMEBUFFER, Ne(0U)));
-  Expectation tex = EXPECT_CALL(*context, bindTexture(GL_TEXTURE_2D, Ne(0U)));
-  Expectation bind_tex =
-      EXPECT_CALL(*context, bindTexImage2DCHROMIUM(GL_TEXTURE_2D, 1))
-          .After(tex, image);
-  EXPECT_CALL(
-      *context,
-      framebufferTexture2D(
-          GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0, GL_TEXTURE_2D, Ne(0U), _))
-      .After(fb, bind_tex);
+  GLenum targets[] = { GL_TEXTURE_2D, GL_TEXTURE_RECTANGLE_ARB };
+  for (size_t i = 0; i < 2; ++i) {
+    GLenum target = targets[i];
+    MockedContext* context;
+    scoped_ptr<BrowserGpuMemoryBufferManager> gpu_memory_buffer_manager(
+        new StubBrowserGpuMemoryBufferManager);
+    scoped_ptr<BufferQueue> output_surface = CreateOutputSurfaceWithMock(
+        target, &context, gpu_memory_buffer_manager.get());
+    EXPECT_CALL(*context, bindTexture(target, Ne(0U)));
+    EXPECT_CALL(*context, destroyImageCHROMIUM(1));
+    Expectation image =
+        EXPECT_CALL(*context, createImageCHROMIUM(_, 0, 0, GL_RGBA))
+            .WillOnce(Return(1));
+    Expectation fb =
+        EXPECT_CALL(*context, bindFramebuffer(GL_FRAMEBUFFER, Ne(0U)));
+    Expectation tex = EXPECT_CALL(*context, bindTexture(target, Ne(0U)));
+    Expectation bind_tex =
+        EXPECT_CALL(*context, bindTexImage2DCHROMIUM(target, 1))
+            .After(tex, image);
+    EXPECT_CALL(
+        *context,
+        framebufferTexture2D(
+            GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0, target, Ne(0U), _))
+        .After(fb, bind_tex);
 
-  output_surface->BindFramebuffer();
+    output_surface->BindFramebuffer();
+  }
 }
 
 TEST(BufferQueueStandaloneTest, CheckBoundFramebuffer) {
@@ -267,7 +296,7 @@ TEST(BufferQueueStandaloneTest, CheckBoundFramebuffer) {
   gl_helper.reset(new GLHelper(context_provider->ContextGL(),
                                context_provider->ContextSupport()));
 
-  output_surface.reset(new BufferQueue(context_provider, GL_RGBA,
+  output_surface.reset(new BufferQueue(context_provider, GL_TEXTURE_2D, GL_RGBA,
                                        gl_helper.get(),
                                        gpu_memory_buffer_manager.get(), 1));
   output_surface->Initialize();
@@ -299,7 +328,7 @@ TEST_F(BufferQueueTest, PartialSwapReuse) {
   SendDamagedFrame(small_damage);
   SendDamagedFrame(large_damage);
   // Verify that the damage has propagated.
-  EXPECT_EQ(next_frame().damage, large_damage);
+  EXPECT_EQ(next_frame()->damage, large_damage);
 }
 
 TEST_F(BufferQueueTest, PartialSwapFullFrame) {
@@ -311,7 +340,7 @@ TEST_F(BufferQueueTest, PartialSwapFullFrame) {
   SendDamagedFrame(small_damage);
   SendFullFrame();
   SendFullFrame();
-  EXPECT_EQ(next_frame().damage, screen_rect);
+  EXPECT_EQ(next_frame()->damage, screen_rect);
 }
 
 TEST_F(BufferQueueTest, PartialSwapOverlapping) {
@@ -325,7 +354,7 @@ TEST_F(BufferQueueTest, PartialSwapOverlapping) {
   SendFullFrame();
   SendDamagedFrame(small_damage);
   SendDamagedFrame(overlapping_damage);
-  EXPECT_EQ(next_frame().damage, overlapping_damage);
+  EXPECT_EQ(next_frame()->damage, overlapping_damage);
 }
 
 TEST_F(BufferQueueTest, MultipleBindCalls) {
@@ -344,27 +373,27 @@ TEST_F(BufferQueueTest, CheckDoubleBuffering) {
   output_surface_->BindFramebuffer();
   EXPECT_EQ(1, CountBuffers());
   EXPECT_NE(0U, current_surface());
-  EXPECT_FALSE(displayed_frame().texture);
+  EXPECT_FALSE(displayed_frame());
   SwapBuffers();
   EXPECT_EQ(1U, in_flight_surfaces().size());
   output_surface_->PageFlipComplete();
   EXPECT_EQ(0U, in_flight_surfaces().size());
-  EXPECT_TRUE(displayed_frame().texture);
+  EXPECT_TRUE(displayed_frame()->texture);
   output_surface_->BindFramebuffer();
   EXPECT_EQ(2, CountBuffers());
   CheckUnique();
   EXPECT_NE(0U, current_surface());
   EXPECT_EQ(0U, in_flight_surfaces().size());
-  EXPECT_TRUE(displayed_frame().texture);
+  EXPECT_TRUE(displayed_frame()->texture);
   SwapBuffers();
   CheckUnique();
   EXPECT_EQ(1U, in_flight_surfaces().size());
-  EXPECT_TRUE(displayed_frame().texture);
+  EXPECT_TRUE(displayed_frame()->texture);
   output_surface_->PageFlipComplete();
   CheckUnique();
   EXPECT_EQ(0U, in_flight_surfaces().size());
   EXPECT_EQ(1U, available_surfaces().size());
-  EXPECT_TRUE(displayed_frame().texture);
+  EXPECT_TRUE(displayed_frame()->texture);
   output_surface_->BindFramebuffer();
   EXPECT_EQ(2, CountBuffers());
   CheckUnique();
@@ -376,7 +405,7 @@ TEST_F(BufferQueueTest, CheckTripleBuffering) {
 
   // This bit is the same sequence tested in the doublebuffering case.
   output_surface_->BindFramebuffer();
-  EXPECT_FALSE(displayed_frame().texture);
+  EXPECT_FALSE(displayed_frame());
   SwapBuffers();
   output_surface_->PageFlipComplete();
   output_surface_->BindFramebuffer();
@@ -385,19 +414,19 @@ TEST_F(BufferQueueTest, CheckTripleBuffering) {
   EXPECT_EQ(2, CountBuffers());
   CheckUnique();
   EXPECT_EQ(1U, in_flight_surfaces().size());
-  EXPECT_TRUE(displayed_frame().texture);
+  EXPECT_TRUE(displayed_frame()->texture);
   output_surface_->BindFramebuffer();
   EXPECT_EQ(3, CountBuffers());
   CheckUnique();
   EXPECT_NE(0U, current_surface());
   EXPECT_EQ(1U, in_flight_surfaces().size());
-  EXPECT_TRUE(displayed_frame().texture);
+  EXPECT_TRUE(displayed_frame()->texture);
   output_surface_->PageFlipComplete();
   EXPECT_EQ(3, CountBuffers());
   CheckUnique();
   EXPECT_NE(0U, current_surface());
   EXPECT_EQ(0U, in_flight_surfaces().size());
-  EXPECT_TRUE(displayed_frame().texture);
+  EXPECT_TRUE(displayed_frame()->texture);
   EXPECT_EQ(1U, available_surfaces().size());
 }
 
@@ -410,9 +439,9 @@ TEST_F(BufferQueueTest, CheckCorrectBufferOrdering) {
 
   EXPECT_EQ(kSwapCount, in_flight_surfaces().size());
   for (size_t i = 0; i < kSwapCount; ++i) {
-    unsigned int next_texture_id = in_flight_surfaces().front().texture;
+    unsigned int next_texture_id = in_flight_surfaces().front()->texture;
     output_surface_->PageFlipComplete();
-    EXPECT_EQ(displayed_frame().texture, next_texture_id);
+    EXPECT_EQ(displayed_frame()->texture, next_texture_id);
   }
 }
 
@@ -428,7 +457,7 @@ TEST_F(BufferQueueTest, ReshapeWithInFlightSurfaces) {
 
   for (size_t i = 0; i < kSwapCount; ++i) {
     output_surface_->PageFlipComplete();
-    EXPECT_EQ(0u, displayed_frame().texture);
+    EXPECT_FALSE(displayed_frame());
   }
 
   // The dummy surfacess left should be discarded.
@@ -453,16 +482,16 @@ TEST_F(BufferQueueTest, SwapAfterReshape) {
 
   for (size_t i = 0; i < kSwapCount; ++i) {
     output_surface_->PageFlipComplete();
-    EXPECT_EQ(0u, displayed_frame().texture);
+    EXPECT_FALSE(displayed_frame());
   }
 
   CheckUnique();
 
   for (size_t i = 0; i < kSwapCount; ++i) {
-    unsigned int next_texture_id = in_flight_surfaces().front().texture;
+    unsigned int next_texture_id = in_flight_surfaces().front()->texture;
     output_surface_->PageFlipComplete();
-    EXPECT_EQ(displayed_frame().texture, next_texture_id);
-    EXPECT_NE(0u, displayed_frame().texture);
+    EXPECT_EQ(displayed_frame()->texture, next_texture_id);
+    EXPECT_TRUE(displayed_frame());
   }
 }
 
@@ -488,37 +517,37 @@ TEST_F(BufferQueueMockedContextTest, RecreateBuffers) {
   // being drawn to.
   ASSERT_EQ(1U, in_flight_surfaces().size());
   ASSERT_EQ(1U, available_surfaces().size());
-  EXPECT_TRUE(displayed_frame().texture);
-  EXPECT_TRUE(current_frame().texture);
+  EXPECT_TRUE(displayed_frame());
+  EXPECT_TRUE(current_frame());
 
-  auto current = current_frame();
-  auto displayed = displayed_frame();
-  auto in_flight = in_flight_surfaces().front();
-  auto available = available_surfaces().front();
+  auto* current = current_frame();
+  auto* displayed = displayed_frame();
+  auto* in_flight = in_flight_surfaces().front().get();
+  auto* available = available_surfaces().front().get();
 
   // Expect all 4 images to be destroyed, 3 of the existing textures to be
   // copied from and 3 new images to be created.
   EXPECT_CALL(*context_, createImageCHROMIUM(_, 0, 0, GL_RGBA)).Times(3);
-  Expectation copy1 =
-      EXPECT_CALL(*mock_output_surface_,
-                  CopyBufferDamage(_, displayed.texture, _, _)).Times(1);
-  Expectation copy2 =
-      EXPECT_CALL(*mock_output_surface_,
-                  CopyBufferDamage(_, current.texture, _, _)).Times(1);
-  Expectation copy3 =
-      EXPECT_CALL(*mock_output_surface_,
-                  CopyBufferDamage(_, in_flight.texture, _, _)).Times(1);
+  Expectation copy1 = EXPECT_CALL(*mock_output_surface_,
+                                  CopyBufferDamage(_, displayed->texture, _, _))
+                          .Times(1);
+  Expectation copy2 = EXPECT_CALL(*mock_output_surface_,
+                                  CopyBufferDamage(_, current->texture, _, _))
+                          .Times(1);
+  Expectation copy3 = EXPECT_CALL(*mock_output_surface_,
+                                  CopyBufferDamage(_, in_flight->texture, _, _))
+                          .Times(1);
 
-  EXPECT_CALL(*context_, destroyImageCHROMIUM(displayed.image))
+  EXPECT_CALL(*context_, destroyImageCHROMIUM(displayed->image))
       .Times(1)
       .After(copy1);
-  EXPECT_CALL(*context_, destroyImageCHROMIUM(current.image))
+  EXPECT_CALL(*context_, destroyImageCHROMIUM(current->image))
       .Times(1)
       .After(copy2);
-  EXPECT_CALL(*context_, destroyImageCHROMIUM(in_flight.image))
+  EXPECT_CALL(*context_, destroyImageCHROMIUM(in_flight->image))
       .Times(1)
       .After(copy3);
-  EXPECT_CALL(*context_, destroyImageCHROMIUM(available.image)).Times(1);
+  EXPECT_CALL(*context_, destroyImageCHROMIUM(available->image)).Times(1);
   // After copying, we expect the framebuffer binding to be updated.
   EXPECT_CALL(*context_, bindFramebuffer(_, _))
       .After(copy1)
@@ -537,8 +566,57 @@ TEST_F(BufferQueueMockedContextTest, RecreateBuffers) {
   // be replaced but still valid.
   EXPECT_EQ(1U, in_flight_surfaces().size());
   EXPECT_EQ(0U, available_surfaces().size());
-  EXPECT_TRUE(displayed_frame().texture);
-  EXPECT_TRUE(current_frame().texture);
+  EXPECT_TRUE(displayed_frame());
+  EXPECT_TRUE(current_frame());
+}
+
+TEST_F(BufferQueueTest, AllocateFails) {
+  output_surface_->Reshape(screen_size, 1.0f);
+
+  // Succeed in the two swaps.
+  output_surface_->BindFramebuffer();
+  EXPECT_TRUE(current_frame());
+  output_surface_->SwapBuffers(screen_rect);
+
+  // Fail the next surface allocation.
+  gpu_memory_buffer_manager_->set_allocate_succeeds(false);
+  output_surface_->BindFramebuffer();
+  EXPECT_FALSE(current_frame());
+  output_surface_->SwapBuffers(screen_rect);
+  EXPECT_FALSE(current_frame());
+
+  // Try another swap. It should copy the buffer damage from the back
+  // surface.
+  gpu_memory_buffer_manager_->set_allocate_succeeds(true);
+  output_surface_->BindFramebuffer();
+  unsigned int source_texture = in_flight_surfaces().front()->texture;
+  unsigned int target_texture = current_frame()->texture;
+  testing::Mock::VerifyAndClearExpectations(mock_output_surface_);
+  EXPECT_CALL(*mock_output_surface_,
+              CopyBufferDamage(target_texture, source_texture, small_damage, _))
+      .Times(1);
+  output_surface_->SwapBuffers(small_damage);
+  testing::Mock::VerifyAndClearExpectations(mock_output_surface_);
+
+  // Destroy the just-created buffer, and try another swap. The copy should
+  // come from the displayed surface (because both in-flight surfaces are
+  // gone now).
+  output_surface_->PageFlipComplete();
+  in_flight_surfaces().back().reset();
+  EXPECT_EQ(2u, in_flight_surfaces().size());
+  for (auto& surface : in_flight_surfaces())
+    EXPECT_FALSE(surface);
+  output_surface_->BindFramebuffer();
+  source_texture = displayed_frame()->texture;
+  EXPECT_TRUE(current_frame());
+  EXPECT_TRUE(displayed_frame());
+  target_texture = current_frame()->texture;
+  testing::Mock::VerifyAndClearExpectations(mock_output_surface_);
+  EXPECT_CALL(*mock_output_surface_,
+              CopyBufferDamage(target_texture, source_texture, small_damage, _))
+      .Times(1);
+  output_surface_->SwapBuffers(small_damage);
+  testing::Mock::VerifyAndClearExpectations(mock_output_surface_);
 }
 
 }  // namespace
