@@ -5,6 +5,9 @@
 #ifndef CONTENT_BROWSER_COMPOSITOR_REFLECTOR_TEXTURE_H_
 #define CONTENT_BROWSER_COMPOSITOR_REFLECTOR_TEXTURE_H_
 
+#include <stdint.h>
+
+#include "base/macros.h"
 #include "base/memory/ref_counted.h"
 #include "base/memory/scoped_ptr.h"
 #include "content/browser/compositor/owned_mailbox.h"
@@ -31,13 +34,13 @@ class CONTENT_EXPORT ReflectorTexture {
   void CopyTextureFullImage(const gfx::Size& size);
   void CopyTextureSubImage(const gfx::Rect& size);
 
-  uint32 texture_id() const { return texture_id_; }
+  uint32_t texture_id() const { return texture_id_; }
   scoped_refptr<OwnedMailbox> mailbox() { return mailbox_; }
 
  private:
   scoped_refptr<OwnedMailbox> mailbox_;
   scoped_ptr<GLHelper> gl_helper_;
-  uint32 texture_id_;
+  uint32_t texture_id_;
 
   DISALLOW_COPY_AND_ASSIGN(ReflectorTexture);
 };
