@@ -5,8 +5,11 @@
 #ifndef CONTENT_BROWSER_FILE_DESCRIPTOR_INFO_IMPL_H_
 #define CONTENT_BROWSER_FILE_DESCRIPTOR_INFO_IMPL_H_
 
+#include <stddef.h>
+
+#include <vector>
+
 #include "base/memory/scoped_ptr.h"
-#include "base/memory/scoped_vector.h"
 #include "content/common/content_export.h"
 #include "content/public/browser/file_descriptor_info.h"
 
@@ -34,7 +37,7 @@ class FileDescriptorInfoImpl : public FileDescriptorInfo {
   void AddToMapping(int id, base::PlatformFile fd);
   bool HasID(int id) const;
   base::FileHandleMappingVector mapping_;
-  ScopedVector<base::ScopedFD> owned_descriptors_;
+  std::vector<base::ScopedFD> owned_descriptors_;
 };
 }
 
