@@ -3,6 +3,7 @@
 // found in the LICENSE file.
 
 #include "content/browser/frame_host/navigation_request_info.h"
+#include "content/common/service_worker/service_worker_types.h"
 
 namespace content {
 
@@ -10,6 +11,7 @@ NavigationRequestInfo::NavigationRequestInfo(
     const CommonNavigationParams& common_params,
     const BeginNavigationParams& begin_params,
     const GURL& first_party_for_cookies,
+    const url::Origin& request_initiator,
     bool is_main_frame,
     bool parent_is_main_frame,
     int frame_tree_node_id,
@@ -17,11 +19,11 @@ NavigationRequestInfo::NavigationRequestInfo(
     : common_params(common_params),
       begin_params(begin_params),
       first_party_for_cookies(first_party_for_cookies),
+      request_initiator(request_initiator),
       is_main_frame(is_main_frame),
       parent_is_main_frame(parent_is_main_frame),
       frame_tree_node_id(frame_tree_node_id),
-      request_body(request_body) {
-}
+      request_body(request_body) {}
 
 NavigationRequestInfo::~NavigationRequestInfo() {}
 
