@@ -5,14 +5,14 @@
 #ifndef CONTENT_BROWSER_GAMEPAD_GAMEPAD_PLATFORM_DATA_FETCHER_LINUX_H_
 #define CONTENT_BROWSER_GAMEPAD_GAMEPAD_PLATFORM_DATA_FETCHER_LINUX_H_
 
+#include <stddef.h>
+
 #include <string>
 
-#include "base/basictypes.h"
 #include "base/compiler_specific.h"
+#include "base/macros.h"
 #include "base/memory/scoped_ptr.h"
 #include "content/browser/gamepad/gamepad_data_fetcher.h"
-#include "content/browser/gamepad/gamepad_standard_mappings.h"
-#include "third_party/WebKit/public/platform/WebGamepads.h"
 
 extern "C" {
 struct udev_device;
@@ -36,15 +36,8 @@ class GamepadPlatformDataFetcherLinux : public GamepadDataFetcher {
   void EnumerateDevices();
   void ReadDeviceData(size_t index);
 
-  // File descriptors for the /dev/input/js* devices. -1 if not in use.
-  int device_fds_[blink::WebGamepads::itemsLengthCap];
-
-  // Functions to map from device data to standard layout, if available. May
-  // be null if no mapping is available.
-  GamepadStandardMappingFunction mappers_[blink::WebGamepads::itemsLengthCap];
-
-  // Data that's returned to the consumer.
-  blink::WebGamepads data_;
+  // File descriptor for the /dev/input/js* devices. -1 if not in use.
+  int device_fd_[blink::WebGamepads::itemsLengthCap];
 
   scoped_ptr<UdevLinux> udev_;
 
