@@ -4,6 +4,9 @@
 
 #include "content/browser/gamepad/raw_input_data_fetcher_win.h"
 
+#include <stddef.h>
+
+#include "base/macros.h"
 #include "base/trace_event/trace_event.h"
 #include "content/common/gamepad_hardware_buffer.h"
 #include "content/common/gamepad_messages.h"
@@ -58,9 +61,9 @@ void RawInputDataFetcher::WillDestroyCurrentMessageLoop() {
 }
 
 RAWINPUTDEVICE* RawInputDataFetcher::GetRawInputDevices(DWORD flags) {
-  int usage_count = arraysize(DeviceUsages);
+  size_t usage_count = arraysize(DeviceUsages);
   scoped_ptr<RAWINPUTDEVICE[]> devices(new RAWINPUTDEVICE[usage_count]);
-  for (int i = 0; i < usage_count; ++i) {
+  for (size_t i = 0; i < usage_count; ++i) {
     devices[i].dwFlags = flags;
     devices[i].usUsagePage = 1;
     devices[i].usUsage = DeviceUsages[i];
@@ -187,7 +190,7 @@ RawGamepadInfo* RawInputDataFetcher::ParseGamepadInfo(HANDLE hDevice) {
   }
   DCHECK_EQ(0u, result);
 
-  scoped_ptr<uint8[]> di_buffer(new uint8[size]);
+  scoped_ptr<uint8_t[]> di_buffer(new uint8_t[size]);
   RID_DEVICE_INFO* device_info =
       reinterpret_cast<RID_DEVICE_INFO*>(di_buffer.get());
   result = GetRawInputDeviceInfo(hDevice, RIDI_DEVICEINFO,
@@ -200,8 +203,8 @@ RawGamepadInfo* RawInputDataFetcher::ParseGamepadInfo(HANDLE hDevice) {
 
   // Make sure this device is of a type that we want to observe.
   bool valid_type = false;
-  for (int i = 0; i < arraysize(DeviceUsages); ++i) {
-    if (device_info->hid.usUsage == DeviceUsages[i]) {
+  for (USHORT device_usage : DeviceUsages) {
+    if (device_info->hid.usUsage == device_usage) {
       valid_type = true;
       break;
     }
@@ -266,7 +269,7 @@ RawGamepadInfo* RawInputDataFetcher::ParseGamepadInfo(HANDLE hDevice) {
   }
   DCHECK_EQ(0u, result);
 
-  gamepad_info->ppd_buffer.reset(new uint8[size]);
+  gamepad_info->ppd_buffer.reset(new uint8_t[size]);
   gamepad_info->preparsed_data =
       reinterpret_cast<PHIDP_PREPARSED_DATA>(gamepad_info->ppd_buffer.get());
   result = GetRawInputDeviceInfo(hDevice, RIDI_PREPARSEDDATA,
@@ -389,8 +392,10 @@ void RawInputDataFetcher::UpdateGamepad(
       for (uint32_t j = 0; j < buttons_length; j++) {
         int32_t button_index = usages[j].Usage - 1;
         if (button_index >= 0 &&
-            button_index < blink::WebGamepad::buttonsLengthCap)
+            button_index <
+                static_cast<int>(blink::WebGamepad::buttonsLengthCap)) {
           gamepad_info->buttons[button_index] = true;
+        }
       }
     }
   }
@@ -440,7 +445,7 @@ LRESULT RawInputDataFetcher::OnInput(HRAWINPUT input_handle) {
   DCHECK_EQ(0u, result);
 
   // Retrieve the input record.
-  scoped_ptr<uint8[]> buffer(new uint8[size]);
+  scoped_ptr<uint8_t[]> buffer(new uint8_t[size]);
   RAWINPUT* input = reinterpret_cast<RAWINPUT*>(buffer.get());
   result = GetRawInputData(
       input_handle, RID_INPUT, buffer.get(), &size, sizeof(RAWINPUTHEADER));
