@@ -2,8 +2,11 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+#include <stddef.h>
+
 #include "base/json/json_reader.h"
 #include "base/json/json_writer.h"
+#include "base/macros.h"
 #include "base/memory/scoped_ptr.h"
 #include "base/strings/string_number_conversions.h"
 #include "base/strings/string_util.h"
@@ -14,6 +17,7 @@
 #include "content/browser/geolocation/location_arbitrator_impl.h"
 #include "content/browser/geolocation/network_location_provider.h"
 #include "content/browser/geolocation/wifi_data_provider.h"
+#include "net/base/net_errors.h"
 #include "net/url_request/test_url_fetcher_factory.h"
 #include "net/url_request/url_request_status.h"
 #include "testing/gtest/include/gtest/gtest.h"
@@ -40,7 +44,7 @@ class MessageLoopQuitListener {
                         const Geoposition& position) {
     EXPECT_EQ(client_message_loop_, base::MessageLoop::current());
     updated_provider_ = provider;
-    client_message_loop_->Quit();
+    client_message_loop_->QuitWhenIdle();
   }
 
   base::MessageLoop* client_message_loop_;
@@ -258,9 +262,8 @@ class GeolocationNetworkProviderTest : public testing::Test {
     const std::string& upload_data = request.upload_data();
     ASSERT_FALSE(upload_data.empty());
     std::string json_parse_error_msg;
-    scoped_ptr<base::Value> parsed_json(
-        base::JSONReader::DeprecatedReadAndReturnError(
-            upload_data, base::JSON_PARSE_RFC, NULL, &json_parse_error_msg));
+    scoped_ptr<base::Value> parsed_json = base::JSONReader::ReadAndReturnError(
+        upload_data, base::JSON_PARSE_RFC, NULL, &json_parse_error_msg);
     EXPECT_TRUE(json_parse_error_msg.empty());
     ASSERT_TRUE(parsed_json.get() != NULL);
 
@@ -437,7 +440,7 @@ TEST_F(GeolocationNetworkProviderTest, MultipleWifiScansComplete) {
   // ...reply with a network error.
 
   fetcher->set_url(test_server_url_);
-  fetcher->set_status(net::URLRequestStatus(net::URLRequestStatus::FAILED, -1));
+  fetcher->set_status(net::URLRequestStatus::FromError(net::ERR_FAILED));
   fetcher->set_response_code(200);  // should be ignored
   fetcher->SetResponseString(std::string());
   fetcher->delegate()->OnURLFetchComplete(fetcher);
