@@ -4,9 +4,7 @@
 
 #include "content/browser/gpu/gpu_internals_ui.h"
 
-#if defined(OS_LINUX) && defined(USE_X11)
-#include <X11/Xlib.h>
-#endif
+#include <stddef.h>
 
 #include <string>
 
@@ -15,10 +13,13 @@
 #include "base/command_line.h"
 #include "base/environment.h"
 #include "base/i18n/time_formatting.h"
+#include "base/macros.h"
 #include "base/strings/string_number_conversions.h"
 #include "base/strings/stringprintf.h"
 #include "base/sys_info.h"
 #include "base/values.h"
+#include "build/build_config.h"
+#include "content/browser/gpu/browser_gpu_memory_buffer_manager.h"
 #include "content/browser/gpu/compositor_util.h"
 #include "content/browser/gpu/gpu_data_manager_impl.h"
 #include "content/grit/content_resources.h"
@@ -36,6 +37,9 @@
 #include "third_party/angle/src/common/version.h"
 #include "ui/gl/gpu_switching_manager.h"
 
+#if defined(OS_LINUX) && defined(USE_X11)
+#include <X11/Xlib.h>
+#endif
 #if defined(OS_WIN)
 #include "ui/base/win/shell.h"
 #endif
@@ -112,6 +116,8 @@ base::DictionaryValue* GpuInfoAsDictionaryValue() {
   basic_info->Append(NewDescriptionValuePair(
       "Initialization time",
       base::Int64ToString(gpu_info.initialization_time.InMilliseconds())));
+  basic_info->Append(NewDescriptionValuePair(
+      "In-process GPU", new base::FundamentalValue(gpu_info.in_process_gpu)));
   basic_info->Append(NewDescriptionValuePair(
       "Sandboxed", new base::FundamentalValue(gpu_info.sandboxed)));
   basic_info->Append(NewDescriptionValuePair(
@@ -144,6 +150,10 @@ base::DictionaryValue* GpuInfoAsDictionaryValue() {
   }
 #endif
 
+  std::string disabled_extensions;
+  GpuDataManagerImpl::GetInstance()->GetDisabledExtensions(
+      &disabled_extensions);
+
   basic_info->Append(
       NewDescriptionValuePair("Driver vendor", gpu_info.driver_vendor));
   basic_info->Append(NewDescriptionValuePair("Driver version",
@@ -154,6 +164,8 @@ base::DictionaryValue* GpuInfoAsDictionaryValue() {
                                              gpu_info.pixel_shader_version));
   basic_info->Append(NewDescriptionValuePair("Vertex shader version",
                                              gpu_info.vertex_shader_version));
+  basic_info->Append(NewDescriptionValuePair("Max. MSAA samples",
+                                             gpu_info.max_msaa_samples));
   basic_info->Append(NewDescriptionValuePair("Machine model name",
                                              gpu_info.machine_model_name));
   basic_info->Append(NewDescriptionValuePair("Machine model version",
@@ -166,6 +178,8 @@ base::DictionaryValue* GpuInfoAsDictionaryValue() {
                                              gpu_info.gl_version));
   basic_info->Append(NewDescriptionValuePair("GL_EXTENSIONS",
                                              gpu_info.gl_extensions));
+  basic_info->Append(NewDescriptionValuePair("Disabled Extensions",
+                                             disabled_extensions));
   basic_info->Append(NewDescriptionValuePair("Window system binding vendor",
                                              gpu_info.gl_ws_vendor));
   basic_info->Append(NewDescriptionValuePair("Window system binding version",
@@ -213,27 +227,98 @@ base::DictionaryValue* GpuInfoAsDictionaryValue() {
   info->Set("basic_info", basic_info);
 
 #if defined(OS_WIN)
-  base::ListValue* perf_info = new base::ListValue();
-  perf_info->Append(NewDescriptionValuePair(
-      "Graphics",
-      base::StringPrintf("%.1f", gpu_info.performance_stats.graphics)));
-  perf_info->Append(NewDescriptionValuePair(
-      "Gaming",
-      base::StringPrintf("%.1f", gpu_info.performance_stats.gaming)));
-  perf_info->Append(NewDescriptionValuePair(
-      "Overall",
-      base::StringPrintf("%.1f", gpu_info.performance_stats.overall)));
-  info->Set("performance_info", perf_info);
-
-  base::Value* dx_info = gpu_info.dx_diagnostics.children.size() ?
-    DxDiagNodeToList(gpu_info.dx_diagnostics) :
-    base::Value::CreateNullValue();
-  info->Set("diagnostics", dx_info);
+  scoped_ptr<base::Value> dx_info = base::Value::CreateNullValue();
+  if (gpu_info.dx_diagnostics.children.size())
+    dx_info.reset(DxDiagNodeToList(gpu_info.dx_diagnostics));
+  info->Set("diagnostics", dx_info.Pass());
 #endif
 
   return info;
 }
 
+const char* BufferFormatToString(gfx::BufferFormat format) {
+  switch (format) {
+    case gfx::BufferFormat::ATC:
+      return "ATC";
+    case gfx::BufferFormat::ATCIA:
+      return "ATCIA";
+    case gfx::BufferFormat::DXT1:
+      return "DXT1";
+    case gfx::BufferFormat::DXT5:
+      return "DXT5";
+    case gfx::BufferFormat::ETC1:
+      return "ETC1";
+    case gfx::BufferFormat::R_8:
+      return "R_8";
+    case gfx::BufferFormat::RGBA_4444:
+      return "RGBA_4444";
+    case gfx::BufferFormat::RGBX_8888:
+      return "RGBX_8888";
+    case gfx::BufferFormat::RGBA_8888:
+      return "RGBA_8888";
+    case gfx::BufferFormat::BGRX_8888:
+      return "BGRX_8888";
+    case gfx::BufferFormat::BGRA_8888:
+      return "BGRA_8888";
+    case gfx::BufferFormat::YUV_420:
+      return "YUV_420";
+    case gfx::BufferFormat::YUV_420_BIPLANAR:
+      return "YUV_420_BIPLANAR";
+    case gfx::BufferFormat::UYVY_422:
+      return "UYVY_422";
+  }
+  NOTREACHED();
+  return nullptr;
+}
+
+const char* BufferUsageToString(gfx::BufferUsage usage) {
+  switch (usage) {
+    case gfx::BufferUsage::GPU_READ:
+      return "GPU_READ";
+    case gfx::BufferUsage::SCANOUT:
+      return "SCANOUT";
+    case gfx::BufferUsage::GPU_READ_CPU_READ_WRITE:
+      return "GPU_READ_CPU_READ_WRITE";
+    case gfx::BufferUsage::GPU_READ_CPU_READ_WRITE_PERSISTENT:
+      return "GPU_READ_CPU_READ_WRITE_PERSISTENT";
+  }
+  NOTREACHED();
+  return nullptr;
+}
+
+base::DictionaryValue* GpuMemoryBufferInfoAsDictionaryValue() {
+  base::ListValue* gpu_memory_buffer_info = new base::ListValue();
+
+  BrowserGpuMemoryBufferManager* gpu_memory_buffer_manager =
+      BrowserGpuMemoryBufferManager::current();
+
+  for (size_t format = 0;
+       format < static_cast<size_t>(gfx::BufferFormat::LAST) + 1; format++) {
+    std::string native_usage_support;
+    for (size_t usage = 0;
+         usage < static_cast<size_t>(gfx::BufferUsage::LAST) + 1; usage++) {
+      if (gpu_memory_buffer_manager->IsNativeGpuMemoryBufferConfiguration(
+              static_cast<gfx::BufferFormat>(format),
+              static_cast<gfx::BufferUsage>(usage)))
+        native_usage_support = base::StringPrintf(
+            "%s%s %s", native_usage_support.c_str(),
+            native_usage_support.empty() ? "" : ",",
+            BufferUsageToString(static_cast<gfx::BufferUsage>(usage)));
+    }
+    if (native_usage_support.empty())
+      native_usage_support = base::StringPrintf("Software only");
+
+    gpu_memory_buffer_info->Append(NewDescriptionValuePair(
+        BufferFormatToString(static_cast<gfx::BufferFormat>(format)),
+        native_usage_support));
+  }
+
+  base::DictionaryValue* info = new base::DictionaryValue();
+  info->Set("gpu_memory_buffer_info", gpu_memory_buffer_info);
+
+  return info;
+}
+
 // This class receives javascript messages from the renderer.
 // Note that the WebUI infrastructure runs on the UI thread, therefore all of
 // this class's methods are expected to run on the UI thread.
@@ -288,7 +373,7 @@ GpuMessageHandler::~GpuMessageHandler() {
 
 /* BrowserBridge.callAsync prepends a requestID to these messages. */
 void GpuMessageHandler::RegisterMessages() {
-  DCHECK(BrowserThread::CurrentlyOn(BrowserThread::UI));
+  DCHECK_CURRENTLY_ON(BrowserThread::UI);
 
   web_ui()->RegisterMessageCallback("browserBridgeInitialized",
       base::Bind(&GpuMessageHandler::OnBrowserBridgeInitialized,
@@ -347,7 +432,7 @@ void GpuMessageHandler::OnCallAsync(const base::ListValue* args) {
 
 void GpuMessageHandler::OnBrowserBridgeInitialized(
     const base::ListValue* args) {
-  DCHECK(BrowserThread::CurrentlyOn(BrowserThread::UI));
+  DCHECK_CURRENTLY_ON(BrowserThread::UI);
 
   // Watch for changes in GPUInfo
   if (!observing_) {
@@ -367,7 +452,7 @@ void GpuMessageHandler::OnBrowserBridgeInitialized(
 
 base::Value* GpuMessageHandler::OnRequestClientInfo(
     const base::ListValue* list) {
-  DCHECK(BrowserThread::CurrentlyOn(BrowserThread::UI));
+  DCHECK_CURRENTLY_ON(BrowserThread::UI);
 
   base::DictionaryValue* dict = new base::DictionaryValue();
 
@@ -388,7 +473,7 @@ base::Value* GpuMessageHandler::OnRequestClientInfo(
 }
 
 base::Value* GpuMessageHandler::OnRequestLogMessages(const base::ListValue*) {
-  DCHECK(BrowserThread::CurrentlyOn(BrowserThread::UI));
+  DCHECK_CURRENTLY_ON(BrowserThread::UI);
 
   return GpuDataManagerImpl::GetInstance()->GetLogMessages();
 }
@@ -397,17 +482,28 @@ void GpuMessageHandler::OnGpuInfoUpdate() {
   // Get GPU Info.
   scoped_ptr<base::DictionaryValue> gpu_info_val(GpuInfoAsDictionaryValue());
 
+
   // Add in blacklisting features
   base::DictionaryValue* feature_status = new base::DictionaryValue;
   feature_status->Set("featureStatus", GetFeatureStatus());
   feature_status->Set("problems", GetProblems());
-  feature_status->Set("workarounds", GetDriverBugWorkarounds());
-  if (feature_status)
-    gpu_info_val->Set("featureStatus", feature_status);
+  base::ListValue* workarounds = new base::ListValue();
+  for (const std::string& workaround : GetDriverBugWorkarounds())
+    workarounds->AppendString(workaround);
+  feature_status->Set("workarounds", workarounds);
+  gpu_info_val->Set("featureStatus", feature_status);
 
   // Send GPU Info to javascript.
   web_ui()->CallJavascriptFunction("browserBridge.onGpuInfoUpdate",
       *(gpu_info_val.get()));
+
+  // Get GpuMemoryBuffer Info.
+  scoped_ptr<base::DictionaryValue> gpu_memory_buffer_info_val(
+      GpuMemoryBufferInfoAsDictionaryValue());
+
+  // Send GpuMemoryBuffer Info to javascript.
+  web_ui()->CallJavascriptFunction("browserBridge.onGpuMemoryBufferInfoUpdate",
+                                   *(gpu_memory_buffer_info_val.get()));
 }
 
 void GpuMessageHandler::OnGpuSwitched() {
