@@ -5,6 +5,9 @@
 #ifndef CONTENT_BROWSER_RENDERER_HOST_RENDER_WIDGET_HOST_VIEW_AURA_H_
 #define CONTENT_BROWSER_RENDERER_HOST_RENDER_WIDGET_HOST_VIEW_AURA_H_
 
+#include <stddef.h>
+#include <stdint.h>
+
 #include <map>
 #include <set>
 #include <string>
@@ -12,10 +15,12 @@
 
 #include "base/callback.h"
 #include "base/gtest_prod_util.h"
+#include "base/macros.h"
 #include "base/memory/linked_ptr.h"
 #include "base/memory/ref_counted.h"
 #include "base/memory/scoped_ptr.h"
 #include "base/memory/weak_ptr.h"
+#include "build/build_config.h"
 #include "content/browser/accessibility/browser_accessibility_manager.h"
 #include "content/browser/compositor/delegated_frame_host.h"
 #include "content/browser/compositor/image_transport_factory.h"
@@ -31,13 +36,14 @@
 #include "ui/aura/window_tree_host_observer.h"
 #include "ui/base/ime/text_input_client.h"
 #include "ui/base/touch/selection_bound.h"
-#include "ui/base/touch/touch_editing_controller.h"
 #include "ui/events/gestures/motion_event_aura.h"
 #include "ui/gfx/display_observer.h"
 #include "ui/gfx/geometry/insets.h"
 #include "ui/gfx/geometry/rect.h"
 #include "ui/wm/public/activation_delegate.h"
 
+struct ViewHostMsg_TextInputState_Params;
+
 namespace aura {
 class WindowTracker;
 namespace client {
@@ -54,6 +60,7 @@ class DelegatedFrameData;
 namespace gfx {
 class Canvas;
 class Display;
+class Rect;
 }
 
 namespace gpu {
@@ -65,6 +72,7 @@ class CompositorLock;
 class InputMethod;
 class LocatedEvent;
 class Texture;
+class TouchSelectionController;
 }
 
 namespace content {
@@ -76,6 +84,7 @@ class OverscrollController;
 class RenderFrameHostImpl;
 class RenderWidgetHostImpl;
 class RenderWidgetHostView;
+class TouchSelectionControllerClientAura;
 
 // RenderWidgetHostView class hierarchy described in render_widget_host_view.h.
 class CONTENT_EXPORT RenderWidgetHostViewAura
@@ -90,49 +99,6 @@ class CONTENT_EXPORT RenderWidgetHostViewAura
       public aura::client::FocusChangeObserver,
       public aura::client::CursorClientObserver {
  public:
-  // Displays and controls touch editing elements such as selection handles.
-  class TouchEditingClient {
-   public:
-    TouchEditingClient() {}
-
-    // Tells the client to start showing touch editing handles.
-    virtual void StartTouchEditing() = 0;
-
-    // Notifies the client that touch editing is no longer needed. |quick|
-    // determines whether the handles should fade out quickly or slowly.
-    virtual void EndTouchEditing(bool quick) = 0;
-
-    // Notifies the client that the selection bounds need to be updated.
-    virtual void OnSelectionOrCursorChanged(
-        const ui::SelectionBound& anchor,
-        const ui::SelectionBound& focus) = 0;
-
-    // Notifies the client that the current text input type as changed.
-    virtual void OnTextInputTypeChanged(ui::TextInputType type) = 0;
-
-    // Notifies the client that an input event is about to be sent to the
-    // renderer. Returns true if the client wants to stop event propagation.
-    virtual bool HandleInputEvent(const ui::Event* event) = 0;
-
-    // Notifies the client that a gesture event ack was received.
-    virtual void GestureEventAck(int gesture_event_type) = 0;
-
-    // Notifies the client that the fling has ended, so it can activate touch
-    // editing if needed.
-    virtual void DidStopFlinging() = 0;
-
-    // This is called when the view is destroyed, so that the client can
-    // perform any necessary clean-up.
-    virtual void OnViewDestroyed() = 0;
-
-   protected:
-    virtual ~TouchEditingClient() {}
-  };
-
-  void set_touch_editing_client(TouchEditingClient* client) {
-    touch_editing_client_ = client;
-  }
-
   // When |is_guest_view_hack| is true, this view isn't really the view for
   // the |widget|, a RenderWidgetHostViewGuest is.
   //
@@ -169,10 +135,8 @@ class CONTENT_EXPORT RenderWidgetHostViewAura
   void Focus() override;
   void UpdateCursor(const WebCursor& cursor) override;
   void SetIsLoading(bool is_loading) override;
-  void TextInputTypeChanged(ui::TextInputType type,
-                            ui::TextInputMode input_mode,
-                            bool can_compose_inline,
-                            int flags) override;
+  void TextInputStateChanged(
+      const ViewHostMsg_TextInputState_Params& params) override;
   void ImeCancelComposition() override;
   void ImeCompositionRangeChanged(
       const gfx::Range& range,
@@ -190,19 +154,19 @@ class CONTENT_EXPORT RenderWidgetHostViewAura
   void CopyFromCompositingSurface(
       const gfx::Rect& src_subrect,
       const gfx::Size& dst_size,
-      ReadbackRequestCallback& callback,
+      const ReadbackRequestCallback& callback,
       const SkColorType preferred_color_type) override;
   void CopyFromCompositingSurfaceToVideoFrame(
       const gfx::Rect& src_subrect,
       const scoped_refptr<media::VideoFrame>& target,
-      const base::Callback<void(bool)>& callback) override;
+      const base::Callback<void(const gfx::Rect&, bool)>& callback) override;
   bool CanCopyToVideoFrame() const override;
-  bool CanSubscribeFrame() const override;
   void BeginFrameSubscription(
       scoped_ptr<RenderWidgetHostViewFrameSubscriber> subscriber) override;
   void EndFrameSubscription() override;
   bool HasAcceleratedSurface(const gfx::Size& desired_size) override;
   void GetScreenInfo(blink::WebScreenInfo* results) override;
+  bool GetScreenColorProfile(std::vector<char>* color_profile) override;
   gfx::Rect GetBoundsInRootWindow() override;
   void WheelEventAck(const blink::WebMouseWheelEvent& event,
                      InputEventAckState ack_result) override;
@@ -213,7 +177,6 @@ class CONTENT_EXPORT RenderWidgetHostViewAura
   scoped_ptr<SyntheticGestureTarget> CreateSyntheticGestureTarget() override;
   InputEventAckState FilterInputEvent(
       const blink::WebInputEvent& input_event) override;
-  gfx::GLSurfaceHandle GetCompositingSurface() override;
   BrowserAccessibilityManager* CreateBrowserAccessibilityManager(
       BrowserAccessibilityDelegate* delegate) override;
   gfx::AcceleratedWidget AccessibilityGetAcceleratedWidget() override;
@@ -222,11 +185,23 @@ class CONTENT_EXPORT RenderWidgetHostViewAura
                                const SkBitmap& zoomed_bitmap) override;
   bool LockMouse() override;
   void UnlockMouse() override;
-  void OnSwapCompositorFrame(uint32 output_surface_id,
+  void OnSwapCompositorFrame(uint32_t output_surface_id,
                              scoped_ptr<cc::CompositorFrame> frame) override;
+  void ClearCompositorFrame() override;
   void DidStopFlinging() override;
   void OnDidNavigateMainFrameToNewPage() override;
+  void LockCompositingSurface() override;
+  void UnlockCompositingSurface() override;
   uint32_t GetSurfaceIdNamespace() override;
+  uint32_t SurfaceIdNamespaceAtPoint(const gfx::Point& point,
+                                     gfx::Point* transformed_point) override;
+  void ProcessMouseEvent(const blink::WebMouseEvent& event) override;
+  void ProcessMouseWheelEvent(const blink::WebMouseWheelEvent& event) override;
+  void ProcessTouchEvent(const blink::WebTouchEvent& event,
+                         const ui::LatencyInfo& latency) override;
+  void TransformPointToLocalCoordSpace(const gfx::Point& point,
+                                       cc::SurfaceId original_surface,
+                                       gfx::Point* transformed_point) override;
 
 #if defined(OS_WIN)
   void SetParentNativeViewAccessible(
@@ -239,13 +214,13 @@ class CONTENT_EXPORT RenderWidgetHostViewAura
   void ConfirmCompositionText() override;
   void ClearCompositionText() override;
   void InsertText(const base::string16& text) override;
-  void InsertChar(base::char16 ch, int flags) override;
+  void InsertChar(const ui::KeyEvent& event) override;
   ui::TextInputType GetTextInputType() const override;
   ui::TextInputMode GetTextInputMode() const override;
   int GetTextInputFlags() const override;
   bool CanComposeInline() const override;
   gfx::Rect GetCaretBounds() const override;
-  bool GetCompositionCharacterBounds(uint32 index,
+  bool GetCompositionCharacterBounds(uint32_t index,
                                      gfx::Rect* rect) const override;
   bool HasCompositionText() const override;
   bool GetTextRange(gfx::Range* range) const override;
@@ -310,8 +285,6 @@ class CONTENT_EXPORT RenderWidgetHostViewAura
   void OnHostMoved(const aura::WindowTreeHost* host,
                    const gfx::Point& new_origin) override;
 
-  void OnTextInputStateChanged(const ViewHostMsg_TextInputState_Params& params);
-
 #if defined(OS_WIN)
   // Sets the cutout rects from constrained windows. These are rectangles that
   // windowed NPAPI plugins shouldn't paint in. Overwrites any previous cutout
@@ -344,6 +317,14 @@ class CONTENT_EXPORT RenderWidgetHostViewAura
 
   void SnapToPhysicalPixelBoundary();
 
+  ui::TouchSelectionController* selection_controller() const {
+    return selection_controller_.get();
+  }
+
+  TouchSelectionControllerClientAura* selection_controller_client() const {
+    return selection_controller_client_.get();
+  }
+
   OverscrollController* overscroll_controller() const {
     return overscroll_controller_.get();
   }
@@ -351,6 +332,11 @@ class CONTENT_EXPORT RenderWidgetHostViewAura
   // Called when the context menu is about to be displayed.
   void OnShowContextMenu();
 
+  // Used in tests to set a mock client for touch selection controller. It will
+  // create a new touch selection controller for the new client.
+  void SetSelectionControllerClientForTest(
+      scoped_ptr<TouchSelectionControllerClientAura> client);
+
  protected:
   ~RenderWidgetHostViewAura() override;
 
@@ -399,6 +385,8 @@ class CONTENT_EXPORT RenderWidgetHostViewAura
   class WindowAncestorObserver;
   friend class WindowAncestorObserver;
 
+  void CreateAuraWindow();
+
   void UpdateCursorIfOverSelf();
 
   // Tracks whether SnapToPhysicalPixelBoundary() has been called.
@@ -504,14 +492,31 @@ class CONTENT_EXPORT RenderWidgetHostViewAura
                               bool mouse_locked,
                               bool selection_popup);
 
+  // Returns true when we can do SurfaceHitTesting for the event type.
+  bool ShouldRouteEvent(const ui::Event* event) const;
+
   // Called when the parent window bounds change.
   void HandleParentBoundsChanged();
 
   // Called when the parent window hierarchy for our window changes.
   void ParentHierarchyChanged();
 
+  // Helper function to be called whenever new selection information is
+  // received. It will update selection controller.
+  void SelectionUpdated(bool is_editable,
+                        bool is_empty_text_form_control,
+                        const ui::SelectionBound& start,
+                        const ui::SelectionBound& end);
+
+  // Helper function to create a selection controller.
+  void CreateSelectionController();
+
+  // Performs gesture handling needed for touch text selection. Sets event as
+  // handled if it should not be further processed.
+  void HandleGestureForTouchSelection(ui::GestureEvent* event);
+
   // The model object.
-  RenderWidgetHostImpl* host_;
+  RenderWidgetHostImpl* const host_;
 
   aura::Window* window_;
 
@@ -647,7 +652,8 @@ class CONTENT_EXPORT RenderWidgetHostViewAura
 
   bool has_snapped_to_boundary_;
 
-  TouchEditingClient* touch_editing_client_;
+  scoped_ptr<TouchSelectionControllerClientAura> selection_controller_client_;
+  scoped_ptr<ui::TouchSelectionController> selection_controller_;
 
   scoped_ptr<OverscrollController> overscroll_controller_;
 
@@ -675,7 +681,9 @@ class CONTENT_EXPORT RenderWidgetHostViewAura
 
   // This flag when set ensures that we send over a notification to blink that
   // the current view has focus. Defaults to false.
-  bool set_focus_on_mouse_down_;
+  bool set_focus_on_mouse_down_or_key_event_;
+
+  float device_scale_factor_;
 
   base::WeakPtrFactory<RenderWidgetHostViewAura> weak_ptr_factory_;
 
