@@ -2,12 +2,14 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+#include <stddef.h>
+
 #include <map>
 #include <set>
 #include <vector>
 
 #include "base/atomic_sequence_num.h"
-#include "base/basictypes.h"
+#include "base/macros.h"
 #include "base/memory/scoped_ptr.h"
 #include "base/memory/scoped_vector.h"
 #include "base/strings/string16.h"
@@ -208,7 +210,7 @@ class MockRendererProcessHost {
     CHECK(queued_messages_.size());
     scoped_ptr<IPC::Message> msg(*queued_messages_.begin());
     queued_messages_.weak_erase(queued_messages_.begin());
-    return msg.Pass();
+    return msg;
   }
 
   void FastShutdownIfPossible() {
@@ -243,7 +245,7 @@ void PostCreateWorker(MockRendererProcessHost* renderer,
                       const std::string& name,
                       unsigned long long document_id,
                       int render_frame_route_id,
-                      int* connector_route_id) {
+                      ViewHostMsg_CreateWorker_Reply* reply) {
   ViewHostMsg_CreateWorker_Params params;
   params.url = GURL(url);
   params.name = base::ASCIIToUTF16(name);
@@ -251,8 +253,10 @@ void PostCreateWorker(MockRendererProcessHost* renderer,
   params.security_policy_type = blink::WebContentSecurityPolicyTypeReport;
   params.document_id = document_id;
   params.render_frame_route_id = render_frame_route_id;
-  EXPECT_TRUE(renderer->OnMessageReceived(
-      new ViewHostMsg_CreateWorker(params, connector_route_id)));
+  params.creation_context_type =
+      blink::WebSharedWorkerCreationContextTypeSecure;
+  EXPECT_TRUE(
+      renderer->OnMessageReceived(new ViewHostMsg_CreateWorker(params, reply)));
 }
 
 class MockSharedWorkerConnector {
@@ -262,8 +266,7 @@ class MockSharedWorkerConnector {
         temporary_remote_port_route_id_(0),
         remote_port_id_(0),
         local_port_route_id_(0),
-        local_port_id_(0),
-        route_id_(0) {}
+        local_port_id_(0) {}
   void Create(const std::string& url,
               const std::string& name,
               unsigned long long document_id,
@@ -273,18 +276,14 @@ class MockSharedWorkerConnector {
                           &remote_port_id_,
                           &local_port_route_id_,
                           &local_port_id_);
-    PostCreateWorker(renderer_host_,
-                     url,
-                     name,
-                     document_id,
-                     render_frame_route_id,
-                     &route_id_);
+    PostCreateWorker(renderer_host_, url, name, document_id,
+                     render_frame_route_id, &create_worker_reply_);
   }
   void SendQueueMessages() {
     EXPECT_TRUE(renderer_host_->OnMessageReceived(
         new MessagePortHostMsg_QueueMessages(remote_port_id_)));
   }
-  void SendPostMessage(std::string data) {
+  void SendPostMessage(const std::string& data) {
     const std::vector<TransferredMessagePort> empty_ports;
     EXPECT_TRUE(
         renderer_host_->OnMessageReceived(new MessagePortHostMsg_PostMessage(
@@ -294,7 +293,8 @@ class MockSharedWorkerConnector {
   void SendConnect() {
     EXPECT_TRUE(
         renderer_host_->OnMessageReceived(new ViewHostMsg_ForwardToWorker(
-            WorkerMsg_Connect(route_id_, remote_port_id_, MSG_ROUTING_NONE))));
+            WorkerMsg_Connect(create_worker_reply_.route_id, remote_port_id_,
+                              MSG_ROUTING_NONE))));
   }
   void SendSendQueuedMessages(
       const std::vector<QueuedMessage>& queued_messages) {
@@ -308,7 +308,10 @@ class MockSharedWorkerConnector {
   int remote_port_id() { return remote_port_id_; }
   int local_port_route_id() { return local_port_route_id_; }
   int local_port_id() { return local_port_id_; }
-  int route_id() { return route_id_; }
+  int route_id() { return create_worker_reply_.route_id; }
+  blink::WebWorkerCreationError creation_error() {
+    return create_worker_reply_.error;
+  }
 
  private:
   MockRendererProcessHost* renderer_host_;
@@ -316,7 +319,7 @@ class MockSharedWorkerConnector {
   int remote_port_id_;
   int local_port_route_id_;
   int local_port_id_;
-  int route_id_;
+  ViewHostMsg_CreateWorker_Reply create_worker_reply_;
 };
 
 void CheckWorkerProcessMsgCreateWorker(
@@ -731,6 +734,8 @@ TEST_F(SharedWorkerServiceImplTest, CreateWorkerTest) {
                        kDocumentIDs[1],
                        kRenderFrameRouteIDs[1]);
     EXPECT_EQ(MSG_ROUTING_NONE, connector1->route_id());
+    EXPECT_EQ(blink::WebWorkerCreationErrorURLMismatch,
+              connector1->creation_error());
     EXPECT_EQ(0U, renderer_host1->QueuedMessageCount());
     RunAllPendingInMessageLoop();
     EXPECT_EQ(0U, renderer_host1->QueuedMessageCount());
