@@ -4,6 +4,8 @@
 
 #include "content/browser/web_contents/web_contents_android.h"
 
+#include <stdint.h>
+
 #include "base/android/jni_android.h"
 #include "base/android/jni_array.h"
 #include "base/android/jni_string.h"
@@ -18,7 +20,7 @@
 #include "content/browser/android/interstitial_page_delegate_android.h"
 #include "content/browser/frame_host/interstitial_page_impl.h"
 #include "content/browser/media/android/browser_media_player_manager.h"
-#include "content/browser/media/media_web_contents_observer.h"
+#include "content/browser/media/android/media_web_contents_observer_android.h"
 #include "content/browser/renderer_host/render_view_host_impl.h"
 #include "content/browser/web_contents/web_contents_impl.h"
 #include "content/common/devtools_messages.h"
@@ -27,18 +29,23 @@
 #include "content/common/view_messages.h"
 #include "content/public/browser/browser_context.h"
 #include "content/public/browser/browser_thread.h"
+#include "content/public/browser/message_port_provider.h"
+#include "content/public/browser/render_widget_host.h"
 #include "content/public/browser/web_contents.h"
 #include "content/public/common/content_switches.h"
 #include "jni/WebContentsImpl_jni.h"
 #include "net/android/network_library.h"
 #include "ui/accessibility/ax_node_data.h"
 #include "ui/gfx/android/device_display_info.h"
+#include "ui/gfx/android/java_bitmap.h"
+#include "ui/gfx/geometry/rect.h"
 
 using base::android::AttachCurrentThread;
 using base::android::ConvertJavaStringToUTF8;
 using base::android::ConvertJavaStringToUTF16;
 using base::android::ConvertUTF8ToJavaString;
 using base::android::ConvertUTF16ToJavaString;
+using base::android::JavaParamRef;
 using base::android::ScopedJavaGlobalRef;
 using base::android::ToJavaIntArray;
 
@@ -61,9 +68,35 @@ void JavaScriptResultCallback(const ScopedJavaGlobalRef<jobject>& callback,
       env, j_json.obj(), callback.obj());
 }
 
-ScopedJavaLocalRef<jobject> WalkAXTreeDepthFirst(JNIEnv* env,
-      BrowserAccessibilityAndroid* node, float scale_factor,
-      float y_offset, float x_scroll) {
+struct AccessibilitySnapshotParams {
+  AccessibilitySnapshotParams(float scale,
+                              float horizontal_scroll,
+                              float vertical_offset)
+      : scale_factor(scale),
+        x_scroll(horizontal_scroll),
+        y_offset(vertical_offset),
+        has_tree_data(false),
+        should_select_leaf_nodes(false) {}
+
+  float scale_factor;
+  float x_scroll;
+  float y_offset;
+  bool has_tree_data;
+  // The current text selection within this tree, if any, expressed as the
+  // node ID and character offset of the anchor (selection start) and focus
+  // (selection end).
+  int32_t sel_anchor_object_id;
+  int32_t sel_anchor_offset;
+  int32_t sel_focus_object_id;
+  int32_t sel_focus_offset;
+  // if the flag is true, mark the leaf node as selected.
+  bool should_select_leaf_nodes;
+};
+
+ScopedJavaLocalRef<jobject> WalkAXTreeDepthFirst(
+    JNIEnv* env,
+    BrowserAccessibilityAndroid* node,
+    AccessibilitySnapshotParams* params) {
   ScopedJavaLocalRef<jstring> j_text =
       ConvertUTF16ToJavaString(env, node->GetText());
   ScopedJavaLocalRef<jstring> j_class =
@@ -76,37 +109,55 @@ ScopedJavaLocalRef<jobject> WalkAXTreeDepthFirst(JNIEnv* env,
   int color = 0;
   int bgcolor = 0;
   int text_style = 0;
+
   if (node->HasFloatAttribute(ui::AX_ATTR_FONT_SIZE)) {
     color = node->GetIntAttribute(ui::AX_ATTR_COLOR);
     bgcolor = node->GetIntAttribute(ui::AX_ATTR_BACKGROUND_COLOR);
     size =  node->GetFloatAttribute(ui::AX_ATTR_FONT_SIZE);
     text_style = node->GetIntAttribute(ui::AX_ATTR_TEXT_STYLE);
   }
+  float scale_factor = params->scale_factor;
   ScopedJavaLocalRef<jobject> j_node =
-      Java_WebContentsImpl_createAccessibilitySnapshotNode(env,
-          scale_factor * location.x() - x_scroll,
-          scale_factor * location.y() + y_offset,
+      Java_WebContentsImpl_createAccessibilitySnapshotNode(
+          env, scale_factor * location.x() - params->x_scroll,
+          scale_factor * location.y() + params->y_offset,
           scale_factor * node->GetScrollX(), scale_factor * node->GetScrollY(),
           scale_factor * location.width(), scale_factor * location.height(),
           j_text.obj(), color, bgcolor, scale_factor * size, text_style,
           j_class.obj());
 
-  for(uint32 i = 0; i < node->PlatformChildCount(); i++) {
+  if (params->has_tree_data && node->PlatformIsLeaf()) {
+    int start_selection = 0;
+    int end_selection = 0;
+    if (params->sel_anchor_object_id == node->GetId()) {
+      start_selection = params->sel_anchor_offset;
+      params->should_select_leaf_nodes = true;
+    }
+    if (params->should_select_leaf_nodes)
+      end_selection = node->GetText().length();
+
+    if (params->sel_focus_object_id == node->GetId()) {
+      end_selection = params->sel_focus_offset;
+      params->should_select_leaf_nodes = false;
+    }
+    if (end_selection > 0)
+      Java_WebContentsImpl_setAccessibilitySnapshotSelection(
+          env, j_node.obj(), start_selection, end_selection);
+  }
+
+  for (uint32_t i = 0; i < node->PlatformChildCount(); i++) {
     BrowserAccessibilityAndroid* child =
         static_cast<BrowserAccessibilityAndroid*>(
             node->PlatformGetChild(i));
-    Java_WebContentsImpl_addAccessibilityNodeAsChild(env,
-        j_node.obj(), WalkAXTreeDepthFirst(env, child, scale_factor, y_offset,
-            x_scroll).obj());
+    Java_WebContentsImpl_addAccessibilityNodeAsChild(
+        env, j_node.obj(), WalkAXTreeDepthFirst(env, child, params).obj());
   }
   return j_node;
 }
 
 // Walks over the AXTreeUpdate and creates a light weight snapshot.
 void AXTreeSnapshotCallback(const ScopedJavaGlobalRef<jobject>& callback,
-                            float scale_factor,
-                            float y_offset,
-                            float x_scroll,
+                            AccessibilitySnapshotParams* params,
                             const ui::AXTreeUpdate& result) {
   JNIEnv* env = base::android::AttachCurrentThread();
   if (result.nodes.empty()) {
@@ -119,8 +170,14 @@ void AXTreeSnapshotCallback(const ScopedJavaGlobalRef<jobject>& callback,
   manager->set_prune_tree_for_screen_reader(false);
   BrowserAccessibilityAndroid* root =
       static_cast<BrowserAccessibilityAndroid*>(manager->GetRoot());
-  ScopedJavaLocalRef<jobject> j_root =
-      WalkAXTreeDepthFirst(env, root, scale_factor, y_offset, x_scroll);
+  if (result.has_tree_data) {
+    params->has_tree_data = true;
+    params->sel_anchor_object_id = result.tree_data.sel_anchor_object_id;
+    params->sel_anchor_offset = result.tree_data.sel_anchor_offset;
+    params->sel_focus_object_id = result.tree_data.sel_focus_object_id;
+    params->sel_focus_offset = result.tree_data.sel_focus_offset;
+  }
+  ScopedJavaLocalRef<jobject> j_root = WalkAXTreeDepthFirst(env, root, params);
   Java_WebContentsImpl_onAccessibilitySnapshot(
       env, j_root.obj(), callback.obj());
 }
@@ -128,9 +185,8 @@ void AXTreeSnapshotCallback(const ScopedJavaGlobalRef<jobject>& callback,
 void ReleaseAllMediaPlayers(WebContents* web_contents,
                             RenderFrameHost* render_frame_host) {
   BrowserMediaPlayerManager* manager =
-      static_cast<WebContentsImpl*>(web_contents)->
-          media_web_contents_observer()->GetMediaPlayerManager(
-              render_frame_host);
+      MediaWebContentsObserverAndroid::FromWebContents(web_contents)
+          ->GetMediaPlayerManager(render_frame_host);
   if (manager)
     manager->ReleaseAllMediaPlayers();
 }
@@ -155,7 +211,7 @@ WebContents* WebContents::FromJavaWebContents(
 
 // static
 static void DestroyWebContents(JNIEnv* env,
-                               jclass clazz,
+                               const JavaParamRef<jclass>& clazz,
                                jlong jweb_contents_android_ptr) {
   WebContentsAndroid* web_contents_android =
       reinterpret_cast<WebContentsAndroid*>(jweb_contents_android_ptr);
@@ -170,22 +226,22 @@ static void DestroyWebContents(JNIEnv* env,
 }
 
 // static
-jobject FromNativePtr(JNIEnv* env,
-                      jclass clazz,
-                      jlong web_contents_ptr) {
+ScopedJavaLocalRef<jobject> FromNativePtr(JNIEnv* env,
+                                          const JavaParamRef<jclass>& clazz,
+                                          jlong web_contents_ptr) {
   WebContentsAndroid* web_contents_android =
       reinterpret_cast<WebContentsAndroid*>(web_contents_ptr);
 
   if (!web_contents_android)
-    return 0;
+    return ScopedJavaLocalRef<jobject>();
 
   // Check to make sure this object hasn't been destroyed.
   if (g_allocated_web_contents_androids.Get().find(web_contents_android) ==
       g_allocated_web_contents_androids.Get().end()) {
-    return 0;
+    return ScopedJavaLocalRef<jobject>();
   }
 
-  return web_contents_android->GetJavaObject().Release();
+  return web_contents_android->GetJavaObject();
 }
 
 // static
@@ -196,6 +252,7 @@ bool WebContentsAndroid::Register(JNIEnv* env) {
 WebContentsAndroid::WebContentsAndroid(WebContents* web_contents)
     : web_contents_(web_contents),
       navigation_controller_(&(web_contents->GetController())),
+      synchronous_compositor_client_(nullptr),
       weak_factory_(this) {
   g_allocated_web_contents_androids.Get().insert(this);
   JNIEnv* env = AttachCurrentThread();
@@ -225,52 +282,65 @@ WebContentsAndroid::GetJavaObject() {
 }
 
 ScopedJavaLocalRef<jstring> WebContentsAndroid::GetTitle(
-    JNIEnv* env, jobject obj) const {
+    JNIEnv* env,
+    const JavaParamRef<jobject>& obj) const {
   return base::android::ConvertUTF16ToJavaString(env,
                                                  web_contents_->GetTitle());
 }
 
 ScopedJavaLocalRef<jstring> WebContentsAndroid::GetVisibleURL(
-    JNIEnv* env, jobject obj) const {
+    JNIEnv* env,
+    const JavaParamRef<jobject>& obj) const {
   return base::android::ConvertUTF8ToJavaString(
       env, web_contents_->GetVisibleURL().spec());
 }
 
-bool WebContentsAndroid::IsLoading(JNIEnv* env, jobject obj) const {
+bool WebContentsAndroid::IsLoading(JNIEnv* env,
+                                   const JavaParamRef<jobject>& obj) const {
   return web_contents_->IsLoading();
 }
 
-bool WebContentsAndroid::IsLoadingToDifferentDocument(JNIEnv* env,
-                                                      jobject obj) const {
+bool WebContentsAndroid::IsLoadingToDifferentDocument(
+    JNIEnv* env,
+    const JavaParamRef<jobject>& obj) const {
   return web_contents_->IsLoadingToDifferentDocument();
 }
 
-void WebContentsAndroid::Stop(JNIEnv* env, jobject obj) {
+void WebContentsAndroid::Stop(JNIEnv* env, const JavaParamRef<jobject>& obj) {
   web_contents_->Stop();
 }
 
-void WebContentsAndroid::Cut(JNIEnv* env, jobject obj) {
+void WebContentsAndroid::Cut(JNIEnv* env, const JavaParamRef<jobject>& obj) {
   web_contents_->Cut();
 }
 
-void WebContentsAndroid::Copy(JNIEnv* env, jobject obj) {
+void WebContentsAndroid::Copy(JNIEnv* env, const JavaParamRef<jobject>& obj) {
   web_contents_->Copy();
 }
 
-void WebContentsAndroid::Paste(JNIEnv* env, jobject obj) {
+void WebContentsAndroid::Paste(JNIEnv* env, const JavaParamRef<jobject>& obj) {
   web_contents_->Paste();
 }
 
-void WebContentsAndroid::SelectAll(JNIEnv* env, jobject obj) {
+void WebContentsAndroid::Replace(JNIEnv* env,
+                                 const JavaParamRef<jobject>& obj,
+                                 const JavaParamRef<jstring>& jstr) {
+  web_contents_->Replace(base::android::ConvertJavaStringToUTF16(env, jstr));
+}
+
+void WebContentsAndroid::SelectAll(JNIEnv* env,
+                                   const JavaParamRef<jobject>& obj) {
   web_contents_->SelectAll();
 }
 
-void WebContentsAndroid::Unselect(JNIEnv* env, jobject obj) {
+void WebContentsAndroid::Unselect(JNIEnv* env,
+                                  const JavaParamRef<jobject>& obj) {
   web_contents_->Unselect();
 }
 
-void WebContentsAndroid::InsertCSS(
-    JNIEnv* env, jobject jobj, jstring jcss) {
+void WebContentsAndroid::InsertCSS(JNIEnv* env,
+                                   const JavaParamRef<jobject>& jobj,
+                                   const JavaParamRef<jstring>& jcss) {
   web_contents_->InsertCSS(base::android::ConvertJavaStringToUTF8(env, jcss));
 }
 
@@ -282,60 +352,65 @@ RenderWidgetHostViewAndroid*
     rwhv = web_contents_->GetInterstitialPage()
                ->GetMainFrame()
                ->GetRenderViewHost()
+               ->GetWidget()
                ->GetView();
   }
   return static_cast<RenderWidgetHostViewAndroid*>(rwhv);
 }
 
-jint WebContentsAndroid::GetBackgroundColor(JNIEnv* env, jobject obj) {
+jint WebContentsAndroid::GetBackgroundColor(JNIEnv* env,
+                                            const JavaParamRef<jobject>& obj) {
   RenderWidgetHostViewAndroid* rwhva = GetRenderWidgetHostViewAndroid();
   if (!rwhva)
     return SK_ColorWHITE;
   return rwhva->GetCachedBackgroundColor();
 }
 
-ScopedJavaLocalRef<jstring> WebContentsAndroid::GetURL(JNIEnv* env,
-                                                       jobject obj) const {
+ScopedJavaLocalRef<jstring> WebContentsAndroid::GetURL(
+    JNIEnv* env,
+    const JavaParamRef<jobject>& obj) const {
   return ConvertUTF8ToJavaString(env, web_contents_->GetURL().spec());
 }
 
 ScopedJavaLocalRef<jstring> WebContentsAndroid::GetLastCommittedURL(
     JNIEnv* env,
-    jobject) const {
+    const JavaParamRef<jobject>&) const {
   return ConvertUTF8ToJavaString(env,
                                  web_contents_->GetLastCommittedURL().spec());
 }
 
-
-jboolean WebContentsAndroid::IsIncognito(JNIEnv* env, jobject obj) {
+jboolean WebContentsAndroid::IsIncognito(JNIEnv* env,
+                                         const JavaParamRef<jobject>& obj) {
   return web_contents_->GetBrowserContext()->IsOffTheRecord();
 }
 
-void WebContentsAndroid::ResumeLoadingCreatedWebContents(JNIEnv* env,
-                                                         jobject obj) {
+void WebContentsAndroid::ResumeLoadingCreatedWebContents(
+    JNIEnv* env,
+    const JavaParamRef<jobject>& obj) {
   web_contents_->ResumeLoadingCreatedWebContents();
 }
 
-void WebContentsAndroid::OnHide(JNIEnv* env, jobject obj) {
+void WebContentsAndroid::OnHide(JNIEnv* env, const JavaParamRef<jobject>& obj) {
   web_contents_->WasHidden();
 }
 
-void WebContentsAndroid::OnShow(JNIEnv* env, jobject obj) {
+void WebContentsAndroid::OnShow(JNIEnv* env, const JavaParamRef<jobject>& obj) {
   web_contents_->WasShown();
 }
 
-void WebContentsAndroid::ReleaseMediaPlayers(JNIEnv* env, jobject jobj) {
+void WebContentsAndroid::ReleaseMediaPlayers(
+    JNIEnv* env,
+    const JavaParamRef<jobject>& jobj) {
 #if defined(ENABLE_BROWSER_CDMS)
   web_contents_->ForEachFrame(
       base::Bind(&ReleaseAllMediaPlayers, base::Unretained(web_contents_)));
 #endif // defined(ENABLE_BROWSER_CDMS)
 }
 
-void WebContentsAndroid::ShowInterstitialPage(
-    JNIEnv* env,
-    jobject obj,
-    jstring jurl,
-    jlong delegate_ptr) {
+void WebContentsAndroid::ShowInterstitialPage(JNIEnv* env,
+                                              const JavaParamRef<jobject>& obj,
+                                              const JavaParamRef<jstring>& jurl,
+                                              jlong delegate_ptr) {
   GURL url(base::android::ConvertJavaStringToUTF8(env, jurl));
   InterstitialPageDelegateAndroid* delegate =
       reinterpret_cast<InterstitialPageDelegateAndroid*>(delegate_ptr);
@@ -345,25 +420,33 @@ void WebContentsAndroid::ShowInterstitialPage(
   interstitial->Show();
 }
 
-jboolean WebContentsAndroid::IsShowingInterstitialPage(JNIEnv* env,
-                                                        jobject obj) {
+jboolean WebContentsAndroid::IsShowingInterstitialPage(
+    JNIEnv* env,
+    const JavaParamRef<jobject>& obj) {
   return web_contents_->ShowingInterstitialPage();
 }
 
+jboolean WebContentsAndroid::FocusLocationBarByDefault(
+    JNIEnv* env,
+    const JavaParamRef<jobject>& obj) {
+  return web_contents_->FocusLocationBarByDefault();
+}
+
 jboolean WebContentsAndroid::IsRenderWidgetHostViewReady(
     JNIEnv* env,
-    jobject obj) {
+    const JavaParamRef<jobject>& obj) {
   RenderWidgetHostViewAndroid* view = GetRenderWidgetHostViewAndroid();
   return view && view->HasValidFrame();
 }
 
-void WebContentsAndroid::ExitFullscreen(JNIEnv* env, jobject obj) {
+void WebContentsAndroid::ExitFullscreen(JNIEnv* env,
+                                        const JavaParamRef<jobject>& obj) {
   web_contents_->ExitFullscreen();
 }
 
 void WebContentsAndroid::UpdateTopControlsState(
     JNIEnv* env,
-    jobject obj,
+    const JavaParamRef<jobject>& obj,
     bool enable_hiding,
     bool enable_showing,
     bool animate) {
@@ -376,7 +459,8 @@ void WebContentsAndroid::UpdateTopControlsState(
                                                 animate));
 }
 
-void WebContentsAndroid::ShowImeIfNeeded(JNIEnv* env, jobject obj) {
+void WebContentsAndroid::ShowImeIfNeeded(JNIEnv* env,
+                                         const JavaParamRef<jobject>& obj) {
   RenderViewHost* host = web_contents_->GetRenderViewHost();
   if (!host)
     return;
@@ -385,7 +469,7 @@ void WebContentsAndroid::ShowImeIfNeeded(JNIEnv* env, jobject obj) {
 
 void WebContentsAndroid::ScrollFocusedEditableNodeIntoView(
     JNIEnv* env,
-    jobject obj) {
+    const JavaParamRef<jobject>& obj) {
   RenderViewHost* host = web_contents_->GetRenderViewHost();
   if (!host)
     return;
@@ -393,24 +477,28 @@ void WebContentsAndroid::ScrollFocusedEditableNodeIntoView(
       host->GetRoutingID(), gfx::Rect()));
 }
 
-void WebContentsAndroid::SelectWordAroundCaret(JNIEnv* env, jobject obj) {
+void WebContentsAndroid::SelectWordAroundCaret(
+    JNIEnv* env,
+    const JavaParamRef<jobject>& obj) {
   RenderViewHost* host = web_contents_->GetRenderViewHost();
   if (!host)
     return;
   host->SelectWordAroundCaret();
 }
 
-void WebContentsAndroid::AdjustSelectionByCharacterOffset(JNIEnv* env,
-                                                          jobject obj,
-                                                          jint start_adjust,
-                                                          jint end_adjust) {
+void WebContentsAndroid::AdjustSelectionByCharacterOffset(
+    JNIEnv* env,
+    const JavaParamRef<jobject>& obj,
+    jint start_adjust,
+    jint end_adjust) {
   web_contents_->AdjustSelectionByCharacterOffset(start_adjust, end_adjust);
 }
 
-void WebContentsAndroid::EvaluateJavaScript(JNIEnv* env,
-                                            jobject obj,
-                                            jstring script,
-                                            jobject callback) {
+void WebContentsAndroid::EvaluateJavaScript(
+    JNIEnv* env,
+    const JavaParamRef<jobject>& obj,
+    const JavaParamRef<jstring>& script,
+    const JavaParamRef<jobject>& callback) {
   RenderViewHost* rvh = web_contents_->GetRenderViewHost();
   DCHECK(rvh);
 
@@ -440,10 +528,45 @@ void WebContentsAndroid::EvaluateJavaScript(JNIEnv* env,
       ConvertJavaStringToUTF16(env, script), js_callback);
 }
 
-void WebContentsAndroid::AddMessageToDevToolsConsole(JNIEnv* env,
-                                                     jobject jobj,
-                                                     jint level,
-                                                     jstring message) {
+void WebContentsAndroid::EvaluateJavaScriptForTests(
+    JNIEnv* env,
+    const JavaParamRef<jobject>& obj,
+    const JavaParamRef<jstring>& script,
+    const JavaParamRef<jobject>& callback) {
+  RenderViewHost* rvh = web_contents_->GetRenderViewHost();
+  DCHECK(rvh);
+
+  if (!rvh->IsRenderViewLive()) {
+    if (!static_cast<WebContentsImpl*>(web_contents_)->
+        CreateRenderViewForInitialEmptyDocument()) {
+      LOG(ERROR) << "Failed to create RenderView in EvaluateJavaScriptForTests";
+      return;
+    }
+  }
+
+  if (!callback) {
+    // No callback requested.
+    web_contents_->GetMainFrame()->ExecuteJavaScriptForTests(
+        ConvertJavaStringToUTF16(env, script));
+    return;
+  }
+
+  // Secure the Java callback in a scoped object and give ownership of it to the
+  // base::Callback.
+  ScopedJavaGlobalRef<jobject> j_callback;
+  j_callback.Reset(env, callback);
+  RenderFrameHost::JavaScriptResultCallback js_callback =
+      base::Bind(&JavaScriptResultCallback, j_callback);
+
+  web_contents_->GetMainFrame()->ExecuteJavaScriptForTests(
+      ConvertJavaStringToUTF16(env, script), js_callback);
+}
+
+void WebContentsAndroid::AddMessageToDevToolsConsole(
+    JNIEnv* env,
+    const JavaParamRef<jobject>& jobj,
+    jint level,
+    const JavaParamRef<jstring>& message) {
   DCHECK_GE(level, 0);
   DCHECK_LE(level, CONSOLE_MESSAGE_LEVEL_LAST);
 
@@ -452,22 +575,38 @@ void WebContentsAndroid::AddMessageToDevToolsConsole(JNIEnv* env,
       ConvertJavaStringToUTF8(env, message));
 }
 
+void WebContentsAndroid::SendMessageToFrame(
+    JNIEnv* env,
+    const JavaParamRef<jobject>& obj,
+    const JavaParamRef<jstring>& frame_name,
+    const JavaParamRef<jstring>& message,
+    const JavaParamRef<jstring>& target_origin) {
+  base::string16 source_origin;
+  base::string16 j_target_origin(ConvertJavaStringToUTF16(env, target_origin));
+  base::string16 j_message(ConvertJavaStringToUTF16(env, message));
+  std::vector<content::TransferredMessagePort> ports;
+  content::MessagePortProvider::PostMessageToFrame(
+      web_contents_, source_origin, j_target_origin, j_message, ports);
+}
+
 jboolean WebContentsAndroid::HasAccessedInitialDocument(
     JNIEnv* env,
-    jobject jobj) {
+    const JavaParamRef<jobject>& jobj) {
   return static_cast<WebContentsImpl*>(web_contents_)->
       HasAccessedInitialDocument();
 }
 
-jint WebContentsAndroid::GetThemeColor(JNIEnv* env, jobject obj) {
+jint WebContentsAndroid::GetThemeColor(JNIEnv* env,
+                                       const JavaParamRef<jobject>& obj) {
   return web_contents_->GetThemeColor();
 }
 
-void WebContentsAndroid::RequestAccessibilitySnapshot(JNIEnv* env,
-                                                      jobject obj,
-                                                      jobject callback,
-                                                      jfloat y_offset,
-                                                      jfloat x_scroll) {
+void WebContentsAndroid::RequestAccessibilitySnapshot(
+    JNIEnv* env,
+    const JavaParamRef<jobject>& obj,
+    const JavaParamRef<jobject>& callback,
+    jfloat y_offset,
+    jfloat x_scroll) {
   // Secure the Java callback in a scoped object and give ownership of it to the
   // base::Callback.
   ScopedJavaGlobalRef<jobject> j_callback;
@@ -475,19 +614,88 @@ void WebContentsAndroid::RequestAccessibilitySnapshot(JNIEnv* env,
   gfx::DeviceDisplayInfo device_info;
   ContentViewCoreImpl* contentViewCore =
       ContentViewCoreImpl::FromWebContents(web_contents_);
+
+  AccessibilitySnapshotParams* params = new AccessibilitySnapshotParams(
+      contentViewCore->GetScaleFactor(), y_offset, x_scroll);
   WebContentsImpl::AXTreeSnapshotCallback snapshot_callback =
-      base::Bind(&AXTreeSnapshotCallback, j_callback,
-          contentViewCore->GetScaleFactor(), y_offset, x_scroll);
+      base::Bind(&AXTreeSnapshotCallback, j_callback, base::Owned(params));
   static_cast<WebContentsImpl*>(web_contents_)->RequestAXTreeSnapshot(
       snapshot_callback);
 }
 
-void WebContentsAndroid::ResumeMediaSession(JNIEnv* env, jobject obj) {
+void WebContentsAndroid::ResumeMediaSession(JNIEnv* env,
+                                            const JavaParamRef<jobject>& obj) {
   web_contents_->ResumeMediaSession();
 }
 
-void WebContentsAndroid::SuspendMediaSession(JNIEnv* env, jobject obj) {
+void WebContentsAndroid::SuspendMediaSession(JNIEnv* env,
+                                             const JavaParamRef<jobject>& obj) {
   web_contents_->SuspendMediaSession();
 }
 
+void WebContentsAndroid::StopMediaSession(JNIEnv* env,
+                                          const JavaParamRef<jobject>& obj) {
+  web_contents_->StopMediaSession();
+}
+
+ScopedJavaLocalRef<jstring> WebContentsAndroid::GetEncoding(
+    JNIEnv* env,
+    const JavaParamRef<jobject>& obj) const {
+  return base::android::ConvertUTF8ToJavaString(env,
+                                                web_contents_->GetEncoding());
+}
+
+void WebContentsAndroid::GetContentBitmap(
+    JNIEnv* env,
+    const JavaParamRef<jobject>& obj,
+    const JavaParamRef<jobject>& jcallback,
+    const JavaParamRef<jobject>& color_type,
+    jfloat scale,
+    jfloat x,
+    jfloat y,
+    jfloat width,
+    jfloat height) {
+  RenderWidgetHostViewAndroid* view = GetRenderWidgetHostViewAndroid();
+  const ReadbackRequestCallback result_callback =
+      base::Bind(&WebContentsAndroid::OnFinishGetContentBitmap,
+                 weak_factory_.GetWeakPtr(),
+                 base::Owned(new ScopedJavaGlobalRef<jobject>(env, obj)),
+                 base::Owned(new ScopedJavaGlobalRef<jobject>(env, jcallback)));
+  SkColorType pref_color_type = gfx::ConvertToSkiaColorType(color_type.obj());
+  if (!view || pref_color_type == kUnknown_SkColorType) {
+    result_callback.Run(SkBitmap(), READBACK_FAILED);
+    return;
+  }
+  if (!view->IsSurfaceAvailableForCopy()) {
+    result_callback.Run(SkBitmap(), READBACK_SURFACE_UNAVAILABLE);
+    return;
+  }
+  view->GetScaledContentBitmap(scale,
+                               pref_color_type,
+                               gfx::Rect(x, y, width, height),
+                               result_callback);
+}
+
+void WebContentsAndroid::OnContextMenuClosed(JNIEnv* env,
+                                             const JavaParamRef<jobject>& obj) {
+  static_cast<WebContentsImpl*>(web_contents_)
+      ->NotifyContextMenuClosed(CustomContextMenuContext());
+}
+
+void WebContentsAndroid::OnFinishGetContentBitmap(
+    ScopedJavaGlobalRef<jobject>* obj,
+    ScopedJavaGlobalRef<jobject>* callback,
+    const SkBitmap& bitmap,
+    ReadbackResponse response) {
+  JNIEnv* env = base::android::AttachCurrentThread();
+  ScopedJavaLocalRef<jobject> java_bitmap;
+  if (response == READBACK_SUCCESS)
+    java_bitmap = gfx::ConvertToJavaBitmap(&bitmap);
+  Java_WebContentsImpl_onGetContentBitmapFinished(env,
+                                                  obj->obj(),
+                                                  callback->obj(),
+                                                  java_bitmap.obj(),
+                                                  response);
+}
+
 }  // namespace content
