@@ -4,6 +4,9 @@
 
 #include "content/child/v8_value_converter_impl.h"
 
+#include <stddef.h>
+#include <stdint.h>
+
 #include <cmath>
 #include <string>
 
@@ -399,8 +402,8 @@ base::Value* V8ValueConverterImpl::FromV8Array(
   base::ListValue* result = new base::ListValue();
 
   // Only fields with integer keys are carried over to the ListValue.
-  for (uint32 i = 0; i < val->Length(); ++i) {
-    v8::TryCatch try_catch;
+  for (uint32_t i = 0; i < val->Length(); ++i) {
+    v8::TryCatch try_catch(isolate);
     v8::Local<v8::Value> child_v8 = val->Get(i);
     if (try_catch.HasCaught()) {
       LOG(ERROR) << "Getter for index " << i << " threw an exception.";
@@ -500,7 +503,7 @@ base::Value* V8ValueConverterImpl::FromV8Object(
   scoped_ptr<base::DictionaryValue> result(new base::DictionaryValue());
   v8::Local<v8::Array> property_names(val->GetOwnPropertyNames());
 
-  for (uint32 i = 0; i < property_names->Length(); ++i) {
+  for (uint32_t i = 0; i < property_names->Length(); ++i) {
     v8::Local<v8::Value> key(property_names->Get(i));
 
     // Extend this test to cover more types as necessary and if sensible.
@@ -513,7 +516,7 @@ base::Value* V8ValueConverterImpl::FromV8Object(
 
     v8::String::Utf8Value name_utf8(key);
 
-    v8::TryCatch try_catch;
+    v8::TryCatch try_catch(isolate);
     v8::Local<v8::Value> child_v8 = val->Get(key);
 
     if (try_catch.HasCaught()) {
