@@ -4,6 +4,9 @@
 
 #include "content/common/child_process_sandbox_support_impl_linux.h"
 
+#include <stddef.h>
+#include <stdint.h>
+
 #include "base/pickle.h"
 #include "base/posix/unix_domain_socket_linux.h"
 #include "content/common/sandbox_linux/sandbox_linux.h"
