@@ -9,6 +9,18 @@
 
 namespace content {
 
+gfx::NativeCursor WebCursor::GetNativeCursor() {
+  return gfx::kNullCursor;
+}
+
+#if defined(USE_AURA)
+// In the future when we want to support cursors of various kinds in Aura on
+// Android, we should switch to using webcursor_aura rather than add an
+// implementation here.
+void WebCursor::SetDisplayInfo(const gfx::Display& display) {
+}
+#endif
+
 void WebCursor::InitPlatformData() {
 }
 
