@@ -10,15 +10,20 @@ namespace content {
 
 FrameReplicationState::FrameReplicationState()
     : sandbox_flags(blink::WebSandboxFlags::None),
-      scope(blink::WebTreeScopeType::Document) {
-}
+      scope(blink::WebTreeScopeType::Document),
+      should_enforce_strict_mixed_content_checking(false) {}
 
 FrameReplicationState::FrameReplicationState(
     blink::WebTreeScopeType scope,
     const std::string& name,
-    blink::WebSandboxFlags sandbox_flags)
-    : origin(), sandbox_flags(sandbox_flags), name(name), scope(scope) {
-}
+    blink::WebSandboxFlags sandbox_flags,
+    bool should_enforce_strict_mixed_content_checking)
+    : origin(),
+      sandbox_flags(sandbox_flags),
+      name(name),
+      scope(scope),
+      should_enforce_strict_mixed_content_checking(
+          should_enforce_strict_mixed_content_checking) {}
 
 FrameReplicationState::~FrameReplicationState() {
 }
