@@ -5,6 +5,8 @@
 #ifndef CONTENT_PUBLIC_BROWSER_CONTENT_BROWSER_CLIENT_H_
 #define CONTENT_PUBLIC_BROWSER_CONTENT_BROWSER_CLIENT_H_
 
+#include <stddef.h>
+
 #include <map>
 #include <string>
 #include <utility>
@@ -15,7 +17,9 @@
 #include "base/memory/scoped_ptr.h"
 #include "base/memory/scoped_vector.h"
 #include "base/values.h"
+#include "build/build_config.h"
 #include "content/public/browser/certificate_request_result_type.h"
+#include "content/public/browser/navigation_throttle.h"
 #include "content/public/common/content_client.h"
 #include "content/public/common/media_stream_request.h"
 #include "content/public/common/resource_type.h"
@@ -54,6 +58,10 @@ class GLShareGroup;
 class ImageSkia;
 }
 
+namespace media {
+class CdmFactory;
+}
+
 namespace mojo {
 class ApplicationDelegate;
 }
@@ -80,6 +88,7 @@ class SelectFilePolicy;
 namespace storage {
 class ExternalMountPoints;
 class FileSystemBackend;
+class QuotaEvictionPolicy;
 }
 
 namespace content {
@@ -97,6 +106,7 @@ class DevToolsManagerDelegate;
 class ExternalVideoSurfaceContainer;
 class LocationProvider;
 class MediaObserver;
+class NavigationHandle;
 class NavigatorConnectContext;
 class NavigatorConnectServiceFactory;
 class PlatformNotificationService;
@@ -179,6 +189,27 @@ class CONTENT_EXPORT ContentBrowserClient {
   virtual bool ShouldUseProcessPerSite(BrowserContext* browser_context,
                                        const GURL& effective_url);
 
+  // Returns true if site isolation should be enabled for |effective_url|. This
+  // call allows the embedder to supplement the site isolation policy enforced
+  // by the content layer.
+  //
+  // Will only be called if both of the following happen:
+  //   1. The embedder asked to be consulted, by returning true from
+  //      ContentClient::IsSupplementarySiteIsolationModeEnabled().
+  //   2. The content layer didn't decide to isolate |effective_url| according
+  //      to its internal policy (e.g. because of --site-per-process).
+  virtual bool DoesSiteRequireDedicatedProcess(BrowserContext* browser_context,
+                                               const GURL& effective_url);
+
+  // Returns true unless the effective URL is part of a site that cannot live in
+  // a process restricted to just that site.  This is only called if site
+  // isolation is enabled for this URL, and is a bug workaround.
+  //
+  // TODO(nick): Remove this function once https://crbug.com/160576 is fixed,
+  // and origin lock can be applied to all URLs.
+  virtual bool ShouldLockToOrigin(BrowserContext* browser_context,
+                                  const GURL& effective_url);
+
   // Returns a list additional WebUI schemes, if any.  These additional schemes
   // act as aliases to the chrome: scheme.  The additional schemes may or may
   // not serve specific WebUI pages depending on the particular URLDataSource
@@ -192,6 +223,11 @@ class CONTENT_EXPORT ContentBrowserClient {
   virtual void GetAdditionalWebUIHostsToIgnoreParititionCheck(
       std::vector<std::string>* hosts) {}
 
+  // Called when WebUI objects are created to get aggregate usage data (i.e. is
+  // chrome://downloads used more than chrome://bookmarks?). Only internal (e.g.
+  // chrome://) URLs are logged. Returns whether the URL was actually logged.
+  virtual bool LogWebUIUrl(const GURL& web_ui_url) const;
+
   // Creates the main net::URLRequestContextGetter. Should only be called once
   // per ContentBrowserClient object.
   // TODO(ajwong): Remove once http://crbug.com/159193 is resolved.
@@ -218,8 +254,17 @@ class CONTENT_EXPORT ContentBrowserClient {
   // more conservative check than IsSuitableHost, since it is used after a
   // navigation has committed to ensure that the process did not exceed its
   // authority.
+  // This is called on the UI thread.
   virtual bool CanCommitURL(RenderProcessHost* process_host, const GURL& url);
 
+  // Returns true if no URL within |origin| is allowed to commit in the given
+  // process.  Must return false if there exists at least one URL in |origin|
+  // that is allowed to commit.
+  // This is called on the IO thread.
+  virtual bool IsIllegalOrigin(ResourceContext* resource_context,
+                               int child_process_id,
+                               const GURL& origin);
+
   // Returns whether a URL should be allowed to open from a specific context.
   // This also applies in cases where the new URL will open in another process.
   virtual bool ShouldAllowOpenURL(SiteInstance* site_instance, const GURL& url);
@@ -294,6 +339,8 @@ class CONTENT_EXPORT ContentBrowserClient {
 
   // Allow the embedder to control if a Service Worker can be associated
   // with the given scope.
+  // Invalid (-1) process/frame ids indicate this is for starting a service
+  // worker, which is not necessarily associated with a particular frame.
   // This is called on the IO thread.
   virtual bool AllowServiceWorker(const GURL& scope,
                                   const GURL& first_party,
@@ -318,7 +365,7 @@ class CONTENT_EXPORT ContentBrowserClient {
                               ResourceContext* context,
                               int render_process_id,
                               int render_frame_id,
-                              net::CookieOptions* options);
+                              const net::CookieOptions& options);
 
   // This is called on the IO thread.
   virtual bool AllowSaveLocalState(ResourceContext* context);
@@ -362,6 +409,9 @@ class CONTENT_EXPORT ContentBrowserClient {
                                         ResourceContext* context);
 #endif  // defined(ENABLE_WEBRTC)
 
+  // Allow the embedder to control whether we can use <keygen>.
+  virtual bool AllowKeygen(const GURL& url, content::ResourceContext* context);
+
   // Allow the embedder to override the request context based on the URL for
   // certain operations, like cookie access. Returns nullptr to indicate the
   // regular request context should be used.
@@ -407,14 +457,18 @@ class CONTENT_EXPORT ContentBrowserClient {
   // Create and return a new quota permission context.
   virtual QuotaPermissionContext* CreateQuotaPermissionContext();
 
+  // Gives the embedder a chance to register a custom QuotaEvictionPolicy for
+  // temporary storage.
+  virtual scoped_ptr<storage::QuotaEvictionPolicy>
+  GetTemporaryStorageEvictionPolicy(BrowserContext* context);
+
   // Informs the embedder that a certificate error has occured.  If
   // |overridable| is true and if |strict_enforcement| is false, the user
   // can ignore the error and continue. The embedder can call the callback
   // asynchronously. If |result| is not set to
   // CERTIFICATE_REQUEST_RESULT_TYPE_CONTINUE, the request will be cancelled
   // or denied immediately, and the callback won't be run.
-  virtual void AllowCertificateError(int render_process_id,
-                                     int render_frame_id,
+  virtual void AllowCertificateError(WebContents* web_contents,
                                      int cert_error,
                                      const net::SSLInfo& ssl_info,
                                      const GURL& request_url,
@@ -515,6 +569,9 @@ class CONTENT_EXPORT ContentBrowserClient {
   // else we should do with the file.
   virtual std::string GetDefaultDownloadName();
 
+  // Returns the path to the browser shader disk cache root.
+  virtual base::FilePath GetShaderDiskCacheDirectory();
+
   // Notification that a pepper plugin has just been spawned. This allows the
   // embedder to add filters onto the host to implement interfaces.
   // This is called on the IO thread.
@@ -590,22 +647,47 @@ class CONTENT_EXPORT ContentBrowserClient {
       BrowserContext* browser_context,
       const GURL& url);
 
-  // Allows to override browser Mojo services exposed through the
+  // Allows to register browser Mojo services exposed through the
   // RenderProcessHost.
-  virtual void OverrideRenderProcessMojoServices(ServiceRegistry* registry) {}
+  virtual void RegisterRenderProcessMojoServices(ServiceRegistry* registry) {}
 
-  // Allows to override browser Mojo services exposed through the
+  // Allows to register browser Mojo services exposed through the
   // FrameMojoShell.
-  virtual void OverrideFrameMojoShellServices(
+  virtual void RegisterFrameMojoShellServices(
+      ServiceRegistry* registry,
+      RenderFrameHost* render_frame_host) {}
+
+  // Allows to register browser Mojo services exposed through the
+  // RenderFrameHost.
+  virtual void RegisterRenderFrameMojoServices(
       ServiceRegistry* registry,
       RenderFrameHost* render_frame_host) {}
 
   using StaticMojoApplicationMap =
       std::map<GURL, base::Callback<scoped_ptr<mojo::ApplicationDelegate>()>>;
 
-  // Registers in-process Mojo application loaders with the browser's global
-  // Mojo shell.
-  virtual void RegisterMojoApplications(StaticMojoApplicationMap* apps) {}
+  // Registers Mojo applications to be loaded in the browser process by the
+  // browser's global Mojo shell.
+  virtual void RegisterInProcessMojoApplications(
+      StaticMojoApplicationMap* apps) {}
+
+  using OutOfProcessMojoApplicationMap = std::map<GURL, base::string16>;
+
+  // Registers Mojo applications to be loaded out of the browser process, in a
+  // sandboxed utility process. The value of each map entry should be the
+  // process name to use for the application's host process when launched.
+  virtual void RegisterOutOfProcessMojoApplications(
+      OutOfProcessMojoApplicationMap* apps) {}
+
+  // Registers Mojo applications to be loaded out of the browser process (in
+  // a utility process) without the sandbox.
+  //
+  // WARNING: This path is NOT recommended! If a Mojo application needs a
+  // service that is only available out of the sandbox, it could ask the browser
+  // process to provide it (e.g. through OverrideFrameMojoShellServices()). Only
+  // use this method when that approach does not work.
+  virtual void RegisterUnsandboxedOutOfProcessMojoApplications(
+      OutOfProcessMojoApplicationMap* apps) {}
 
   // Registers additional navigator.connect service factories available in a
   // particular NavigatorConnectContext.
@@ -633,6 +715,18 @@ class CONTENT_EXPORT ContentBrowserClient {
   // Allows the embedder to record |metric| for a specific |url|.
   virtual void RecordURLMetric(const std::string& metric, const GURL& url) {}
 
+  // Allows the embedder to register one or more NavigationThrottles for the
+  // navigation indicated by |navigation_handle|.  A NavigationThrottle is used
+  // to control the flow of a navigation on the UI thread. The embedder is
+  // guaranteed that the throttles will be executed in the order they were
+  // provided.
+  virtual ScopedVector<NavigationThrottle> CreateThrottlesForNavigation(
+      NavigationHandle* navigation_handle);
+
+  // Creates and returns a factory used for creating CDM instances for playing
+  // protected content.
+  virtual scoped_ptr<media::CdmFactory> CreateCdmFactory();
+
   // Populates |mappings| with all files that need to be mapped before launching
   // a child process.
 #if defined(OS_ANDROID)
@@ -655,14 +749,22 @@ class CONTENT_EXPORT ContentBrowserClient {
   // This is called on the PROCESS_LAUNCHER thread before the renderer process
   // is launched. It gives the embedder a chance to add loosen the sandbox
   // policy.
-  virtual void PreSpawnRenderer(sandbox::TargetPolicy* policy,
-                                bool* success) {}
+  virtual bool PreSpawnRenderer(sandbox::TargetPolicy* policy);
 
   // Returns the AppContainer SID for the specified sandboxed process type, or
   // empty string if this sandboxed process type does not support living inside
   // an AppContainer.
   virtual base::string16 GetAppContainerSidForSandboxType(
       int sandbox_type) const;
+
+  // Returns whether the Win32k lockdown process mitigation should be applied to
+  // a process hosting a plugin with the specified |mime_type|.
+  virtual bool IsWin32kLockdownEnabledForMimeType(
+      const std::string& mime_type) const;
+
+  // Returns true if processes should be launched with a /prefetch:# argument.
+  // See the kPrefetchArgument* constants in content_switches.cc for details.
+  virtual bool ShouldUseWindowsPrefetchArgument() const;
 #endif
 
 #if defined(VIDEO_HOLE)
