@@ -5,14 +5,16 @@
 #ifndef CONTENT_PUBLIC_BROWSER_WEB_CONTENTS_H_
 #define CONTENT_PUBLIC_BROWSER_WEB_CONTENTS_H_
 
+#include <stdint.h>
+
 #include <set>
 
-#include "base/basictypes.h"
 #include "base/callback_forward.h"
 #include "base/files/file_path.h"
 #include "base/process/kill.h"
 #include "base/strings/string16.h"
 #include "base/supports_user_data.h"
+#include "build/build_config.h"
 #include "content/common/content_export.h"
 #include "content/public/browser/invalidate_type.h"
 #include "content/public/browser/navigation_controller.h"
@@ -58,6 +60,7 @@ class WebContentsDelegate;
 struct CustomContextMenuContext;
 struct DropData;
 struct Manifest;
+struct PageImportanceSignals;
 struct RendererPreferences;
 
 // WebContents is the core class in content/. A WebContents renders web content
@@ -112,11 +115,13 @@ class WebContents : public PageNavigator,
     // (e.g., for blocked popups).
     bool created_with_opener;
 
-    // The routing ids of the RenderView and of the main RenderFrame. Either
-    // both must be provided, or both must be MSG_ROUTING_NONE to have the
-    // WebContents make the assignment.
-    int routing_id;
-    int main_frame_routing_id;
+    // The routing ids of the RenderView, main RenderFrame, and the widget for
+    // the main RenderFrame. Either all routing IDs must be provided or all must
+    // be MSG_ROUTING_NONE to have WebContents make the assignment. If provided,
+    // these routing IDs are associated with |site_instance->GetProcess()|.
+    int32_t routing_id;
+    int32_t main_frame_routing_id;
+    int32_t main_frame_widget_routing_id;
 
     // The name of the top-level frame of the new window. It is non-empty
     // when creating a named window (e.g. <a target="foo"> or
@@ -160,10 +165,9 @@ class WebContents : public PageNavigator,
       const CreateParams& params,
       const SessionStorageNamespaceMap& session_storage_namespace_map);
 
-  // Returns a WebContents that wraps the RenderViewHost, or nullptr if the
+  // Returns the WebContents that owns the RenderViewHost, or nullptr if the
   // render view host's delegate isn't a WebContents.
-  CONTENT_EXPORT static WebContents* FromRenderViewHost(
-      const RenderViewHost* rvh);
+  CONTENT_EXPORT static WebContents* FromRenderViewHost(RenderViewHost* rvh);
 
   CONTENT_EXPORT static WebContents* FromRenderFrameHost(RenderFrameHost* rfh);
 
@@ -210,6 +214,12 @@ class WebContents : public PageNavigator,
   // Returns the focused frame for the currently active view.
   virtual RenderFrameHost* GetFocusedFrame() = 0;
 
+  // Returns the current RenderFrameHost for a given FrameTreeNode ID if it is
+  // part of this tab. See RenderFrameHost::GetFrameTreeNodeId for documentation
+  // on this ID.
+  virtual RenderFrameHost* FindFrameByFrameTreeNodeId(
+      int frame_tree_node_id) = 0;
+
   // Calls |on_frame| for each frame in the currently active view.
   // Note: The RenderFrameHost parameter is not guaranteed to have a live
   // RenderFrame counterpart in the renderer process. Callbacks should check
@@ -218,9 +228,10 @@ class WebContents : public PageNavigator,
   virtual void ForEachFrame(
       const base::Callback<void(RenderFrameHost*)>& on_frame) = 0;
 
-  // Sends the given IPC to all frames in the currently active view. This is a
-  // convenience method instead of calling ForEach.
-  virtual void SendToAllFrames(IPC::Message* message) = 0;
+  // Sends the given IPC to all frames in the currently active view and returns
+  // the number of sent messages (i.e. the number of processed frames). This is
+  // a convenience method instead of calling ForEach.
+  virtual int SendToAllFrames(IPC::Message* message) = 0;
 
   // Gets the current RenderViewHost for this tab.
   virtual RenderViewHost* GetRenderViewHost() const = 0;
@@ -248,8 +259,10 @@ class WebContents : public PageNavigator,
   // Create a WebUI page for the given url. In most cases, this doesn't need to
   // be called by embedders since content will create its own WebUI objects as
   // necessary. However if the embedder wants to create its own WebUI object and
-  // keep track of it manually, it can use this.
-  virtual WebUI* CreateWebUI(const GURL& url) = 0;
+  // keep track of it manually, it can use this. |frame_name| is used to
+  // identify the frame and cannot be empty.
+  virtual WebUI* CreateSubframeWebUI(const GURL& url,
+                                     const std::string& frame_name) = 0;
 
   // Returns the committed WebUI if one exists, otherwise the pending one.
   virtual WebUI* GetWebUI() const = 0;
@@ -276,6 +289,8 @@ class WebContents : public PageNavigator,
       gfx::NativeViewAccessible accessible_parent) = 0;
 #endif
 
+  virtual const PageImportanceSignals& GetPageImportanceSignals() const = 0;
+
   // Tab navigation state ------------------------------------------------------
 
   // Returns the current navigation properties, which if a navigation is
@@ -287,11 +302,11 @@ class WebContents : public PageNavigator,
   // this WebContents.  Page IDs are specific to a given SiteInstance and
   // WebContents, corresponding to a specific RenderView in the renderer.
   // Page IDs increase with each new page that is loaded by a tab.
-  virtual int32 GetMaxPageID() = 0;
+  virtual int32_t GetMaxPageID() = 0;
 
   // The max page ID for any page that the given SiteInstance has loaded in
   // this WebContents.
-  virtual int32 GetMaxPageIDForSiteInstance(SiteInstance* site_instance) = 0;
+  virtual int32_t GetMaxPageIDForSiteInstance(SiteInstance* site_instance) = 0;
 
   // Returns the SiteInstance associated with the current page.
   virtual SiteInstance* GetSiteInstance() const = 0;
@@ -318,11 +333,8 @@ class WebContents : public PageNavigator,
   virtual const base::string16& GetLoadStateHost() const = 0;
 
   // Returns the upload progress.
-  virtual uint64 GetUploadSize() const = 0;
-  virtual uint64 GetUploadPosition() const = 0;
-
-  // Returns a set of the site URLs currently committed in this tab.
-  virtual std::set<GURL> GetSitesInTab() const = 0;
+  virtual uint64_t GetUploadSize() const = 0;
+  virtual uint64_t GetUploadPosition() const = 0;
 
   // Returns the character encoding of the page.
   virtual const std::string& GetEncoding() const = 0;
@@ -521,8 +533,7 @@ class WebContents : public PageNavigator,
   // Generate an MHTML representation of the current page in the given file.
   virtual void GenerateMHTML(
       const base::FilePath& file,
-      const base::Callback<void(
-          int64 /* size of the file */)>& callback) = 0;
+      const base::Callback<void(int64_t /* size of the file */)>& callback) = 0;
 
   // Returns the contents MIME type after a navigation.
   virtual const std::string& GetContentsMimeType() const = 0;
@@ -574,8 +585,8 @@ class WebContents : public PageNavigator,
   virtual int GetMinimumZoomPercent() const = 0;
   virtual int GetMaximumZoomPercent() const = 0;
 
-  // Set the renderer's page scale back to one.
-  virtual void ResetPageScale() = 0;
+  // Set the renderer's page scale to the given factor.
+  virtual void SetPageScale(float page_scale_factor) = 0;
 
   // Gets the preferred size of the contents.
   virtual gfx::Size GetPreferredSize() const = 0;
@@ -637,7 +648,7 @@ class WebContents : public PageNavigator,
   // removed since we can then embed iframes in different processes.
   virtual bool IsSubframe() const = 0;
 
-  // Finds text on a page.
+  // Finds text on a page. |search_text| should not be empty.
   virtual void Find(int request_id,
                     const base::string16& search_text,
                     const blink::WebFindOptions& options) = 0;
@@ -655,7 +666,14 @@ class WebContents : public PageNavigator,
   typedef base::Callback<void(const Manifest&)> GetManifestCallback;
 
   // Requests the Manifest of the main frame's document.
-  virtual void GetManifest(const GetManifestCallback&) = 0;
+  virtual void GetManifest(const GetManifestCallback& callback) = 0;
+
+  typedef base::Callback<void(bool)> HasManifestCallback;
+
+  // Returns true if the main frame has a <link> to a web manifest, otherwise
+  // false. This method does not guarantee that the manifest exists at the
+  // specified location or is valid.
+  virtual void HasManifest(const HasManifestCallback& callback) = 0;
 
   // Requests the renderer to exit fullscreen.
   virtual void ExitFullscreen() = 0;
@@ -673,10 +691,13 @@ class WebContents : public PageNavigator,
   virtual void ResumeMediaSession() = 0;
   // Requests to suspend the current media session.
   virtual void SuspendMediaSession() = 0;
-
+  // Requests to stop the current media session.
+  virtual void StopMediaSession() = 0;
+#if !defined(USE_AURA)
   CONTENT_EXPORT static WebContents* FromJavaWebContents(
       jobject jweb_contents_android);
   virtual base::android::ScopedJavaLocalRef<jobject> GetJavaWebContents() = 0;
+#endif  // !USE_AURA
 #elif defined(OS_MACOSX)
   // Allowing other views disables optimizations which assume that only a single
   // WebContents is present.
