@@ -2,6 +2,7 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+#include "build/build_config.h"
 #include "content/public/common/sandboxed_process_launcher_delegate.h"
 
 namespace content {
@@ -15,6 +16,15 @@ bool SandboxedProcessLauncherDelegate::ShouldSandbox() {
   return true;
 }
 
+bool SandboxedProcessLauncherDelegate::DisableDefaultPolicy() {
+  return false;
+}
+
+bool SandboxedProcessLauncherDelegate::PreSpawnTarget(
+    sandbox::TargetPolicy* policy) {
+  return true;
+}
+
 #elif(OS_POSIX)
 bool SandboxedProcessLauncherDelegate::ShouldUseZygote() {
   return false;
