@@ -4,31 +4,31 @@
 
 #include "content/renderer/pepper/event_conversion.h"
 
-#include "base/basictypes.h"
+#include <stddef.h>
+#include <stdint.h>
+#include <string.h>
+
 #include "base/i18n/char_iterator.h"
 #include "base/logging.h"
 #include "base/memory/scoped_ptr.h"
+#include "base/strings/string16.h"
 #include "base/strings/string_util.h"
 #include "base/strings/stringprintf.h"
 #include "base/strings/utf_string_conversion_utils.h"
 #include "base/strings/utf_string_conversions.h"
+#include "build/build_config.h"
 #include "content/common/input/web_touch_event_traits.h"
 #include "ppapi/c/pp_input_event.h"
 #include "ppapi/shared_impl/ppb_input_event_shared.h"
-#include "ppapi/shared_impl/time_conversion.h"
 #include "third_party/WebKit/public/platform/WebGamepads.h"
-#include "third_party/WebKit/public/platform/WebString.h"
 #include "third_party/WebKit/public/web/WebInputEvent.h"
 #include "ui/events/keycodes/dom/keycode_converter.h"
 
-using ppapi::EventTimeToPPTimeTicks;
 using ppapi::InputEventData;
-using ppapi::PPTimeTicksToEventTime;
 using blink::WebInputEvent;
 using blink::WebKeyboardEvent;
 using blink::WebMouseEvent;
 using blink::WebMouseWheelEvent;
-using blink::WebString;
 using blink::WebTouchEvent;
 using blink::WebTouchPoint;
 using blink::WebUChar;
@@ -124,7 +124,7 @@ PP_InputEvent_Type ConvertEventTypes(WebInputEvent::Type wetype) {
 InputEventData GetEventWithCommonFieldsAndType(const WebInputEvent& web_event) {
   InputEventData result;
   result.event_type = ConvertEventTypes(web_event.type);
-  result.event_time_stamp = EventTimeToPPTimeTicks(web_event.timeStampSeconds);
+  result.event_time_stamp = web_event.timeStampSeconds;
   return result;
 }
 
@@ -135,14 +135,8 @@ void AppendKeyEvent(const WebInputEvent& event,
   InputEventData result = GetEventWithCommonFieldsAndType(event);
   result.event_modifiers = key_event.modifiers;
   result.key_code = key_event.windowsKeyCode;
-#if defined(OS_MACOSX)
-  // Workaround for |domCode| not being set on OS X. crbug.com/493833
-  result.code = ui::KeycodeConverter::DomCodeToCodeString(
-      ui::KeycodeConverter::NativeKeycodeToDomCode(key_event.nativeKeyCode));
-#else
   result.code = ui::KeycodeConverter::DomCodeToCodeString(
       static_cast<ui::DomCode>(key_event.domCode));
-#endif
   result_events->push_back(result);
 }
 
@@ -349,7 +343,7 @@ WebTouchEvent* BuildTouchEvent(const InputEventData& event) {
       NOTREACHED();
   }
   WebTouchEventTraits::ResetType(
-      type, PPTimeTicksToEventTime(event.event_time_stamp), web_event);
+      type, event.event_time_stamp, web_event);
   web_event->touchesLength = 0;
 
   // First add all changed touches, then add only the remaining unset
@@ -381,7 +375,7 @@ WebKeyboardEvent* BuildKeyEvent(const InputEventData& event) {
     default:
       NOTREACHED();
   }
-  key_event->timeStampSeconds = PPTimeTicksToEventTime(event.event_time_stamp);
+  key_event->timeStampSeconds = event.event_time_stamp;
   key_event->modifiers = event.event_modifiers;
   key_event->windowsKeyCode = event.key_code;
   key_event->setKeyIdentifierFromWindowsKeyCode();
@@ -391,7 +385,7 @@ WebKeyboardEvent* BuildKeyEvent(const InputEventData& event) {
 WebKeyboardEvent* BuildCharEvent(const InputEventData& event) {
   WebKeyboardEvent* key_event = new WebKeyboardEvent();
   key_event->type = WebInputEvent::Char;
-  key_event->timeStampSeconds = PPTimeTicksToEventTime(event.event_time_stamp);
+  key_event->timeStampSeconds = event.event_time_stamp;
   key_event->modifiers = event.event_modifiers;
 
   // Make sure to not read beyond the buffer in case some bad code doesn't
@@ -430,8 +424,7 @@ WebMouseEvent* BuildMouseEvent(const InputEventData& event) {
     default:
       NOTREACHED();
   }
-  mouse_event->timeStampSeconds =
-      PPTimeTicksToEventTime(event.event_time_stamp);
+  mouse_event->timeStampSeconds = event.event_time_stamp;
   mouse_event->modifiers = event.event_modifiers;
   mouse_event->button = static_cast<WebMouseEvent::Button>(event.mouse_button);
   if (mouse_event->type == WebInputEvent::MouseMove) {
@@ -453,8 +446,7 @@ WebMouseEvent* BuildMouseEvent(const InputEventData& event) {
 WebMouseWheelEvent* BuildMouseWheelEvent(const InputEventData& event) {
   WebMouseWheelEvent* mouse_wheel_event = new WebMouseWheelEvent();
   mouse_wheel_event->type = WebInputEvent::MouseWheel;
-  mouse_wheel_event->timeStampSeconds =
-      PPTimeTicksToEventTime(event.event_time_stamp);
+  mouse_wheel_event->timeStampSeconds = event.event_time_stamp;
   mouse_wheel_event->modifiers = event.event_modifiers;
   mouse_wheel_event->deltaX = event.wheel_delta.x;
   mouse_wheel_event->deltaY = event.wheel_delta.y;
@@ -534,10 +526,9 @@ void GetKeyCode(const std::string& char_text,
       }
     }
     if (!vk_code) {
-      WebString web_char_text =
-          WebString::fromUTF8(char_text.data(), char_text.size());
-      DCHECK_EQ(web_char_text.length(), 1U);
-      vk_text = vk_code = web_char_text.at(0);
+      base::string16 char_text16 = base::UTF8ToUTF16(char_text);
+      DCHECK_EQ(char_text16.size(), 1U);
+      vk_text = vk_code = char_text16[0];
       *needs_shift_modifier =
           (vk_code & 0xFF) >= 'A' && (vk_code & 0xFF) <= 'Z';
       if ((vk_code & 0xFF) >= 'a' && (vk_code & 0xFF) <= 'z')
@@ -633,12 +624,12 @@ WebInputEvent* CreateWebInputEvent(const InputEventData& event) {
 
 // Generate a coherent sequence of input events to simulate a user event.
 // From src/components/test_runner/event_sender.cc.
-std::vector<linked_ptr<WebInputEvent> > CreateSimulatedWebInputEvents(
+std::vector<scoped_ptr<WebInputEvent>> CreateSimulatedWebInputEvents(
     const ppapi::InputEventData& event,
     int plugin_x,
     int plugin_y) {
-  std::vector<linked_ptr<WebInputEvent> > events;
-  linked_ptr<WebInputEvent> original_event(CreateWebInputEvent(event));
+  std::vector<scoped_ptr<WebInputEvent>> events;
+  scoped_ptr<WebInputEvent> original_event(CreateWebInputEvent(event));
 
   switch (event.event_type) {
     case PP_INPUTEVENT_TYPE_MOUSEDOWN:
@@ -650,7 +641,7 @@ std::vector<linked_ptr<WebInputEvent> > CreateSimulatedWebInputEvents(
     case PP_INPUTEVENT_TYPE_TOUCHMOVE:
     case PP_INPUTEVENT_TYPE_TOUCHEND:
     case PP_INPUTEVENT_TYPE_TOUCHCANCEL:
-      events.push_back(original_event);
+      events.push_back(std::move(original_event));
       break;
 
     case PP_INPUTEVENT_TYPE_WHEEL: {
@@ -658,7 +649,7 @@ std::vector<linked_ptr<WebInputEvent> > CreateSimulatedWebInputEvents(
           static_cast<WebMouseWheelEvent*>(original_event.get());
       web_mouse_wheel_event->x = plugin_x;
       web_mouse_wheel_event->y = plugin_y;
-      events.push_back(original_event);
+      events.push_back(std::move(original_event));
       break;
     }
 
@@ -672,7 +663,7 @@ std::vector<linked_ptr<WebInputEvent> > CreateSimulatedWebInputEvents(
       if (web_keyboard_event->type == WebInputEvent::KeyDown)
         web_keyboard_event->type = WebInputEvent::RawKeyDown;
 #endif
-      events.push_back(original_event);
+      events.push_back(std::move(original_event));
       break;
     }
 
@@ -708,16 +699,16 @@ std::vector<linked_ptr<WebInputEvent> > CreateSimulatedWebInputEvents(
 
       *key_up_event = *web_char_event = *key_down_event;
 
-      events.push_back(linked_ptr<WebInputEvent>(key_down_event.release()));
+      events.push_back(make_scoped_ptr<WebInputEvent>(key_down_event.release()));
 
       if (generate_char) {
         web_char_event->type = WebInputEvent::Char;
         web_char_event->keyIdentifier[0] = '\0';
-        events.push_back(original_event);
+        events.push_back(std::move(original_event));
       }
 
       key_up_event->type = WebInputEvent::KeyUp;
-      events.push_back(linked_ptr<WebInputEvent>(key_up_event.release()));
+      events.push_back(make_scoped_ptr<WebInputEvent>(key_up_event.release()));
       break;
     }
 
