@@ -5,9 +5,13 @@
 #ifndef CONTENT_RENDERER_PEPPER_PPB_GRAPHICS_3D_IMPL_H_
 #define CONTENT_RENDERER_PEPPER_PPB_GRAPHICS_3D_IMPL_H_
 
+#include <stdint.h>
+
+#include "base/macros.h"
 #include "base/memory/shared_memory.h"
 #include "base/memory/weak_ptr.h"
 #include "gpu/command_buffer/common/mailbox.h"
+#include "gpu/command_buffer/common/sync_token.h"
 #include "ppapi/shared_impl/ppb_graphics_3d_shared.h"
 #include "ppapi/shared_impl/resource.h"
 
@@ -29,12 +33,13 @@ class PPB_Graphics3D_Impl : public ppapi::PPB_Graphics3D_Shared {
       PP_Resource share_context,
       const int32_t* attrib_list,
       gpu::Capabilities* capabilities,
-      base::SharedMemoryHandle* shared_state_handle);
+      base::SharedMemoryHandle* shared_state_handle,
+      uint64_t* command_buffer_id);
 
   // PPB_Graphics3D_API trusted implementation.
   PP_Bool SetGetBuffer(int32_t transfer_buffer_id) override;
   scoped_refptr<gpu::Buffer> CreateTransferBuffer(uint32_t size,
-                                                  int32* id) override;
+                                                  int32_t* id) override;
   PP_Bool DestroyTransferBuffer(int32_t id) override;
   PP_Bool Flush(int32_t put_offset) override;
   gpu::CommandBuffer::State WaitForTokenInRange(int32_t start,
@@ -44,6 +49,7 @@ class PPB_Graphics3D_Impl : public ppapi::PPB_Graphics3D_Shared {
   uint32_t InsertSyncPoint() override;
   uint32_t InsertFutureSyncPoint() override;
   void RetireSyncPoint(uint32_t) override;
+  void EnsureWorkVisible() override;
 
   // Binds/unbinds the graphics of this context with the associated instance.
   // Returns true if binding/unbinding is successful.
@@ -56,12 +62,12 @@ class PPB_Graphics3D_Impl : public ppapi::PPB_Graphics3D_Shared {
   // These messages are used to send Flush callbacks to the plugin.
   void ViewInitiatedPaint();
 
-  void GetBackingMailbox(gpu::Mailbox* mailbox, uint32* sync_point) {
+  void GetBackingMailbox(gpu::Mailbox* mailbox, gpu::SyncToken* sync_token) {
     *mailbox = mailbox_;
-    *sync_point = sync_point_;
+    *sync_token = sync_token_;
   }
 
-  int GetCommandBufferRouteId();
+  CommandBufferProxyImpl* GetCommandBufferProxy();
 
   GpuChannelHost* channel() { return channel_.get(); }
 
@@ -70,7 +76,7 @@ class PPB_Graphics3D_Impl : public ppapi::PPB_Graphics3D_Shared {
   // ppapi::PPB_Graphics3D_Shared overrides.
   gpu::CommandBuffer* GetCommandBuffer() override;
   gpu::GpuControl* GetGpuControl() override;
-  int32 DoSwapBuffers() override;
+  int32_t DoSwapBuffers(const gpu::SyncToken& sync_token) override;
 
  private:
   explicit PPB_Graphics3D_Impl(PP_Instance instance);
@@ -79,7 +85,8 @@ class PPB_Graphics3D_Impl : public ppapi::PPB_Graphics3D_Shared {
   bool InitRaw(PPB_Graphics3D_API* share_context,
                const int32_t* attrib_list,
                gpu::Capabilities* capabilities,
-               base::SharedMemoryHandle* shared_state_handle);
+               base::SharedMemoryHandle* shared_state_handle,
+               uint64_t* command_buffer_id);
 
   // Notifications received from the GPU process.
   void OnSwapBuffers();
@@ -94,10 +101,10 @@ class PPB_Graphics3D_Impl : public ppapi::PPB_Graphics3D_Shared {
   bool commit_pending_;
 
   gpu::Mailbox mailbox_;
-  uint32 sync_point_;
+  gpu::SyncToken sync_token_;
   bool has_alpha_;
   scoped_refptr<GpuChannelHost> channel_;
-  CommandBufferProxyImpl* command_buffer_;
+  scoped_ptr<CommandBufferProxyImpl> command_buffer_;
 
   base::WeakPtrFactory<PPB_Graphics3D_Impl> weak_ptr_factory_;
 
