@@ -342,9 +342,6 @@ class CONTENT_EXPORT RenderWidget
 
   bool host_closing() const { return host_closing_; }
 
-  uint32_t GetContentSourceId();
-  void IncrementContentSourceId();
-
  protected:
   // Friend RefCounted so that the dtor can be non-public. Using this class
   // without ref-counting is an error.
@@ -758,19 +755,6 @@ class CONTENT_EXPORT RenderWidget
   scoped_ptr<scheduler::RenderWidgetSchedulingState>
       render_widget_scheduling_state_;
 
-  // This is initialized to zero and is incremented on each non-same-page
-  // navigation commit by RenderFrameImpl. At that time it is sent to the
-  // compositor so that it can tag compositor frames, and RenderFrameImpl is
-  // responsible for sending it to the browser process to be used to match
-  // each compositor frame to the most recent page navigation before it was
-  // generated.
-  // This only applies to main frames, and is not touched for subframe
-  // RenderWidgets, where there is no concern around displaying unloaded
-  // content.
-  // TODO(kenrb, fsamuel): This should be removed when SurfaceIDs can be used
-  // to replace it. See https://crbug.com/695579.
-  uint32_t current_content_source_id_;
-
   DISALLOW_COPY_AND_ASSIGN(RenderWidget);
 };
 
