@@ -4,6 +4,8 @@
 
 #include "content/renderer/renderer_blink_platform_impl.h"
 
+#include <utility>
+
 #include "base/command_line.h"
 #include "base/files/file_path.h"
 #include "base/lazy_instance.h"
@@ -19,8 +21,10 @@
 #include "build/build_config.h"
 #include "cc/blink/context_provider_web_context.h"
 #include "components/scheduler/child/web_scheduler_impl.h"
+#include "components/scheduler/child/web_task_runner_impl.h"
 #include "components/scheduler/renderer/renderer_scheduler.h"
 #include "components/scheduler/renderer/webthread_impl_for_renderer_scheduler.h"
+#include "components/url_formatter/url_formatter.h"
 #include "content/child/database_util.h"
 #include "content/child/file_info_util.h"
 #include "content/child/fileapi/webfilesystem_impl.h"
@@ -31,16 +35,18 @@
 #include "content/child/simple_webmimeregistry_impl.h"
 #include "content/child/thread_safe_sender.h"
 #include "content/child/web_database_observer_impl.h"
+#include "content/child/web_url_loader_impl.h"
 #include "content/child/webblobregistry_impl.h"
 #include "content/child/webfileutilities_impl.h"
 #include "content/child/webmessageportchannel_impl.h"
 #include "content/common/file_utilities_messages.h"
+#include "content/common/frame_messages.h"
 #include "content/common/gpu/client/context_provider_command_buffer.h"
 #include "content/common/gpu/client/gpu_channel_host.h"
 #include "content/common/gpu/client/webgraphicscontext3d_command_buffer_impl.h"
 #include "content/common/gpu/gpu_process_launch_causes.h"
 #include "content/common/mime_registry_messages.h"
-#include "content/common/view_messages.h"
+#include "content/common/render_process_messages.h"
 #include "content/public/common/content_switches.h"
 #include "content/public/common/service_registry.h"
 #include "content/public/common/webplugininfo.h"
@@ -49,12 +55,16 @@
 #include "content/renderer/cache_storage/webserviceworkercachestorage_impl.h"
 #include "content/renderer/device_sensors/device_light_event_pump.h"
 #include "content/renderer/device_sensors/device_motion_event_pump.h"
+#include "content/renderer/device_sensors/device_orientation_absolute_event_pump.h"
 #include "content/renderer/device_sensors/device_orientation_event_pump.h"
 #include "content/renderer/dom_storage/webstoragenamespace_impl.h"
 #include "content/renderer/gamepad_shared_memory_reader.h"
 #include "content/renderer/media/audio_decoder.h"
+#include "content/renderer/media/canvas_capture_handler.h"
+#include "content/renderer/media/media_recorder_handler.h"
 #include "content/renderer/media/renderer_webaudiodevice_impl.h"
 #include "content/renderer/media/renderer_webmidiaccessor_impl.h"
+#include "content/renderer/media/rtc_certificate_generator.h"
 #include "content/renderer/render_thread_impl.h"
 #include "content/renderer/renderer_clipboard_delegate.h"
 #include "content/renderer/screen_orientation/screen_orientation_observer.h"
@@ -69,7 +79,6 @@
 #include "media/base/mime_util.h"
 #include "media/blink/webcontentdecryptionmodule_impl.h"
 #include "media/filters/stream_parser_factory.h"
-#include "net/base/net_util.h"
 #include "storage/common/database/database_identifier.h"
 #include "storage/common/quota/quota_types.h"
 #include "third_party/WebKit/public/platform/WebBatteryStatusListener.h"
@@ -89,7 +98,6 @@
 
 #if defined(OS_ANDROID)
 #include "content/renderer/android/synchronous_compositor_factory.h"
-#include "content/renderer/media/android/audio_decoder_android.h"
 #include "gpu/blink/webgraphicscontext3d_in_process_command_buffer_impl.h"
 #endif
 
@@ -132,6 +140,7 @@
 using blink::Platform;
 using blink::WebAudioDevice;
 using blink::WebBlobRegistry;
+using blink::WebCanvasCaptureHandler;
 using blink::WebDatabaseObserver;
 using blink::WebFileInfo;
 using blink::WebFileSystem;
@@ -139,12 +148,15 @@ using blink::WebGamepad;
 using blink::WebGamepads;
 using blink::WebIDBFactory;
 using blink::WebMIDIAccessor;
+using blink::WebMediaRecorderHandler;
 using blink::WebMediaStreamCenter;
 using blink::WebMediaStreamCenterClient;
+using blink::WebMediaStreamTrack;
 using blink::WebMimeRegistry;
 using blink::WebRTCPeerConnectionHandler;
 using blink::WebRTCPeerConnectionHandlerClient;
 using blink::WebStorageNamespace;
+using blink::WebSize;
 using blink::WebString;
 using blink::WebURL;
 using blink::WebVector;
@@ -163,30 +175,29 @@ base::LazyInstance<blink::WebDeviceOrientationData>::Leaky
 // not owned by us.
 blink::WebBatteryStatusListener* g_test_battery_status_listener = nullptr;
 
-} // namespace
+}  // namespace
 
 //------------------------------------------------------------------------------
 
 class RendererBlinkPlatformImpl::MimeRegistry
     : public SimpleWebMimeRegistryImpl {
  public:
-  virtual blink::WebMimeRegistry::SupportsType supportsMediaMIMEType(
+  blink::WebMimeRegistry::SupportsType supportsMediaMIMEType(
       const blink::WebString& mime_type,
       const blink::WebString& codecs,
-      const blink::WebString& key_system);
-  virtual bool supportsMediaSourceMIMEType(const blink::WebString& mime_type,
-                                           const blink::WebString& codecs);
-  virtual blink::WebString mimeTypeForExtension(
-      const blink::WebString& file_extension);
-  virtual blink::WebString mimeTypeFromFile(
-      const blink::WebString& file_path);
+      const blink::WebString& key_system) override;
+  bool supportsMediaSourceMIMEType(const blink::WebString& mime_type,
+                                   const blink::WebString& codecs) override;
+  blink::WebString mimeTypeForExtension(
+      const blink::WebString& file_extension) override;
 };
 
 class RendererBlinkPlatformImpl::FileUtilities : public WebFileUtilitiesImpl {
  public:
   explicit FileUtilities(ThreadSafeSender* sender)
       : thread_safe_sender_(sender) {}
-  virtual bool getFileInfo(const WebString& path, WebFileInfo& result);
+  bool getFileInfo(const WebString& path, WebFileInfo& result) override;
+
  private:
   bool SendSyncMessageFromAnyThread(IPC::SyncMessage* msg) const;
   scoped_refptr<ThreadSafeSender> thread_safe_sender_;
@@ -199,17 +210,17 @@ class RendererBlinkPlatformImpl::SandboxSupport
   virtual ~SandboxSupport() {}
 
 #if defined(OS_MACOSX)
-  virtual bool loadFont(
-      NSFont* src_font,
-      CGFontRef* container,
-      uint32* font_id);
+  bool loadFont(NSFont* src_font,
+                CGFontRef* container,
+                uint32_t* font_id) override;
 #elif defined(OS_POSIX)
-  virtual void getFallbackFontForCharacter(
+  void getFallbackFontForCharacter(
       blink::WebUChar32 character,
       const char* preferred_locale,
-      blink::WebFallbackFont* fallbackFont);
-  virtual void getRenderStyleForStrike(
-      const char* family, int sizeAndStyle, blink::WebFontRenderStyle* out);
+      blink::WebFallbackFont* fallbackFont) override;
+  void getRenderStyleForStrike(const char* family,
+                               int sizeAndStyle,
+                               blink::WebFontRenderStyle* out) override;
 
  private:
   // WebKit likes to ask us for the correct font family to use for a set of
@@ -226,15 +237,16 @@ class RendererBlinkPlatformImpl::SandboxSupport
 RendererBlinkPlatformImpl::RendererBlinkPlatformImpl(
     scheduler::RendererScheduler* renderer_scheduler)
     : BlinkPlatformImpl(renderer_scheduler->DefaultTaskRunner()),
-      main_thread_(
-          new scheduler::WebThreadImplForRendererScheduler(renderer_scheduler)),
+      main_thread_(renderer_scheduler->CreateMainThread()),
       clipboard_delegate_(new RendererClipboardDelegate),
       clipboard_(new WebClipboardImpl(clipboard_delegate_.get())),
       mime_registry_(new RendererBlinkPlatformImpl::MimeRegistry),
       sudden_termination_disables_(0),
       plugin_refresh_allowed_(true),
       default_task_runner_(renderer_scheduler->DefaultTaskRunner()),
-      web_scrollbar_behavior_(new WebScrollbarBehaviorImpl) {
+      loading_task_runner_(renderer_scheduler->LoadingTaskRunner()),
+      web_scrollbar_behavior_(new WebScrollbarBehaviorImpl),
+      renderer_scheduler_(renderer_scheduler) {
 #if !defined(OS_ANDROID) && !defined(OS_WIN)
   if (g_sandbox_enabled && sandboxEnabled()) {
     sandbox_support_.reset(new RendererBlinkPlatformImpl::SandboxSupport);
@@ -259,8 +271,40 @@ RendererBlinkPlatformImpl::~RendererBlinkPlatformImpl() {
   WebFileSystemImpl::DeleteThreadSpecificInstance();
 }
 
+void RendererBlinkPlatformImpl::Shutdown() {
+#if !defined(OS_ANDROID) && !defined(OS_WIN)
+  // SandboxSupport contains a map of WebFontFamily objects, which hold
+  // WebCStrings, which become invalidated when blink is shut down. Hence, we
+  // need to clear that map now, just before blink::shutdown() is called.
+  sandbox_support_.reset();
+#endif
+}
+
 //------------------------------------------------------------------------------
 
+double RendererBlinkPlatformImpl::currentTimeSeconds() {
+  return renderer_scheduler_->CurrentTimeSeconds();
+}
+
+double RendererBlinkPlatformImpl::monotonicallyIncreasingTimeSeconds() {
+  return renderer_scheduler_->MonotonicallyIncreasingTimeSeconds();
+}
+
+//------------------------------------------------------------------------------
+
+blink::WebURLLoader* RendererBlinkPlatformImpl::createURLLoader() {
+  ChildThreadImpl* child_thread = ChildThreadImpl::current();
+  // There may be no child thread in RenderViewTests.  These tests can still use
+  // data URLs to bypass the ResourceDispatcher.
+  scoped_ptr<scheduler::WebTaskRunnerImpl> task_runner(
+      new scheduler::WebTaskRunnerImpl(
+        loading_task_runner_->BelongsToCurrentThread()
+            ? loading_task_runner_ : base::ThreadTaskRunnerHandle::Get()));
+  return new content::WebURLLoaderImpl(
+      child_thread ? child_thread->resource_dispatcher() : NULL,
+      std::move(task_runner));
+}
+
 blink::WebThread* RendererBlinkPlatformImpl::currentThread() {
   if (main_thread_->isCurrentThread())
     return main_thread_.get();
@@ -344,15 +388,16 @@ RendererBlinkPlatformImpl::prescientNetworking() {
 }
 
 void RendererBlinkPlatformImpl::cacheMetadata(const blink::WebURL& url,
-                                              int64 response_time,
+                                              int64_t response_time,
                                               const char* data,
                                               size_t size) {
   // Let the browser know we generated cacheable metadata for this resource. The
   // browser may cache it and return it on subsequent responses to speed
   // the processing of this resource.
   std::vector<char> copy(data, data + size);
-  RenderThread::Get()->Send(new ViewHostMsg_DidGenerateCacheableMetadata(
-      url, base::Time::FromInternalValue(response_time), copy));
+  RenderThread::Get()->Send(
+      new RenderProcessHostMsg_DidGenerateCacheableMetadata(
+          url, base::Time::FromInternalValue(response_time), copy));
 }
 
 WebString RendererBlinkPlatformImpl::defaultLocale() {
@@ -376,7 +421,7 @@ void RendererBlinkPlatformImpl::suddenTerminationChanged(bool enabled) {
 
   RenderThread* thread = RenderThread::Get();
   if (thread)  // NULL in unittests.
-    thread->Send(new ViewHostMsg_SuddenTerminationChanged(enabled));
+    thread->Send(new RenderProcessHostMsg_SuddenTerminationChanged(enabled));
 }
 
 WebStorageNamespace* RendererBlinkPlatformImpl::createLocalStorageNamespace() {
@@ -414,9 +459,6 @@ RendererBlinkPlatformImpl::MimeRegistry::supportsMediaMIMEType(
     const WebString& codecs,
     const WebString& key_system) {
   const std::string mime_type_ascii = ToASCIIOrEmpty(mime_type);
-  // Not supporting the container is a flat-out no.
-  if (!media::IsSupportedMediaMimeType(mime_type_ascii))
-    return IsNotSupported;
 
   if (!key_system.isEmpty()) {
     // Check whether the key system is supported with the mime_type and codecs.
@@ -426,35 +468,23 @@ RendererBlinkPlatformImpl::MimeRegistry::supportsMediaMIMEType(
       return IsNotSupported;
 
     std::string key_system_ascii =
-        media::GetUnprefixedKeySystemName(base::UTF16ToASCII(key_system));
-    std::vector<std::string> strict_codecs;
-    media::ParseCodecString(ToASCIIOrEmpty(codecs), &strict_codecs, true);
+        media::GetUnprefixedKeySystemName(base::UTF16ToASCII(
+            base::StringPiece16(key_system)));
+    std::vector<std::string> codec_vector;
+    media::ParseCodecString(ToASCIIOrEmpty(codecs), &codec_vector, true);
 
     if (!media::PrefixedIsSupportedKeySystemWithMediaMimeType(
-            mime_type_ascii, strict_codecs, key_system_ascii)) {
+            mime_type_ascii, codec_vector, key_system_ascii)) {
       return IsNotSupported;
     }
 
     // Continue processing the mime_type and codecs.
   }
 
-  // Check list of strict codecs to see if it is supported.
-  if (media::IsStrictMediaMimeType(mime_type_ascii)) {
-    // Check if the codecs are a perfect match.
-    std::vector<std::string> strict_codecs;
-    media::ParseCodecString(ToASCIIOrEmpty(codecs), &strict_codecs, false);
-    return static_cast<WebMimeRegistry::SupportsType> (
-        media::IsSupportedStrictMediaMimeType(mime_type_ascii, strict_codecs));
-  }
-
-  // If we don't recognize the codec, it's possible we support it.
-  std::vector<std::string> parsed_codecs;
-  media::ParseCodecString(ToASCIIOrEmpty(codecs), &parsed_codecs, true);
-  if (!media::AreSupportedMediaCodecs(parsed_codecs))
-    return MayBeSupported;
-
-  // Otherwise we have a perfect match.
-  return IsSupported;
+  std::vector<std::string> codec_vector;
+  media::ParseCodecString(ToASCIIOrEmpty(codecs), &codec_vector, false);
+  return static_cast<WebMimeRegistry::SupportsType>(
+      media::IsSupportedMediaFormat(mime_type_ascii, codec_vector));
 }
 
 bool RendererBlinkPlatformImpl::MimeRegistry::supportsMediaSourceMIMEType(
@@ -483,20 +513,6 @@ WebString RendererBlinkPlatformImpl::MimeRegistry::mimeTypeForExtension(
   return base::ASCIIToUTF16(mime_type);
 }
 
-WebString RendererBlinkPlatformImpl::MimeRegistry::mimeTypeFromFile(
-    const WebString& file_path) {
-  if (IsPluginProcess())
-    return SimpleWebMimeRegistryImpl::mimeTypeFromFile(file_path);
-
-  // The sandbox restricts our access to the registry, so we need to proxy
-  // these calls over to the browser process.
-  std::string mime_type;
-  RenderThread::Get()->Send(new MimeRegistryMsg_GetMimeTypeFromFile(
-      base::FilePath::FromUTF16Unsafe(file_path),
-      &mime_type));
-  return base::ASCIIToUTF16(mime_type);
-}
-
 //------------------------------------------------------------------------------
 
 bool RendererBlinkPlatformImpl::FileUtilities::getFileInfo(
@@ -529,11 +545,11 @@ bool RendererBlinkPlatformImpl::FileUtilities::SendSyncMessageFromAnyThread(
 
 bool RendererBlinkPlatformImpl::SandboxSupport::loadFont(NSFont* src_font,
                                                          CGFontRef* out,
-                                                         uint32* font_id) {
-  uint32 font_data_size;
+                                                         uint32_t* font_id) {
+  uint32_t font_data_size;
   FontDescriptor src_font_descriptor(src_font);
   base::SharedMemoryHandle font_data;
-  if (!RenderThread::Get()->Send(new ViewHostMsg_LoadFont(
+  if (!RenderThread::Get()->Send(new RenderProcessHostMsg_LoadFont(
         src_font_descriptor, &font_data_size, &font_data, font_id))) {
     *out = NULL;
     *font_id = 0;
@@ -542,7 +558,7 @@ bool RendererBlinkPlatformImpl::SandboxSupport::loadFont(NSFont* src_font,
 
   if (font_data_size == 0 || font_data == base::SharedMemory::NULLHandle() ||
       *font_id == 0) {
-    LOG(ERROR) << "Bad response from ViewHostMsg_LoadFont() for " <<
+    LOG(ERROR) << "Bad response from RenderProcessHostMsg_LoadFont() for " <<
         src_font_descriptor.font_name;
     *out = NULL;
     *font_id = 0;
@@ -629,13 +645,6 @@ bool RendererBlinkPlatformImpl::databaseSetFileSize(
 }
 
 bool RendererBlinkPlatformImpl::canAccelerate2dCanvas() {
-#if defined(OS_ANDROID)
-  if (SynchronousCompositorFactory* factory =
-          SynchronousCompositorFactory::GetInstance()) {
-    return factory->GetGPUInfo().SupportsAccelerated2dCanvas();
-  }
-#endif
-
   RenderThreadImpl* thread = RenderThreadImpl::current();
   GpuChannelHost* host = thread->EstablishGpuChannelSync(
       CAUSE_FOR_GPU_LAUNCH_CANVAS_2D);
@@ -651,6 +660,11 @@ bool RendererBlinkPlatformImpl::isThreadedCompositingEnabled() {
   return thread && thread->compositor_task_runner().get();
 }
 
+bool RendererBlinkPlatformImpl::isThreadedAnimationEnabled() {
+  RenderThreadImpl* thread = RenderThreadImpl::current();
+  return thread ? thread->IsThreadedAnimationEnabled() : true;
+}
+
 double RendererBlinkPlatformImpl::audioHardwareSampleRate() {
   RenderThreadImpl* thread = RenderThreadImpl::current();
   return thread->GetAudioHardwareConfig()->GetOutputSampleRate();
@@ -686,8 +700,6 @@ WebAudioDevice* RendererBlinkPlatformImpl::createAudioDevice(
   // The |channels| does not exactly identify the channel layout of the
   // device. The switch statement below assigns a best guess to the channel
   // layout based on number of channels.
-  // TODO(crogers): WebKit should give the channel layout instead of the hard
-  // channel count.
   media::ChannelLayout layout = media::CHANNEL_LAYOUT_UNSUPPORTED;
   switch (channels) {
     case 1:
@@ -715,37 +727,31 @@ WebAudioDevice* RendererBlinkPlatformImpl::createAudioDevice(
       layout = media::CHANNEL_LAYOUT_7_1;
       break;
     default:
-      layout = media::CHANNEL_LAYOUT_STEREO;
+      // If the layout is not supported (more than 9 channels), falls back to
+      // discrete mode.
+      layout = media::CHANNEL_LAYOUT_DISCRETE;
   }
 
   int session_id = 0;
   if (input_device_id.isNull() ||
-      !base::StringToInt(base::UTF16ToUTF8(input_device_id), &session_id)) {
+      !base::StringToInt(base::UTF16ToUTF8(
+          base::StringPiece16(input_device_id)), &session_id)) {
     if (input_channels > 0)
       DLOG(WARNING) << "createAudioDevice(): request for audio input ignored";
 
     input_channels = 0;
   }
 
-  media::AudioParameters params(
-      media::AudioParameters::AUDIO_PCM_LOW_LATENCY,
-      layout, static_cast<int>(sample_rate), 16, buffer_size,
-      media::AudioParameters::NO_EFFECTS);
+  // For CHANNEL_LAYOUT_DISCRETE, pass the explicit channel count along with
+  // the channel layout when creating an |AudioParameters| object.
+  media::AudioParameters params(media::AudioParameters::AUDIO_PCM_LOW_LATENCY,
+                                layout, static_cast<int>(sample_rate), 16,
+                                buffer_size);
+  params.set_channels_for_discrete(channels);
 
   return new RendererWebAudioDeviceImpl(params, callback, session_id);
 }
 
-#if defined(OS_ANDROID)
-bool RendererBlinkPlatformImpl::loadAudioResource(
-    blink::WebAudioBus* destination_bus,
-    const char* audio_file_data,
-    size_t data_size) {
-  return DecodeAudioFileData(destination_bus,
-                             audio_file_data,
-                             data_size,
-                             thread_safe_sender_);
-}
-#else
 bool RendererBlinkPlatformImpl::loadAudioResource(
     blink::WebAudioBus* destination_bus,
     const char* audio_file_data,
@@ -753,7 +759,6 @@ bool RendererBlinkPlatformImpl::loadAudioResource(
   return DecodeAudioFileData(
       destination_bus, audio_file_data, data_size);
 }
-#endif  // defined(OS_ANDROID)
 
 //------------------------------------------------------------------------------
 
@@ -774,24 +779,19 @@ void RendererBlinkPlatformImpl::getPluginList(
   std::vector<WebPluginInfo> plugins;
   if (!plugin_refresh_allowed_)
     refresh = false;
-  RenderThread::Get()->Send(
-      new ViewHostMsg_GetPlugins(refresh, &plugins));
-  for (size_t i = 0; i < plugins.size(); ++i) {
-    const WebPluginInfo& plugin = plugins[i];
-
+  RenderThread::Get()->Send(new FrameHostMsg_GetPlugins(refresh, &plugins));
+  for (const WebPluginInfo& plugin : plugins) {
     builder->addPlugin(
         plugin.name, plugin.desc,
         plugin.path.BaseName().AsUTF16Unsafe());
 
-    for (size_t j = 0; j < plugin.mime_types.size(); ++j) {
-      const WebPluginMimeType& mime_type = plugin.mime_types[j];
-
+    for (const WebPluginMimeType& mime_type : plugin.mime_types) {
       builder->addMediaTypeToLastPlugin(
           WebString::fromUTF8(mime_type.mime_type), mime_type.description);
 
-      for (size_t k = 0; k < mime_type.file_extensions.size(); ++k) {
+      for (const auto& extension : mime_type.file_extensions) {
         builder->addFileExtensionToLastMediaType(
-            WebString::fromUTF8(mime_type.file_extensions[k]));
+            WebString::fromUTF8(extension));
       }
     }
   }
@@ -809,13 +809,12 @@ blink::WebPublicSuffixList* RendererBlinkPlatformImpl::publicSuffixList() {
 blink::WebString RendererBlinkPlatformImpl::signedPublicKeyAndChallengeString(
     unsigned key_size_index,
     const blink::WebString& challenge,
-    const blink::WebURL& url) {
+    const blink::WebURL& url,
+    const blink::WebURL& top_origin) {
   std::string signed_public_key;
-  RenderThread::Get()->Send(new ViewHostMsg_Keygen(
-      static_cast<uint32>(key_size_index),
-      challenge.utf8(),
-      GURL(url),
-      &signed_public_key));
+  RenderThread::Get()->Send(new RenderProcessHostMsg_Keygen(
+      static_cast<uint32_t>(key_size_index), challenge.utf8(), GURL(url),
+      GURL(top_origin), &signed_public_key));
   return WebString::fromUTF8(signed_public_key);
 }
 
@@ -830,7 +829,7 @@ void RendererBlinkPlatformImpl::screenColorProfile(
   // safe send to avoid crashing trying to access RenderThread::Get(),
   // which is not accessible from arbitrary threads.
   thread_safe_sender_->Send(
-      new ViewHostMsg_GetMonitorColorProfile(&profile));
+      new RenderProcessHostMsg_GetMonitorColorProfile(&profile));
   *to_profile = profile;
 #else
   // On other platforms, the primary monitor color profile can be read
@@ -857,7 +856,7 @@ WebBlobRegistry* RendererBlinkPlatformImpl::blobRegistry() {
 
 void RendererBlinkPlatformImpl::sampleGamepads(WebGamepads& gamepads) {
   PlatformEventObserverBase* observer =
-      platform_event_observers_.Lookup(blink::WebPlatformEventGamepad);
+      platform_event_observers_.Lookup(blink::WebPlatformEventTypeGamepad);
   if (!observer)
     return;
   static_cast<RendererGamepadProvider*>(observer)->SampleGamepads(gamepads);
@@ -865,6 +864,17 @@ void RendererBlinkPlatformImpl::sampleGamepads(WebGamepads& gamepads) {
 
 //------------------------------------------------------------------------------
 
+WebMediaRecorderHandler*
+RendererBlinkPlatformImpl::createMediaRecorderHandler() {
+#if defined(ENABLE_WEBRTC)
+  return new content::MediaRecorderHandler();
+#else
+  return nullptr;
+#endif
+}
+
+//------------------------------------------------------------------------------
+
 WebRTCPeerConnectionHandler*
 RendererBlinkPlatformImpl::createRTCPeerConnectionHandler(
     WebRTCPeerConnectionHandlerClient* client) {
@@ -890,6 +900,17 @@ RendererBlinkPlatformImpl::createRTCPeerConnectionHandler(
 
 //------------------------------------------------------------------------------
 
+blink::WebRTCCertificateGenerator*
+RendererBlinkPlatformImpl::createRTCCertificateGenerator() {
+#if defined(ENABLE_WEBRTC)
+  return new RTCCertificateGenerator();
+#else
+  return nullptr;
+#endif  // defined(ENABLE_WEBRTC)
+}
+
+//------------------------------------------------------------------------------
+
 WebMediaStreamCenter* RendererBlinkPlatformImpl::createMediaStreamCenter(
     WebMediaStreamCenterClient* client) {
   RenderThreadImpl* render_thread = RenderThreadImpl::current();
@@ -908,19 +929,23 @@ bool RendererBlinkPlatformImpl::SetSandboxEnabledForTesting(bool enable) {
 
 //------------------------------------------------------------------------------
 
-blink::WebSpeechSynthesizer* RendererBlinkPlatformImpl::createSpeechSynthesizer(
-    blink::WebSpeechSynthesizerClient* client) {
-  return GetContentClient()->renderer()->OverrideSpeechSynthesizer(client);
+WebCanvasCaptureHandler* RendererBlinkPlatformImpl::createCanvasCaptureHandler(
+    const WebSize& size,
+    double frame_rate,
+    WebMediaStreamTrack* track) {
+#if defined(ENABLE_WEBRTC)
+  return CanvasCaptureHandler::CreateCanvasCaptureHandler(
+      size, frame_rate, RenderThread::Get()->GetIOMessageLoopProxy(), track);
+#else
+  return nullptr;
+#endif  // defined(ENABLE_WEBRTC)
 }
 
 //------------------------------------------------------------------------------
 
-bool RendererBlinkPlatformImpl::processMemorySizesInBytes(
-    size_t* private_bytes,
-    size_t* shared_bytes) {
-  content::RenderThread::Get()->Send(
-      new ViewHostMsg_GetProcessMemorySizes(private_bytes, shared_bytes));
-  return true;
+blink::WebSpeechSynthesizer* RendererBlinkPlatformImpl::createSpeechSynthesizer(
+    blink::WebSpeechSynthesizerClient* client) {
+  return GetContentClient()->renderer()->OverrideSpeechSynthesizer(client);
 }
 
 //------------------------------------------------------------------------------
@@ -935,59 +960,65 @@ blink::WebGraphicsContext3D*
 RendererBlinkPlatformImpl::createOffscreenGraphicsContext3D(
     const blink::WebGraphicsContext3D::Attributes& attributes,
     blink::WebGraphicsContext3D* share_context) {
-  return createOffscreenGraphicsContext3D(attributes, share_context, NULL);
+  blink::WebGraphicsContext3D::WebGraphicsInfo gl_info;
+  return createOffscreenGraphicsContext3D(attributes, share_context, &gl_info);
+}
+
+static void Collect3DContextInformationOnFailure(
+    blink::WebGraphicsContext3D* share_context,
+    blink::WebGraphicsContext3D::WebGraphicsInfo* gl_info,
+    GpuChannelHost* host) {
+  DCHECK(gl_info);
+  std::string error_message("OffscreenContext Creation failed, ");
+  if (host) {
+    const gpu::GPUInfo& gpu_info = host->gpu_info();
+    gl_info->vendorId = gpu_info.gpu.vendor_id;
+    gl_info->deviceId = gpu_info.gpu.device_id;
+    switch (gpu_info.context_info_state) {
+      case gpu::kCollectInfoSuccess:
+      case gpu::kCollectInfoNonFatalFailure:
+        gl_info->rendererInfo = WebString::fromUTF8(gpu_info.gl_renderer);
+        gl_info->vendorInfo = WebString::fromUTF8(gpu_info.gl_vendor);
+        gl_info->driverVersion = WebString::fromUTF8(gpu_info.driver_version);
+        gl_info->resetNotificationStrategy =
+            gpu_info.gl_reset_notification_strategy;
+        gl_info->sandboxed = gpu_info.sandboxed;
+        gl_info->processCrashCount = gpu_info.process_crash_count;
+        gl_info->amdSwitchable = gpu_info.amd_switchable;
+        gl_info->optimus = gpu_info.optimus;
+        break;
+      case gpu::kCollectInfoFatalFailure:
+      case gpu::kCollectInfoNone:
+        error_message.append(
+            "Failed to collect gpu information, GLSurface or GLContext "
+            "creation failed");
+        gl_info->errorMessage = WebString::fromUTF8(error_message);
+        break;
+      default:
+        NOTREACHED();
+    }
+  } else {
+    error_message.append("GpuChannelHost creation failed");
+    gl_info->errorMessage = WebString::fromUTF8(error_message);
+  }
 }
 
 blink::WebGraphicsContext3D*
 RendererBlinkPlatformImpl::createOffscreenGraphicsContext3D(
     const blink::WebGraphicsContext3D::Attributes& attributes,
     blink::WebGraphicsContext3D* share_context,
-    blink::WebGLInfo* gl_info) {
-  if (!RenderThreadImpl::current())
+    blink::WebGraphicsContext3D::WebGraphicsInfo* gl_info) {
+  DCHECK(gl_info);
+  if (!RenderThreadImpl::current()) {
+    std::string error_message("Failed to run in Current RenderThreadImpl");
+    gl_info->errorMessage = WebString::fromUTF8(error_message);
     return NULL;
-
-#if defined(OS_ANDROID)
-  if (SynchronousCompositorFactory* factory =
-      SynchronousCompositorFactory::GetInstance()) {
-    scoped_ptr<gpu_blink::WebGraphicsContext3DInProcessCommandBufferImpl>
-        in_process_context(
-            factory->CreateOffscreenGraphicsContext3D(attributes));
-    if (!in_process_context ||
-        !in_process_context->InitializeOnCurrentThread())
-      return NULL;
-    return in_process_context.release();
   }
-#endif
 
   scoped_refptr<GpuChannelHost> gpu_channel_host(
       RenderThreadImpl::current()->EstablishGpuChannelSync(
           CAUSE_FOR_GPU_LAUNCH_WEBGRAPHICSCONTEXT3DCOMMANDBUFFERIMPL_INITIALIZE));
 
-  if (gpu_channel_host.get() && gl_info) {
-    const gpu::GPUInfo& gpu_info = gpu_channel_host->gpu_info();
-    switch (gpu_info.context_info_state) {
-      case gpu::kCollectInfoSuccess:
-      case gpu::kCollectInfoNonFatalFailure:
-        gl_info->vendorInfo.assign(
-            blink::WebString::fromUTF8(gpu_info.gl_vendor));
-        gl_info->rendererInfo.assign(
-            blink::WebString::fromUTF8(gpu_info.gl_renderer));
-        gl_info->driverVersion.assign(
-            blink::WebString::fromUTF8(gpu_info.driver_version));
-        gl_info->vendorId = gpu_info.gpu.vendor_id;
-        gl_info->deviceId = gpu_info.gpu.device_id;
-        break;
-      case gpu::kCollectInfoFatalFailure:
-      case gpu::kCollectInfoNone:
-        gl_info->contextInfoCollectionFailure.assign(blink::WebString::fromUTF8(
-            "GPUInfoCollectionFailure: GPU initialization Failed. GPU "
-            "Info not Collected."));
-        break;
-      default:
-        NOTREACHED();
-    };
-  }
-
   WebGraphicsContext3DCommandBufferImpl::SharedMemoryLimits limits;
   bool lose_context_when_out_of_memory = false;
   scoped_ptr<WebGraphicsContext3DCommandBufferImpl> context(
@@ -1001,8 +1032,14 @@ RendererBlinkPlatformImpl::createOffscreenGraphicsContext3D(
 
   // Most likely the GPU process exited and the attempt to reconnect to it
   // failed. Need to try to restore the context again later.
-  if (!context || !context->InitializeOnCurrentThread())
+  if (!context || !context->InitializeOnCurrentThread() ||
+      gl_info->testFailContext) {
+    // Collect Graphicsinfo if there is a context failure or it is failed
+    // purposefully in case of layout tests.
+    Collect3DContextInformationOnFailure(share_context, gl_info,
+                                         gpu_channel_host.get());
       return NULL;
+  }
   return context.release();
 }
 
@@ -1028,7 +1065,7 @@ blink::WebCompositorSupport* RendererBlinkPlatformImpl::compositorSupport() {
 blink::WebString RendererBlinkPlatformImpl::convertIDNToUnicode(
     const blink::WebString& host,
     const blink::WebString& languages) {
-  return net::IDNToUnicode(host.utf8(), languages.utf8());
+  return url_formatter::IDNToUnicode(host.utf8(), languages.utf8());
 }
 
 //------------------------------------------------------------------------------
@@ -1070,7 +1107,7 @@ void RendererBlinkPlatformImpl::SetMockDeviceOrientationDataForTesting(
 
 void RendererBlinkPlatformImpl::vibrate(unsigned int milliseconds) {
   GetConnectedVibrationManagerService()->Vibrate(
-      base::checked_cast<int64>(milliseconds));
+      base::checked_cast<int64_t>(milliseconds));
   vibration_manager_.reset();
 }
 
@@ -1103,15 +1140,17 @@ RendererBlinkPlatformImpl::CreatePlatformEventObserverFromType(
     thread = NULL;
 
   switch (type) {
-    case blink::WebPlatformEventDeviceMotion:
+    case blink::WebPlatformEventTypeDeviceMotion:
       return new DeviceMotionEventPump(thread);
-    case blink::WebPlatformEventDeviceOrientation:
+    case blink::WebPlatformEventTypeDeviceOrientation:
       return new DeviceOrientationEventPump(thread);
-    case blink::WebPlatformEventDeviceLight:
+    case blink::WebPlatformEventTypeDeviceOrientationAbsolute:
+      return new DeviceOrientationAbsoluteEventPump(thread);
+    case blink::WebPlatformEventTypeDeviceLight:
       return new DeviceLightEventPump(thread);
-    case blink::WebPlatformEventGamepad:
+    case blink::WebPlatformEventTypeGamepad:
       return new GamepadSharedMemoryReader(thread);
-    case blink::WebPlatformEventScreenOrientation:
+    case blink::WebPlatformEventTypeScreenOrientation:
       return new ScreenOrientationObserver();
     default:
       // A default statement is required to prevent compilation errors when
@@ -1126,7 +1165,7 @@ RendererBlinkPlatformImpl::CreatePlatformEventObserverFromType(
 void RendererBlinkPlatformImpl::SetPlatformEventObserverForTesting(
     blink::WebPlatformEventType type,
     scoped_ptr<PlatformEventObserverBase> observer) {
-  DCHECK(type != blink::WebPlatformEventBattery);
+  DCHECK(type != blink::WebPlatformEventTypeBattery);
 
   if (platform_event_observers_.Lookup(type))
     platform_event_observers_.Remove(type);
@@ -1136,7 +1175,7 @@ void RendererBlinkPlatformImpl::SetPlatformEventObserverForTesting(
 void RendererBlinkPlatformImpl::startListening(
     blink::WebPlatformEventType type,
     blink::WebPlatformEventListener* listener) {
-  if (type == blink::WebPlatformEventBattery) {
+  if (type == blink::WebPlatformEventTypeBattery) {
     if (RenderThreadImpl::current() &&
         RenderThreadImpl::current()->layout_test_mode()) {
       g_test_battery_status_listener =
@@ -1153,7 +1192,7 @@ void RendererBlinkPlatformImpl::startListening(
     observer = CreatePlatformEventObserverFromType(type);
     if (!observer)
       return;
-    platform_event_observers_.AddWithID(observer, static_cast<int32>(type));
+    platform_event_observers_.AddWithID(observer, static_cast<int32_t>(type));
   }
   observer->Start(listener);
 
@@ -1163,9 +1202,10 @@ void RendererBlinkPlatformImpl::startListening(
   // using this broken pattern.
   if (RenderThreadImpl::current() &&
       RenderThreadImpl::current()->layout_test_mode() &&
-      (type == blink::WebPlatformEventDeviceMotion ||
-       type == blink::WebPlatformEventDeviceOrientation ||
-       type == blink::WebPlatformEventDeviceLight)) {
+      (type == blink::WebPlatformEventTypeDeviceMotion ||
+       type == blink::WebPlatformEventTypeDeviceOrientation ||
+       type == blink::WebPlatformEventTypeDeviceOrientationAbsolute ||
+       type == blink::WebPlatformEventTypeDeviceLight)) {
     SendFakeDeviceEventDataForTesting(type);
   }
 }
@@ -1178,15 +1218,16 @@ void RendererBlinkPlatformImpl::SendFakeDeviceEventDataForTesting(
   void* data = 0;
 
   switch (type) {
-  case blink::WebPlatformEventDeviceMotion:
+  case blink::WebPlatformEventTypeDeviceMotion:
     if (!(g_test_device_motion_data == 0))
       data = &g_test_device_motion_data.Get();
     break;
-  case blink::WebPlatformEventDeviceOrientation:
+  case blink::WebPlatformEventTypeDeviceOrientation:
+  case blink::WebPlatformEventTypeDeviceOrientationAbsolute:
     if (!(g_test_device_orientation_data == 0))
       data = &g_test_device_orientation_data.Get();
     break;
-  case blink::WebPlatformEventDeviceLight:
+  case blink::WebPlatformEventTypeDeviceLight:
     if (g_test_device_light_data >= 0)
       data = &g_test_device_light_data;
     break;
@@ -1205,7 +1246,7 @@ void RendererBlinkPlatformImpl::SendFakeDeviceEventDataForTesting(
 
 void RendererBlinkPlatformImpl::stopListening(
     blink::WebPlatformEventType type) {
-  if (type == blink::WebPlatformEventBattery) {
+  if (type == blink::WebPlatformEventTypeBattery) {
     g_test_battery_status_listener = nullptr;
     battery_status_dispatcher_.reset();
     return;
