@@ -5,11 +5,14 @@
 #ifndef GOOGLE_APIS_DRIVE_DRIVE_API_REQUESTS_H_
 #define GOOGLE_APIS_DRIVE_DRIVE_API_REQUESTS_H_
 
+#include <stdint.h>
 #include <string>
+#include <utility>
 #include <vector>
 
 #include "base/callback_forward.h"
 #include "base/location.h"
+#include "base/macros.h"
 #include "base/memory/scoped_ptr.h"
 #include "base/sequenced_task_runner.h"
 #include "base/task_runner_util.h"
@@ -166,7 +169,7 @@ class DriveApiDataRequest : public DriveApiPartialFieldRequest {
   void OnDataParsed(DriveApiErrorCode error, scoped_ptr<DataType> value) {
     if (!value)
       error = DRIVE_PARSE_ERROR;
-    callback_.Run(error, value.Pass());
+    callback_.Run(error, std::move(value));
     OnProcessURLFetchResultsComplete();
   }
 
@@ -249,6 +252,12 @@ class FilesAuthorizeRequest : public DriveApiDataRequest<FileResource> {
 
 //============================ FilesInsertRequest =============================
 
+// Enumeration type for specifying visibility of files.
+enum FileVisibility {
+  FILE_VISIBILITY_DEFAULT,
+  FILE_VISIBILITY_PRIVATE,
+};
+
 // This class performs the request for creating a resource.
 // This request is mapped to
 // https://developers.google.com/drive/v2/reference/files/insert
@@ -261,6 +270,11 @@ class FilesInsertRequest : public DriveApiDataRequest<FileResource> {
                      const FileResourceCallback& callback);
   ~FilesInsertRequest() override;
 
+  // Optional parameter
+  void set_visibility(FileVisibility visibility) {
+    visibility_ = visibility;
+  }
+
   // Optional request body.
   const base::Time& last_viewed_by_me_date() const {
     return last_viewed_by_me_date_;
@@ -302,6 +316,7 @@ class FilesInsertRequest : public DriveApiDataRequest<FileResource> {
  private:
   const DriveApiUrlGenerator url_generator_;
 
+  FileVisibility visibility_;
   base::Time last_viewed_by_me_date_;
   std::string mime_type_;
   base::Time modified_date_;
@@ -409,6 +424,11 @@ class FilesCopyRequest : public DriveApiDataRequest<FileResource> {
   const std::string& file_id() const { return file_id_; }
   void set_file_id(const std::string& file_id) { file_id_ = file_id; }
 
+  // Optional parameter
+  void set_visibility(FileVisibility visibility) {
+    visibility_ = visibility;
+  }
+
   // Optional request body.
   const std::vector<std::string>& parents() const { return parents_; }
   void add_parent(const std::string& parent) { parents_.push_back(parent); }
@@ -434,6 +454,7 @@ class FilesCopyRequest : public DriveApiDataRequest<FileResource> {
   const DriveApiUrlGenerator url_generator_;
 
   std::string file_id_;
+  FileVisibility visibility_;
   base::Time modified_date_;
   std::vector<std::string> parents_;
   std::string title_;
@@ -618,8 +639,8 @@ class ChangesListRequest : public DriveApiDataRequest<ChangeList> {
     page_token_ = page_token;
   }
 
-  int64 start_change_id() const { return start_change_id_; }
-  void set_start_change_id(int64 start_change_id) {
+  int64_t start_change_id() const { return start_change_id_; }
+  void set_start_change_id(int64_t start_change_id) {
     start_change_id_ = start_change_id;
   }
 
@@ -632,7 +653,7 @@ class ChangesListRequest : public DriveApiDataRequest<ChangeList> {
   bool include_deleted_;
   int max_results_;
   std::string page_token_;
-  int64 start_change_id_;
+  int64_t start_change_id_;
 
   DISALLOW_COPY_AND_ASSIGN(ChangesListRequest);
 };
@@ -800,7 +821,7 @@ class InitiateUploadNewFileRequest : public InitiateUploadRequestBase {
   InitiateUploadNewFileRequest(RequestSender* sender,
                                const DriveApiUrlGenerator& url_generator,
                                const std::string& content_type,
-                               int64 content_length,
+                               int64_t content_length,
                                const std::string& parent_resource_id,
                                const std::string& title,
                                const InitiateUploadCallback& callback);
@@ -855,7 +876,7 @@ class InitiateUploadExistingFileRequest : public InitiateUploadRequestBase {
   InitiateUploadExistingFileRequest(RequestSender* sender,
                                     const DriveApiUrlGenerator& url_generator,
                                     const std::string& content_type,
-                                    int64 content_length,
+                                    int64_t content_length,
                                     const std::string& resource_id,
                                     const std::string& etag,
                                     const InitiateUploadCallback& callback);
@@ -919,9 +940,9 @@ class ResumeUploadRequest : public ResumeUploadRequestBase {
   // |callback| must not be null. |progress_callback| may be null.
   ResumeUploadRequest(RequestSender* sender,
                       const GURL& upload_location,
-                      int64 start_position,
-                      int64 end_position,
-                      int64 content_length,
+                      int64_t start_position,
+                      int64_t end_position,
+                      int64_t content_length,
                       const std::string& content_type,
                       const base::FilePath& local_file_path,
                       const UploadRangeCallback& callback,
@@ -934,8 +955,8 @@ class ResumeUploadRequest : public ResumeUploadRequestBase {
                               scoped_ptr<base::Value> value) override;
   // content::UrlFetcherDelegate overrides.
   void OnURLFetchUploadProgress(const net::URLFetcher* source,
-                                int64 current,
-                                int64 total) override;
+                                int64_t current,
+                                int64_t total) override;
 
  private:
   const UploadRangeCallback callback_;
@@ -953,7 +974,7 @@ class GetUploadStatusRequest : public GetUploadStatusRequestBase {
   // |callback| must not be null.
   GetUploadStatusRequest(RequestSender* sender,
                          const GURL& upload_url,
-                         int64 content_length,
+                         int64_t content_length,
                          const UploadRangeCallback& callback);
   ~GetUploadStatusRequest() override;
 
@@ -981,7 +1002,7 @@ class MultipartUploadNewFileDelegate : public MultipartUploadRequestBase {
                                  const std::string& title,
                                  const std::string& parent_resource_id,
                                  const std::string& content_type,
-                                 int64 content_length,
+                                 int64_t content_length,
                                  const base::Time& modified_date,
                                  const base::Time& last_viewed_by_me_date,
                                  const base::FilePath& local_file_path,
@@ -1018,7 +1039,7 @@ class MultipartUploadExistingFileDelegate : public MultipartUploadRequestBase {
       const std::string& resource_id,
       const std::string& parent_resource_id,
       const std::string& content_type,
-      int64 content_length,
+      int64_t content_length,
       const base::Time& modified_date,
       const base::Time& last_viewed_by_me_date,
       const base::FilePath& local_file_path,
@@ -1130,8 +1151,8 @@ class SingleBatchableDelegateRequest : public UrlFetchRequestBase {
   void RunCallbackOnPrematureFailure(DriveApiErrorCode code) override;
   void ProcessURLFetchResults(const net::URLFetcher* source) override;
   void OnURLFetchUploadProgress(const net::URLFetcher* source,
-                                int64 current,
-                                int64 total) override;
+                                int64_t current,
+                                int64_t total) override;
   scoped_ptr<BatchableDelegate> delegate_;
 
   // Note: This should remain the last member so it'll be destroyed and
@@ -1149,8 +1170,8 @@ class BatchUploadChildEntry {
   ~BatchUploadChildEntry();
   scoped_ptr<BatchableDelegate> request;
   bool prepared;
-  int64 data_offset;
-  int64 data_size;
+  int64_t data_offset;
+  int64_t data_size;
 
  private:
   DISALLOW_COPY_AND_ASSIGN(BatchUploadChildEntry);
@@ -1195,8 +1216,8 @@ class BatchUploadRequest : public UrlFetchRequestBase {
 
   // content::UrlFetcherDelegate overrides.
   void OnURLFetchUploadProgress(const net::URLFetcher* source,
-                                int64 current,
-                                int64 total) override;
+                                int64_t current,
+                                int64_t total) override;
 
  private:
   typedef void* RequestID;
@@ -1228,7 +1249,7 @@ class BatchUploadRequest : public UrlFetchRequestBase {
   ContentTypeAndData upload_content_;
 
   // Last reported progress value.
-  int64 last_progress_value_;
+  int64_t last_progress_value_;
 
   // Note: This should remain the last member so it'll be destroyed and
   // invalidate its weak pointers before any other members are destroyed.
