@@ -4,12 +4,14 @@
 
 #include "google_apis/gcm/engine/gcm_store_impl.h"
 
-#include "base/basictypes.h"
+#include <utility>
+
 #include "base/bind.h"
 #include "base/callback.h"
 #include "base/files/file_path.h"
 #include "base/files/file_util.h"
 #include "base/logging.h"
+#include "base/macros.h"
 #include "base/metrics/histogram_macros.h"
 #include "base/profiler/scoped_tracker.h"
 #include "base/sequenced_task_runner.h"
@@ -189,8 +191,8 @@ class GCMStoreImpl::Backend
   void Load(StoreOpenMode open_mode, const LoadCallback& callback);
   void Close();
   void Destroy(const UpdateCallback& callback);
-  void SetDeviceCredentials(uint64 device_android_id,
-                            uint64 device_security_token,
+  void SetDeviceCredentials(uint64_t device_android_id,
+                            uint64_t device_security_token,
                             const UpdateCallback& callback);
   void AddRegistration(const std::string& serialized_key,
                        const std::string& serialized_value,
@@ -209,7 +211,7 @@ class GCMStoreImpl::Backend
       const base::Callback<void(bool, const AppIdToMessageCountMap&)>
           callback);
   void AddUserSerialNumber(const std::string& username,
-                           int64 serial_number,
+                           int64_t serial_number,
                            const UpdateCallback& callback);
   void RemoveUserSerialNumber(const std::string& username,
                               const UpdateCallback& callback);
@@ -245,7 +247,7 @@ class GCMStoreImpl::Backend
   ~Backend();
 
   LoadStatus OpenStoreAndLoadData(StoreOpenMode open_mode, LoadResult* result);
-  bool LoadDeviceCredentials(uint64* android_id, uint64* security_token);
+  bool LoadDeviceCredentials(uint64_t* android_id, uint64_t* security_token);
   bool LoadRegistrations(std::map<std::string, std::string>* registrations);
   bool LoadIncomingMessages(std::vector<std::string>* incoming_messages);
   bool LoadOutgoingMessages(OutgoingMessageMap* outgoing_messages);
@@ -271,8 +273,7 @@ GCMStoreImpl::Backend::Backend(
     scoped_ptr<Encryptor> encryptor)
     : path_(path),
       foreground_task_runner_(foreground_task_runner),
-      encryptor_(encryptor.Pass()) {
-}
+      encryptor_(std::move(encryptor)) {}
 
 GCMStoreImpl::Backend::~Backend() {}
 
@@ -294,9 +295,13 @@ LoadStatus GCMStoreImpl::Backend::OpenStoreAndLoadData(StoreOpenMode open_mode,
   leveldb::Options options;
   options.create_if_missing = open_mode == CREATE_IF_MISSING;
   options.reuse_logs = leveldb_env::kDefaultLogReuseOptionValue;
+  options.paranoid_checks = true;
   leveldb::DB* db;
   leveldb::Status status =
       leveldb::DB::Open(options, path_.AsUTF8Unsafe(), &db);
+  UMA_HISTOGRAM_ENUMERATION("GCM.Database.Open",
+                            leveldb_env::GetLevelDBStatusUMAValue(status),
+                            leveldb_env::LEVELDB_STATUS_MAX);
   if (!status.ok()) {
     LOG(ERROR) << "Failed to open database " << path_.value() << ": "
                << status.ToString();
@@ -353,7 +358,8 @@ void GCMStoreImpl::Backend::Load(StoreOpenMode open_mode,
   int gcm_registration_count = 0;
   int instance_id_token_count = 0;
   for (const auto& registration : result->registrations) {
-    if (base::StartsWithASCII(registration.first, "iid-", true))
+    if (base::StartsWith(registration.first, "iid-",
+                         base::CompareCase::SENSITIVE))
       instance_id_token_count++;
     else
       gcm_registration_count++;
@@ -361,7 +367,7 @@ void GCMStoreImpl::Backend::Load(StoreOpenMode open_mode,
 
   // Only record histograms if GCM had already been set up for this device.
   if (result->device_android_id != 0 && result->device_security_token != 0) {
-    int64 file_size = 0;
+    int64_t file_size = 0;
     if (base::GetFileSize(path_, &file_size)) {
       UMA_HISTOGRAM_COUNTS("GCM.StoreSizeKB",
                            static_cast<int>(file_size / 1024));
@@ -413,8 +419,8 @@ void GCMStoreImpl::Backend::Destroy(const UpdateCallback& callback) {
 }
 
 void GCMStoreImpl::Backend::SetDeviceCredentials(
-    uint64 device_android_id,
-    uint64 device_security_token,
+    uint64_t device_android_id,
+    uint64_t device_security_token,
     const UpdateCallback& callback) {
   DVLOG(1) << "Saving device credentials with AID " << device_android_id;
   if (!db_.get()) {
@@ -631,7 +637,7 @@ void GCMStoreImpl::Backend::SetLastCheckinInfo(
     const UpdateCallback& callback) {
   leveldb::WriteBatch write_batch;
 
-  int64 last_checkin_time_internal = time.ToInternalValue();
+  int64_t last_checkin_time_internal = time.ToInternalValue();
   write_batch.Put(MakeSlice(kLastCheckinTimeKey),
                   MakeSlice(base::Int64ToString(last_checkin_time_internal)));
 
@@ -894,8 +900,8 @@ void GCMStoreImpl::Backend::SetValue(const std::string& key,
   foreground_task_runner_->PostTask(FROM_HERE, base::Bind(callback, s.ok()));
 }
 
-bool GCMStoreImpl::Backend::LoadDeviceCredentials(uint64* android_id,
-                                                  uint64* security_token) {
+bool GCMStoreImpl::Backend::LoadDeviceCredentials(uint64_t* android_id,
+                                                  uint64_t* security_token) {
   leveldb::ReadOptions read_options;
   read_options.verify_checksums = true;
 
@@ -986,7 +992,7 @@ bool GCMStoreImpl::Backend::LoadOutgoingMessages(
       LOG(ERROR) << "Error reading incoming message with key " << s.ToString();
       return false;
     }
-    uint8 tag = iter->value().data()[0];
+    uint8_t tag = iter->value().data()[0];
     std::string id = ParseOutgoingKey(iter->key().ToString());
     scoped_ptr<google::protobuf::MessageLite> message(
         BuildProtobufFromTag(tag));
@@ -997,7 +1003,7 @@ bool GCMStoreImpl::Backend::LoadOutgoingMessages(
       return false;
     }
     DVLOG(1) << "Found outgoing message with id " << id << " of type "
-             << base::IntToString(tag);
+             << base::UintToString(tag);
     (*outgoing_messages)[id] = make_linked_ptr(message.release());
   }
 
@@ -1014,7 +1020,7 @@ bool GCMStoreImpl::Backend::LoadLastCheckinInfo(
   leveldb::Status s = db_->Get(read_options,
                                MakeSlice(kLastCheckinTimeKey),
                                &result);
-  int64 time_internal = 0LL;
+  int64_t time_internal = 0LL;
   if (s.ok() && !base::StringToInt64(result, &time_internal)) {
     LOG(ERROR) << "Failed to restore last checkin time. Using default = 0.";
     time_internal = 0LL;
@@ -1095,7 +1101,7 @@ bool GCMStoreImpl::Backend::LoadLastTokenFetchTime(
   std::string result;
   leveldb::Status s =
       db_->Get(read_options, MakeSlice(kLastTokenFetchTimeKey), &result);
-  int64 time_internal = 0LL;
+  int64_t time_internal = 0LL;
   if (s.ok() && !base::StringToInt64(result, &time_internal)) {
     LOG(ERROR) <<
         "Failed to restore last token fetching time. Using default = 0.";
@@ -1160,10 +1166,9 @@ GCMStoreImpl::GCMStoreImpl(
     scoped_ptr<Encryptor> encryptor)
     : backend_(new Backend(path,
                            base::ThreadTaskRunnerHandle::Get(),
-                           encryptor.Pass())),
+                           std::move(encryptor))),
       blocking_task_runner_(blocking_task_runner),
-      weak_ptr_factory_(this) {
-}
+      weak_ptr_factory_(this) {}
 
 GCMStoreImpl::~GCMStoreImpl() {}
 
@@ -1192,8 +1197,8 @@ void GCMStoreImpl::Destroy(const UpdateCallback& callback) {
       base::Bind(&GCMStoreImpl::Backend::Destroy, backend_, callback));
 }
 
-void GCMStoreImpl::SetDeviceCredentials(uint64 device_android_id,
-                                        uint64 device_security_token,
+void GCMStoreImpl::SetDeviceCredentials(uint64_t device_android_id,
+                                        uint64_t device_security_token,
                                         const UpdateCallback& callback) {
   blocking_task_runner_->PostTask(
       FROM_HERE,
@@ -1425,7 +1430,7 @@ void GCMStoreImpl::LoadContinuation(const LoadCallback& callback,
       FROM_HERE_WITH_EXPLICIT_FUNCTION(
           "477117 GCMStoreImpl::LoadContinuation"));
   if (!result->success) {
-    callback.Run(result.Pass());
+    callback.Run(std::move(result));
     return;
   }
   int num_throttled_apps = 0;
@@ -1443,7 +1448,7 @@ void GCMStoreImpl::LoadContinuation(const LoadCallback& callback,
       num_throttled_apps++;
   }
   UMA_HISTOGRAM_COUNTS("GCM.NumThrottledApps", num_throttled_apps);
-  callback.Run(result.Pass());
+  callback.Run(std::move(result));
 }
 
 void GCMStoreImpl::AddOutgoingMessageContinuation(
