@@ -5,21 +5,23 @@
 #ifndef GPU_COMMAND_BUFFER_SERVICE_FEATURE_INFO_H_
 #define GPU_COMMAND_BUFFER_SERVICE_FEATURE_INFO_H_
 
-#include <set>
 #include <string>
-#include "base/containers/hash_tables.h"
+#include "base/macros.h"
 #include "base/memory/ref_counted.h"
-#include "base/sys_info.h"
+#include "base/memory/scoped_ptr.h"
 #include "gpu/command_buffer/service/gles2_cmd_decoder.h"
 #include "gpu/command_buffer/service/gles2_cmd_validation.h"
 #include "gpu/config/gpu_driver_bug_workaround_type.h"
 #include "gpu/gpu_export.h"
-#include "ui/gl/gl_version_info.h"
 
 namespace base {
 class CommandLine;
 }
 
+namespace gfx {
+struct GLVersionInfo;
+}
+
 namespace gpu {
 namespace gles2 {
 
@@ -40,6 +42,7 @@ class GPU_EXPORT FeatureInfo : public base::RefCounted<FeatureInfo> {
     bool multisampled_render_to_texture;
     // Use the IMG GLenum values and functions rather than EXT.
     bool use_img_for_multisampled_render_to_texture;
+    bool chromium_screen_space_antialiasing;
     bool oes_standard_derivatives;
     bool oes_egl_image_external;
     bool oes_depth24;
@@ -56,6 +59,7 @@ class GPU_EXPORT FeatureInfo : public base::RefCounted<FeatureInfo> {
     bool use_arb_occlusion_query2_for_occlusion_query_boolean;
     bool use_arb_occlusion_query_for_occlusion_query_boolean;
     bool native_vertex_array_object;
+    bool ext_texture_format_astc;
     bool ext_texture_format_atc;
     bool ext_texture_format_bgra8888;
     bool ext_texture_format_dxt1;
@@ -74,12 +78,17 @@ class GPU_EXPORT FeatureInfo : public base::RefCounted<FeatureInfo> {
     bool angle_texture_usage;
     bool ext_texture_storage;
     bool chromium_path_rendering;
+    bool chromium_framebuffer_mixed_samples;
     bool blend_equation_advanced;
     bool blend_equation_advanced_coherent;
     bool ext_texture_rg;
+    bool chromium_image_ycbcr_420v;
+    bool chromium_image_ycbcr_422;
     bool enable_subscribe_uniform;
     bool emulate_primitive_restart_fixed_index;
     bool ext_render_buffer_format_bgra8888;
+    bool ext_multisample_compatibility;
+    bool ext_blend_func_extended;
   };
 
   struct Workarounds {
@@ -105,13 +114,18 @@ class GPU_EXPORT FeatureInfo : public base::RefCounted<FeatureInfo> {
   FeatureInfo(const base::CommandLine& command_line);
 
   // Initializes the feature information. Needs a current GL context.
-  bool Initialize();
-  bool Initialize(const DisallowedFeatures& disallowed_features);
+  bool Initialize(ContextType context_type,
+                  const DisallowedFeatures& disallowed_features);
+
+  // Helper that defaults to no disallowed features and a GLES2 context.
+  bool InitializeForTesting();
 
   const Validators* validators() const {
     return &validators_;
   }
 
+  ContextType context_type() const { return context_type_; }
+
   const std::string& extensions() const {
     return extensions_;
   }
@@ -136,6 +150,10 @@ class GPU_EXPORT FeatureInfo : public base::RefCounted<FeatureInfo> {
     return unsafe_es3_apis_enabled_;
   }
 
+  bool disable_shader_translator() const { return disable_shader_translator_; }
+
+  bool IsWebGLContext() const;
+
  private:
   friend class base::RefCounted<FeatureInfo>;
   friend class BufferManagerClientSideArraysTest;
@@ -143,13 +161,15 @@ class GPU_EXPORT FeatureInfo : public base::RefCounted<FeatureInfo> {
   ~FeatureInfo();
 
   void AddExtensionString(const char* s);
-  void InitializeBasicState(const base::CommandLine& command_line);
+  void InitializeBasicState(const base::CommandLine* command_line);
   void InitializeFeatures();
 
   Validators validators_;
 
   DisallowedFeatures disallowed_features_;
 
+  ContextType context_type_;
+
   // The extensions string returned by glGetString(GL_EXTENSIONS);
   std::string extensions_;
 
@@ -164,6 +184,10 @@ class GPU_EXPORT FeatureInfo : public base::RefCounted<FeatureInfo> {
 
   bool unsafe_es3_apis_enabled_;
 
+  // Whether the command line switch kEnableGLPathRendering is passed in.
+  bool enable_gl_path_rendering_switch_;
+
+  bool disable_shader_translator_;
   scoped_ptr<gfx::GLVersionInfo> gl_version_info_;
 
   DISALLOW_COPY_AND_ASSIGN(FeatureInfo);
