@@ -5,6 +5,10 @@
 #ifndef GPU_COMMAND_BUFFER_SERVICE_GLES2_CMD_DECODER_UNITTEST_BASE_H_
 #define GPU_COMMAND_BUFFER_SERVICE_GLES2_CMD_DECODER_UNITTEST_BASE_H_
 
+#include <stddef.h>
+#include <stdint.h>
+
+#include "base/message_loop/message_loop.h"
 #include "gpu/command_buffer/common/gles2_cmd_format.h"
 #include "gpu/command_buffer/common/gles2_cmd_utils.h"
 #include "gpu/command_buffer/service/buffer_manager.h"
@@ -17,14 +21,16 @@
 #include "gpu/command_buffer/service/program_manager.h"
 #include "gpu/command_buffer/service/query_manager.h"
 #include "gpu/command_buffer/service/renderbuffer_manager.h"
+#include "gpu/command_buffer/service/sampler_manager.h"
 #include "gpu/command_buffer/service/shader_manager.h"
 #include "gpu/command_buffer/service/test_helper.h"
 #include "gpu/command_buffer/service/texture_manager.h"
 #include "gpu/command_buffer/service/valuebuffer_manager.h"
 #include "gpu/command_buffer/service/vertex_array_manager.h"
 #include "testing/gtest/include/gtest/gtest.h"
-#include "ui/gl/gl_surface_stub.h"
 #include "ui/gl/gl_mock.h"
+#include "ui/gl/gl_surface_stub.h"
+#include "ui/gl/gl_version_info.h"
 
 namespace base {
 class CommandLine;
@@ -43,7 +49,7 @@ class GLES2DecoderTestBase : public ::testing::TestWithParam<bool> {
   // Template to call glGenXXX functions.
   template <typename T>
   void GenHelper(GLuint client_id) {
-    int8 buffer[sizeof(T) + sizeof(client_id)];
+    int8_t buffer[sizeof(T) + sizeof(client_id)];
     T& cmd = *reinterpret_cast<T*>(&buffer);
     cmd.Init(1, &client_id);
     EXPECT_EQ(error::kNoError,
@@ -95,8 +101,8 @@ class GLES2DecoderTestBase : public ::testing::TestWithParam<bool> {
   }
 
   template <typename T>
-  T GetSharedMemoryAsWithOffset(uint32 offset) {
-    void* ptr = reinterpret_cast<int8*>(shared_memory_address_) + offset;
+  T GetSharedMemoryAsWithOffset(uint32_t offset) {
+    void* ptr = reinterpret_cast<int8_t*>(shared_memory_address_) + offset;
     return reinterpret_cast<T>(ptr);
   }
 
@@ -133,8 +139,8 @@ class GLES2DecoderTestBase : public ::testing::TestWithParam<bool> {
     return decoder_->GetQueryManager()->GetQuery(client_id);
   }
 
-  bool GetSamplerServiceId(GLuint client_id, GLuint* service_id) const {
-    return group_->GetSamplerServiceId(client_id, service_id);
+  Sampler* GetSampler(GLuint client_id) {
+    return group_->sampler_manager()->GetSampler(client_id);
   }
 
   bool GetTransformFeedbackServiceId(
@@ -175,11 +181,14 @@ class GLES2DecoderTestBase : public ::testing::TestWithParam<bool> {
   void DoFenceSync(GLuint client_id, GLuint service_id);
 
   void SetBucketData(uint32_t bucket_id, const void* data, uint32_t data_size);
-  void SetBucketAsCString(uint32 bucket_id, const char* str);
+  void SetBucketAsCString(uint32_t bucket_id, const char* str);
   // If we want a valid bucket, just set |count_in_header| as |count|,
   // and set |str_end| as 0.
-  void SetBucketAsCStrings(uint32 bucket_id, GLsizei count, const char** str,
-                           GLsizei count_in_header, char str_end);
+  void SetBucketAsCStrings(uint32_t bucket_id,
+                           GLsizei count,
+                           const char** str,
+                           GLsizei count_in_header,
+                           char str_end);
 
   void set_memory_tracker(MemoryTracker* memory_tracker) {
     memory_tracker_ = memory_tracker;
@@ -199,7 +208,7 @@ class GLES2DecoderTestBase : public ::testing::TestWithParam<bool> {
     bool bind_generates_resource;
     bool lose_context_when_out_of_memory;
     bool use_native_vao;  // default is true.
-    unsigned webgl_version;  // default to 0, i.e., not WebGL context.
+    ContextType context_type;
   };
 
   void InitDecoder(const InitState& init);
@@ -224,6 +233,10 @@ class GLES2DecoderTestBase : public ::testing::TestWithParam<bool> {
     return decoder_.get();
   }
 
+  uint32_t GetAndClearBackbufferClearBitsForTest() const {
+    return decoder_->GetAndClearBackbufferClearBitsForTest();
+  }
+
   typedef TestHelper::AttribInfo AttribInfo;
   typedef TestHelper::UniformInfo UniformInfo;
 
@@ -235,7 +248,7 @@ class GLES2DecoderTestBase : public ::testing::TestWithParam<bool> {
       GLuint fragment_shader_client_id, GLuint fragment_shader_service_id);
 
   void SetupInitCapabilitiesExpectations(bool es3_capable);
-  void SetupInitStateExpectations();
+  void SetupInitStateExpectations(bool es3_capable);
   void ExpectEnableDisable(GLenum cap, bool enable);
 
   // Setups up a shader for testing glUniform.
@@ -284,22 +297,36 @@ class GLES2DecoderTestBase : public ::testing::TestWithParam<bool> {
   void DoDeleteShader(GLuint client_id, GLuint service_id);
   void DoDeleteTexture(GLuint client_id, GLuint service_id);
 
-  void DoCompressedTexImage2D(
-      GLenum target, GLint level, GLenum format,
-      GLsizei width, GLsizei height, GLint border,
-      GLsizei size, uint32 bucket_id);
+  void DoCompressedTexImage2D(GLenum target,
+                              GLint level,
+                              GLenum format,
+                              GLsizei width,
+                              GLsizei height,
+                              GLint border,
+                              GLsizei size,
+                              uint32_t bucket_id);
   void DoBindTexImage2DCHROMIUM(GLenum target, GLint image_id);
-  void DoTexImage2D(
-      GLenum target, GLint level, GLenum internal_format,
-      GLsizei width, GLsizei height, GLint border,
-      GLenum format, GLenum type,
-      uint32 shared_memory_id, uint32 shared_memory_offset);
-  void DoTexImage2DConvertInternalFormat(
-      GLenum target, GLint level, GLenum requested_internal_format,
-      GLsizei width, GLsizei height, GLint border,
-      GLenum format, GLenum type,
-      uint32 shared_memory_id, uint32 shared_memory_offset,
-      GLenum expected_internal_format);
+  void DoTexImage2D(GLenum target,
+                    GLint level,
+                    GLenum internal_format,
+                    GLsizei width,
+                    GLsizei height,
+                    GLint border,
+                    GLenum format,
+                    GLenum type,
+                    uint32_t shared_memory_id,
+                    uint32_t shared_memory_offset);
+  void DoTexImage2DConvertInternalFormat(GLenum target,
+                                         GLint level,
+                                         GLenum requested_internal_format,
+                                         GLsizei width,
+                                         GLsizei height,
+                                         GLint border,
+                                         GLenum format,
+                                         GLenum type,
+                                         uint32_t shared_memory_id,
+                                         uint32_t shared_memory_offset,
+                                         GLenum expected_internal_format);
   void DoRenderbufferStorage(
       GLenum target, GLenum internal_format, GLenum actual_format,
       GLsizei width, GLsizei height, GLenum error);
@@ -329,6 +356,8 @@ class GLES2DecoderTestBase : public ::testing::TestWithParam<bool> {
 
   void DoScissor(GLint x, GLint y, GLsizei width, GLsizei height);
 
+  void DoPixelStorei(GLenum pname, GLint param);
+
   void SetupVertexBuffer();
   void SetupAllNeededVertexBuffers();
 
@@ -427,9 +456,7 @@ class GLES2DecoderTestBase : public ::testing::TestWithParam<bool> {
   void AddExpectationsForBindVertexArrayOES();
   void AddExpectationsForRestoreAttribState(GLuint attrib);
 
-  GLvoid* BufferOffset(unsigned i) {
-    return static_cast<int8 *>(NULL)+(i);
-  }
+  GLvoid* BufferOffset(unsigned i) { return static_cast<int8_t*>(NULL) + (i); }
 
   template <typename Command, typename Result>
   bool IsObjectHelper(GLuint client_id) {
@@ -479,17 +506,17 @@ class GLES2DecoderTestBase : public ::testing::TestWithParam<bool> {
   static const GLuint kServiceTransformFeedbackId = 311;
   static const GLuint kServiceSyncId = 312;
 
-  static const int32 kSharedMemoryId = 401;
+  static const int32_t kSharedMemoryId = 401;
   static const size_t kSharedBufferSize = 2048;
-  static const uint32 kSharedMemoryOffset = 132;
-  static const int32 kInvalidSharedMemoryId = 402;
-  static const uint32 kInvalidSharedMemoryOffset = kSharedBufferSize + 1;
-  static const uint32 kInitialResult = 0xBDBDBDBDu;
-  static const uint8 kInitialMemoryValue = 0xBDu;
+  static const uint32_t kSharedMemoryOffset = 132;
+  static const int32_t kInvalidSharedMemoryId = 402;
+  static const uint32_t kInvalidSharedMemoryOffset = kSharedBufferSize + 1;
+  static const uint32_t kInitialResult = 0xBDBDBDBDu;
+  static const uint8_t kInitialMemoryValue = 0xBDu;
 
-  static const uint32 kNewClientId = 501;
-  static const uint32 kNewServiceId = 502;
-  static const uint32 kInvalidClientId = 601;
+  static const uint32_t kNewClientId = 501;
+  static const uint32_t kNewServiceId = 502;
+  static const uint32_t kInvalidClientId = 601;
 
   static const GLuint kServiceVertexShaderId = 321;
   static const GLuint kServiceFragmentShaderId = 322;
@@ -533,28 +560,59 @@ class GLES2DecoderTestBase : public ::testing::TestWithParam<bool> {
   static const char* kUniform1Name;
   static const char* kUniform2Name;
   static const char* kUniform3Name;
+  static const char* kUniform4Name;
+  static const char* kUniform5Name;
+  static const char* kUniform6Name;
+  static const char* kUniform7Name;
   static const GLint kUniform1Size = 1;
   static const GLint kUniform2Size = 3;
   static const GLint kUniform3Size = 2;
+  static const GLint kUniform4Size = 1;
+  static const GLint kUniform5Size = 1;
+  static const GLint kUniform6Size = 1;
+  static const GLint kUniform7Size = 1;
   static const GLint kUniform1RealLocation = 3;
   static const GLint kUniform2RealLocation = 10;
   static const GLint kUniform2ElementRealLocation = 12;
   static const GLint kUniform3RealLocation = 20;
+  static const GLint kUniform4RealLocation = 22;
+  static const GLint kUniform5RealLocation = 30;
+  static const GLint kUniform6RealLocation = 32;
+  static const GLint kUniform7RealLocation = 44;
   static const GLint kUniform1FakeLocation = 0;               // These are
   static const GLint kUniform2FakeLocation = 1;               // hardcoded
   static const GLint kUniform2ElementFakeLocation = 0x10001;  // to match
   static const GLint kUniform3FakeLocation = 2;               // ProgramManager.
+  static const GLint kUniform4FakeLocation = 3;               //
+  static const GLint kUniform5FakeLocation = 4;               //
+  static const GLint kUniform6FakeLocation = 5;               //
+  static const GLint kUniform7FakeLocation = 6;               //
   static const GLint kUniform1DesiredLocation = -1;
   static const GLint kUniform2DesiredLocation = -1;
   static const GLint kUniform3DesiredLocation = -1;
+  static const GLint kUniform4DesiredLocation = -1;
+  static const GLint kUniform5DesiredLocation = -1;
+  static const GLint kUniform6DesiredLocation = -1;
+  static const GLint kUniform7DesiredLocation = -1;
   static const GLenum kUniform1Type = GL_SAMPLER_2D;
   static const GLenum kUniform2Type = GL_INT_VEC2;
   static const GLenum kUniform3Type = GL_FLOAT_VEC3;
+  static const GLenum kUniform4Type = GL_UNSIGNED_INT;
+  static const GLenum kUniform5Type = GL_UNSIGNED_INT_VEC2;
+  static const GLenum kUniform6Type = GL_UNSIGNED_INT_VEC3;
+  static const GLenum kUniform7Type = GL_UNSIGNED_INT_VEC4;
   static const GLenum kUniformSamplerExternalType = GL_SAMPLER_EXTERNAL_OES;
   static const GLenum kUniformCubemapType = GL_SAMPLER_CUBE;
   static const GLint kInvalidUniformLocation = 30;
   static const GLint kBadUniformIndex = 1000;
 
+  static const GLint kOutputVariable1Size = 0;
+  static const GLenum kOutputVariable1Type = GL_FLOAT_VEC4;
+  static const GLuint kOutputVariable1ColorName = 7;
+  static const GLuint kOutputVariable1Index = 0;
+  static const char* kOutputVariable1Name;
+  static const char* kOutputVariable1NameESSL3;
+
   // Use StrictMock to make 100% sure we know how GL will be called.
   scoped_ptr< ::testing::StrictMock< ::gfx::MockGLInterface> > gl_;
   scoped_refptr<gfx::GLSurfaceStub> surface_;
@@ -579,15 +637,15 @@ class GLES2DecoderTestBase : public ::testing::TestWithParam<bool> {
   GLuint client_transformfeedback_id_;
   GLuint client_sync_id_;
 
-  uint32 shared_memory_id_;
-  uint32 shared_memory_offset_;
+  uint32_t shared_memory_id_;
+  uint32_t shared_memory_offset_;
   void* shared_memory_address_;
   void* shared_memory_base_;
 
   GLuint service_renderbuffer_id_;
   bool service_renderbuffer_valid_;
 
-  uint32 immediate_buffer_[64];
+  uint32_t immediate_buffer_[64];
 
   const bool ignore_cached_state_for_test_;
   bool cached_color_mask_red_;
@@ -613,6 +671,8 @@ class GLES2DecoderTestBase : public ::testing::TestWithParam<bool> {
 
   EnableFlags enable_flags_;
 
+  int shader_language_version_;
+
  private:
   class MockCommandBufferEngine : public CommandBufferEngine {
    public:
@@ -620,21 +680,21 @@ class GLES2DecoderTestBase : public ::testing::TestWithParam<bool> {
 
     ~MockCommandBufferEngine() override;
 
-    scoped_refptr<gpu::Buffer> GetSharedMemoryBuffer(int32 shm_id) override;
+    scoped_refptr<gpu::Buffer> GetSharedMemoryBuffer(int32_t shm_id) override;
 
     void ClearSharedMemory() {
       memset(valid_buffer_->memory(), kInitialMemoryValue, kSharedBufferSize);
     }
 
-    void set_token(int32 token) override;
+    void set_token(int32_t token) override;
 
-    bool SetGetBuffer(int32 /* transfer_buffer_id */) override;
+    bool SetGetBuffer(int32_t /* transfer_buffer_id */) override;
 
     // Overridden from CommandBufferEngine.
-    bool SetGetOffset(int32 offset) override;
+    bool SetGetOffset(int32_t offset) override;
 
     // Overridden from CommandBufferEngine.
-    int32 GetGetOffset() override;
+    int32_t GetGetOffset() override;
 
    private:
     scoped_refptr<gpu::Buffer> valid_buffer_;
@@ -679,9 +739,12 @@ class GLES2DecoderTestBase : public ::testing::TestWithParam<bool> {
   void AddExpectationsForVertexAttribManager();
   void SetupMockGLBehaviors();
 
+  void SetupInitStateManualExpectations(bool es3_capable);
+
   scoped_ptr< ::testing::StrictMock<MockCommandBufferEngine> > engine_;
   scoped_refptr<ContextGroup> group_;
   MockGLStates gl_states_;
+  base::MessageLoop message_loop_;
 };
 
 class GLES2DecoderWithShaderTestBase : public GLES2DecoderTestBase {
