@@ -7,6 +7,7 @@
 
 #include "base/memory/ref_counted.h"
 #include "base/memory/scoped_ptr.h"
+#include "base/message_loop/message_loop.h"
 #include "testing/gmock/include/gmock/gmock.h"
 #include "testing/gtest/include/gtest/gtest.h"
 #include "ui/gl/gl_mock.h"
@@ -22,11 +23,13 @@ namespace gles2 {
 class GpuServiceTest : public testing::Test {
  public:
   GpuServiceTest();
-  virtual ~GpuServiceTest();
+  ~GpuServiceTest() override;
 
  protected:
-  virtual void SetUp() OVERRIDE;
-  virtual void TearDown() OVERRIDE;
+  void SetUpWithGLVersion(const char* gl_version, const char* gl_extensions);
+  void SetUp() override;
+  void TearDown() override;
+  gfx::GLContext* GetGLContext();
 
   scoped_ptr< ::testing::StrictMock< ::gfx::MockGLInterface> > gl_;
 
@@ -34,9 +37,10 @@ class GpuServiceTest : public testing::Test {
   bool ran_setup_;
   bool ran_teardown_;
   scoped_refptr<gfx::GLContextStubWithExtensions> context_;
+  base::MessageLoop message_loop_;
 };
 
 }  // namespace gles2
 }  // namespace gpu
 
-#endif  // GPU_COMMAND_BUFFER_SERVICE_MAILBOX_SYNCHRONIZER_H_
+#endif  // GPU_COMMAND_BUFFER_SERVICE_GPU_SERVICE_TEST_H_
