@@ -4,8 +4,11 @@
 
 #include "gpu/command_buffer/service/stream_texture_manager_in_process_android.h"
 
+#include <stdint.h>
+
 #include "base/bind.h"
 #include "base/callback.h"
+#include "base/macros.h"
 #include "gpu/command_buffer/service/texture_manager.h"
 #include "ui/gfx/geometry/size.h"
 #include "ui/gl/android/surface_texture.h"
@@ -17,33 +20,37 @@ namespace gpu {
 namespace {
 
 // Simply wraps a SurfaceTexture reference as a GLImage.
-class GLImageImpl : public gfx::GLImage {
+class GLImageImpl : public gl::GLImage {
  public:
-  GLImageImpl(const scoped_refptr<gfx::SurfaceTexture>& surface_texture,
+  GLImageImpl(uint32_t texture_id,
+              gles2::TextureManager* texture_manager,
+              const scoped_refptr<gfx::SurfaceTexture>& surface_texture,
               const base::Closure& release_callback);
 
-  // implement gfx::GLImage
+  // implement gl::GLImage
   void Destroy(bool have_context) override;
   gfx::Size GetSize() override;
   unsigned GetInternalFormat() override;
   bool BindTexImage(unsigned target) override;
   void ReleaseTexImage(unsigned target) override;
+  bool CopyTexImage(unsigned target) override;
   bool CopyTexSubImage(unsigned target,
                        const gfx::Point& offset,
                        const gfx::Rect& rect) override;
-  void WillUseTexImage() override;
-  void DidUseTexImage() override {}
-  void WillModifyTexImage() override {}
-  void DidModifyTexImage() override {}
   bool ScheduleOverlayPlane(gfx::AcceleratedWidget widget,
                             int z_order,
                             gfx::OverlayTransform transform,
                             const gfx::Rect& bounds_rect,
                             const gfx::RectF& crop_rect) override;
+  void OnMemoryDump(base::trace_event::ProcessMemoryDump* pmd,
+                    uint64_t process_tracing_id,
+                    const std::string& dump_name) override;
 
  private:
   ~GLImageImpl() override;
 
+  uint32_t texture_id_;
+  gles2::TextureManager* texture_manager_;
   scoped_refptr<gfx::SurfaceTexture> surface_texture_;
   base::Closure release_callback_;
 
@@ -51,9 +58,14 @@ class GLImageImpl : public gfx::GLImage {
 };
 
 GLImageImpl::GLImageImpl(
+    uint32_t texture_id,
+    gles2::TextureManager* texture_manager,
     const scoped_refptr<gfx::SurfaceTexture>& surface_texture,
     const base::Closure& release_callback)
-    : surface_texture_(surface_texture), release_callback_(release_callback) {}
+    : texture_id_(texture_id),
+      texture_manager_(texture_manager),
+      surface_texture_(surface_texture),
+      release_callback_(release_callback) {}
 
 GLImageImpl::~GLImageImpl() {
   release_callback_.Run();
@@ -80,16 +92,39 @@ void GLImageImpl::ReleaseTexImage(unsigned target) {
   NOTREACHED();
 }
 
+bool GLImageImpl::CopyTexImage(unsigned target) {
+  if (target != GL_TEXTURE_EXTERNAL_OES)
+    return false;
+
+  GLint texture_id;
+  glGetIntegerv(GL_TEXTURE_BINDING_EXTERNAL_OES, &texture_id);
+  DCHECK(texture_id);
+
+  // The following code only works if we're being asked to copy into
+  // |texture_id_|. Copying into a different texture is not supported.
+  if (static_cast<unsigned>(texture_id) != texture_id_)
+    return false;
+
+  surface_texture_->UpdateTexImage();
+
+  gles2::Texture* texture =
+      texture_manager_->GetTextureForServiceId(texture_id_);
+  if (texture) {
+    // By setting image state to UNBOUND instead of COPIED we ensure that
+    // CopyTexImage() is called each time the surface texture is used for
+    // drawing.
+    texture->SetLevelImage(GL_TEXTURE_EXTERNAL_OES, 0, this,
+                           gles2::Texture::UNBOUND);
+  }
+  return true;
+}
+
 bool GLImageImpl::CopyTexSubImage(unsigned target,
                                   const gfx::Point& offset,
                                   const gfx::Rect& rect) {
   return false;
 }
 
-void GLImageImpl::WillUseTexImage() {
-  surface_texture_->UpdateTexImage();
-}
-
 bool GLImageImpl::ScheduleOverlayPlane(gfx::AcceleratedWidget widget,
                                        int z_order,
                                        gfx::OverlayTransform transform,
@@ -99,6 +134,12 @@ bool GLImageImpl::ScheduleOverlayPlane(gfx::AcceleratedWidget widget,
   return false;
 }
 
+void GLImageImpl::OnMemoryDump(base::trace_event::ProcessMemoryDump* pmd,
+                               uint64_t process_tracing_id,
+                               const std::string& dump_name) {
+  // TODO(ericrk): Implement GLImage OnMemoryDump. crbug.com/514914
+}
+
 }  // anonymous namespace
 
 StreamTextureManagerInProcess::StreamTextureManagerInProcess()
@@ -112,7 +153,7 @@ StreamTextureManagerInProcess::~StreamTextureManagerInProcess() {
 }
 
 GLuint StreamTextureManagerInProcess::CreateStreamTexture(
-    uint32 client_texture_id,
+    uint32_t client_texture_id,
     gles2::TextureManager* texture_manager) {
   CalledOnValidThread();
 
@@ -126,20 +167,21 @@ GLuint StreamTextureManagerInProcess::CreateStreamTexture(
   scoped_refptr<gfx::SurfaceTexture> surface_texture(
       gfx::SurfaceTexture::Create(texture->service_id()));
 
-  uint32 stream_id = next_id_++;
+  uint32_t stream_id = next_id_++;
   base::Closure release_callback =
       base::Bind(&StreamTextureManagerInProcess::OnReleaseStreamTexture,
                  weak_factory_.GetWeakPtr(), stream_id);
-  scoped_refptr<gfx::GLImage> gl_image(new GLImageImpl(surface_texture,
-                                       release_callback));
+  scoped_refptr<gl::GLImage> gl_image(
+      new GLImageImpl(texture->service_id(), texture_manager, surface_texture,
+                      release_callback));
 
   gfx::Size size = gl_image->GetSize();
   texture_manager->SetTarget(texture, GL_TEXTURE_EXTERNAL_OES);
   texture_manager->SetLevelInfo(texture, GL_TEXTURE_EXTERNAL_OES, 0, GL_RGBA,
                                 size.width(), size.height(), 1, 0, GL_RGBA,
                                 GL_UNSIGNED_BYTE, gfx::Rect(size));
-  texture_manager->SetLevelImage(
-      texture, GL_TEXTURE_EXTERNAL_OES, 0, gl_image.get());
+  texture_manager->SetLevelImage(texture, GL_TEXTURE_EXTERNAL_OES, 0,
+                                 gl_image.get(), gles2::Texture::UNBOUND);
 
   {
     base::AutoLock lock(map_lock_);
@@ -152,7 +194,7 @@ GLuint StreamTextureManagerInProcess::CreateStreamTexture(
   return stream_id;
 }
 
-void StreamTextureManagerInProcess::OnReleaseStreamTexture(uint32 stream_id) {
+void StreamTextureManagerInProcess::OnReleaseStreamTexture(uint32_t stream_id) {
   CalledOnValidThread();
   base::AutoLock lock(map_lock_);
   textures_.erase(stream_id);
@@ -160,7 +202,7 @@ void StreamTextureManagerInProcess::OnReleaseStreamTexture(uint32 stream_id) {
 
 // This can get called from any thread.
 scoped_refptr<gfx::SurfaceTexture>
-StreamTextureManagerInProcess::GetSurfaceTexture(uint32 stream_id) {
+StreamTextureManagerInProcess::GetSurfaceTexture(uint32_t stream_id) {
   base::AutoLock lock(map_lock_);
   TextureMap::const_iterator it = textures_.find(stream_id);
   if (it != textures_.end())
