@@ -4,6 +4,10 @@
 
 #include "ipc/ipc_channel_proxy.h"
 
+#include <stddef.h>
+#include <stdint.h>
+#include <utility>
+
 #include "base/bind.h"
 #include "base/compiler_specific.h"
 #include "base/location.h"
@@ -12,6 +16,7 @@
 #include "base/profiler/scoped_tracker.h"
 #include "base/single_thread_task_runner.h"
 #include "base/thread_task_runner_handle.h"
+#include "build/build_config.h"
 #include "ipc/ipc_channel_factory.h"
 #include "ipc/ipc_listener.h"
 #include "ipc/ipc_logging.h"
@@ -32,7 +37,8 @@ ChannelProxy::Context::Context(
       channel_connected_called_(false),
       channel_send_thread_safe_(false),
       message_filter_router_(new MessageFilterRouter()),
-      peer_pid_(base::kNullProcessId) {
+      peer_pid_(base::kNullProcessId),
+      attachment_broker_endpoint_(false) {
   DCHECK(ipc_task_runner_.get());
   // The Listener thread where Messages are handled must be a separate thread
   // to avoid oversubscribing the IO thread. If you trigger this error, you
@@ -52,20 +58,13 @@ void ChannelProxy::Context::ClearIPCTaskRunner() {
   ipc_task_runner_ = NULL;
 }
 
-void ChannelProxy::Context::SetListenerTaskRunner(
-    scoped_refptr<base::SingleThreadTaskRunner> task_runner) {
-  DCHECK(ipc_task_runner_.get() != task_runner.get());
-  DCHECK(listener_task_runner_->BelongsToCurrentThread());
-  DCHECK(task_runner->BelongsToCurrentThread());
-  listener_task_runner_ = task_runner;
-}
-
 void ChannelProxy::Context::CreateChannel(scoped_ptr<ChannelFactory> factory) {
   base::AutoLock l(channel_lifetime_lock_);
   DCHECK(!channel_);
   channel_id_ = factory->GetName();
   channel_ = factory->BuildChannel(this);
   channel_send_thread_safe_ = channel_->IsSendThreadSafe();
+  channel_->SetAttachmentBrokerEndpoint(attachment_broker_endpoint_);
 }
 
 bool ChannelProxy::Context::TryFilters(const Message& message) {
@@ -106,7 +105,7 @@ bool ChannelProxy::Context::OnMessageReceivedNoFilter(const Message& message) {
 }
 
 // Called on the IPC::Channel thread
-void ChannelProxy::Context::OnChannelConnected(int32 peer_pid) {
+void ChannelProxy::Context::OnChannelConnected(int32_t peer_pid) {
   // We cache off the peer_pid so it can be safely accessed from both threads.
   peer_pid_ = channel_->GetPeerPID();
 
@@ -347,6 +346,10 @@ void ChannelProxy::Context::Send(Message* message) {
                             base::Passed(scoped_ptr<Message>(message))));
 }
 
+bool ChannelProxy::Context::IsChannelSendThreadSafe() const {
+  return channel_send_thread_safe_;
+}
+
 //-----------------------------------------------------------------------------
 
 // static
@@ -354,11 +357,10 @@ scoped_ptr<ChannelProxy> ChannelProxy::Create(
     const IPC::ChannelHandle& channel_handle,
     Channel::Mode mode,
     Listener* listener,
-    const scoped_refptr<base::SingleThreadTaskRunner>& ipc_task_runner,
-    AttachmentBroker* broker) {
+    const scoped_refptr<base::SingleThreadTaskRunner>& ipc_task_runner) {
   scoped_ptr<ChannelProxy> channel(new ChannelProxy(listener, ipc_task_runner));
-  channel->Init(channel_handle, mode, true, broker);
-  return channel.Pass();
+  channel->Init(channel_handle, mode, true);
+  return channel;
 }
 
 // static
@@ -367,8 +369,8 @@ scoped_ptr<ChannelProxy> ChannelProxy::Create(
     Listener* listener,
     const scoped_refptr<base::SingleThreadTaskRunner>& ipc_task_runner) {
   scoped_ptr<ChannelProxy> channel(new ChannelProxy(listener, ipc_task_runner));
-  channel->Init(factory.Pass(), true);
-  return channel.Pass();
+  channel->Init(std::move(factory), true);
+  return channel;
 }
 
 ChannelProxy::ChannelProxy(Context* context)
@@ -396,8 +398,7 @@ ChannelProxy::~ChannelProxy() {
 
 void ChannelProxy::Init(const IPC::ChannelHandle& channel_handle,
                         Channel::Mode mode,
-                        bool create_pipe_now,
-                        AttachmentBroker* broker) {
+                        bool create_pipe_now) {
 #if defined(OS_POSIX)
   // When we are creating a server on POSIX, we need its file descriptor
   // to be created immediately so that it can be accessed and passed
@@ -407,7 +408,7 @@ void ChannelProxy::Init(const IPC::ChannelHandle& channel_handle,
     create_pipe_now = true;
   }
 #endif  // defined(OS_POSIX)
-  Init(ChannelFactory::Create(channel_handle, mode, broker), create_pipe_now);
+  Init(ChannelFactory::Create(channel_handle, mode), create_pipe_now);
 }
 
 void ChannelProxy::Init(scoped_ptr<ChannelFactory> factory,
@@ -420,11 +421,11 @@ void ChannelProxy::Init(scoped_ptr<ChannelFactory> factory,
     // low-level pipe so that the client can connect.  Without creating
     // the pipe immediately, it is possible for a listener to attempt
     // to connect and get an error since the pipe doesn't exist yet.
-    context_->CreateChannel(factory.Pass());
+    context_->CreateChannel(std::move(factory));
   } else {
     context_->ipc_task_runner()->PostTask(
-        FROM_HERE, base::Bind(&Context::CreateChannel,
-                              context_.get(), Passed(factory.Pass())));
+        FROM_HERE, base::Bind(&Context::CreateChannel, context_.get(),
+                              base::Passed(&factory)));
   }
 
   // complete initialization on the background thread
@@ -432,6 +433,7 @@ void ChannelProxy::Init(scoped_ptr<ChannelFactory> factory,
       FROM_HERE, base::Bind(&Context::OnChannelOpened, context_.get()));
 
   did_init_ = true;
+  OnChannelInit();
 }
 
 void ChannelProxy::Close() {
@@ -484,19 +486,20 @@ void ChannelProxy::RemoveFilter(MessageFilter* filter) {
                             make_scoped_refptr(filter)));
 }
 
-void ChannelProxy::SetListenerTaskRunner(
-    scoped_refptr<base::SingleThreadTaskRunner> task_runner) {
-  DCHECK(CalledOnValidThread());
-
-  context()->SetListenerTaskRunner(task_runner);
-}
-
 void ChannelProxy::ClearIPCTaskRunner() {
   DCHECK(CalledOnValidThread());
 
   context()->ClearIPCTaskRunner();
 }
 
+base::ProcessId ChannelProxy::GetPeerPID() const {
+  return context_->peer_pid_;
+}
+
+void ChannelProxy::OnSetAttachmentBrokerEndpoint() {
+  context()->set_attachment_broker_endpoint(is_attachment_broker_endpoint());
+}
+
 #if defined(OS_POSIX) && !defined(OS_NACL_SFI)
 // See the TODO regarding lazy initialization of the channel in
 // ChannelProxy::Init().
@@ -519,6 +522,9 @@ base::ScopedFD ChannelProxy::TakeClientFileDescriptor() {
 }
 #endif
 
+void ChannelProxy::OnChannelInit() {
+}
+
 //-----------------------------------------------------------------------------
 
 }  // namespace IPC
