@@ -5,6 +5,8 @@
 #include "ipc/ipc_channel_win.h"
 
 #include <windows.h>
+#include <stddef.h>
+#include <stdint.h>
 
 #include "base/auto_reset.h"
 #include "base/bind.h"
@@ -17,6 +19,7 @@
 #include "base/strings/utf_string_conversions.h"
 #include "base/threading/thread_checker.h"
 #include "base/win/scoped_handle.h"
+#include "ipc/attachment_broker.h"
 #include "ipc/ipc_listener.h"
 #include "ipc/ipc_logging.h"
 #include "ipc/ipc_message_attachment_set.h"
@@ -37,8 +40,7 @@ ChannelWin::State::~State() {
 
 ChannelWin::ChannelWin(const IPC::ChannelHandle& channel_handle,
                        Mode mode,
-                       Listener* listener,
-                       AttachmentBroker* broker)
+                       Listener* listener)
     : ChannelReader(listener),
       input_state_(this),
       output_state_(this),
@@ -47,12 +49,12 @@ ChannelWin::ChannelWin(const IPC::ChannelHandle& channel_handle,
       processing_incoming_(false),
       validate_client_(false),
       client_secret_(0),
-      broker_(broker),
       weak_factory_(this) {
   CreatePipe(channel_handle, mode);
 }
 
 ChannelWin::~ChannelWin() {
+  CleanUp();
   Close();
 }
 
@@ -74,30 +76,45 @@ void ChannelWin::Close() {
   }
 
   while (!output_queue_.empty()) {
-    Message* m = output_queue_.front();
+    OutputElement* element = output_queue_.front();
     output_queue_.pop();
-    delete m;
+    delete element;
   }
 }
 
 bool ChannelWin::Send(Message* message) {
-  // TODO(erikchen): Remove this DCHECK once ChannelWin fully supports
-  // brokerable attachments. http://crbug.com/493414.
-  DCHECK(!message->HasAttachments());
   DCHECK(thread_check_->CalledOnValidThread());
   DVLOG(2) << "sending message @" << message << " on channel @" << this
            << " with type " << message->type()
            << " (" << output_queue_.size() << " in queue)";
 
+  if (!prelim_queue_.empty()) {
+    prelim_queue_.push(message);
+    return true;
+  }
+
+  if (message->HasBrokerableAttachments() &&
+      peer_pid_ == base::kNullProcessId) {
+    prelim_queue_.push(message);
+    return true;
+  }
+
+  return ProcessMessageForDelivery(message);
+}
+
+bool ChannelWin::ProcessMessageForDelivery(Message* message) {
   // Sending a brokerable attachment requires a call to Channel::Send(), so
-  // Send() may be re-entrant. Brokered attachments must be sent before the
-  // Message itself.
+  // both Send() and ProcessMessageForDelivery() may be re-entrant.
   if (message->HasBrokerableAttachments()) {
-    DCHECK(broker_);
-    for (const BrokerableAttachment* attachment :
-         message->attachment_set()->PeekBrokerableAttachments()) {
-      if (!broker_->SendAttachmentToProcess(attachment, peer_pid_))
+    DCHECK(GetAttachmentBroker());
+    DCHECK(peer_pid_ != base::kNullProcessId);
+    for (const scoped_refptr<IPC::BrokerableAttachment>& attachment :
+         message->attachment_set()->GetBrokerableAttachments()) {
+      if (!GetAttachmentBroker()->SendAttachmentToProcess(attachment,
+                                                          peer_pid_)) {
+        delete message;
         return false;
+      }
     }
   }
 
@@ -105,8 +122,24 @@ bool ChannelWin::Send(Message* message) {
   Logging::GetInstance()->OnSendMessage(message, "");
 #endif
 
-  message->TraceMessageBegin();
-  output_queue_.push(message);
+  TRACE_EVENT_WITH_FLOW0(TRACE_DISABLED_BY_DEFAULT("ipc.flow"),
+                         "ChannelWin::ProcessMessageForDelivery",
+                         message->flags(),
+                         TRACE_EVENT_FLAG_FLOW_OUT);
+
+  // |output_queue_| takes ownership of |message|.
+  OutputElement* element = new OutputElement(message);
+  output_queue_.push(element);
+
+#if USE_ATTACHMENT_BROKER
+  if (message->HasBrokerableAttachments()) {
+    // |output_queue_| takes ownership of |ids.buffer|.
+    Message::SerializedAttachmentIds ids =
+        message->SerializedIdsOfBrokerableAttachments();
+    output_queue_.push(new OutputElement(ids.buffer, ids.size));
+  }
+#endif
+
   // ensure waiting to write
   if (!waiting_connect_) {
     if (!output_state_.is_pending) {
@@ -118,8 +151,33 @@ bool ChannelWin::Send(Message* message) {
   return true;
 }
 
+void ChannelWin::FlushPrelimQueue() {
+  DCHECK_NE(peer_pid_, base::kNullProcessId);
+
+  // Due to the possibly re-entrant nature of ProcessMessageForDelivery(), it
+  // is critical that |prelim_queue_| appears empty.
+  std::queue<Message*> prelim_queue;
+  prelim_queue_.swap(prelim_queue);
+
+  while (!prelim_queue.empty()) {
+    Message* m = prelim_queue.front();
+    bool success = ProcessMessageForDelivery(m);
+    prelim_queue.pop();
+
+    if (!success)
+      break;
+  }
+
+  // Delete any unprocessed messages.
+  while (!prelim_queue.empty()) {
+    Message* m = prelim_queue.front();
+    delete m;
+    prelim_queue.pop();
+  }
+}
+
 AttachmentBroker* ChannelWin::GetAttachmentBroker() {
-  return broker_;
+  return AttachmentBroker::GetGlobal();
 }
 
 base::ProcessId ChannelWin::GetPeerPID() const {
@@ -172,22 +230,26 @@ ChannelWin::ReadState ChannelWin::ReadData(
   return READ_PENDING;
 }
 
-bool ChannelWin::WillDispatchInputMessage(Message* msg) {
+bool ChannelWin::ShouldDispatchInputMessage(Message* msg) {
   // Make sure we get a hello when client validation is required.
   if (validate_client_)
     return IsHelloMessage(*msg);
   return true;
 }
 
+bool ChannelWin::GetNonBrokeredAttachments(Message* msg) {
+  return true;
+}
+
 void ChannelWin::HandleInternalMessage(const Message& msg) {
   DCHECK_EQ(msg.type(), static_cast<unsigned>(Channel::HELLO_MESSAGE_TYPE));
   // The hello message contains one parameter containing the PID.
   base::PickleIterator it(msg);
-  int32 claimed_pid;
+  int32_t claimed_pid;
   bool failed = !it.ReadInt(&claimed_pid);
 
   if (!failed && validate_client_) {
-    int32 secret;
+    int32_t secret;
     failed = it.ReadInt(&secret) ? (secret != client_secret_) : true;
   }
 
@@ -201,7 +263,18 @@ void ChannelWin::HandleInternalMessage(const Message& msg) {
   peer_pid_ = claimed_pid;
   // Validation completed.
   validate_client_ = false;
+
   listener()->OnChannelConnected(claimed_pid);
+
+  FlushPrelimQueue();
+}
+
+base::ProcessId ChannelWin::GetSenderPID() {
+  return GetPeerPID();
+}
+
+bool ChannelWin::IsAttachmentBrokerEndpoint() {
+  return is_attachment_broker_endpoint();
 }
 
 bool ChannelWin::DidEmptyInputBuffers() {
@@ -210,8 +283,8 @@ bool ChannelWin::DidEmptyInputBuffers() {
 }
 
 // static
-const base::string16 ChannelWin::PipeName(
-    const std::string& channel_id, int32* secret) {
+const base::string16 ChannelWin::PipeName(const std::string& channel_id,
+                                          int32_t* secret) {
   std::string name("\\\\.\\pipe\\chrome.");
 
   // Prevent the shared secret from ending up in the pipe name.
@@ -306,14 +379,15 @@ bool ChannelWin::CreatePipe(const IPC::ChannelHandle &channel_handle,
 
   // Don't send the secret to the untrusted process, and don't send a secret
   // if the value is zero (for IPC backwards compatability).
-  int32 secret = validate_client_ ? 0 : client_secret_;
+  int32_t secret = validate_client_ ? 0 : client_secret_;
   if (!m->WriteInt(GetCurrentProcessId()) ||
       (secret && !m->WriteUInt32(secret))) {
     pipe_.Close();
     return false;
   }
 
-  output_queue_.push(m.release());
+  OutputElement* element = new OutputElement(m.release());
+  output_queue_.push(element);
   return true;
 }
 
@@ -403,9 +477,9 @@ bool ChannelWin::ProcessOutgoingMessages(
     }
     // Message was sent.
     CHECK(!output_queue_.empty());
-    Message* m = output_queue_.front();
+    OutputElement* element = output_queue_.front();
     output_queue_.pop();
-    delete m;
+    delete element;
   }
 
   if (output_queue_.empty())
@@ -415,11 +489,11 @@ bool ChannelWin::ProcessOutgoingMessages(
     return false;
 
   // Write to pipe...
-  Message* m = output_queue_.front();
-  DCHECK(m->size() <= INT_MAX);
+  OutputElement* element = output_queue_.front();
+  DCHECK(element->size() <= INT_MAX);
   BOOL ok = WriteFile(pipe_.Get(),
-                      m->data(),
-                      static_cast<uint32>(m->size()),
+                      element->data(),
+                      static_cast<uint32_t>(element->size()),
                       NULL,
                       &output_state_.context.overlapped);
   if (!ok) {
@@ -427,8 +501,11 @@ bool ChannelWin::ProcessOutgoingMessages(
     if (write_error == ERROR_IO_PENDING) {
       output_state_.is_pending = true;
 
-      DVLOG(2) << "sent pending message @" << m << " on channel @" << this
-               << " with type " << m->type();
+      const Message* m = element->get_message();
+      if (m) {
+        DVLOG(2) << "sent pending message @" << m << " on channel @" << this
+                 << " with type " << m->type();
+      }
 
       return true;
     }
@@ -436,8 +513,11 @@ bool ChannelWin::ProcessOutgoingMessages(
     return false;
   }
 
-  DVLOG(2) << "sent message @" << m << " on channel @" << this
-           << " with type " << m->type();
+  const Message* m = element->get_message();
+  if (m) {
+    DVLOG(2) << "sent message @" << m << " on channel @" << this
+             << " with type " << m->type();
+  }
 
   output_state_.is_pending = true;
   return true;
@@ -470,17 +550,18 @@ void ChannelWin::OnIOCompleted(
     if (input_state_.is_pending) {
       // This is the normal case for everything except the initialization step.
       input_state_.is_pending = false;
-      if (!bytes_transfered)
+      if (!bytes_transfered) {
         ok = false;
-      else if (pipe_.IsValid())
-        ok = AsyncReadComplete(bytes_transfered);
+      } else if (pipe_.IsValid()) {
+        ok = (AsyncReadComplete(bytes_transfered) != DISPATCH_ERROR);
+      }
     } else {
       DCHECK(!bytes_transfered);
     }
 
     // Request more data.
     if (ok)
-      ok = ProcessIncomingMessages();
+      ok = (ProcessIncomingMessages() != DISPATCH_ERROR);
   } else {
     DCHECK(context == &output_state_.context);
     CHECK(output_state_.is_pending);
@@ -499,10 +580,8 @@ void ChannelWin::OnIOCompleted(
 // static
 scoped_ptr<Channel> Channel::Create(const IPC::ChannelHandle& channel_handle,
                                     Mode mode,
-                                    Listener* listener,
-                                    AttachmentBroker* broker) {
-  return scoped_ptr<Channel>(
-      new ChannelWin(channel_handle, mode, listener, broker));
+                                    Listener* listener) {
+  return scoped_ptr<Channel>(new ChannelWin(channel_handle, mode, listener));
 }
 
 // static
