@@ -4,10 +4,18 @@
 
 #include "ipc/mojo/ipc_channel_mojo.h"
 
+#include <stddef.h>
+#include <stdint.h>
+#include <memory>
+#include <utility>
+
 #include "base/bind.h"
 #include "base/bind_helpers.h"
+#include "base/command_line.h"
 #include "base/lazy_instance.h"
+#include "base/macros.h"
 #include "base/thread_task_runner_handle.h"
+#include "build/build_config.h"
 #include "ipc/ipc_listener.h"
 #include "ipc/ipc_logging.h"
 #include "ipc/ipc_message_attachment_set.h"
@@ -15,6 +23,7 @@
 #include "ipc/mojo/client_channel.mojom.h"
 #include "ipc/mojo/ipc_mojo_bootstrap.h"
 #include "ipc/mojo/ipc_mojo_handle_attachment.h"
+#include "mojo/public/cpp/bindings/binding.h"
 #include "third_party/mojo/src/mojo/edk/embedder/embedder.h"
 
 #if defined(OS_POSIX) && !defined(OS_NACL)
@@ -25,31 +34,29 @@ namespace IPC {
 
 namespace {
 
+// TODO(jam): do more tests on using channel on same thread if it supports it (
+// i.e. with use-new-edk and Windows). Also see message_pipe_dispatcher.cc
+bool g_use_channel_on_io_thread_only = true;
+
 class MojoChannelFactory : public ChannelFactory {
  public:
   MojoChannelFactory(scoped_refptr<base::TaskRunner> io_runner,
                      ChannelHandle channel_handle,
-                     Channel::Mode mode,
-                     AttachmentBroker* broker)
-      : io_runner_(io_runner),
-        channel_handle_(channel_handle),
-        mode_(mode),
-        broker_(broker) {}
+                     Channel::Mode mode)
+      : io_runner_(io_runner), channel_handle_(channel_handle), mode_(mode) {}
 
   std::string GetName() const override {
     return channel_handle_.name;
   }
 
   scoped_ptr<Channel> BuildChannel(Listener* listener) override {
-    return ChannelMojo::Create(io_runner_, channel_handle_, mode_, listener,
-                               broker_);
+    return ChannelMojo::Create(io_runner_, channel_handle_, mode_, listener);
   }
 
  private:
   scoped_refptr<base::TaskRunner> io_runner_;
   ChannelHandle channel_handle_;
   Channel::Mode mode_;
-  AttachmentBroker* broker_;
 };
 
 //------------------------------------------------------------------------------
@@ -58,21 +65,46 @@ class ClientChannelMojo : public ChannelMojo, public ClientChannel {
  public:
   ClientChannelMojo(scoped_refptr<base::TaskRunner> io_runner,
                     const ChannelHandle& handle,
-                    Listener* listener,
-                    AttachmentBroker* broker);
-  ~ClientChannelMojo() override;
+                    Listener* listener)
+      : ChannelMojo(io_runner, handle, Channel::MODE_CLIENT, listener),
+        binding_(this),
+        weak_factory_(this) {
+  }
+  ~ClientChannelMojo() override {}
+
   // MojoBootstrap::Delegate implementation
-  void OnPipeAvailable(mojo::embedder::ScopedPlatformHandle handle) override;
+  void OnPipeAvailable(mojo::embedder::ScopedPlatformHandle handle,
+                       int32_t peer_pid) override {
+    if (base::CommandLine::ForCurrentProcess()->HasSwitch("use-new-edk")) {
+      InitMessageReader(
+          mojo::embedder::CreateChannel(
+              std::move(handle),
+              base::Callback<void(mojo::embedder::ChannelInfo*)>(),
+              scoped_refptr<base::TaskRunner>()),
+          peer_pid);
+      return;
+    }
+    CreateMessagingPipe(
+        std::move(handle),
+        base::Bind(&ClientChannelMojo::BindPipe, weak_factory_.GetWeakPtr()));
+  }
 
   // ClientChannel implementation
   void Init(
       mojo::ScopedMessagePipeHandle pipe,
       int32_t peer_pid,
-      const mojo::Callback<void(int32_t)>& callback) override;
+      const mojo::Callback<void(int32_t)>& callback) override {
+    InitMessageReader(std::move(pipe), static_cast<base::ProcessId>(peer_pid));
+   callback.Run(GetSelfPID());
+  }
 
  private:
-  void BindPipe(mojo::ScopedMessagePipeHandle handle);
-  void OnConnectionError();
+  void BindPipe(mojo::ScopedMessagePipeHandle handle) {
+    binding_.Bind(std::move(handle));
+  }
+  void OnConnectionError() {
+    listener()->OnChannelError();
+  }
 
   mojo::Binding<ClientChannel> binding_;
   base::WeakPtrFactory<ClientChannelMojo> weak_factory_;
@@ -80,62 +112,78 @@ class ClientChannelMojo : public ChannelMojo, public ClientChannel {
   DISALLOW_COPY_AND_ASSIGN(ClientChannelMojo);
 };
 
-ClientChannelMojo::ClientChannelMojo(scoped_refptr<base::TaskRunner> io_runner,
-                                     const ChannelHandle& handle,
-                                     Listener* listener,
-                                     AttachmentBroker* broker)
-    : ChannelMojo(io_runner, handle, Channel::MODE_CLIENT, listener, broker),
-      binding_(this),
-      weak_factory_(this) {
-}
-
-ClientChannelMojo::~ClientChannelMojo() {
-}
-
-void ClientChannelMojo::OnPipeAvailable(
-    mojo::embedder::ScopedPlatformHandle handle) {
-  CreateMessagingPipe(handle.Pass(), base::Bind(&ClientChannelMojo::BindPipe,
-                                                weak_factory_.GetWeakPtr()));
-}
-
-void ClientChannelMojo::Init(
-    mojo::ScopedMessagePipeHandle pipe,
-    int32_t peer_pid,
-    const mojo::Callback<void(int32_t)>& callback) {
-  InitMessageReader(pipe.Pass(), static_cast<base::ProcessId>(peer_pid));
-  callback.Run(GetSelfPID());
-}
-
-void ClientChannelMojo::BindPipe(mojo::ScopedMessagePipeHandle handle) {
-  binding_.Bind(handle.Pass());
-}
-
-void ClientChannelMojo::OnConnectionError() {
-  listener()->OnChannelError();
-}
-
 //------------------------------------------------------------------------------
 
 class ServerChannelMojo : public ChannelMojo {
  public:
   ServerChannelMojo(scoped_refptr<base::TaskRunner> io_runner,
                     const ChannelHandle& handle,
-                    Listener* listener,
-                    AttachmentBroker* broker);
-  ~ServerChannelMojo() override;
+                    Listener* listener)
+      : ChannelMojo(io_runner, handle, Channel::MODE_SERVER, listener),
+        weak_factory_(this) {
+  }
+  ~ServerChannelMojo() override {
+    Close();
+  }
 
   // MojoBootstrap::Delegate implementation
-  void OnPipeAvailable(mojo::embedder::ScopedPlatformHandle handle) override;
+  void OnPipeAvailable(mojo::embedder::ScopedPlatformHandle handle,
+                       int32_t peer_pid) override {
+    if (base::CommandLine::ForCurrentProcess()->HasSwitch("use-new-edk")) {
+      message_pipe_ = mojo::embedder::CreateChannel(
+          std::move(handle),
+          base::Callback<void(mojo::embedder::ChannelInfo*)>(),
+          scoped_refptr<base::TaskRunner>());
+      if (!message_pipe_.is_valid()) {
+        LOG(WARNING) << "mojo::CreateMessagePipe failed: ";
+        listener()->OnChannelError();
+        return;
+      }
+      InitMessageReader(std::move(message_pipe_), peer_pid);
+      return;
+    }
+
+    mojo::ScopedMessagePipeHandle peer;
+    MojoResult create_result =
+        mojo::CreateMessagePipe(nullptr, &message_pipe_, &peer);
+    if (create_result != MOJO_RESULT_OK) {
+      LOG(WARNING) << "mojo::CreateMessagePipe failed: " << create_result;
+      listener()->OnChannelError();
+      return;
+    }
+    CreateMessagingPipe(
+        std::move(handle),
+        base::Bind(&ServerChannelMojo::InitClientChannel,
+                   weak_factory_.GetWeakPtr(), base::Passed(&peer)));
+  }
   // Channel override
-  void Close() override;
+  void Close() override {
+    client_channel_.reset();
+    message_pipe_.reset();
+    ChannelMojo::Close();
+  }
 
  private:
   void InitClientChannel(mojo::ScopedMessagePipeHandle peer_handle,
-                         mojo::ScopedMessagePipeHandle handle);
-  void OnConnectionError();
+                         mojo::ScopedMessagePipeHandle handle) {
+    client_channel_.Bind(
+        mojo::InterfacePtrInfo<ClientChannel>(std::move(handle), 0u));
+    client_channel_.set_connection_error_handler(base::Bind(
+        &ServerChannelMojo::OnConnectionError, base::Unretained(this)));
+    client_channel_->Init(
+        std::move(peer_handle), static_cast<int32_t>(GetSelfPID()),
+        base::Bind(&ServerChannelMojo::ClientChannelWasInitialized,
+                   base::Unretained(this)));
+  }
+
+  void OnConnectionError() {
+    listener()->OnChannelError();
+  }
 
   // ClientChannelClient implementation
-  void ClientChannelWasInitialized(int32_t peer_pid);
+  void ClientChannelWasInitialized(int32_t peer_pid) {
+    InitMessageReader(std::move(message_pipe_), peer_pid);
+  }
 
   mojo::InterfacePtr<ClientChannel> client_channel_;
   mojo::ScopedMessagePipeHandle message_pipe_;
@@ -144,61 +192,6 @@ class ServerChannelMojo : public ChannelMojo {
   DISALLOW_COPY_AND_ASSIGN(ServerChannelMojo);
 };
 
-ServerChannelMojo::ServerChannelMojo(scoped_refptr<base::TaskRunner> io_runner,
-                                     const ChannelHandle& handle,
-                                     Listener* listener,
-                                     AttachmentBroker* broker)
-    : ChannelMojo(io_runner, handle, Channel::MODE_SERVER, listener, broker),
-      weak_factory_(this) {
-}
-
-ServerChannelMojo::~ServerChannelMojo() {
-  Close();
-}
-
-void ServerChannelMojo::OnPipeAvailable(
-    mojo::embedder::ScopedPlatformHandle handle) {
-  mojo::ScopedMessagePipeHandle peer;
-  MojoResult create_result =
-      mojo::CreateMessagePipe(nullptr, &message_pipe_, &peer);
-  if (create_result != MOJO_RESULT_OK) {
-    LOG(WARNING) << "mojo::CreateMessagePipe failed: " << create_result;
-    listener()->OnChannelError();
-    return;
-  }
-  CreateMessagingPipe(
-      handle.Pass(),
-      base::Bind(&ServerChannelMojo::InitClientChannel,
-                 weak_factory_.GetWeakPtr(), base::Passed(&peer)));
-}
-
-void ServerChannelMojo::Close() {
-  client_channel_.reset();
-  message_pipe_.reset();
-  ChannelMojo::Close();
-}
-
-void ServerChannelMojo::InitClientChannel(
-    mojo::ScopedMessagePipeHandle peer_handle,
-    mojo::ScopedMessagePipeHandle handle) {
-  client_channel_.Bind(
-      mojo::InterfacePtrInfo<ClientChannel>(handle.Pass(), 0u));
-  client_channel_.set_connection_error_handler(base::Bind(
-      &ServerChannelMojo::OnConnectionError, base::Unretained(this)));
-  client_channel_->Init(
-      peer_handle.Pass(), static_cast<int32_t>(GetSelfPID()),
-      base::Bind(&ServerChannelMojo::ClientChannelWasInitialized,
-                 base::Unretained(this)));
-}
-
-void ServerChannelMojo::OnConnectionError() {
-  listener()->OnChannelError();
-}
-
-void ServerChannelMojo::ClientChannelWasInitialized(int32_t peer_pid) {
-  InitMessageReader(message_pipe_.Pass(), peer_pid);
-}
-
 #if defined(OS_POSIX) && !defined(OS_NACL)
 
 base::ScopedFD TakeOrDupFile(internal::PlatformFileAttachment* attachment) {
@@ -244,15 +237,14 @@ scoped_ptr<ChannelMojo> ChannelMojo::Create(
     scoped_refptr<base::TaskRunner> io_runner,
     const ChannelHandle& channel_handle,
     Mode mode,
-    Listener* listener,
-    AttachmentBroker* broker) {
+    Listener* listener) {
   switch (mode) {
     case Channel::MODE_CLIENT:
       return make_scoped_ptr(
-          new ClientChannelMojo(io_runner, channel_handle, listener, broker));
+          new ClientChannelMojo(io_runner, channel_handle, listener));
     case Channel::MODE_SERVER:
       return make_scoped_ptr(
-          new ServerChannelMojo(io_runner, channel_handle, listener, broker));
+          new ServerChannelMojo(io_runner, channel_handle, listener));
     default:
       NOTREACHED();
       return nullptr;
@@ -262,26 +254,23 @@ scoped_ptr<ChannelMojo> ChannelMojo::Create(
 // static
 scoped_ptr<ChannelFactory> ChannelMojo::CreateServerFactory(
     scoped_refptr<base::TaskRunner> io_runner,
-    const ChannelHandle& channel_handle,
-    AttachmentBroker* broker) {
-  return make_scoped_ptr(new MojoChannelFactory(io_runner, channel_handle,
-                                                Channel::MODE_SERVER, broker));
+    const ChannelHandle& channel_handle) {
+  return make_scoped_ptr(
+      new MojoChannelFactory(io_runner, channel_handle, Channel::MODE_SERVER));
 }
 
 // static
 scoped_ptr<ChannelFactory> ChannelMojo::CreateClientFactory(
     scoped_refptr<base::TaskRunner> io_runner,
-    const ChannelHandle& channel_handle,
-    AttachmentBroker* broker) {
-  return make_scoped_ptr(new MojoChannelFactory(io_runner, channel_handle,
-                                                Channel::MODE_CLIENT, broker));
+    const ChannelHandle& channel_handle) {
+  return make_scoped_ptr(
+      new MojoChannelFactory(io_runner, channel_handle, Channel::MODE_CLIENT));
 }
 
 ChannelMojo::ChannelMojo(scoped_refptr<base::TaskRunner> io_runner,
                          const ChannelHandle& handle,
                          Mode mode,
-                         Listener* listener,
-                         AttachmentBroker* broker)
+                         Listener* listener)
     : listener_(listener),
       peer_pid_(base::kNullProcessId),
       io_runner_(io_runner),
@@ -290,8 +279,9 @@ ChannelMojo::ChannelMojo(scoped_refptr<base::TaskRunner> io_runner,
       weak_factory_(this) {
   // Create MojoBootstrap after all members are set as it touches
   // ChannelMojo from a different thread.
-  bootstrap_ = MojoBootstrap::Create(handle, mode, this, broker);
-  if (io_runner == base::MessageLoop::current()->task_runner()) {
+  bootstrap_ = MojoBootstrap::Create(handle, mode, this);
+  if (!g_use_channel_on_io_thread_only ||
+      io_runner == base::MessageLoop::current()->task_runner()) {
     InitOnIOThread();
   } else {
     io_runner->PostTask(FROM_HERE, base::Bind(&ChannelMojo::InitOnIOThread,
@@ -313,9 +303,11 @@ void ChannelMojo::CreateMessagingPipe(
     const CreateMessagingPipeCallback& callback) {
   auto return_callback = base::Bind(&ChannelMojo::OnMessagingPipeCreated,
                                     weak_factory_.GetWeakPtr(), callback);
-  if (base::ThreadTaskRunnerHandle::Get() == io_runner_) {
-    CreateMessagingPipeOnIOThread(
-        handle.Pass(), base::ThreadTaskRunnerHandle::Get(), return_callback);
+  if (!g_use_channel_on_io_thread_only ||
+      base::ThreadTaskRunnerHandle::Get() == io_runner_) {
+    CreateMessagingPipeOnIOThread(std::move(handle),
+                                  base::ThreadTaskRunnerHandle::Get(),
+                                  return_callback);
   } else {
     io_runner_->PostTask(
         FROM_HERE,
@@ -332,9 +324,9 @@ void ChannelMojo::CreateMessagingPipeOnIOThread(
     const CreateMessagingPipeOnIOThreadCallback& callback) {
   mojo::embedder::ChannelInfo* channel_info;
   mojo::ScopedMessagePipeHandle pipe =
-      mojo::embedder::CreateChannelOnIOThread(handle.Pass(), &channel_info);
+      mojo::embedder::CreateChannelOnIOThread(std::move(handle), &channel_info);
   if (base::ThreadTaskRunnerHandle::Get() == callback_runner) {
-    callback.Run(pipe.Pass(), channel_info);
+    callback.Run(std::move(pipe), channel_info);
   } else {
     callback_runner->PostTask(
         FROM_HERE, base::Bind(callback, base::Passed(&pipe), channel_info));
@@ -348,7 +340,7 @@ void ChannelMojo::OnMessagingPipeCreated(
   DCHECK(!channel_info_.get());
   channel_info_ = scoped_ptr<mojo::embedder::ChannelInfo, ChannelInfoDeleter>(
       channel_info, ChannelInfoDeleter(io_runner_));
-  callback.Run(handle.Pass());
+  callback.Run(std::move(handle));
 }
 
 bool ChannelMojo::Connect() {
@@ -363,7 +355,7 @@ void ChannelMojo::Close() {
     // |message_reader_| has to be cleared inside the lock,
     // but the instance has to be deleted outside.
     base::AutoLock l(lock_);
-    to_be_deleted = message_reader_.Pass();
+    to_be_deleted = std::move(message_reader_);
     // We might Close() before we Connect().
     waiting_connect_ = false;
   }
@@ -382,7 +374,7 @@ namespace {
 // ClosingDeleter calls |CloseWithErrorIfPending| before deleting the
 // |MessagePipeReader|.
 struct ClosingDeleter {
-  typedef base::DefaultDeleter<internal::MessagePipeReader> DefaultType;
+  typedef std::default_delete<internal::MessagePipeReader> DefaultType;
 
   void operator()(internal::MessagePipeReader* ptr) const {
     ptr->CloseWithErrorIfPending();
@@ -395,7 +387,7 @@ struct ClosingDeleter {
 void ChannelMojo::InitMessageReader(mojo::ScopedMessagePipeHandle pipe,
                                     int32_t peer_pid) {
   scoped_ptr<internal::MessagePipeReader, ClosingDeleter> reader(
-      new internal::MessagePipeReader(pipe.Pass(), this));
+      new internal::MessagePipeReader(std::move(pipe), this));
 
   {
     base::AutoLock l(lock_);
@@ -447,7 +439,7 @@ bool ChannelMojo::Send(Message* message) {
 }
 
 bool ChannelMojo::IsSendThreadSafe() const {
-  return false;
+  return true;
 }
 
 base::ProcessId ChannelMojo::GetPeerPID() const {
@@ -486,8 +478,9 @@ MojoResult ChannelMojo::ReadFromMessageAttachmentSet(
   // of FDs, so just to dup()-and-own them is the safest option.
   if (message->HasAttachments()) {
     MessageAttachmentSet* set = message->attachment_set();
-    for (unsigned i = 0; i < set->size(); ++i) {
-      scoped_refptr<MessageAttachment> attachment = set->GetAttachmentAt(i);
+    for (unsigned i = 0; i < set->num_non_brokerable_attachments(); ++i) {
+      scoped_refptr<MessageAttachment> attachment =
+          set->GetNonBrokerableAttachmentAt(i);
       switch (attachment->GetType()) {
         case MessageAttachment::TYPE_PLATFORM_FILE:
 #if defined(OS_POSIX) && !defined(OS_NACL)
@@ -497,19 +490,19 @@ MojoResult ChannelMojo::ReadFromMessageAttachmentSet(
                   attachment.get()));
           if (!file.is_valid()) {
             DPLOG(WARNING) << "Failed to dup FD to transmit.";
-            set->CommitAll();
+            set->CommitAllDescriptors();
             return MOJO_RESULT_UNKNOWN;
           }
 
           MojoHandle wrapped_handle;
-          MojoResult wrap_result = CreatePlatformHandleWrapper(
+          MojoResult wrap_result = mojo::embedder::CreatePlatformHandleWrapper(
               mojo::embedder::ScopedPlatformHandle(
                   mojo::embedder::PlatformHandle(file.release())),
               &wrapped_handle);
           if (MOJO_RESULT_OK != wrap_result) {
             LOG(WARNING) << "Pipe failed to wrap handles. Closing: "
                          << wrap_result;
-            set->CommitAll();
+            set->CommitAllDescriptors();
             return wrap_result;
           }
 
@@ -533,7 +526,7 @@ MojoResult ChannelMojo::ReadFromMessageAttachmentSet(
       }
     }
 
-    set->CommitAll();
+    set->CommitAllDescriptors();
   }
 
   return MOJO_RESULT_OK;
