@@ -4,16 +4,20 @@
 
 #include "base/android/jni_string.h"
 #include "jni/MandolineActivity_jni.h"
-#include "mandoline/ui/browser/public/interfaces/launch_handler.mojom.h"
+#include "mandoline/ui/desktop_ui/public/interfaces/launch_handler.mojom.h"
 #include "mojo/runner/android/main.h"
 #include "mojo/runner/context.h"
+#include "mojo/shell/connect_util.h"
 
 namespace mandoline {
 
-static void LaunchURL(JNIEnv* env, jclass clazz, jstring jurl) {
+static void LaunchURL(JNIEnv* env,
+                      const JavaParamRef<jclass>& clazz,
+                      const JavaParamRef<jstring>& jurl) {
   LaunchHandlerPtr launch_handler;
-  mojo::runner::GetContext()->application_manager()->ConnectToService(
-      GURL("mojo:browser"), &launch_handler);
+  mojo::shell::ConnectToService(
+      mojo::runner::GetContext()->application_manager(), GURL("mojo:phone_ui"),
+      &launch_handler);
   launch_handler->LaunchURL(
       base::android::ConvertJavaStringToUTF8(env, jurl));
 }
