@@ -9,6 +9,7 @@
 #include <initguid.h>
 #include <mmsystem.h>
 #include <setupapi.h>
+#include <stddef.h>
 
 #include "base/bind.h"
 #include "base/bind_helpers.h"
@@ -16,6 +17,7 @@
 #include "base/files/file_path.h"
 #include "base/memory/scoped_ptr.h"
 #include "base/message_loop/message_loop.h"
+#include "base/metrics/histogram.h"
 #include "base/path_service.h"
 #include "base/process/launch.h"
 #include "base/strings/string_number_conversions.h"
@@ -111,8 +113,9 @@ static base::string16 GetDeviceAndDriverInfo(HDEVINFO device_info,
 static int NumberOfWaveOutBuffers() {
   // Use the user provided buffer count if provided.
   int buffers = 0;
-  std::string buffers_str(CommandLine::ForCurrentProcess()->GetSwitchValueASCII(
-      switches::kWaveOutBuffers));
+  std::string buffers_str(
+      base::CommandLine::ForCurrentProcess()->GetSwitchValueASCII(
+          switches::kWaveOutBuffers));
   if (base::StringToInt(buffers_str, &buffers) && buffers > 0) {
     return buffers;
   }
@@ -195,7 +198,7 @@ base::string16 AudioManagerWin::GetAudioInputDeviceModel() {
     return base::string16();  // No audio capture device.
 
   base::string16 device_interface_name;
-  base::string16::value_type* name_ptr = WriteInto(&device_interface_name,
+  base::string16::value_type* name_ptr = base::WriteInto(&device_interface_name,
       device_interface_name_size / bytes_in_char16);
   waveInMessage(reinterpret_cast<HWAVEIN>(device_id),
                 DRV_QUERYDEVICEINTERFACE,
@@ -257,9 +260,9 @@ void AudioManagerWin::ShowAudioInputSettings() {
   base::FilePath path;
   PathService::Get(base::DIR_SYSTEM, &path);
   path = path.Append(program);
-  CommandLine command_line(path);
+  base::CommandLine command_line(path);
   command_line.AppendArg(argument);
-  base::LaunchProcess(command_line, base::LaunchOptions(), NULL);
+  base::LaunchProcess(command_line, base::LaunchOptions());
 }
 
 void AudioManagerWin::GetAudioDeviceNamesImpl(
@@ -281,10 +284,15 @@ void AudioManagerWin::GetAudioDeviceNamesImpl(
       GetOutputDeviceNamesWin(device_names);
   }
 
-  // Always add default device parameters as first element.
   if (!device_names->empty()) {
     AudioDeviceName name;
-    name.device_name = AudioManagerBase::kDefaultDeviceName;
+    if (enumeration_type() == kMMDeviceEnumeration) {
+      name.device_name = AudioManager::GetCommunicationsDeviceName();
+      name.unique_id = AudioManagerBase::kCommunicationsDeviceId;
+      device_names->push_front(name);
+    }
+    // Always add default device parameters as first element.
+    name.device_name = AudioManager::GetDefaultDeviceName();
     name.unique_id = AudioManagerBase::kDefaultDeviceId;
     device_names->push_front(name);
   }
@@ -301,22 +309,23 @@ void AudioManagerWin::GetAudioOutputDeviceNames(
 
 AudioParameters AudioManagerWin::GetInputStreamParameters(
     const std::string& device_id) {
+  HRESULT hr = E_FAIL;
   AudioParameters parameters;
-  if (!core_audio_supported()) {
+  if (core_audio_supported()) {
+    hr = CoreAudioUtil::GetPreferredAudioParameters(device_id, false,
+                                                    &parameters);
+  }
+
+  if (FAILED(hr) || !parameters.IsValid()) {
     // Windows Wave implementation is being used.
-    parameters = AudioParameters(
-        AudioParameters::AUDIO_PCM_LINEAR, CHANNEL_LAYOUT_STEREO, 48000,
-        16, kFallbackBufferSize, AudioParameters::NO_EFFECTS);
-  } else  {
-    parameters = WASAPIAudioInputStream::GetInputStreamParameters(device_id);
+    parameters =
+        AudioParameters(AudioParameters::AUDIO_PCM_LINEAR,
+                        CHANNEL_LAYOUT_STEREO, 48000, 16, kFallbackBufferSize);
   }
 
   int user_buffer_size = GetUserBufferSize();
-  if (user_buffer_size) {
-    parameters.Reset(parameters.format(), parameters.channel_layout(),
-                     parameters.channels(), parameters.sample_rate(),
-                     parameters.bits_per_sample(), user_buffer_size);
-  }
+  if (user_buffer_size)
+    parameters.set_frames_per_buffer(user_buffer_size);
 
   return parameters;
 }
@@ -371,11 +380,12 @@ AudioOutputStream* AudioManagerWin::MakeLowLatencyOutputStream(
   // Pass an empty string to indicate that we want the default device
   // since we consistently only check for an empty string in
   // WASAPIAudioOutputStream.
+  bool communications = device_id == AudioManagerBase::kCommunicationsDeviceId;
   return new WASAPIAudioOutputStream(this,
-      device_id == AudioManagerBase::kDefaultDeviceId ?
+      communications || device_id == AudioManagerBase::kDefaultDeviceId ?
           std::string() : device_id,
       params,
-      params.effects() & AudioParameters::DUCKING ? eCommunications : eConsole);
+      communications ? eCommunications : eConsole);
 }
 
 // Factory for the implementations of AudioInputStream for AUDIO_PCM_LINEAR
@@ -393,6 +403,7 @@ AudioInputStream* AudioManagerWin::MakeLowLatencyInputStream(
   DCHECK_EQ(AudioParameters::AUDIO_PCM_LOW_LATENCY, params.format());
   DVLOG(1) << "MakeLowLatencyInputStream: " << device_id;
   AudioInputStream* stream = NULL;
+  UMA_HISTOGRAM_BOOLEAN("Media.WindowsCoreAudioInput", core_audio_supported());
   if (!core_audio_supported()) {
     // Fall back to Windows Wave implementation on Windows XP or lower.
     DVLOG(1) << "Using WaveIn since WASAPI requires at least Vista.";
@@ -416,7 +427,7 @@ AudioParameters AudioManagerWin::GetPreferredOutputStreamParameters(
   DLOG_IF(ERROR, !core_audio_supported() && !output_device_id.empty())
       << "CoreAudio is required to open non-default devices.";
 
-  const CommandLine* cmd_line = CommandLine::ForCurrentProcess();
+  const base::CommandLine* cmd_line = base::CommandLine::ForCurrentProcess();
   ChannelLayout channel_layout = CHANNEL_LAYOUT_STEREO;
   int sample_rate = 48000;
   int buffer_size = kFallbackBufferSize;
@@ -437,9 +448,9 @@ AudioParameters AudioManagerWin::GetPreferredOutputStreamParameters(
     } else {
       AudioParameters params;
       HRESULT hr = CoreAudioUtil::GetPreferredAudioParameters(
-          output_device_id.empty() ?
-              GetDefaultOutputDeviceID() : output_device_id,
-          &params);
+          output_device_id.empty() ? GetDefaultOutputDeviceID()
+                                   : output_device_id,
+          true, &params);
       if (SUCCEEDED(hr)) {
         bits_per_sample = params.bits_per_sample();
         buffer_size = params.frames_per_buffer();
@@ -480,8 +491,8 @@ AudioParameters AudioManagerWin::GetPreferredOutputStreamParameters(
           // Open up using the same channel layout as the source if it is
           // supported by the hardware.
           channel_layout = input_params.channel_layout();
-          VLOG(1) << "Hardware channel layout is not used; using same layout"
-                  << " as the source instead (" << channel_layout << ")";
+          DVLOG(1) << "Hardware channel layout is not used; using same layout"
+                   << " as the source instead (" << channel_layout << ")";
         }
       }
     }
@@ -504,9 +515,10 @@ AudioParameters AudioManagerWin::GetPreferredOutputStreamParameters(
   if (user_buffer_size)
     buffer_size = user_buffer_size;
 
-  return AudioParameters(
-      AudioParameters::AUDIO_PCM_LOW_LATENCY, channel_layout,
-      sample_rate, bits_per_sample, buffer_size, effects);
+  AudioParameters params(AudioParameters::AUDIO_PCM_LOW_LATENCY, channel_layout,
+                         sample_rate, bits_per_sample, buffer_size);
+  params.set_effects(effects);
+  return params;
 }
 
 AudioInputStream* AudioManagerWin::CreatePCMWaveInAudioInputStream(
