@@ -29,7 +29,7 @@ void CdmCallbackPromise<T...>::resolve(const T&... result) {
 
 template <typename... T>
 void CdmCallbackPromise<T...>::reject(MediaKeys::Exception exception_code,
-                                      uint32 system_code,
+                                      uint32_t system_code,
                                       const std::string& error_message) {
   MarkPromiseSettled();
   reject_cb_.Run(exception_code, system_code, error_message);
