@@ -7,6 +7,7 @@
 
 #include <string>
 
+#include "base/macros.h"
 #include "base/memory/scoped_ptr.h"
 #include "media/base/media_export.h"
 #include "media/base/media_keys.h"
@@ -17,7 +18,7 @@ namespace media {
 
 // Callback used when CDM is created. |error_message| only used if
 // MediaKeys is null (i.e. CDM can't be created).
-using CdmCreatedCB = base::Callback<void(scoped_ptr<MediaKeys>,
+using CdmCreatedCB = base::Callback<void(const scoped_refptr<MediaKeys>&,
                                          const std::string& error_message)>;
 
 struct CdmConfig;
