@@ -2,6 +2,8 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+#include <stdint.h>
+
 #if defined(_MSC_VER)
 #include <intrin.h>
 #else
@@ -13,11 +15,11 @@
 
 namespace media {
 
-void ConvertYUVAToARGB_MMX(const uint8* yplane,
-                           const uint8* uplane,
-                           const uint8* vplane,
-                           const uint8* aplane,
-                           uint8* rgbframe,
+void ConvertYUVAToARGB_MMX(const uint8_t* yplane,
+                           const uint8_t* uplane,
+                           const uint8_t* vplane,
+                           const uint8_t* aplane,
+                           uint8_t* rgbframe,
                            int width,
                            int height,
                            int ystride,
@@ -27,11 +29,11 @@ void ConvertYUVAToARGB_MMX(const uint8* yplane,
                            YUVType yuv_type) {
   unsigned int y_shift = GetVerticalShift(yuv_type);
   for (int y = 0; y < height; ++y) {
-    uint8* rgb_row = rgbframe + y * rgbstride;
-    const uint8* y_ptr = yplane + y * ystride;
-    const uint8* u_ptr = uplane + (y >> y_shift) * uvstride;
-    const uint8* v_ptr = vplane + (y >> y_shift) * uvstride;
-    const uint8* a_ptr = aplane + y * astride;
+    uint8_t* rgb_row = rgbframe + y * rgbstride;
+    const uint8_t* y_ptr = yplane + y * ystride;
+    const uint8_t* u_ptr = uplane + (y >> y_shift) * uvstride;
+    const uint8_t* v_ptr = vplane + (y >> y_shift) * uvstride;
+    const uint8_t* a_ptr = aplane + y * astride;
 
     ConvertYUVAToARGBRow_MMX(y_ptr,
                              u_ptr,
@@ -45,10 +47,10 @@ void ConvertYUVAToARGB_MMX(const uint8* yplane,
   EmptyRegisterState();
 }
 
-void ConvertYUVToRGB32_SSE(const uint8* yplane,
-                           const uint8* uplane,
-                           const uint8* vplane,
-                           uint8* rgbframe,
+void ConvertYUVToRGB32_SSE(const uint8_t* yplane,
+                           const uint8_t* uplane,
+                           const uint8_t* vplane,
+                           uint8_t* rgbframe,
                            int width,
                            int height,
                            int ystride,
@@ -57,10 +59,10 @@ void ConvertYUVToRGB32_SSE(const uint8* yplane,
                            YUVType yuv_type) {
   unsigned int y_shift = GetVerticalShift(yuv_type);
   for (int y = 0; y < height; ++y) {
-    uint8* rgb_row = rgbframe + y * rgbstride;
-    const uint8* y_ptr = yplane + y * ystride;
-    const uint8* u_ptr = uplane + (y >> y_shift) * uvstride;
-    const uint8* v_ptr = vplane + (y >> y_shift) * uvstride;
+    uint8_t* rgb_row = rgbframe + y * rgbstride;
+    const uint8_t* y_ptr = yplane + y * ystride;
+    const uint8_t* u_ptr = uplane + (y >> y_shift) * uvstride;
+    const uint8_t* v_ptr = vplane + (y >> y_shift) * uvstride;
 
     ConvertYUVToRGB32Row_SSE(y_ptr,
                              u_ptr,
