@@ -5,12 +5,17 @@
 #ifndef MEDIA_BASE_STREAM_PARSER_BUFFER_H_
 #define MEDIA_BASE_STREAM_PARSER_BUFFER_H_
 
+#include <stddef.h>
+#include <stdint.h>
+
 #include <deque>
 
+#include "base/macros.h"
 #include "media/base/decoder_buffer.h"
 #include "media/base/demuxer_stream.h"
 #include "media/base/media_export.h"
 #include "media/base/stream_parser.h"
+#include "media/base/timestamp_constants.h"
 
 namespace media {
 
@@ -59,19 +64,17 @@ class DecodeTimestamp {
     return DecodeTimestamp(ts_ - rhs);
   }
 
-  int64 operator/(const base::TimeDelta& rhs) const {
-    return ts_ / rhs;
-  }
+  int64_t operator/(const base::TimeDelta& rhs) const { return ts_ / rhs; }
 
   static DecodeTimestamp FromSecondsD(double seconds) {
     return DecodeTimestamp(base::TimeDelta::FromSecondsD(seconds));
   }
 
-  static DecodeTimestamp FromMilliseconds(int64 milliseconds) {
+  static DecodeTimestamp FromMilliseconds(int64_t milliseconds) {
     return DecodeTimestamp(base::TimeDelta::FromMilliseconds(milliseconds));
   }
 
-  static DecodeTimestamp FromMicroseconds(int64 microseconds) {
+  static DecodeTimestamp FromMicroseconds(int64_t microseconds) {
     return DecodeTimestamp(base::TimeDelta::FromMicroseconds(microseconds));
   }
 
@@ -82,8 +85,8 @@ class DecodeTimestamp {
   }
 
   double InSecondsF() const { return ts_.InSecondsF(); }
-  int64 InMilliseconds() const { return ts_.InMilliseconds(); }
-  int64 InMicroseconds() const { return ts_.InMicroseconds(); }
+  int64_t InMilliseconds() const { return ts_.InMilliseconds(); }
+  int64_t InMicroseconds() const { return ts_.InMicroseconds(); }
 
   // TODO(acolwell): Remove once all the hacks are gone. This method is called
   // by hacks where a decode time is being used as a presentation time.
@@ -109,13 +112,18 @@ class MEDIA_EXPORT StreamParserBuffer : public DecoderBuffer {
 
   static scoped_refptr<StreamParserBuffer> CreateEOSBuffer();
 
-  static scoped_refptr<StreamParserBuffer> CopyFrom(
-      const uint8* data, int data_size, bool is_key_frame, Type type,
-      TrackId track_id);
-  static scoped_refptr<StreamParserBuffer> CopyFrom(
-      const uint8* data, int data_size,
-      const uint8* side_data, int side_data_size, bool is_key_frame, Type type,
-      TrackId track_id);
+  static scoped_refptr<StreamParserBuffer> CopyFrom(const uint8_t* data,
+                                                    int data_size,
+                                                    bool is_key_frame,
+                                                    Type type,
+                                                    TrackId track_id);
+  static scoped_refptr<StreamParserBuffer> CopyFrom(const uint8_t* data,
+                                                    int data_size,
+                                                    const uint8_t* side_data,
+                                                    int side_data_size,
+                                                    bool is_key_frame,
+                                                    Type type,
+                                                    TrackId track_id);
 
   // Decode timestamp. If not explicitly set, or set to kNoTimestamp(), the
   // value will be taken from the normal timestamp.
@@ -134,6 +142,7 @@ class MEDIA_EXPORT StreamParserBuffer : public DecoderBuffer {
   // Gets the parser's media type associated with this buffer. Value is
   // meaningless for EOS buffers.
   Type type() const { return type_; }
+  const char* GetTypeName() const;
 
   // Gets the parser's track ID associated with this buffer. Value is
   // meaningless for EOS buffers.
@@ -177,9 +186,12 @@ class MEDIA_EXPORT StreamParserBuffer : public DecoderBuffer {
   }
 
  private:
-  StreamParserBuffer(const uint8* data, int data_size,
-                     const uint8* side_data, int side_data_size,
-                     bool is_key_frame, Type type,
+  StreamParserBuffer(const uint8_t* data,
+                     int data_size,
+                     const uint8_t* side_data,
+                     int side_data_size,
+                     bool is_key_frame,
+                     Type type,
                      TrackId track_id);
   ~StreamParserBuffer() override;
 
