@@ -5,6 +5,7 @@
 #ifndef MEDIA_BLINK_MOCK_WEBURLLOADER_H_
 #define MEDIA_BLINK_MOCK_WEBURLLOADER_H_
 
+#include "base/macros.h"
 #include "testing/gmock/include/gmock/gmock.h"
 #include "third_party/WebKit/public/platform/WebURLLoader.h"
 
@@ -23,6 +24,7 @@ class MockWebURLLoader : public blink::WebURLLoader {
                                         blink::WebURLLoaderClient* client));
   MOCK_METHOD0(cancel, void());
   MOCK_METHOD1(setDefersLoading, void(bool value));
+  MOCK_METHOD1(setLoadingTaskRunner, void(blink::WebTaskRunner* task_runner));
 
  private:
   DISALLOW_COPY_AND_ASSIGN(MockWebURLLoader);
