@@ -6,9 +6,12 @@
 #define MEDIA_CAST_COMMON_TRANSPORT_ENCRYPTION_HANDLER_H_
 
 // Helper class to handle encryption for the Cast Transport library.
+
+#include <stdint.h>
+
 #include <string>
 
-#include "base/basictypes.h"
+#include "base/macros.h"
 #include "base/memory/scoped_ptr.h"
 #include "base/strings/string_piece.h"
 #include "base/threading/non_thread_safe.h"
@@ -26,13 +29,13 @@ class TransportEncryptionHandler : public base::NonThreadSafe {
   TransportEncryptionHandler();
   ~TransportEncryptionHandler();
 
-  bool Initialize(std::string aes_key, std::string aes_iv_mask);
+  bool Initialize(const std::string& aes_key, const std::string& aes_iv_mask);
 
-  bool Encrypt(uint32 frame_id,
+  bool Encrypt(uint32_t frame_id,
                const base::StringPiece& data,
                std::string* encrypted_data);
 
-  bool Decrypt(uint32 frame_id,
+  bool Decrypt(uint32_t frame_id,
                const base::StringPiece& ciphertext,
                std::string* plaintext);
 
