@@ -5,7 +5,11 @@
 #ifndef MEDIA_CAST_LOGGING_STATS_EVENT_SUBSCRIBER_H_
 #define MEDIA_CAST_LOGGING_STATS_EVENT_SUBSCRIBER_H_
 
+#include <stddef.h>
+#include <stdint.h>
+
 #include "base/gtest_prod_util.h"
+#include "base/macros.h"
 #include "base/memory/linked_ptr.h"
 #include "base/memory/scoped_ptr.h"
 #include "base/threading/thread_checker.h"
@@ -90,20 +94,20 @@ class StatsEventSubscriber : public RawEventSubscriber {
     // Overflow bucket: >= max
     // |min| must be less than |max|.
     // |width| must divide |max - min| evenly.
-    SimpleHistogram(int64 min, int64 max, int64 width);
+    SimpleHistogram(int64_t min, int64_t max, int64_t width);
 
     ~SimpleHistogram();
 
-    void Add(int64 sample);
+    void Add(int64_t sample);
 
     void Reset();
 
     scoped_ptr<base::ListValue> GetHistogram() const;
 
    private:
-    int64 min_;
-    int64 max_;
-    int64 width_;
+    int64_t min_;
+    int64_t max_;
+    int64_t width_;
     std::vector<int> buckets_;
   };
 
@@ -182,10 +186,9 @@ class StatsEventSubscriber : public RawEventSubscriber {
 
   typedef std::map<CastStat, double> StatsMap;
   typedef std::map<CastStat, linked_ptr<SimpleHistogram> > HistogramMap;
-  typedef std::map<RtpTimestamp, FrameInfo> FrameInfoMap;
-  typedef std::map<
-      std::pair<RtpTimestamp, uint16>,
-      std::pair<base::TimeTicks, CastLoggingEvent> >
+  typedef std::map<RtpTimeTicks, FrameInfo> FrameInfoMap;
+  typedef std::map<std::pair<RtpTimeTicks, uint16_t>,
+                   std::pair<base::TimeTicks, CastLoggingEvent>>
       PacketEventTimeMap;
   typedef std::map<CastLoggingEvent, FrameLogStats> FrameStatsMap;
   typedef std::map<CastLoggingEvent, PacketLogStats> PacketStatsMap;
@@ -203,7 +206,7 @@ class StatsEventSubscriber : public RawEventSubscriber {
   void UpdateFirstLastEventTime(base::TimeTicks timestamp,
                                 bool is_receiver_event);
   bool GetReceiverOffset(base::TimeDelta* offset);
-  void MaybeInsertFrameInfo(RtpTimestamp rtp_timestamp,
+  void MaybeInsertFrameInfo(RtpTimeTicks rtp_timestamp,
                             const FrameInfo& frame_info);
   void RecordFrameCaptureTime(const FrameEvent& frame_event);
   void RecordCaptureLatency(const FrameEvent& frame_event);
