@@ -7,10 +7,11 @@
 #include <algorithm>
 #include <cstring>
 #include <sstream>
+#include <utility>
 
 #include "base/bind.h"
 #include "base/logging.h"
-#include "base/stl_util.h"
+#include "base/macros.h"
 #include "base/time/time.h"
 #include "base/trace_event/trace_event.h"
 #include "media/base/cdm_callback_promise.h"
@@ -24,8 +25,7 @@
 #include "url/gurl.h"
 
 #if defined(CLEAR_KEY_CDM_USE_FAKE_AUDIO_DECODER)
-#include "base/basictypes.h"
-const int64 kNoTimestamp = kint64min;
+const int64_t kNoTimestamp = INT64_MIN;
 #endif  // CLEAR_KEY_CDM_USE_FAKE_AUDIO_DECODER
 
 #if defined(CLEAR_KEY_CDM_USE_FFMPEG_DECODER)
@@ -70,15 +70,15 @@ const char kExternalClearKeyCrashKeySystem[] =
 // These constants need to be in sync with
 // chrome/test/data/media/encrypted_media_utils.js
 const char kLoadableSessionId[] = "LoadableSession";
-const uint8 kLoadableSessionKeyId[] = "0123456789012345";
-const uint8 kLoadableSessionKey[] =
-    {0xeb, 0xdd, 0x62, 0xf1, 0x68, 0x14, 0xd2, 0x7b,
-     0x68, 0xef, 0x12, 0x2a, 0xfc, 0xe4, 0xae, 0x3c};
+const uint8_t kLoadableSessionKeyId[] = "0123456789012345";
+const uint8_t kLoadableSessionKey[] = {0xeb, 0xdd, 0x62, 0xf1, 0x68, 0x14,
+                                       0xd2, 0x7b, 0x68, 0xef, 0x12, 0x2a,
+                                       0xfc, 0xe4, 0xae, 0x3c};
 
-const int64 kSecondsPerMinute = 60;
-const int64 kMsPerSecond = 1000;
-const int64 kInitialTimerDelayMs = 200;
-const int64 kMaxTimerDelayMs = 1 * kSecondsPerMinute * kMsPerSecond;
+const int64_t kSecondsPerMinute = 60;
+const int64_t kMsPerSecond = 1000;
+const int64_t kInitialTimerDelayMs = 200;
+const int64_t kMaxTimerDelayMs = 1 * kSecondsPerMinute * kMsPerSecond;
 // Renewal message header. For prefixed EME, if a key message starts with
 // |kRenewalHeader|, it's a renewal message. Otherwise, it's a key request.
 // FIXME(jrummell): Remove this once prefixed EME goes away.
@@ -113,7 +113,7 @@ static scoped_refptr<media::DecoderBuffer> CopyDecoderBufferFrom(
                   input_buffer.iv_size),
       subsamples));
 
-  output_buffer->set_decrypt_config(decrypt_config.Pass());
+  output_buffer->set_decrypt_config(std::move(decrypt_config));
   output_buffer->set_timestamp(
       base::TimeDelta::FromMicroseconds(input_buffer.timestamp));
 
@@ -183,12 +183,14 @@ cdm::KeyStatus ConvertKeyStatus(media::CdmKeyInformation::KeyStatus status) {
       return cdm::kInternalError;
     case media::CdmKeyInformation::KeyStatus::EXPIRED:
       return cdm::kExpired;
-    case media::CdmKeyInformation::KeyStatus::OUTPUT_NOT_ALLOWED:
-      return cdm::kOutputNotAllowed;
+    case media::CdmKeyInformation::KeyStatus::OUTPUT_RESTRICTED:
+      return cdm::kOutputRestricted;
     case media::CdmKeyInformation::KeyStatus::OUTPUT_DOWNSCALED:
       return cdm::kOutputDownscaled;
     case media::CdmKeyInformation::KeyStatus::KEY_STATUS_PENDING:
       return cdm::kStatusPending;
+    case media::CdmKeyInformation::KeyStatus::RELEASED:
+      return cdm::kReleased;
   }
   NOTREACHED();
   return cdm::kInternalError;
@@ -202,7 +204,7 @@ void ConvertCdmKeysInfo(const std::vector<media::CdmKeyInformation*>& keys_info,
   keys_vector->reserve(keys_info.size());
   for (const auto& key_info : keys_info) {
     cdm::KeyInformation key;
-    key.key_id = vector_as_array(&key_info->key_id);
+    key.key_id = key_info->key_id.data();
     key.key_id_size = key_info->key_id.size();
     key.status = ConvertKeyStatus(key_info->status);
     key.system_code = key_info->system_code;
@@ -210,23 +212,15 @@ void ConvertCdmKeysInfo(const std::vector<media::CdmKeyInformation*>& keys_info,
   }
 }
 
-template<typename Type>
-class ScopedResetter {
- public:
-  explicit ScopedResetter(Type* object) : object_(object) {}
-  ~ScopedResetter() { object_->Reset(); }
-
- private:
-  Type* const object_;
-};
-
 void INITIALIZE_CDM_MODULE() {
+  DVLOG(1) << __FUNCTION__;
 #if defined(CLEAR_KEY_CDM_USE_FFMPEG_DECODER)
   av_register_all();
 #endif  // CLEAR_KEY_CDM_USE_FFMPEG_DECODER
 }
 
 void DeinitializeCdmModule() {
+  DVLOG(1) << __FUNCTION__;
 }
 
 void* CreateCdmInstance(int cdm_interface_version,
@@ -253,7 +247,8 @@ void* CreateCdmInstance(int cdm_interface_version,
     return NULL;
 
   // TODO(jrummell): Obtain the proper origin for this instance.
-  return new media::ClearKeyCdm(host, key_system_string, GURL::EmptyGURL());
+  GURL empty_gurl;
+  return new media::ClearKeyCdm(host, key_system_string, empty_gurl);
 }
 
 const char* GetCdmVersion() {
@@ -265,12 +260,12 @@ namespace media {
 ClearKeyCdm::ClearKeyCdm(ClearKeyCdmHost* host,
                          const std::string& key_system,
                          const GURL& origin)
-    : decryptor_(
+    : decryptor_(new AesDecryptor(
           origin,
           base::Bind(&ClearKeyCdm::OnSessionMessage, base::Unretained(this)),
           base::Bind(&ClearKeyCdm::OnSessionClosed, base::Unretained(this)),
           base::Bind(&ClearKeyCdm::OnSessionKeysChange,
-                     base::Unretained(this))),
+                     base::Unretained(this)))),
       host_(host),
       key_system_(key_system),
       has_received_keys_change_event_for_emulated_loadsession_(false),
@@ -294,11 +289,11 @@ void ClearKeyCdm::Initialize(bool /* allow_distinctive_identifier */,
 }
 
 void ClearKeyCdm::CreateSessionAndGenerateRequest(
-    uint32 promise_id,
+    uint32_t promise_id,
     cdm::SessionType session_type,
     cdm::InitDataType init_data_type,
-    const uint8* init_data,
-    uint32 init_data_size) {
+    const uint8_t* init_data,
+    uint32_t init_data_size) {
   DVLOG(1) << __FUNCTION__;
 
   scoped_ptr<media::NewSessionCdmPromise> promise(
@@ -309,10 +304,10 @@ void ClearKeyCdm::CreateSessionAndGenerateRequest(
           base::Bind(&ClearKeyCdm::OnPromiseFailed,
                      base::Unretained(this),
                      promise_id)));
-  decryptor_.CreateSessionAndGenerateRequest(
+  decryptor_->CreateSessionAndGenerateRequest(
       ConvertSessionType(session_type), ConvertInitDataType(init_data_type),
       std::vector<uint8_t>(init_data, init_data + init_data_size),
-      promise.Pass());
+      std::move(promise));
 
   if (key_system_ == kExternalClearKeyFileIOTestKeySystem)
     StartFileIOTest();
@@ -321,7 +316,7 @@ void ClearKeyCdm::CreateSessionAndGenerateRequest(
 // Loads a emulated stored session. Currently only |kLoadableSessionId|
 // (containing a |kLoadableSessionKey| for |kLoadableSessionKeyId|) is
 // supported.
-void ClearKeyCdm::LoadSession(uint32 promise_id,
+void ClearKeyCdm::LoadSession(uint32_t promise_id,
                               cdm::SessionType session_type,
                               const char* session_id,
                               uint32_t session_id_length) {
@@ -345,16 +340,16 @@ void ClearKeyCdm::LoadSession(uint32 promise_id,
           base::Bind(&ClearKeyCdm::OnPromiseFailed,
                      base::Unretained(this),
                      promise_id)));
-  decryptor_.CreateSessionAndGenerateRequest(
+  decryptor_->CreateSessionAndGenerateRequest(
       MediaKeys::TEMPORARY_SESSION, EmeInitDataType::WEBM,
-      std::vector<uint8_t>(), promise.Pass());
+      std::vector<uint8_t>(), std::move(promise));
 }
 
-void ClearKeyCdm::UpdateSession(uint32 promise_id,
+void ClearKeyCdm::UpdateSession(uint32_t promise_id,
                                 const char* session_id,
                                 uint32_t session_id_length,
-                                const uint8* response,
-                                uint32 response_size) {
+                                const uint8_t* response,
+                                uint32_t response_size) {
   DVLOG(1) << __FUNCTION__;
   std::string web_session_str(session_id, session_id_length);
 
@@ -367,9 +362,9 @@ void ClearKeyCdm::UpdateSession(uint32 promise_id,
                  promise_id),
       base::Bind(&ClearKeyCdm::OnPromiseFailed, base::Unretained(this),
                  promise_id)));
-  decryptor_.UpdateSession(
+  decryptor_->UpdateSession(
       web_session_str, std::vector<uint8_t>(response, response + response_size),
-      promise.Pass());
+      std::move(promise));
 
   if (!renewal_timer_set_) {
     ScheduleNextRenewal();
@@ -377,7 +372,7 @@ void ClearKeyCdm::UpdateSession(uint32 promise_id,
   }
 }
 
-void ClearKeyCdm::CloseSession(uint32 promise_id,
+void ClearKeyCdm::CloseSession(uint32_t promise_id,
                                const char* session_id,
                                uint32_t session_id_length) {
   DVLOG(1) << __FUNCTION__;
@@ -392,10 +387,10 @@ void ClearKeyCdm::CloseSession(uint32 promise_id,
           &ClearKeyCdm::OnPromiseResolved, base::Unretained(this), promise_id),
       base::Bind(
           &ClearKeyCdm::OnPromiseFailed, base::Unretained(this), promise_id)));
-  decryptor_.CloseSession(web_session_str, promise.Pass());
+  decryptor_->CloseSession(web_session_str, std::move(promise));
 }
 
-void ClearKeyCdm::RemoveSession(uint32 promise_id,
+void ClearKeyCdm::RemoveSession(uint32_t promise_id,
                                 const char* session_id,
                                 uint32_t session_id_length) {
   DVLOG(1) << __FUNCTION__;
@@ -421,10 +416,10 @@ void ClearKeyCdm::RemoveSession(uint32 promise_id,
                  promise_id),
       base::Bind(&ClearKeyCdm::OnPromiseFailed, base::Unretained(this),
                  promise_id)));
-  decryptor_.RemoveSession(web_session_str, promise.Pass());
+  decryptor_->RemoveSession(web_session_str, std::move(promise));
 }
 
-void ClearKeyCdm::SetServerCertificate(uint32 promise_id,
+void ClearKeyCdm::SetServerCertificate(uint32_t promise_id,
                                        const uint8_t* server_certificate_data,
                                        uint32_t server_certificate_data_size) {
   // ClearKey doesn't use a server certificate.
@@ -622,7 +617,7 @@ cdm::Status ClearKeyCdm::DecryptAndDecodeSamples(
 
   return audio_decoder_->DecodeBuffer(data, size, timestamp, audio_frames);
 #elif defined(CLEAR_KEY_CDM_USE_FAKE_AUDIO_DECODER)
-  int64 timestamp_in_microseconds = kNoTimestamp;
+  int64_t timestamp_in_microseconds = kNoTimestamp;
   if (!buffer->end_of_stream()) {
     timestamp_in_microseconds = buffer->GetTimestamp().InMicroseconds();
     DCHECK(timestamp_in_microseconds != kNoTimestamp);
@@ -669,9 +664,8 @@ cdm::Status ClearKeyCdm::DecryptToMediaDecoderBuffer(
   media::Decryptor::Status status = media::Decryptor::kError;
   // The AesDecryptor does not care what the stream type is. Pass kVideo
   // for both audio and video decryption.
-  decryptor_.Decrypt(
-      media::Decryptor::kVideo,
-      buffer,
+  decryptor_->Decrypt(
+      media::Decryptor::kVideo, buffer,
       base::Bind(&CopyDecryptResults, &status, decrypted_buffer));
 
   if (status == media::Decryptor::kError)
@@ -705,14 +699,14 @@ void ClearKeyCdm::LoadLoadableSession() {
       base::Bind(&ClearKeyCdm::OnLoadSessionUpdated, base::Unretained(this)),
       base::Bind(&ClearKeyCdm::OnPromiseFailed, base::Unretained(this),
                  promise_id_for_emulated_loadsession_)));
-  decryptor_.UpdateSession(session_id_for_emulated_loadsession_,
-                           std::vector<uint8_t>(jwk_set.begin(), jwk_set.end()),
-                           promise.Pass());
+  decryptor_->UpdateSession(
+      session_id_for_emulated_loadsession_,
+      std::vector<uint8_t>(jwk_set.begin(), jwk_set.end()), std::move(promise));
 }
 
 void ClearKeyCdm::OnSessionMessage(const std::string& session_id,
                                    MediaKeys::MessageType message_type,
-                                   const std::vector<uint8>& message,
+                                   const std::vector<uint8_t>& message,
                                    const GURL& legacy_destination_url) {
   DVLOG(1) << "OnSessionMessage: " << message.size();
 
@@ -753,8 +747,8 @@ void ClearKeyCdm::OnSessionKeysChange(const std::string& session_id,
   std::vector<cdm::KeyInformation> keys_vector;
   ConvertCdmKeysInfo(keys_info.get(), &keys_vector);
   host_->OnSessionKeysChange(new_session_id.data(), new_session_id.length(),
-                             has_additional_usable_key,
-                             vector_as_array(&keys_vector), keys_vector.size());
+                             has_additional_usable_key, keys_vector.data(),
+                             keys_vector.size());
 }
 
 void ClearKeyCdm::OnSessionClosed(const std::string& session_id) {
@@ -764,7 +758,7 @@ void ClearKeyCdm::OnSessionClosed(const std::string& session_id) {
   host_->OnSessionClosed(new_session_id.data(), new_session_id.length());
 }
 
-void ClearKeyCdm::OnSessionCreated(uint32 promise_id,
+void ClearKeyCdm::OnSessionCreated(uint32_t promise_id,
                                    const std::string& session_id) {
   // Save the latest session ID for renewal and file IO test messages.
   last_session_id_ = session_id;
@@ -773,7 +767,7 @@ void ClearKeyCdm::OnSessionCreated(uint32 promise_id,
                                     session_id.length());
 }
 
-void ClearKeyCdm::OnSessionLoaded(uint32 promise_id,
+void ClearKeyCdm::OnSessionLoaded(uint32_t promise_id,
                                   const std::string& session_id) {
   // Save the latest session ID for renewal and file IO test messages.
   last_session_id_ = session_id;
@@ -785,7 +779,7 @@ void ClearKeyCdm::OnSessionLoaded(uint32 promise_id,
 
   // Delay LoadLoadableSession() to test the case where Decrypt*() calls are
   // made before the session is fully loaded.
-  const int64 kDelayToLoadSessionMs = 500;
+  const int64_t kDelayToLoadSessionMs = 500;
 
   // Defer resolving the promise until the session is loaded.
   promise_id_for_emulated_loadsession_ = promise_id;
@@ -818,19 +812,19 @@ void ClearKeyCdm::OnLoadSessionUpdated() {
     has_received_keys_change_event_for_emulated_loadsession_ = false;
     DCHECK(!keys_vector.empty());
     ConvertCdmKeysInfo(keys_info.get(), &keys_vector);
-    host_->OnSessionKeysChange(
-        kLoadableSessionId, strlen(kLoadableSessionId), !keys_vector.empty(),
-        vector_as_array(&keys_vector), keys_vector.size());
+    host_->OnSessionKeysChange(kLoadableSessionId, strlen(kLoadableSessionId),
+                               !keys_vector.empty(), keys_vector.data(),
+                               keys_vector.size());
   }
 }
 
-void ClearKeyCdm::OnPromiseResolved(uint32 promise_id) {
+void ClearKeyCdm::OnPromiseResolved(uint32_t promise_id) {
   host_->OnResolvePromise(promise_id);
 }
 
-void ClearKeyCdm::OnPromiseFailed(uint32 promise_id,
+void ClearKeyCdm::OnPromiseFailed(uint32_t promise_id,
                                   MediaKeys::Exception exception_code,
-                                  uint32 system_code,
+                                  uint32_t system_code,
                                   const std::string& error_message) {
   host_->OnRejectPromise(promise_id,
                          ConvertException(exception_code),
@@ -840,25 +834,27 @@ void ClearKeyCdm::OnPromiseFailed(uint32 promise_id,
 }
 
 #if defined(CLEAR_KEY_CDM_USE_FAKE_AUDIO_DECODER)
-int64 ClearKeyCdm::CurrentTimeStampInMicroseconds() const {
+int64_t ClearKeyCdm::CurrentTimeStampInMicroseconds() const {
   return output_timestamp_base_in_microseconds_ +
          base::Time::kMicrosecondsPerSecond *
          total_samples_generated_  / samples_per_second_;
 }
 
 int ClearKeyCdm::GenerateFakeAudioFramesFromDuration(
-    int64 duration_in_microseconds,
+    int64_t duration_in_microseconds,
     cdm::AudioFrames* audio_frames) const {
-  int64 samples_to_generate = static_cast<double>(samples_per_second_) *
-      duration_in_microseconds / base::Time::kMicrosecondsPerSecond + 0.5;
+  int64_t samples_to_generate = static_cast<double>(samples_per_second_) *
+                                    duration_in_microseconds /
+                                    base::Time::kMicrosecondsPerSecond +
+                                0.5;
   if (samples_to_generate <= 0)
     return 0;
 
-  int64 bytes_per_sample = channel_count_ * bits_per_channel_ / 8;
+  int64_t bytes_per_sample = channel_count_ * bits_per_channel_ / 8;
   // |frame_size| must be a multiple of |bytes_per_sample|.
-  int64 frame_size = bytes_per_sample * samples_to_generate;
+  int64_t frame_size = bytes_per_sample * samples_to_generate;
 
-  int64 timestamp = CurrentTimeStampInMicroseconds();
+  int64_t timestamp = CurrentTimeStampInMicroseconds();
 
   const int kHeaderSize = sizeof(timestamp) + sizeof(frame_size);
   audio_frames->SetFrameBuffer(host_->Allocate(kHeaderSize + frame_size));
@@ -878,7 +874,7 @@ int ClearKeyCdm::GenerateFakeAudioFramesFromDuration(
 }
 
 cdm::Status ClearKeyCdm::GenerateFakeAudioFrames(
-    int64 timestamp_in_microseconds,
+    int64_t timestamp_in_microseconds,
     cdm::AudioFrames* audio_frames) {
   if (timestamp_in_microseconds == kNoTimestamp)
     return cdm::kNeedMoreData;
