@@ -5,7 +5,8 @@
 #ifndef MEDIA_FORMATS_WEBM_WEBM_CRYPTO_HELPERS_H_
 #define MEDIA_FORMATS_WEBM_WEBM_CRYPTO_HELPERS_H_
 
-#include "base/basictypes.h"
+#include <stdint.h>
+
 #include "base/memory/scoped_ptr.h"
 #include "media/base/decoder_buffer.h"
 
@@ -18,8 +19,10 @@ namespace media {
 // false otherwise, in which case |decrypt_config| and |data_offset| will not be
 // changed. Current encrypted WebM request for comments specification is here
 // http://wiki.webmproject.org/encryption/webm-encryption-rfc
-bool WebMCreateDecryptConfig(const uint8* data, int data_size,
-                             const uint8* key_id, int key_id_size,
+bool WebMCreateDecryptConfig(const uint8_t* data,
+                             int data_size,
+                             const uint8_t* key_id,
+                             int key_id_size,
                              scoped_ptr<DecryptConfig>* decrypt_config,
                              int* data_offset);
 
