@@ -6,15 +6,19 @@
 
 #include "base/bind.h"
 #include "base/logging.h"
+#include "media/mojo/services/mojo_cdm_service.h"
 
 namespace media {
 
-MojoCdmServiceContext::MojoCdmServiceContext() {
-}
+MojoCdmServiceContext::MojoCdmServiceContext() : weak_ptr_factory_(this) {}
 
 MojoCdmServiceContext::~MojoCdmServiceContext() {
 }
 
+base::WeakPtr<MojoCdmServiceContext> MojoCdmServiceContext::GetWeakPtr() {
+  return weak_ptr_factory_.GetWeakPtr();
+}
+
 void MojoCdmServiceContext::RegisterCdm(int cdm_id,
                                         MojoCdmService* cdm_service) {
   DCHECK(!cdm_services_.count(cdm_id));
