@@ -4,20 +4,25 @@
 
 #include "media/mojo/services/mojo_demuxer_stream_impl.h"
 
+#include <stdint.h>
+#include <utility>
+
 #include "base/bind.h"
+#include "base/numerics/safe_conversions.h"
 #include "media/base/audio_decoder_config.h"
 #include "media/base/decoder_buffer.h"
 #include "media/base/video_decoder_config.h"
 #include "media/mojo/services/media_type_converters.h"
-#include "third_party/mojo/src/mojo/public/cpp/system/data_pipe.h"
+#include "mojo/public/cpp/system/data_pipe.h"
 
 namespace media {
 
 MojoDemuxerStreamImpl::MojoDemuxerStreamImpl(
     media::DemuxerStream* stream,
-    mojo::InterfaceRequest<mojo::DemuxerStream> request)
-    : binding_(this, request.Pass()), stream_(stream), weak_factory_(this) {
-}
+    mojo::InterfaceRequest<interfaces::DemuxerStream> request)
+    : binding_(this, std::move(request)),
+      stream_(stream),
+      weak_factory_(this) {}
 
 MojoDemuxerStreamImpl::~MojoDemuxerStreamImpl() {
 }
@@ -44,25 +49,25 @@ void MojoDemuxerStreamImpl::Initialize(const InitializeCallback& callback) {
   }
 
   mojo::DataPipe data_pipe(options);
-  stream_pipe_ = data_pipe.producer_handle.Pass();
+  stream_pipe_ = std::move(data_pipe.producer_handle);
 
   // Prepare the initial config.
-  mojo::AudioDecoderConfigPtr audio_config;
-  mojo::VideoDecoderConfigPtr video_config;
+  interfaces::AudioDecoderConfigPtr audio_config;
+  interfaces::VideoDecoderConfigPtr video_config;
   if (stream_->type() == media::DemuxerStream::AUDIO) {
     audio_config =
-        mojo::AudioDecoderConfig::From(stream_->audio_decoder_config());
+        interfaces::AudioDecoderConfig::From(stream_->audio_decoder_config());
   } else if (stream_->type() == media::DemuxerStream::VIDEO) {
     video_config =
-        mojo::VideoDecoderConfig::From(stream_->video_decoder_config());
+        interfaces::VideoDecoderConfig::From(stream_->video_decoder_config());
   } else {
     NOTREACHED() << "Unsupported stream type: " << stream_->type();
     return;
   }
 
-  callback.Run(static_cast<mojo::DemuxerStream::Type>(stream_->type()),
-               data_pipe.consumer_handle.Pass(), audio_config.Pass(),
-               video_config.Pass());
+  callback.Run(static_cast<interfaces::DemuxerStream::Type>(stream_->type()),
+               std::move(data_pipe.consumer_handle), std::move(audio_config),
+               std::move(video_config));
 }
 
 void MojoDemuxerStreamImpl::Read(const ReadCallback& callback)  {
@@ -74,8 +79,8 @@ void MojoDemuxerStreamImpl::OnBufferReady(
     const ReadCallback& callback,
     media::DemuxerStream::Status status,
     const scoped_refptr<media::DecoderBuffer>& buffer) {
-  mojo::AudioDecoderConfigPtr audio_config;
-  mojo::VideoDecoderConfigPtr video_config;
+  interfaces::AudioDecoderConfigPtr audio_config;
+  interfaces::VideoDecoderConfigPtr video_config;
 
   if (status == media::DemuxerStream::kConfigChanged) {
     DVLOG(2) << __FUNCTION__ << ": ConfigChange!";
@@ -83,45 +88,46 @@ void MojoDemuxerStreamImpl::OnBufferReady(
     // Status obtained via Run() below.
     if (stream_->type() == media::DemuxerStream::AUDIO) {
       audio_config =
-          mojo::AudioDecoderConfig::From(stream_->audio_decoder_config());
+          interfaces::AudioDecoderConfig::From(stream_->audio_decoder_config());
     } else if (stream_->type() == media::DemuxerStream::VIDEO) {
       video_config =
-          mojo::VideoDecoderConfig::From(stream_->video_decoder_config());
+          interfaces::VideoDecoderConfig::From(stream_->video_decoder_config());
     } else {
       NOTREACHED() << "Unsupported config change encountered for type: "
                    << stream_->type();
     }
 
-    callback.Run(mojo::DemuxerStream::STATUS_CONFIG_CHANGED,
-                 mojo::MediaDecoderBufferPtr(), audio_config.Pass(),
-                 video_config.Pass());
+    callback.Run(interfaces::DemuxerStream::STATUS_CONFIG_CHANGED,
+                 interfaces::DecoderBufferPtr(), std::move(audio_config),
+                 std::move(video_config));
     return;
   }
 
   if (status == media::DemuxerStream::kAborted) {
-    callback.Run(mojo::DemuxerStream::STATUS_ABORTED,
-                 mojo::MediaDecoderBufferPtr(), audio_config.Pass(),
-                 video_config.Pass());
+    callback.Run(interfaces::DemuxerStream::STATUS_ABORTED,
+                 interfaces::DecoderBufferPtr(), std::move(audio_config),
+                 std::move(video_config));
     return;
   }
 
   DCHECK_EQ(status, media::DemuxerStream::kOk);
   if (!buffer->end_of_stream()) {
-    DCHECK_GT(buffer->data_size(), 0);
+    DCHECK_GT(buffer->data_size(), 0u);
     // Serialize the data section of the DecoderBuffer into our pipe.
-    uint32_t num_bytes = buffer->data_size();
-    CHECK_EQ(WriteDataRaw(stream_pipe_.get(), buffer->data(), &num_bytes,
+    uint32_t bytes_to_write = base::checked_cast<uint32_t>(buffer->data_size());
+    uint32_t bytes_written = bytes_to_write;
+    CHECK_EQ(WriteDataRaw(stream_pipe_.get(), buffer->data(), &bytes_written,
                           MOJO_READ_DATA_FLAG_ALL_OR_NONE),
              MOJO_RESULT_OK);
-    CHECK_EQ(num_bytes, static_cast<uint32_t>(buffer->data_size()));
+    CHECK_EQ(bytes_to_write, bytes_written);
   }
 
   // TODO(dalecurtis): Once we can write framed data to the DataPipe, fill via
   // the producer handle and then read more to keep the pipe full.  Waiting for
   // space can be accomplished using an AsyncWaiter.
-  callback.Run(static_cast<mojo::DemuxerStream::Status>(status),
-               mojo::MediaDecoderBuffer::From(buffer), audio_config.Pass(),
-               video_config.Pass());
+  callback.Run(static_cast<interfaces::DemuxerStream::Status>(status),
+               interfaces::DecoderBuffer::From(buffer), std::move(audio_config),
+               std::move(video_config));
 }
 
 }  // namespace media
