@@ -4,6 +4,8 @@
 
 #include "media/renderers/renderer_impl.h"
 
+#include <utility>
+
 #include "base/bind.h"
 #include "base/callback.h"
 #include "base/callback_helpers.h"
@@ -31,8 +33,8 @@ RendererImpl::RendererImpl(
     scoped_ptr<VideoRenderer> video_renderer)
     : state_(STATE_UNINITIALIZED),
       task_runner_(task_runner),
-      audio_renderer_(audio_renderer.Pass()),
-      video_renderer_(video_renderer.Pass()),
+      audio_renderer_(std::move(audio_renderer)),
+      video_renderer_(std::move(video_renderer)),
       time_source_(NULL),
       time_ticking_(false),
       playback_rate_(0.0),
@@ -121,13 +123,12 @@ void RendererImpl::SetCdm(CdmContext* cdm_context,
 
   cdm_context_ = cdm_context;
 
-  if (decryptor_ready_cb_.is_null()) {
+  if (cdm_ready_cb_.is_null()) {
     cdm_attached_cb.Run(true);
     return;
   }
 
-  base::ResetAndReturn(&decryptor_ready_cb_)
-      .Run(cdm_context->GetDecryptor(), cdm_attached_cb);
+  base::ResetAndReturn(&cdm_ready_cb_).Run(cdm_context, cdm_attached_cb);
 }
 
 void RendererImpl::Flush(const base::Closure& flush_cb) {
@@ -253,27 +254,25 @@ bool RendererImpl::GetWallClockTimes(
   return time_source_->GetWallClockTimes(media_timestamps, wall_clock_times);
 }
 
-void RendererImpl::SetDecryptorReadyCallback(
-    const DecryptorReadyCB& decryptor_ready_cb) {
-  // Cancels the previous decryptor request.
-  if (decryptor_ready_cb.is_null()) {
-    if (!decryptor_ready_cb_.is_null()) {
-      base::ResetAndReturn(&decryptor_ready_cb_)
+void RendererImpl::SetCdmReadyCallback(const CdmReadyCB& cdm_ready_cb) {
+  // Cancels the previous CDM request.
+  if (cdm_ready_cb.is_null()) {
+    if (!cdm_ready_cb_.is_null()) {
+      base::ResetAndReturn(&cdm_ready_cb_)
           .Run(nullptr, base::Bind(IgnoreCdmAttached));
     }
     return;
   }
 
   // We initialize audio and video decoders in sequence.
-  DCHECK(decryptor_ready_cb_.is_null());
+  DCHECK(cdm_ready_cb_.is_null());
 
   if (cdm_context_) {
-    decryptor_ready_cb.Run(cdm_context_->GetDecryptor(),
-                           base::Bind(IgnoreCdmAttached));
+    cdm_ready_cb.Run(cdm_context_, base::Bind(IgnoreCdmAttached));
     return;
   }
 
-  decryptor_ready_cb_ = decryptor_ready_cb;
+  cdm_ready_cb_ = cdm_ready_cb;
 }
 
 void RendererImpl::InitializeAudioRenderer() {
@@ -295,7 +294,7 @@ void RendererImpl::InitializeAudioRenderer() {
   // happen at any time and all future calls must guard against STATE_ERROR.
   audio_renderer_->Initialize(
       demuxer_stream_provider_->GetStream(DemuxerStream::AUDIO), done_cb,
-      base::Bind(&RendererImpl::SetDecryptorReadyCallback, weak_this_),
+      base::Bind(&RendererImpl::SetCdmReadyCallback, weak_this_),
       base::Bind(&RendererImpl::OnUpdateStatistics, weak_this_),
       base::Bind(&RendererImpl::OnBufferingStateChanged, weak_this_,
                  &audio_buffering_state_),
@@ -342,7 +341,7 @@ void RendererImpl::InitializeVideoRenderer() {
 
   video_renderer_->Initialize(
       demuxer_stream_provider_->GetStream(DemuxerStream::VIDEO), done_cb,
-      base::Bind(&RendererImpl::SetDecryptorReadyCallback, weak_this_),
+      base::Bind(&RendererImpl::SetCdmReadyCallback, weak_this_),
       base::Bind(&RendererImpl::OnUpdateStatistics, weak_this_),
       base::Bind(&RendererImpl::OnBufferingStateChanged, weak_this_,
                  &video_buffering_state_),
