@@ -2,9 +2,12 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+#include <stdint.h>
+
 #include <vector>
 
 #include "base/bind.h"
+#include "base/macros.h"
 #include "base/message_loop/message_loop.h"
 #include "base/run_loop.h"
 #include "base/test/simple_test_tick_clock.h"
@@ -24,7 +27,7 @@ using ::testing::StrictMock;
 
 namespace media {
 
-const int64 kStartPlayingTimeInMs = 100;
+const int64_t kStartPlayingTimeInMs = 100;
 
 ACTION_P2(SetBufferingState, cb, buffering_state) {
   cb->Run(buffering_state);
@@ -82,7 +85,7 @@ class RendererImplTest : public ::testing::Test {
       DemuxerStream::Type type) {
     scoped_ptr<StrictMock<MockDemuxerStream> > stream(
         new StrictMock<MockDemuxerStream>(type));
-    return stream.Pass();
+    return stream;
   }
 
   // Sets up expectations to allow the audio renderer to initialize.
@@ -223,13 +226,13 @@ class RendererImplTest : public ::testing::Test {
     base::RunLoop().RunUntilIdle();
   }
 
-  int64 GetMediaTimeMs() {
+  int64_t GetMediaTimeMs() {
     return renderer_impl_->GetMediaTime().InMilliseconds();
   }
 
   bool IsMediaTimeAdvancing(double playback_rate) {
-    int64 start_time_ms = GetMediaTimeMs();
-    const int64 time_to_advance_ms = 100;
+    int64_t start_time_ms = GetMediaTimeMs();
+    const int64_t time_to_advance_ms = 100;
 
     test_tick_clock_.Advance(
         base::TimeDelta::FromMilliseconds(time_to_advance_ms));
