@@ -5,6 +5,10 @@
 #ifndef MEDIA_VIDEO_H264_POC_H_
 #define MEDIA_VIDEO_H264_POC_H_
 
+#include <stdint.h>
+
+#include "base/macros.h"
+
 namespace media {
 
 struct H264SPS;
