@@ -4,28 +4,37 @@
 
 #include "mojo/edk/system/core.h"
 
+#include <stddef.h>
+#include <stdint.h>
+
+#include <utility>
 #include <vector>
 
+#include "base/containers/stack_container.h"
 #include "base/logging.h"
+#include "base/rand_util.h"
 #include "base/time/time.h"
+#include "mojo/edk/embedder/embedder_internal.h"
+#include "mojo/edk/embedder/platform_channel_pair.h"
 #include "mojo/edk/embedder/platform_shared_buffer.h"
 #include "mojo/edk/embedder/platform_support.h"
-#include "mojo/edk/system/constants.h"
+#include "mojo/edk/system/async_waiter.h"
+#include "mojo/edk/system/broker.h"
+#include "mojo/edk/system/configuration.h"
 #include "mojo/edk/system/data_pipe.h"
 #include "mojo/edk/system/data_pipe_consumer_dispatcher.h"
 #include "mojo/edk/system/data_pipe_producer_dispatcher.h"
 #include "mojo/edk/system/dispatcher.h"
 #include "mojo/edk/system/handle_signals_state.h"
-#include "mojo/edk/system/local_data_pipe.h"
-#include "mojo/edk/system/memory.h"
-#include "mojo/edk/system/message_pipe.h"
 #include "mojo/edk/system/message_pipe_dispatcher.h"
 #include "mojo/edk/system/shared_buffer_dispatcher.h"
+#include "mojo/edk/system/wait_set_dispatcher.h"
 #include "mojo/edk/system/waiter.h"
 #include "mojo/public/c/system/macros.h"
+#include "mojo/public/cpp/system/macros.h"
 
 namespace mojo {
-namespace system {
+namespace edk {
 
 // Implementation notes
 //
@@ -47,10 +56,11 @@ namespace system {
 // |Dispatcher|s for the handles that it wants to wait on with a |Waiter|
 // object; this |Waiter| object may be created on the stack of that thread or be
 // kept in thread local storage for that thread (TODO(vtl): future improvement).
-// The |Dispatcher| then adds the |Waiter| to a |WaiterList| that's either owned
-// by that |Dispatcher| (see |SimpleDispatcher|) or by a secondary object (e.g.,
-// |MessagePipe|). To signal/wake a |Waiter|, the object in question -- either a
-// |SimpleDispatcher| or a secondary object -- talks to its |WaiterList|.
+// The |Dispatcher| then adds the |Waiter| to an |AwakableList| that's either
+// owned by that |Dispatcher| (see |SimpleDispatcher|) or by a secondary object
+// (e.g., |MessagePipe|). To signal/wake a |Waiter|, the object in question --
+// either a |SimpleDispatcher| or a secondary object -- talks to its
+// |AwakableList|.
 
 // Thread-safety notes
 //
@@ -76,8 +86,8 @@ namespace system {
 //      held.
 
 // TODO(vtl): This should take a |scoped_ptr<PlatformSupport>| as a parameter.
-Core::Core(scoped_ptr<embedder::PlatformSupport> platform_support)
-    : platform_support_(platform_support.Pass()) {
+Core::Core(PlatformSupport* platform_support)
+    : platform_support_(platform_support) {
 }
 
 Core::~Core() {
@@ -96,6 +106,19 @@ scoped_refptr<Dispatcher> Core::GetDispatcher(MojoHandle handle) {
   return handle_table_.GetDispatcher(handle);
 }
 
+MojoResult Core::AsyncWait(MojoHandle handle,
+                           MojoHandleSignals signals,
+                           const base::Callback<void(MojoResult)>& callback) {
+  scoped_refptr<Dispatcher> dispatcher = GetDispatcher(handle);
+  DCHECK(dispatcher);
+
+  scoped_ptr<AsyncWaiter> waiter = make_scoped_ptr(new AsyncWaiter(callback));
+  MojoResult rv = dispatcher->AddAwakable(waiter.get(), signals, 0, nullptr);
+  if (rv == MOJO_RESULT_OK)
+    ignore_result(waiter.release());
+  return rv;
+}
+
 MojoTimeTicks Core::GetTimeTicksNow() {
   return base::TimeTicks::Now().ToInternalValue();
 }
@@ -123,77 +146,132 @@ MojoResult Core::Close(MojoHandle handle) {
 MojoResult Core::Wait(MojoHandle handle,
                       MojoHandleSignals signals,
                       MojoDeadline deadline,
-                      UserPointer<MojoHandleSignalsState> signals_state) {
+                      MojoHandleSignalsState* signals_state) {
   uint32_t unused = static_cast<uint32_t>(-1);
   HandleSignalsState hss;
-  MojoResult rv = WaitManyInternal(&handle,
-                                   &signals,
-                                   1,
-                                   deadline,
-                                   &unused,
-                                   signals_state.IsNull() ? nullptr : &hss);
-  if (rv != MOJO_RESULT_INVALID_ARGUMENT && !signals_state.IsNull())
-    signals_state.Put(hss);
+  MojoResult rv = WaitManyInternal(&handle, &signals, 1, deadline, &unused,
+                                   signals_state ? &hss : nullptr);
+  if (rv != MOJO_RESULT_INVALID_ARGUMENT && signals_state)
+    *signals_state = hss;
   return rv;
 }
 
-MojoResult Core::WaitMany(UserPointer<const MojoHandle> handles,
-                          UserPointer<const MojoHandleSignals> signals,
+MojoResult Core::WaitMany(const MojoHandle* handles,
+                          const MojoHandleSignals* signals,
                           uint32_t num_handles,
                           MojoDeadline deadline,
-                          UserPointer<uint32_t> result_index,
-                          UserPointer<MojoHandleSignalsState> signals_states) {
+                          uint32_t* result_index,
+                          MojoHandleSignalsState* signals_state) {
   if (num_handles < 1)
     return MOJO_RESULT_INVALID_ARGUMENT;
-  if (num_handles > kMaxWaitManyNumHandles)
+  if (num_handles > GetConfiguration().max_wait_many_num_handles)
     return MOJO_RESULT_RESOURCE_EXHAUSTED;
 
-  UserPointer<const MojoHandle>::Reader handles_reader(handles, num_handles);
-  UserPointer<const MojoHandleSignals>::Reader signals_reader(signals,
-                                                              num_handles);
   uint32_t index = static_cast<uint32_t>(-1);
   MojoResult rv;
-  if (signals_states.IsNull()) {
-    rv = WaitManyInternal(handles_reader.GetPointer(),
-                          signals_reader.GetPointer(),
-                          num_handles,
-                          deadline,
-                          &index,
+  if (!signals_state) {
+    rv = WaitManyInternal(handles, signals, num_handles, deadline, &index,
                           nullptr);
   } else {
-    UserPointer<MojoHandleSignalsState>::Writer signals_states_writer(
-        signals_states, num_handles);
     // Note: The |reinterpret_cast| is safe, since |HandleSignalsState| is a
     // subclass of |MojoHandleSignalsState| that doesn't add any data members.
-    rv = WaitManyInternal(handles_reader.GetPointer(),
-                          signals_reader.GetPointer(),
-                          num_handles,
-                          deadline,
-                          &index,
-                          reinterpret_cast<HandleSignalsState*>(
-                              signals_states_writer.GetPointer()));
-    if (rv != MOJO_RESULT_INVALID_ARGUMENT)
-      signals_states_writer.Commit();
+    rv = WaitManyInternal(handles, signals, num_handles, deadline, &index,
+                          reinterpret_cast<HandleSignalsState*>(signals_state));
   }
-  if (index != static_cast<uint32_t>(-1) && !result_index.IsNull())
-    result_index.Put(index);
+  if (index != static_cast<uint32_t>(-1) && result_index)
+    *result_index = index;
   return rv;
 }
 
+MojoResult Core::CreateWaitSet(MojoHandle* wait_set_handle) {
+  if (!wait_set_handle)
+    return MOJO_RESULT_INVALID_ARGUMENT;
+
+  scoped_refptr<WaitSetDispatcher> dispatcher = new WaitSetDispatcher();
+  MojoHandle h = AddDispatcher(dispatcher);
+  if (h == MOJO_HANDLE_INVALID) {
+    LOG(ERROR) << "Handle table full";
+    dispatcher->Close();
+    return MOJO_RESULT_RESOURCE_EXHAUSTED;
+  }
+
+  *wait_set_handle = h;
+  return MOJO_RESULT_OK;
+}
+
+MojoResult Core::AddHandle(MojoHandle wait_set_handle,
+                           MojoHandle handle,
+                           MojoHandleSignals signals) {
+  scoped_refptr<Dispatcher> wait_set_dispatcher(GetDispatcher(wait_set_handle));
+  if (!wait_set_dispatcher)
+    return MOJO_RESULT_INVALID_ARGUMENT;
+
+  scoped_refptr<Dispatcher> dispatcher(GetDispatcher(handle));
+  if (!dispatcher)
+    return MOJO_RESULT_INVALID_ARGUMENT;
+
+  return wait_set_dispatcher->AddWaitingDispatcher(dispatcher, signals, handle);
+}
+
+MojoResult Core::RemoveHandle(MojoHandle wait_set_handle,
+                              MojoHandle handle) {
+  scoped_refptr<Dispatcher> wait_set_dispatcher(GetDispatcher(wait_set_handle));
+  if (!wait_set_dispatcher)
+    return MOJO_RESULT_INVALID_ARGUMENT;
+
+  scoped_refptr<Dispatcher> dispatcher(GetDispatcher(handle));
+  if (!dispatcher)
+    return MOJO_RESULT_INVALID_ARGUMENT;
+
+  return wait_set_dispatcher->RemoveWaitingDispatcher(dispatcher);
+}
+
+MojoResult Core::GetReadyHandles(MojoHandle wait_set_handle,
+                                 uint32_t* count,
+                                 MojoHandle* handles,
+                                 MojoResult* results,
+                                 MojoHandleSignalsState* signals_state) {
+  if (!handles || !count || !(*count) || !results)
+    return MOJO_RESULT_INVALID_ARGUMENT;
+
+  scoped_refptr<Dispatcher> wait_set_dispatcher(GetDispatcher(wait_set_handle));
+  if (!wait_set_dispatcher)
+    return MOJO_RESULT_INVALID_ARGUMENT;
+
+  DispatcherVector awoken_dispatchers;
+  base::StackVector<uintptr_t, 16> contexts;
+  contexts->assign(*count, MOJO_HANDLE_INVALID);
+
+  MojoResult result = wait_set_dispatcher->GetReadyDispatchers(
+      count, &awoken_dispatchers, results, contexts->data());
+
+  if (result == MOJO_RESULT_OK) {
+    for (size_t i = 0; i < *count; i++) {
+      handles[i] = static_cast<MojoHandle>(contexts[i]);
+      if (signals_state)
+        signals_state[i] = awoken_dispatchers[i]->GetHandleSignalsState();
+    }
+  }
+
+  return result;
+}
+
 MojoResult Core::CreateMessagePipe(
-    UserPointer<const MojoCreateMessagePipeOptions> options,
-    UserPointer<MojoHandle> message_pipe_handle0,
-    UserPointer<MojoHandle> message_pipe_handle1) {
+    const MojoCreateMessagePipeOptions* options,
+    MojoHandle* message_pipe_handle0,
+    MojoHandle* message_pipe_handle1) {
+  CHECK(message_pipe_handle0);
+  CHECK(message_pipe_handle1);
   MojoCreateMessagePipeOptions validated_options = {};
   MojoResult result =
       MessagePipeDispatcher::ValidateCreateOptions(options, &validated_options);
   if (result != MOJO_RESULT_OK)
     return result;
 
-  scoped_refptr<MessagePipeDispatcher> dispatcher0(
-      new MessagePipeDispatcher(validated_options));
-  scoped_refptr<MessagePipeDispatcher> dispatcher1(
-      new MessagePipeDispatcher(validated_options));
+  scoped_refptr<MessagePipeDispatcher> dispatcher0 =
+      MessagePipeDispatcher::Create(validated_options);
+  scoped_refptr<MessagePipeDispatcher> dispatcher1 =
+      MessagePipeDispatcher::Create(validated_options);
 
   std::pair<MojoHandle, MojoHandle> handle_pair;
   {
@@ -208,12 +286,35 @@ MojoResult Core::CreateMessagePipe(
     return MOJO_RESULT_RESOURCE_EXHAUSTED;
   }
 
-  scoped_refptr<MessagePipe> message_pipe(MessagePipe::CreateLocalLocal());
-  dispatcher0->Init(message_pipe, 0);
-  dispatcher1->Init(message_pipe, 1);
+  if (validated_options.flags &
+          MOJO_CREATE_MESSAGE_PIPE_OPTIONS_FLAG_TRANSFERABLE) {
+    ScopedPlatformHandle server_handle, client_handle;
+#if defined(OS_WIN)
+    internal::g_broker->CreatePlatformChannelPair(&server_handle,
+                                                  &client_handle);
+#else
+    PlatformChannelPair channel_pair;
+    server_handle = channel_pair.PassServerHandle();
+    client_handle = channel_pair.PassClientHandle();
+#endif
+    dispatcher0->Init(std::move(server_handle), nullptr, 0u, nullptr, 0u,
+                      nullptr, nullptr);
+    dispatcher1->Init(std::move(client_handle), nullptr, 0u, nullptr, 0u,
+                      nullptr, nullptr);
+  } else {
+    uint64_t pipe_id = 0;
+    // route_id 0 is used internally in RoutedRawChannel. See kInternalRouteId
+    // in routed_raw_channel.cc.
+    // route_id 1 is used by broker communication. See kBrokerRouteId in
+    // broker_messages.h.
+    while (pipe_id < 2)
+      pipe_id = base::RandUint64();
+    dispatcher0->InitNonTransferable(pipe_id);
+    dispatcher1->InitNonTransferable(pipe_id);
+  }
 
-  message_pipe_handle0.Put(handle_pair.first);
-  message_pipe_handle1.Put(handle_pair.second);
+  *message_pipe_handle0 = handle_pair.first;
+  *message_pipe_handle1 = handle_pair.second;
   return MOJO_RESULT_OK;
 }
 
@@ -225,13 +326,13 @@ MojoResult Core::CreateMessagePipe(
 // after the the message has been received and a new handle created (and
 // possibly even after calls have been made on the new handle).
 MojoResult Core::WriteMessage(MojoHandle message_pipe_handle,
-                              UserPointer<const void> bytes,
+                              const void* bytes,
                               uint32_t num_bytes,
-                              UserPointer<const MojoHandle> handles,
+                              const MojoHandle* handles,
                               uint32_t num_handles,
                               MojoWriteMessageFlags flags) {
   scoped_refptr<Dispatcher> dispatcher(GetDispatcher(message_pipe_handle));
-  if (!dispatcher.get())
+  if (!dispatcher)
     return MOJO_RESULT_INVALID_ARGUMENT;
 
   // Easy case: not sending any handles.
@@ -246,11 +347,9 @@ MojoResult Core::WriteMessage(MojoHandle message_pipe_handle,
   // validity, even for dispatchers that don't support |WriteMessage()| and will
   // simply return failure unconditionally. It also breaks the usual
   // left-to-right verification order of arguments.)
-  if (num_handles > kMaxMessageNumHandles)
+  if (num_handles > GetConfiguration().max_message_num_handles)
     return MOJO_RESULT_RESOURCE_EXHAUSTED;
 
-  UserPointer<const MojoHandle>::Reader handles_reader(handles, num_handles);
-
   // We'll need to hold on to the dispatchers so that we can pass them on to
   // |WriteMessage()| and also so that we can unlock their locks afterwards
   // without accessing the handle table. These can be dumb pointers, since their
@@ -263,11 +362,8 @@ MojoResult Core::WriteMessage(MojoHandle message_pipe_handle,
   // handles from the handle table.
   {
     base::AutoLock locker(handle_table_lock_);
-    MojoResult result =
-        handle_table_.MarkBusyAndStartTransport(message_pipe_handle,
-                                                handles_reader.GetPointer(),
-                                                num_handles,
-                                                &transports);
+    MojoResult result = handle_table_.MarkBusyAndStartTransport(
+        message_pipe_handle, handles, num_handles, &transports);
     if (result != MOJO_RESULT_OK)
       return result;
   }
@@ -283,10 +379,9 @@ MojoResult Core::WriteMessage(MojoHandle message_pipe_handle,
   {
     base::AutoLock locker(handle_table_lock_);
     if (rv == MOJO_RESULT_OK) {
-      handle_table_.RemoveBusyHandles(handles_reader.GetPointer(), num_handles);
+      handle_table_.RemoveBusyHandles(handles, num_handles);
     } else {
-      handle_table_.RestoreBusyHandles(handles_reader.GetPointer(),
-                                       num_handles);
+      handle_table_.RestoreBusyHandles(handles, num_handles);
     }
   }
 
@@ -294,47 +389,41 @@ MojoResult Core::WriteMessage(MojoHandle message_pipe_handle,
 }
 
 MojoResult Core::ReadMessage(MojoHandle message_pipe_handle,
-                             UserPointer<void> bytes,
-                             UserPointer<uint32_t> num_bytes,
-                             UserPointer<MojoHandle> handles,
-                             UserPointer<uint32_t> num_handles,
+                             void* bytes,
+                             uint32_t* num_bytes,
+                             MojoHandle* handles,
+                             uint32_t* num_handles,
                              MojoReadMessageFlags flags) {
   scoped_refptr<Dispatcher> dispatcher(GetDispatcher(message_pipe_handle));
-  if (!dispatcher.get())
+  if (!dispatcher)
     return MOJO_RESULT_INVALID_ARGUMENT;
 
-  uint32_t num_handles_value = num_handles.IsNull() ? 0 : num_handles.Get();
-
   MojoResult rv;
+  uint32_t num_handles_value = num_handles ? *num_handles : 0;
   if (num_handles_value == 0) {
     // Easy case: won't receive any handles.
-    rv = dispatcher->ReadMessage(
-        bytes, num_bytes, nullptr, &num_handles_value, flags);
+    rv = dispatcher->ReadMessage(bytes, num_bytes, nullptr, &num_handles_value,
+                                 flags);
   } else {
     DispatcherVector dispatchers;
-    rv = dispatcher->ReadMessage(
-        bytes, num_bytes, &dispatchers, &num_handles_value, flags);
+    rv = dispatcher->ReadMessage(bytes, num_bytes, &dispatchers,
+                                 &num_handles_value, flags);
     if (!dispatchers.empty()) {
       DCHECK_EQ(rv, MOJO_RESULT_OK);
-      DCHECK(!num_handles.IsNull());
+      DCHECK(num_handles);
       DCHECK_LE(dispatchers.size(), static_cast<size_t>(num_handles_value));
 
       bool success;
-      UserPointer<MojoHandle>::Writer handles_writer(handles,
-                                                     dispatchers.size());
       {
         base::AutoLock locker(handle_table_lock_);
-        success = handle_table_.AddDispatcherVector(
-            dispatchers, handles_writer.GetPointer());
+        success = handle_table_.AddDispatcherVector(dispatchers, handles);
       }
-      if (success) {
-        handles_writer.Commit();
-      } else {
+      if (!success) {
         LOG(ERROR) << "Received message with " << dispatchers.size()
                    << " handles, but handle table full";
         // Close dispatchers (outside the lock).
         for (size_t i = 0; i < dispatchers.size(); i++) {
-          if (dispatchers[i].get())
+          if (dispatchers[i])
             dispatchers[i]->Close();
         }
         if (rv == MOJO_RESULT_OK)
@@ -343,25 +432,25 @@ MojoResult Core::ReadMessage(MojoHandle message_pipe_handle,
     }
   }
 
-  if (!num_handles.IsNull())
-    num_handles.Put(num_handles_value);
+  if (num_handles)
+    *num_handles = num_handles_value;
   return rv;
 }
 
 MojoResult Core::CreateDataPipe(
-    UserPointer<const MojoCreateDataPipeOptions> options,
-    UserPointer<MojoHandle> data_pipe_producer_handle,
-    UserPointer<MojoHandle> data_pipe_consumer_handle) {
+    const MojoCreateDataPipeOptions* options,
+    MojoHandle* data_pipe_producer_handle,
+    MojoHandle* data_pipe_consumer_handle) {
   MojoCreateDataPipeOptions validated_options = {};
   MojoResult result =
       DataPipe::ValidateCreateOptions(options, &validated_options);
   if (result != MOJO_RESULT_OK)
     return result;
 
-  scoped_refptr<DataPipeProducerDispatcher> producer_dispatcher(
-      new DataPipeProducerDispatcher());
-  scoped_refptr<DataPipeConsumerDispatcher> consumer_dispatcher(
-      new DataPipeConsumerDispatcher());
+  scoped_refptr<DataPipeProducerDispatcher> producer_dispatcher =
+      DataPipeProducerDispatcher::Create(validated_options);
+  scoped_refptr<DataPipeConsumerDispatcher> consumer_dispatcher =
+      DataPipeConsumerDispatcher::Create(validated_options);
 
   std::pair<MojoHandle, MojoHandle> handle_pair;
   {
@@ -378,34 +467,41 @@ MojoResult Core::CreateDataPipe(
   }
   DCHECK_NE(handle_pair.second, MOJO_HANDLE_INVALID);
 
-  scoped_refptr<DataPipe> data_pipe(new LocalDataPipe(validated_options));
-  producer_dispatcher->Init(data_pipe);
-  consumer_dispatcher->Init(data_pipe);
+  ScopedPlatformHandle server_handle, client_handle;
+#if defined(OS_WIN)
+  internal::g_broker->CreatePlatformChannelPair(&server_handle, &client_handle);
+#else
+  PlatformChannelPair channel_pair;
+  server_handle = channel_pair.PassServerHandle();
+  client_handle = channel_pair.PassClientHandle();
+#endif
+  producer_dispatcher->Init(std::move(server_handle), nullptr, 0u);
+  consumer_dispatcher->Init(std::move(client_handle), nullptr, 0u);
 
-  data_pipe_producer_handle.Put(handle_pair.first);
-  data_pipe_consumer_handle.Put(handle_pair.second);
+  *data_pipe_producer_handle = handle_pair.first;
+  *data_pipe_consumer_handle = handle_pair.second;
   return MOJO_RESULT_OK;
 }
 
 MojoResult Core::WriteData(MojoHandle data_pipe_producer_handle,
-                           UserPointer<const void> elements,
-                           UserPointer<uint32_t> num_bytes,
+                           const void* elements,
+                           uint32_t* num_bytes,
                            MojoWriteDataFlags flags) {
   scoped_refptr<Dispatcher> dispatcher(
       GetDispatcher(data_pipe_producer_handle));
-  if (!dispatcher.get())
+  if (!dispatcher)
     return MOJO_RESULT_INVALID_ARGUMENT;
 
   return dispatcher->WriteData(elements, num_bytes, flags);
 }
 
 MojoResult Core::BeginWriteData(MojoHandle data_pipe_producer_handle,
-                                UserPointer<void*> buffer,
-                                UserPointer<uint32_t> buffer_num_bytes,
+                                void** buffer,
+                                uint32_t* buffer_num_bytes,
                                 MojoWriteDataFlags flags) {
   scoped_refptr<Dispatcher> dispatcher(
       GetDispatcher(data_pipe_producer_handle));
-  if (!dispatcher.get())
+  if (!dispatcher)
     return MOJO_RESULT_INVALID_ARGUMENT;
 
   return dispatcher->BeginWriteData(buffer, buffer_num_bytes, flags);
@@ -415,31 +511,31 @@ MojoResult Core::EndWriteData(MojoHandle data_pipe_producer_handle,
                               uint32_t num_bytes_written) {
   scoped_refptr<Dispatcher> dispatcher(
       GetDispatcher(data_pipe_producer_handle));
-  if (!dispatcher.get())
+  if (!dispatcher)
     return MOJO_RESULT_INVALID_ARGUMENT;
 
   return dispatcher->EndWriteData(num_bytes_written);
 }
 
 MojoResult Core::ReadData(MojoHandle data_pipe_consumer_handle,
-                          UserPointer<void> elements,
-                          UserPointer<uint32_t> num_bytes,
+                          void* elements,
+                          uint32_t* num_bytes,
                           MojoReadDataFlags flags) {
   scoped_refptr<Dispatcher> dispatcher(
       GetDispatcher(data_pipe_consumer_handle));
-  if (!dispatcher.get())
+  if (!dispatcher)
     return MOJO_RESULT_INVALID_ARGUMENT;
 
   return dispatcher->ReadData(elements, num_bytes, flags);
 }
 
 MojoResult Core::BeginReadData(MojoHandle data_pipe_consumer_handle,
-                               UserPointer<const void*> buffer,
-                               UserPointer<uint32_t> buffer_num_bytes,
+                               const void** buffer,
+                               uint32_t* buffer_num_bytes,
                                MojoReadDataFlags flags) {
   scoped_refptr<Dispatcher> dispatcher(
       GetDispatcher(data_pipe_consumer_handle));
-  if (!dispatcher.get())
+  if (!dispatcher)
     return MOJO_RESULT_INVALID_ARGUMENT;
 
   return dispatcher->BeginReadData(buffer, buffer_num_bytes, flags);
@@ -449,16 +545,16 @@ MojoResult Core::EndReadData(MojoHandle data_pipe_consumer_handle,
                              uint32_t num_bytes_read) {
   scoped_refptr<Dispatcher> dispatcher(
       GetDispatcher(data_pipe_consumer_handle));
-  if (!dispatcher.get())
+  if (!dispatcher)
     return MOJO_RESULT_INVALID_ARGUMENT;
 
   return dispatcher->EndReadData(num_bytes_read);
 }
 
 MojoResult Core::CreateSharedBuffer(
-    UserPointer<const MojoCreateSharedBufferOptions> options,
+    const MojoCreateSharedBufferOptions* options,
     uint64_t num_bytes,
-    UserPointer<MojoHandle> shared_buffer_handle) {
+    MojoHandle* shared_buffer_handle) {
   MojoCreateSharedBufferOptions validated_options = {};
   MojoResult result = SharedBufferDispatcher::ValidateCreateOptions(
       options, &validated_options);
@@ -466,30 +562,29 @@ MojoResult Core::CreateSharedBuffer(
     return result;
 
   scoped_refptr<SharedBufferDispatcher> dispatcher;
-  result = SharedBufferDispatcher::Create(
-      platform_support(), validated_options, num_bytes, &dispatcher);
+  result = SharedBufferDispatcher::Create(platform_support_, validated_options,
+                                          num_bytes, &dispatcher);
   if (result != MOJO_RESULT_OK) {
-    DCHECK(!dispatcher.get());
+    DCHECK(!dispatcher);
     return result;
   }
 
-  MojoHandle h = AddDispatcher(dispatcher);
-  if (h == MOJO_HANDLE_INVALID) {
+  *shared_buffer_handle = AddDispatcher(dispatcher);
+  if (*shared_buffer_handle == MOJO_HANDLE_INVALID) {
     LOG(ERROR) << "Handle table full";
     dispatcher->Close();
     return MOJO_RESULT_RESOURCE_EXHAUSTED;
   }
 
-  shared_buffer_handle.Put(h);
   return MOJO_RESULT_OK;
 }
 
 MojoResult Core::DuplicateBufferHandle(
     MojoHandle buffer_handle,
-    UserPointer<const MojoDuplicateBufferHandleOptions> options,
-    UserPointer<MojoHandle> new_buffer_handle) {
+    const MojoDuplicateBufferHandleOptions* options,
+    MojoHandle* new_buffer_handle) {
   scoped_refptr<Dispatcher> dispatcher(GetDispatcher(buffer_handle));
-  if (!dispatcher.get())
+  if (!dispatcher)
     return MOJO_RESULT_INVALID_ARGUMENT;
 
   // Don't verify |options| here; that's the dispatcher's job.
@@ -499,27 +594,26 @@ MojoResult Core::DuplicateBufferHandle(
   if (result != MOJO_RESULT_OK)
     return result;
 
-  MojoHandle new_handle = AddDispatcher(new_dispatcher);
-  if (new_handle == MOJO_HANDLE_INVALID) {
+  *new_buffer_handle = AddDispatcher(new_dispatcher);
+  if (*new_buffer_handle == MOJO_HANDLE_INVALID) {
     LOG(ERROR) << "Handle table full";
     dispatcher->Close();
     return MOJO_RESULT_RESOURCE_EXHAUSTED;
   }
 
-  new_buffer_handle.Put(new_handle);
   return MOJO_RESULT_OK;
 }
 
 MojoResult Core::MapBuffer(MojoHandle buffer_handle,
                            uint64_t offset,
                            uint64_t num_bytes,
-                           UserPointer<void*> buffer,
+                           void** buffer,
                            MojoMapBufferFlags flags) {
   scoped_refptr<Dispatcher> dispatcher(GetDispatcher(buffer_handle));
-  if (!dispatcher.get())
+  if (!dispatcher)
     return MOJO_RESULT_INVALID_ARGUMENT;
 
-  scoped_ptr<embedder::PlatformSharedBufferMapping> mapping;
+  scoped_ptr<PlatformSharedBufferMapping> mapping;
   MojoResult result = dispatcher->MapBuffer(offset, num_bytes, flags, &mapping);
   if (result != MOJO_RESULT_OK)
     return result;
@@ -528,23 +622,23 @@ MojoResult Core::MapBuffer(MojoHandle buffer_handle,
   void* address = mapping->GetBase();
   {
     base::AutoLock locker(mapping_table_lock_);
-    result = mapping_table_.AddMapping(mapping.Pass());
+    result = mapping_table_.AddMapping(std::move(mapping));
   }
   if (result != MOJO_RESULT_OK)
     return result;
 
-  buffer.Put(address);
+  *buffer = address;
   return MOJO_RESULT_OK;
 }
 
-MojoResult Core::UnmapBuffer(UserPointer<void> buffer) {
+MojoResult Core::UnmapBuffer(void* buffer) {
   base::AutoLock locker(mapping_table_lock_);
-  return mapping_table_.RemoveMapping(buffer.GetPointerValue());
+  return mapping_table_.RemoveMapping(buffer);
 }
 
 // Note: We allow |handles| to repeat the same handle multiple times, since
 // different flags may be specified.
-// TODO(vtl): This incurs a performance cost in |RemoveWaiter()|. Analyze this
+// TODO(vtl): This incurs a performance cost in |Remove()|. Analyze this
 // more carefully and address it if necessary.
 MojoResult Core::WaitManyInternal(const MojoHandle* handles,
                                   const MojoHandleSignals* signals,
@@ -552,15 +646,20 @@ MojoResult Core::WaitManyInternal(const MojoHandle* handles,
                                   MojoDeadline deadline,
                                   uint32_t* result_index,
                                   HandleSignalsState* signals_states) {
+  CHECK(handles);
+  CHECK(signals);
   DCHECK_GT(num_handles, 0u);
-  DCHECK_EQ(*result_index, static_cast<uint32_t>(-1));
+  if (result_index) {
+    DCHECK_EQ(*result_index, static_cast<uint32_t>(-1));
+  }
 
   DispatcherVector dispatchers;
   dispatchers.reserve(num_handles);
   for (uint32_t i = 0; i < num_handles; i++) {
     scoped_refptr<Dispatcher> dispatcher = GetDispatcher(handles[i]);
-    if (!dispatcher.get()) {
-      *result_index = i;
+    if (!dispatcher) {
+      if (result_index)
+        *result_index = i;
       return MOJO_RESULT_INVALID_ARGUMENT;
     }
     dispatchers.push_back(dispatcher);
@@ -573,26 +672,30 @@ MojoResult Core::WaitManyInternal(const MojoHandle* handles,
   uint32_t i;
   MojoResult rv = MOJO_RESULT_OK;
   for (i = 0; i < num_handles; i++) {
-    rv = dispatchers[i]->AddWaiter(
+    rv = dispatchers[i]->AddAwakable(
         &waiter, signals[i], i, signals_states ? &signals_states[i] : nullptr);
     if (rv != MOJO_RESULT_OK) {
-      *result_index = i;
+      if (result_index)
+        *result_index = i;
       break;
     }
   }
   uint32_t num_added = i;
 
-  if (rv == MOJO_RESULT_ALREADY_EXISTS)
+  if (rv == MOJO_RESULT_ALREADY_EXISTS) {
     rv = MOJO_RESULT_OK;  // The i-th one is already "triggered".
-  else if (rv == MOJO_RESULT_OK)
-    rv = waiter.Wait(deadline, result_index);
+  } else if (rv == MOJO_RESULT_OK) {
+    uintptr_t uintptr_result = *result_index;
+    rv = waiter.Wait(deadline, &uintptr_result);
+    *result_index = static_cast<uint32_t>(uintptr_result);
+  }
 
   // Make sure no other dispatchers try to wake |waiter| for the current
   // |Wait()|/|WaitMany()| call. (Only after doing this can |waiter| be
   // destroyed, but this would still be required if the waiter were in TLS.)
   for (i = 0; i < num_added; i++) {
-    dispatchers[i]->RemoveWaiter(&waiter,
-                                 signals_states ? &signals_states[i] : nullptr);
+    dispatchers[i]->RemoveAwakable(
+        &waiter, signals_states ? &signals_states[i] : nullptr);
   }
   if (signals_states) {
     for (; i < num_handles; i++)
@@ -602,5 +705,5 @@ MojoResult Core::WaitManyInternal(const MojoHandle* handles,
   return rv;
 }
 
-}  // namespace system
+}  // namespace edk
 }  // namespace mojo
