@@ -4,34 +4,41 @@
 
 #include "mojo/edk/system/raw_channel.h"
 
+#include <stddef.h>
 #include <stdint.h>
-
+#include <stdio.h>
+#include <utility>
 #include <vector>
 
 #include "base/bind.h"
+#include "base/files/file_path.h"
+#include "base/files/file_util.h"
+#include "base/files/scoped_file.h"
+#include "base/files/scoped_temp_dir.h"
 #include "base/location.h"
 #include "base/logging.h"
-#include "base/macros.h"
 #include "base/memory/scoped_ptr.h"
 #include "base/memory/scoped_vector.h"
 #include "base/rand_util.h"
 #include "base/synchronization/lock.h"
 #include "base/synchronization/waitable_event.h"
 #include "base/test/test_io_thread.h"
-#include "base/threading/platform_thread.h"  // For |Sleep()|.
 #include "base/threading/simple_thread.h"
-#include "base/time/time.h"
 #include "build/build_config.h"
+#include "mojo/edk/embedder/embedder.h"
+#include "mojo/edk/embedder/embedder_internal.h"
 #include "mojo/edk/embedder/platform_channel_pair.h"
 #include "mojo/edk/embedder/platform_handle.h"
 #include "mojo/edk/embedder/scoped_platform_handle.h"
 #include "mojo/edk/system/message_in_transit.h"
 #include "mojo/edk/system/test_utils.h"
+#include "mojo/edk/system/transport_data.h"
 #include "mojo/edk/test/test_utils.h"
+#include "mojo/public/cpp/system/macros.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
 namespace mojo {
-namespace system {
+namespace edk {
 namespace {
 
 scoped_ptr<MessageInTransit> MakeTestMessage(uint32_t num_bytes) {
@@ -39,10 +46,8 @@ scoped_ptr<MessageInTransit> MakeTestMessage(uint32_t num_bytes) {
   for (size_t i = 0; i < num_bytes; i++)
     bytes[i] = static_cast<unsigned char>(i + num_bytes);
   return make_scoped_ptr(
-      new MessageInTransit(MessageInTransit::kTypeMessagePipeEndpoint,
-                           MessageInTransit::kSubtypeMessagePipeEndpointData,
-                           num_bytes,
-                           bytes.empty() ? nullptr : &bytes[0]));
+      new MessageInTransit(MessageInTransit::Type::MESSAGE,
+                           num_bytes, bytes.empty() ? nullptr : &bytes[0]));
 }
 
 bool CheckMessageData(const void* bytes, uint32_t num_bytes) {
@@ -55,16 +60,17 @@ bool CheckMessageData(const void* bytes, uint32_t num_bytes) {
 }
 
 void InitOnIOThread(RawChannel* raw_channel, RawChannel::Delegate* delegate) {
-  CHECK(raw_channel->Init(delegate));
+  raw_channel->Init(delegate);
+  raw_channel->EnsureLazyInitialized();
 }
 
-bool WriteTestMessageToHandle(const embedder::PlatformHandle& handle,
+bool WriteTestMessageToHandle(const PlatformHandle& handle,
                               uint32_t num_bytes) {
   scoped_ptr<MessageInTransit> message(MakeTestMessage(num_bytes));
 
   size_t write_size = 0;
-  mojo::test::BlockingWrite(
-      handle, message->main_buffer(), message->main_buffer_size(), &write_size);
+  test::BlockingWrite(handle, message->main_buffer(),
+                      message->main_buffer_size(), &write_size);
   return write_size == message->main_buffer_size();
 }
 
@@ -72,31 +78,33 @@ bool WriteTestMessageToHandle(const embedder::PlatformHandle& handle,
 
 class RawChannelTest : public testing::Test {
  public:
-  RawChannelTest() : io_thread_(base::TestIOThread::kManualStart) {}
+  RawChannelTest() {}
   ~RawChannelTest() override {}
 
   void SetUp() override {
-    embedder::PlatformChannelPair channel_pair;
+    PlatformChannelPair channel_pair;
     handles[0] = channel_pair.PassServerHandle();
-    handles[1] = channel_pair.PassClientHandle();
-    io_thread_.Start();
+    handles[1] = channel_pair.PassClientHandle();\
   }
 
   void TearDown() override {
-    io_thread_.Stop();
     handles[0].reset();
     handles[1].reset();
   }
 
- protected:
-  base::TestIOThread* io_thread() { return &io_thread_; }
+  void FlushIOThread() {
+    base::WaitableEvent event(false, false);
+    internal::g_io_thread_task_runner->PostTask(
+        FROM_HERE,
+        base::Bind(&base::WaitableEvent::Signal, base::Unretained(&event)));
+    event.Wait();
+  }
 
-  embedder::ScopedPlatformHandle handles[2];
+ protected:
+  ScopedPlatformHandle handles[2];
 
  private:
-  base::TestIOThread io_thread_;
-
-  DISALLOW_COPY_AND_ASSIGN(RawChannelTest);
+  MOJO_DISALLOW_COPY_AND_ASSIGN(RawChannelTest);
 };
 
 // RawChannelTest.WriteMessage -------------------------------------------------
@@ -109,7 +117,7 @@ class WriteOnlyRawChannelDelegate : public RawChannel::Delegate {
   // |RawChannel::Delegate| implementation:
   void OnReadMessage(
       const MessageInTransit::View& /*message_view*/,
-      embedder::ScopedPlatformHandleVectorPtr /*platform_handles*/) override {
+      ScopedPlatformHandleVectorPtr /*platform_handles*/) override {
     CHECK(false);  // Should not get called.
   }
   void OnError(Error error) override {
@@ -118,15 +126,15 @@ class WriteOnlyRawChannelDelegate : public RawChannel::Delegate {
   }
 
  private:
-  DISALLOW_COPY_AND_ASSIGN(WriteOnlyRawChannelDelegate);
+  MOJO_DISALLOW_COPY_AND_ASSIGN(WriteOnlyRawChannelDelegate);
 };
 
-static const int64_t kMessageReaderSleepMs = 1;
+static const unsigned kMessageReaderSleepMs = 1;
 static const size_t kMessageReaderMaxPollIterations = 3000;
 
 class TestMessageReaderAndChecker {
  public:
-  explicit TestMessageReaderAndChecker(embedder::PlatformHandle handle)
+  explicit TestMessageReaderAndChecker(PlatformHandle handle)
       : handle_(handle) {}
   ~TestMessageReaderAndChecker() { CHECK(bytes_.empty()); }
 
@@ -135,8 +143,7 @@ class TestMessageReaderAndChecker {
 
     for (size_t i = 0; i < kMessageReaderMaxPollIterations;) {
       size_t read_size = 0;
-      CHECK(mojo::test::NonBlockingRead(
-          handle_, buffer, sizeof(buffer), &read_size));
+      CHECK(test::NonBlockingRead(handle_, buffer, sizeof(buffer), &read_size));
 
       // Append newly-read data to |bytes_|.
       bytes_.insert(bytes_.end(), buffer, buffer + read_size);
@@ -144,8 +151,7 @@ class TestMessageReaderAndChecker {
       // If we have the header....
       size_t message_size;
       if (MessageInTransit::GetNextMessageSize(
-              bytes_.empty() ? nullptr : &bytes_[0],
-              bytes_.size(),
+              bytes_.empty() ? nullptr : &bytes_[0], bytes_.size(),
               &message_size)) {
         // If we've read the whole message....
         if (bytes_.size() >= message_size) {
@@ -172,8 +178,7 @@ class TestMessageReaderAndChecker {
 
       if (static_cast<size_t>(read_size) < sizeof(buffer)) {
         i++;
-        base::PlatformThread::Sleep(
-            base::TimeDelta::FromMilliseconds(kMessageReaderSleepMs));
+        test::Sleep(test::DeadlineFromMilliseconds(kMessageReaderSleepMs));
       }
     }
 
@@ -182,22 +187,22 @@ class TestMessageReaderAndChecker {
   }
 
  private:
-  const embedder::PlatformHandle handle_;
+  const PlatformHandle handle_;
 
   // The start of the received data should always be on a message boundary.
   std::vector<unsigned char> bytes_;
 
-  DISALLOW_COPY_AND_ASSIGN(TestMessageReaderAndChecker);
+  MOJO_DISALLOW_COPY_AND_ASSIGN(TestMessageReaderAndChecker);
 };
 
 // Tests writing (and verifies reading using our own custom reader).
 TEST_F(RawChannelTest, WriteMessage) {
   WriteOnlyRawChannelDelegate delegate;
-  scoped_ptr<RawChannel> rc(RawChannel::Create(handles[0].Pass()));
+  RawChannel* rc = RawChannel::Create(std::move(handles[0]));
   TestMessageReaderAndChecker checker(handles[1].get());
-  io_thread()->PostTaskAndWait(
+  internal::g_io_thread_task_runner->PostTask(
       FROM_HERE,
-      base::Bind(&InitOnIOThread, rc.get(), base::Unretained(&delegate)));
+      base::Bind(&InitOnIOThread, rc, base::Unretained(&delegate)));
 
   // Write and read, for a variety of sizes.
   for (uint32_t size = 1; size < 5 * 1000 * 1000; size += size / 2 + 1) {
@@ -211,8 +216,8 @@ TEST_F(RawChannelTest, WriteMessage) {
   for (uint32_t size = 1; size < 5 * 1000 * 1000; size += size / 2 + 1)
     EXPECT_TRUE(checker.ReadAndCheckNextMessage(size)) << size;
 
-  io_thread()->PostTaskAndWait(
-      FROM_HERE, base::Bind(&RawChannel::Shutdown, base::Unretained(rc.get())));
+  internal::g_io_thread_task_runner->PostTask(
+      FROM_HERE, base::Bind(&RawChannel::Shutdown, base::Unretained(rc)));
 }
 
 // RawChannelTest.OnReadMessage ------------------------------------------------
@@ -225,7 +230,7 @@ class ReadCheckerRawChannelDelegate : public RawChannel::Delegate {
   // |RawChannel::Delegate| implementation (called on the I/O thread):
   void OnReadMessage(
       const MessageInTransit::View& message_view,
-      embedder::ScopedPlatformHandleVectorPtr platform_handles) override {
+      ScopedPlatformHandleVectorPtr platform_handles) override {
     EXPECT_FALSE(platform_handles);
 
     size_t position;
@@ -273,16 +278,16 @@ class ReadCheckerRawChannelDelegate : public RawChannel::Delegate {
   std::vector<uint32_t> expected_sizes_;
   size_t position_;
 
-  DISALLOW_COPY_AND_ASSIGN(ReadCheckerRawChannelDelegate);
+  MOJO_DISALLOW_COPY_AND_ASSIGN(ReadCheckerRawChannelDelegate);
 };
 
 // Tests reading (writing using our own custom writer).
 TEST_F(RawChannelTest, OnReadMessage) {
   ReadCheckerRawChannelDelegate delegate;
-  scoped_ptr<RawChannel> rc(RawChannel::Create(handles[0].Pass()));
-  io_thread()->PostTaskAndWait(
+  RawChannel* rc = RawChannel::Create(std::move(handles[0]));
+  internal::g_io_thread_task_runner->PostTask(
       FROM_HERE,
-      base::Bind(&InitOnIOThread, rc.get(), base::Unretained(&delegate)));
+      base::Bind(&InitOnIOThread, rc, base::Unretained(&delegate)));
 
   // Write and read, for a variety of sizes.
   for (uint32_t size = 1; size < 5 * 1000 * 1000; size += size / 2 + 1) {
@@ -303,8 +308,8 @@ TEST_F(RawChannelTest, OnReadMessage) {
     EXPECT_TRUE(WriteTestMessageToHandle(handles[1].get(), size));
   delegate.Wait();
 
-  io_thread()->PostTaskAndWait(
-      FROM_HERE, base::Bind(&RawChannel::Shutdown, base::Unretained(rc.get())));
+  internal::g_io_thread_task_runner->PostTask(
+      FROM_HERE, base::Bind(&RawChannel::Shutdown, base::Unretained(rc)));
 }
 
 // RawChannelTest.WriteMessageAndOnReadMessage ---------------------------------
@@ -331,7 +336,7 @@ class RawChannelWriterThread : public base::SimpleThread {
   RawChannel* const raw_channel_;
   size_t left_to_write_;
 
-  DISALLOW_COPY_AND_ASSIGN(RawChannelWriterThread);
+  MOJO_DISALLOW_COPY_AND_ASSIGN(RawChannelWriterThread);
 };
 
 class ReadCountdownRawChannelDelegate : public RawChannel::Delegate {
@@ -343,7 +348,7 @@ class ReadCountdownRawChannelDelegate : public RawChannel::Delegate {
   // |RawChannel::Delegate| implementation (called on the I/O thread):
   void OnReadMessage(
       const MessageInTransit::View& message_view,
-      embedder::ScopedPlatformHandleVectorPtr platform_handles) override {
+      ScopedPlatformHandleVectorPtr platform_handles) override {
     EXPECT_FALSE(platform_handles);
 
     EXPECT_LT(count_, expected_count_);
@@ -368,33 +373,33 @@ class ReadCountdownRawChannelDelegate : public RawChannel::Delegate {
   size_t expected_count_;
   size_t count_;
 
-  DISALLOW_COPY_AND_ASSIGN(ReadCountdownRawChannelDelegate);
+  MOJO_DISALLOW_COPY_AND_ASSIGN(ReadCountdownRawChannelDelegate);
 };
 
 TEST_F(RawChannelTest, WriteMessageAndOnReadMessage) {
   static const size_t kNumWriterThreads = 10;
-  static const size_t kNumWriteMessagesPerThread = 4000;
+  static const size_t kNumWriteMessagesPerThread = 400;
 
   WriteOnlyRawChannelDelegate writer_delegate;
-  scoped_ptr<RawChannel> writer_rc(RawChannel::Create(handles[0].Pass()));
-  io_thread()->PostTaskAndWait(FROM_HERE,
-                               base::Bind(&InitOnIOThread,
-                                          writer_rc.get(),
-                                          base::Unretained(&writer_delegate)));
+  RawChannel* writer_rc = RawChannel::Create(std::move(handles[0]));
+  internal::g_io_thread_task_runner->PostTask(
+    FROM_HERE,
+    base::Bind(&InitOnIOThread, writer_rc,
+               base::Unretained(&writer_delegate)));
 
   ReadCountdownRawChannelDelegate reader_delegate(kNumWriterThreads *
                                                   kNumWriteMessagesPerThread);
-  scoped_ptr<RawChannel> reader_rc(RawChannel::Create(handles[1].Pass()));
-  io_thread()->PostTaskAndWait(FROM_HERE,
-                               base::Bind(&InitOnIOThread,
-                                          reader_rc.get(),
-                                          base::Unretained(&reader_delegate)));
+  RawChannel* reader_rc = RawChannel::Create(std::move(handles[1]));
+  internal::g_io_thread_task_runner->PostTask(
+    FROM_HERE,
+    base::Bind(&InitOnIOThread, reader_rc,
+               base::Unretained(&reader_delegate)));
 
   {
     ScopedVector<RawChannelWriterThread> writer_threads;
     for (size_t i = 0; i < kNumWriterThreads; i++) {
       writer_threads.push_back(new RawChannelWriterThread(
-          writer_rc.get(), kNumWriteMessagesPerThread));
+          writer_rc, kNumWriteMessagesPerThread));
     }
     for (size_t i = 0; i < writer_threads.size(); i++)
       writer_threads[i]->Start();
@@ -402,18 +407,18 @@ TEST_F(RawChannelTest, WriteMessageAndOnReadMessage) {
 
   // Sleep a bit, to let any extraneous reads be processed. (There shouldn't be
   // any, but we want to know about them.)
-  base::PlatformThread::Sleep(base::TimeDelta::FromMilliseconds(100));
+  test::Sleep(test::DeadlineFromMilliseconds(100));
 
   // Wait for reading to finish.
   reader_delegate.Wait();
 
-  io_thread()->PostTaskAndWait(
+  internal::g_io_thread_task_runner->PostTask(
       FROM_HERE,
-      base::Bind(&RawChannel::Shutdown, base::Unretained(reader_rc.get())));
+      base::Bind(&RawChannel::Shutdown, base::Unretained(reader_rc)));
 
-  io_thread()->PostTaskAndWait(
+  internal::g_io_thread_task_runner->PostTask(
       FROM_HERE,
-      base::Bind(&RawChannel::Shutdown, base::Unretained(writer_rc.get())));
+      base::Bind(&RawChannel::Shutdown, base::Unretained(writer_rc)));
 }
 
 // RawChannelTest.OnError ------------------------------------------------------
@@ -469,16 +474,17 @@ class ErrorRecordingRawChannelDelegate
   bool expecting_read_error_;
   bool expecting_write_error_;
 
-  DISALLOW_COPY_AND_ASSIGN(ErrorRecordingRawChannelDelegate);
+  MOJO_DISALLOW_COPY_AND_ASSIGN(ErrorRecordingRawChannelDelegate);
 };
 
 // Tests (fatal) errors.
 TEST_F(RawChannelTest, OnError) {
   ErrorRecordingRawChannelDelegate delegate(0, true, true);
-  scoped_ptr<RawChannel> rc(RawChannel::Create(handles[0].Pass()));
-  io_thread()->PostTaskAndWait(
+  RawChannel* rc = RawChannel::Create(std::move(handles[0]));
+  internal::g_io_thread_task_runner->PostTask(
       FROM_HERE,
-      base::Bind(&InitOnIOThread, rc.get(), base::Unretained(&delegate)));
+      base::Bind(&InitOnIOThread, rc, base::Unretained(&delegate)));
+  FlushIOThread();
 
   // Close the handle of the other end, which should make writing fail.
   handles[1].reset();
@@ -495,10 +501,10 @@ TEST_F(RawChannelTest, OnError) {
 
   // Sleep a bit, to make sure we don't get another |OnError()|
   // notification. (If we actually get another one, |OnError()| crashes.)
-  base::PlatformThread::Sleep(base::TimeDelta::FromMilliseconds(20));
+  test::Sleep(test::DeadlineFromMilliseconds(20));
 
-  io_thread()->PostTaskAndWait(
-      FROM_HERE, base::Bind(&RawChannel::Shutdown, base::Unretained(rc.get())));
+  internal::g_io_thread_task_runner->PostTask(
+      FROM_HERE, base::Bind(&RawChannel::Shutdown, base::Unretained(rc)));
 }
 
 // RawChannelTest.ReadUnaffectedByWriteError -----------------------------------
@@ -518,10 +524,11 @@ TEST_F(RawChannelTest, ReadUnaffectedByWriteError) {
   // Only start up reading here. The system buffer should still contain the
   // messages that were written.
   ErrorRecordingRawChannelDelegate delegate(kMessageCount, true, true);
-  scoped_ptr<RawChannel> rc(RawChannel::Create(handles[0].Pass()));
-  io_thread()->PostTaskAndWait(
+  RawChannel* rc = RawChannel::Create(std::move(handles[0]));
+  internal::g_io_thread_task_runner->PostTask(
       FROM_HERE,
-      base::Bind(&InitOnIOThread, rc.get(), base::Unretained(&delegate)));
+      base::Bind(&InitOnIOThread, rc, base::Unretained(&delegate)));
+  FlushIOThread();
 
   EXPECT_FALSE(rc->WriteMessage(MakeTestMessage(1)));
 
@@ -534,155 +541,120 @@ TEST_F(RawChannelTest, ReadUnaffectedByWriteError) {
   // And then we should get a read error.
   delegate.WaitForReadError();
 
-  io_thread()->PostTaskAndWait(
-      FROM_HERE, base::Bind(&RawChannel::Shutdown, base::Unretained(rc.get())));
+  internal::g_io_thread_task_runner->PostTask(
+      FROM_HERE, base::Bind(&RawChannel::Shutdown, base::Unretained(rc)));
 }
 
-// RawChannelTest.WriteMessageAfterShutdown ------------------------------------
+// RawChannelTest.ReadWritePlatformHandles -------------------------------------
 
-// Makes sure that calling |WriteMessage()| after |Shutdown()| behaves
-// correctly.
-TEST_F(RawChannelTest, WriteMessageAfterShutdown) {
-  WriteOnlyRawChannelDelegate delegate;
-  scoped_ptr<RawChannel> rc(RawChannel::Create(handles[0].Pass()));
-  io_thread()->PostTaskAndWait(
-      FROM_HERE,
-      base::Bind(&InitOnIOThread, rc.get(), base::Unretained(&delegate)));
-  io_thread()->PostTaskAndWait(
-      FROM_HERE, base::Bind(&RawChannel::Shutdown, base::Unretained(rc.get())));
-
-  EXPECT_FALSE(rc->WriteMessage(MakeTestMessage(1)));
-}
-
-// RawChannelTest.ShutdownOnReadMessage ----------------------------------------
-
-class ShutdownOnReadMessageRawChannelDelegate : public RawChannel::Delegate {
+class ReadPlatformHandlesCheckerRawChannelDelegate
+    : public RawChannel::Delegate {
  public:
-  explicit ShutdownOnReadMessageRawChannelDelegate(RawChannel* raw_channel)
-      : raw_channel_(raw_channel),
-        done_event_(false, false),
-        did_shutdown_(false) {}
-  ~ShutdownOnReadMessageRawChannelDelegate() override {}
+  ReadPlatformHandlesCheckerRawChannelDelegate() : done_event_(false, false) {}
+  ~ReadPlatformHandlesCheckerRawChannelDelegate() override {}
 
   // |RawChannel::Delegate| implementation (called on the I/O thread):
   void OnReadMessage(
       const MessageInTransit::View& message_view,
-      embedder::ScopedPlatformHandleVectorPtr platform_handles) override {
-    EXPECT_FALSE(platform_handles);
-    EXPECT_FALSE(did_shutdown_);
-    EXPECT_TRUE(
-        CheckMessageData(message_view.bytes(), message_view.num_bytes()));
-    raw_channel_->Shutdown();
-    did_shutdown_ = true;
+      ScopedPlatformHandleVectorPtr platform_handles) override {
+    const char kHello[] = "hello";
+
+    EXPECT_EQ(sizeof(kHello), message_view.num_bytes());
+    EXPECT_STREQ(kHello, static_cast<const char*>(message_view.bytes()));
+
+    ASSERT_TRUE(platform_handles);
+    ASSERT_EQ(2u, platform_handles->size());
+    ScopedPlatformHandle h1(platform_handles->at(0));
+    EXPECT_TRUE(h1.is_valid());
+    ScopedPlatformHandle h2(platform_handles->at(1));
+    EXPECT_TRUE(h2.is_valid());
+    platform_handles->clear();
+
+    {
+      char buffer[100] = {};
+
+      base::ScopedFILE fp(test::FILEFromPlatformHandle(std::move(h1), "rb"));
+      EXPECT_TRUE(fp);
+      rewind(fp.get());
+      EXPECT_EQ(1u, fread(buffer, 1, sizeof(buffer), fp.get()));
+      EXPECT_EQ('1', buffer[0]);
+    }
+
+    {
+      char buffer[100] = {};
+      base::ScopedFILE fp(test::FILEFromPlatformHandle(std::move(h2), "rb"));
+      EXPECT_TRUE(fp);
+      rewind(fp.get());
+      EXPECT_EQ(1u, fread(buffer, 1, sizeof(buffer), fp.get()));
+      EXPECT_EQ('2', buffer[0]);
+    }
+
     done_event_.Signal();
   }
-  void OnError(Error /*error*/) override {
-    CHECK(false);  // Should not get called.
-  }
-
-  // Waits for shutdown.
-  void Wait() {
-    done_event_.Wait();
-    EXPECT_TRUE(did_shutdown_);
-  }
-
- private:
-  RawChannel* const raw_channel_;
-  base::WaitableEvent done_event_;
-  bool did_shutdown_;
-
-  DISALLOW_COPY_AND_ASSIGN(ShutdownOnReadMessageRawChannelDelegate);
-};
-
-TEST_F(RawChannelTest, ShutdownOnReadMessage) {
-  // Write a few messages into the other end.
-  for (size_t count = 0; count < 5; count++)
-    EXPECT_TRUE(WriteTestMessageToHandle(handles[1].get(), 10));
-
-  scoped_ptr<RawChannel> rc(RawChannel::Create(handles[0].Pass()));
-  ShutdownOnReadMessageRawChannelDelegate delegate(rc.get());
-  io_thread()->PostTaskAndWait(
-      FROM_HERE,
-      base::Bind(&InitOnIOThread, rc.get(), base::Unretained(&delegate)));
-
-  // Wait for the delegate, which will shut the |RawChannel| down.
-  delegate.Wait();
-}
-
-// RawChannelTest.ShutdownOnError{Read, Write} ---------------------------------
-
-class ShutdownOnErrorRawChannelDelegate : public RawChannel::Delegate {
- public:
-  ShutdownOnErrorRawChannelDelegate(RawChannel* raw_channel,
-                                    Error shutdown_on_error_type)
-      : raw_channel_(raw_channel),
-        shutdown_on_error_type_(shutdown_on_error_type),
-        done_event_(false, false),
-        did_shutdown_(false) {}
-  ~ShutdownOnErrorRawChannelDelegate() override {}
-
-  // |RawChannel::Delegate| implementation (called on the I/O thread):
-  void OnReadMessage(
-      const MessageInTransit::View& /*message_view*/,
-      embedder::ScopedPlatformHandleVectorPtr /*platform_handles*/) override {
-    CHECK(false);  // Should not get called.
-  }
   void OnError(Error error) override {
-    EXPECT_FALSE(did_shutdown_);
-    if (error != shutdown_on_error_type_)
-      return;
-    raw_channel_->Shutdown();
-    did_shutdown_ = true;
-    done_event_.Signal();
+    // We'll get a read (shutdown) error when the connection is closed.
+    CHECK_EQ(error, ERROR_READ_SHUTDOWN);
   }
 
-  // Waits for shutdown.
-  void Wait() {
-    done_event_.Wait();
-    EXPECT_TRUE(did_shutdown_);
-  }
+  void Wait() { done_event_.Wait(); }
 
  private:
-  RawChannel* const raw_channel_;
-  const Error shutdown_on_error_type_;
   base::WaitableEvent done_event_;
-  bool did_shutdown_;
 
-  DISALLOW_COPY_AND_ASSIGN(ShutdownOnErrorRawChannelDelegate);
+  MOJO_DISALLOW_COPY_AND_ASSIGN(ReadPlatformHandlesCheckerRawChannelDelegate);
 };
 
-TEST_F(RawChannelTest, ShutdownOnErrorRead) {
-  scoped_ptr<RawChannel> rc(RawChannel::Create(handles[0].Pass()));
-  ShutdownOnErrorRawChannelDelegate delegate(
-      rc.get(), RawChannel::Delegate::ERROR_READ_SHUTDOWN);
-  io_thread()->PostTaskAndWait(
+TEST_F(RawChannelTest, ReadWritePlatformHandles) {
+  base::ScopedTempDir temp_dir;
+  ASSERT_TRUE(temp_dir.CreateUniqueTempDir());
+
+  WriteOnlyRawChannelDelegate write_delegate;
+  RawChannel* rc_write = RawChannel::Create(std::move(handles[0]));
+  internal::g_io_thread_task_runner->PostTask(
       FROM_HERE,
-      base::Bind(&InitOnIOThread, rc.get(), base::Unretained(&delegate)));
+      base::Bind(&InitOnIOThread, rc_write, base::Unretained(&write_delegate)));
 
-  // Close the handle of the other end, which should stuff fail.
-  handles[1].reset();
-
-  // Wait for the delegate, which will shut the |RawChannel| down.
-  delegate.Wait();
-}
-
-TEST_F(RawChannelTest, ShutdownOnErrorWrite) {
-  scoped_ptr<RawChannel> rc(RawChannel::Create(handles[0].Pass()));
-  ShutdownOnErrorRawChannelDelegate delegate(rc.get(),
-                                             RawChannel::Delegate::ERROR_WRITE);
-  io_thread()->PostTaskAndWait(
+  ReadPlatformHandlesCheckerRawChannelDelegate read_delegate;
+  RawChannel* rc_read = RawChannel::Create(std::move(handles[1]));
+  internal::g_io_thread_task_runner->PostTask(
       FROM_HERE,
-      base::Bind(&InitOnIOThread, rc.get(), base::Unretained(&delegate)));
+      base::Bind(&InitOnIOThread, rc_read, base::Unretained(&read_delegate)));
 
-  // Close the handle of the other end, which should stuff fail.
-  handles[1].reset();
+  base::FilePath unused;
+  base::ScopedFILE fp1(
+      base::CreateAndOpenTemporaryFileInDir(temp_dir.path(), &unused));
+  EXPECT_EQ(1u, fwrite("1", 1, 1, fp1.get()));
+  base::ScopedFILE fp2(
+      base::CreateAndOpenTemporaryFileInDir(temp_dir.path(), &unused));
+  EXPECT_EQ(1u, fwrite("2", 1, 1, fp2.get()));
 
-  EXPECT_FALSE(rc->WriteMessage(MakeTestMessage(1)));
+  {
+    const char kHello[] = "hello";
+    ScopedPlatformHandleVectorPtr platform_handles(new PlatformHandleVector());
+    platform_handles->push_back(
+        test::PlatformHandleFromFILE(std::move(fp1)).release());
+    platform_handles->push_back(
+        test::PlatformHandleFromFILE(std::move(fp2)).release());
 
-  // Wait for the delegate, which will shut the |RawChannel| down.
-  delegate.Wait();
+    scoped_ptr<MessageInTransit> message(
+        new MessageInTransit(MessageInTransit::Type::MESSAGE,
+                             sizeof(kHello), kHello));
+    message->SetTransportData(make_scoped_ptr(
+        new TransportData(std::move(platform_handles),
+                          rc_write->GetSerializedPlatformHandleSize())));
+    EXPECT_TRUE(rc_write->WriteMessage(std::move(message)));
+  }
+
+  read_delegate.Wait();
+
+  internal::g_io_thread_task_runner->PostTask(
+      FROM_HERE,
+      base::Bind(&RawChannel::Shutdown, base::Unretained(rc_read)));
+  internal::g_io_thread_task_runner->PostTask(
+      FROM_HERE,
+      base::Bind(&RawChannel::Shutdown, base::Unretained(rc_write)));
 }
 
 }  // namespace
-}  // namespace system
+}  // namespace edk
 }  // namespace mojo
