@@ -3,29 +3,29 @@
 // found in the LICENSE file.
 
 // NOTE(vtl): Some of these tests are inherently flaky (e.g., if run on a
-// heavily-loaded system). Sorry. |test::EpsilonTimeout()| may be increased to
+// heavily-loaded system). Sorry. |test::EpsilonDeadline()| may be increased to
 // increase tolerance and reduce observed flakiness (though doing so reduces the
 // meaningfulness of the test).
 
 #include "mojo/edk/system/simple_dispatcher.h"
 
+#include <stdint.h>
+
 #include "base/logging.h"
-#include "base/macros.h"
 #include "base/memory/ref_counted.h"
 #include "base/memory/scoped_vector.h"
 #include "base/synchronization/lock.h"
-#include "base/threading/platform_thread.h"  // For |Sleep()|.
-#include "base/time/time.h"
 #include "mojo/edk/system/test_utils.h"
 #include "mojo/edk/system/waiter.h"
 #include "mojo/edk/system/waiter_test_utils.h"
+#include "mojo/public/cpp/system/macros.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
 namespace mojo {
-namespace system {
+namespace edk {
 namespace {
 
-class MockSimpleDispatcher : public SimpleDispatcher {
+class MockSimpleDispatcher final : public SimpleDispatcher {
  public:
   MockSimpleDispatcher()
       : state_(MOJO_HANDLE_SIGNAL_NONE,
@@ -60,7 +60,7 @@ class MockSimpleDispatcher : public SimpleDispatcher {
     HandleSignalsStateChangedNoLock();
   }
 
-  Type GetType() const override { return kTypeUnknown; }
+  Type GetType() const override { return Type::UNKNOWN; }
 
  private:
   friend class base::RefCountedThreadSafe<MockSimpleDispatcher>;
@@ -82,7 +82,7 @@ class MockSimpleDispatcher : public SimpleDispatcher {
   // Protected by |lock()|:
   HandleSignalsState state_;
 
-  DISALLOW_COPY_AND_ASSIGN(MockSimpleDispatcher);
+  MOJO_DISALLOW_COPY_AND_ASSIGN(MockSimpleDispatcher);
 };
 
 #if defined(OS_WIN)
@@ -96,7 +96,7 @@ TEST(SimpleDispatcherTest, MAYBE_Basic) {
 
   scoped_refptr<MockSimpleDispatcher> d(new MockSimpleDispatcher());
   Waiter w;
-  uint32_t context = 0;
+  uintptr_t context = 0;
   HandleSignalsState hss;
 
   // Try adding a readable waiter when already readable.
@@ -104,7 +104,7 @@ TEST(SimpleDispatcherTest, MAYBE_Basic) {
   d->SetSatisfiedSignals(MOJO_HANDLE_SIGNAL_READABLE);
   hss = HandleSignalsState();
   EXPECT_EQ(MOJO_RESULT_ALREADY_EXISTS,
-            d->AddWaiter(&w, MOJO_HANDLE_SIGNAL_READABLE, 0, &hss));
+            d->AddAwakable(&w, MOJO_HANDLE_SIGNAL_READABLE, 0, &hss));
   EXPECT_EQ(MOJO_HANDLE_SIGNAL_READABLE, hss.satisfied_signals);
   EXPECT_EQ(MOJO_HANDLE_SIGNAL_READABLE | MOJO_HANDLE_SIGNAL_WRITABLE,
             hss.satisfiable_signals);
@@ -114,14 +114,14 @@ TEST(SimpleDispatcherTest, MAYBE_Basic) {
   w.Init();
   d->SetSatisfiedSignals(MOJO_HANDLE_SIGNAL_READABLE);
   ASSERT_EQ(MOJO_RESULT_OK,
-            d->AddWaiter(&w, MOJO_HANDLE_SIGNAL_WRITABLE, 1, nullptr));
+            d->AddAwakable(&w, MOJO_HANDLE_SIGNAL_WRITABLE, 1, nullptr));
   d->SetSatisfiedSignals(MOJO_HANDLE_SIGNAL_WRITABLE);
   stopwatch.Start();
   EXPECT_EQ(MOJO_RESULT_OK, w.Wait(MOJO_DEADLINE_INDEFINITE, &context));
-  EXPECT_LT(stopwatch.Elapsed(), test::EpsilonTimeout());
+  EXPECT_LT(stopwatch.Elapsed(), test::EpsilonDeadline());
   EXPECT_EQ(1u, context);
   hss = HandleSignalsState();
-  d->RemoveWaiter(&w, &hss);
+  d->RemoveAwakable(&w, &hss);
   EXPECT_EQ(MOJO_HANDLE_SIGNAL_WRITABLE, hss.satisfied_signals);
   EXPECT_EQ(MOJO_HANDLE_SIGNAL_READABLE | MOJO_HANDLE_SIGNAL_WRITABLE,
             hss.satisfiable_signals);
@@ -130,14 +130,14 @@ TEST(SimpleDispatcherTest, MAYBE_Basic) {
   w.Init();
   d->SetSatisfiedSignals(MOJO_HANDLE_SIGNAL_READABLE);
   ASSERT_EQ(MOJO_RESULT_OK,
-            d->AddWaiter(&w, MOJO_HANDLE_SIGNAL_WRITABLE, 2, nullptr));
+            d->AddAwakable(&w, MOJO_HANDLE_SIGNAL_WRITABLE, 2, nullptr));
   d->SetSatisfiedSignals(MOJO_HANDLE_SIGNAL_WRITABLE);
   stopwatch.Start();
   EXPECT_EQ(MOJO_RESULT_OK, w.Wait(0, &context));
-  EXPECT_LT(stopwatch.Elapsed(), test::EpsilonTimeout());
+  EXPECT_LT(stopwatch.Elapsed(), test::EpsilonDeadline());
   EXPECT_EQ(2u, context);
   hss = HandleSignalsState();
-  d->RemoveWaiter(&w, &hss);
+  d->RemoveAwakable(&w, &hss);
   EXPECT_EQ(MOJO_HANDLE_SIGNAL_WRITABLE, hss.satisfied_signals);
   EXPECT_EQ(MOJO_HANDLE_SIGNAL_READABLE | MOJO_HANDLE_SIGNAL_WRITABLE,
             hss.satisfiable_signals);
@@ -146,15 +146,14 @@ TEST(SimpleDispatcherTest, MAYBE_Basic) {
   w.Init();
   d->SetSatisfiedSignals(MOJO_HANDLE_SIGNAL_READABLE);
   ASSERT_EQ(MOJO_RESULT_OK,
-            d->AddWaiter(&w, MOJO_HANDLE_SIGNAL_WRITABLE, 3, nullptr));
+            d->AddAwakable(&w, MOJO_HANDLE_SIGNAL_WRITABLE, 3, nullptr));
   d->SetSatisfiedSignals(MOJO_HANDLE_SIGNAL_WRITABLE);
   stopwatch.Start();
-  EXPECT_EQ(MOJO_RESULT_OK,
-            w.Wait(2 * test::EpsilonTimeout().InMicroseconds(), &context));
-  EXPECT_LT(stopwatch.Elapsed(), test::EpsilonTimeout());
+  EXPECT_EQ(MOJO_RESULT_OK, w.Wait(2 * test::EpsilonDeadline(), &context));
+  EXPECT_LT(stopwatch.Elapsed(), test::EpsilonDeadline());
   EXPECT_EQ(3u, context);
   hss = HandleSignalsState();
-  d->RemoveWaiter(&w, &hss);
+  d->RemoveAwakable(&w, &hss);
   EXPECT_EQ(MOJO_HANDLE_SIGNAL_WRITABLE, hss.satisfied_signals);
   EXPECT_EQ(MOJO_HANDLE_SIGNAL_READABLE | MOJO_HANDLE_SIGNAL_WRITABLE,
             hss.satisfiable_signals);
@@ -163,12 +162,12 @@ TEST(SimpleDispatcherTest, MAYBE_Basic) {
   w.Init();
   d->SetSatisfiedSignals(MOJO_HANDLE_SIGNAL_READABLE);
   ASSERT_EQ(MOJO_RESULT_OK,
-            d->AddWaiter(&w, MOJO_HANDLE_SIGNAL_WRITABLE, 4, nullptr));
+            d->AddAwakable(&w, MOJO_HANDLE_SIGNAL_WRITABLE, 4, nullptr));
   stopwatch.Start();
   EXPECT_EQ(MOJO_RESULT_DEADLINE_EXCEEDED, w.Wait(0, nullptr));
-  EXPECT_LT(stopwatch.Elapsed(), test::EpsilonTimeout());
+  EXPECT_LT(stopwatch.Elapsed(), test::EpsilonDeadline());
   hss = HandleSignalsState();
-  d->RemoveWaiter(&w, &hss);
+  d->RemoveAwakable(&w, &hss);
   EXPECT_EQ(MOJO_HANDLE_SIGNAL_READABLE, hss.satisfied_signals);
   EXPECT_EQ(MOJO_HANDLE_SIGNAL_READABLE | MOJO_HANDLE_SIGNAL_WRITABLE,
             hss.satisfiable_signals);
@@ -178,15 +177,15 @@ TEST(SimpleDispatcherTest, MAYBE_Basic) {
   w.Init();
   d->SetSatisfiedSignals(MOJO_HANDLE_SIGNAL_READABLE);
   ASSERT_EQ(MOJO_RESULT_OK,
-            d->AddWaiter(&w, MOJO_HANDLE_SIGNAL_WRITABLE, 5, nullptr));
+            d->AddAwakable(&w, MOJO_HANDLE_SIGNAL_WRITABLE, 5, nullptr));
   stopwatch.Start();
   EXPECT_EQ(MOJO_RESULT_DEADLINE_EXCEEDED,
-            w.Wait(2 * test::EpsilonTimeout().InMicroseconds(), nullptr));
-  base::TimeDelta elapsed = stopwatch.Elapsed();
-  EXPECT_GT(elapsed, (2 - 1) * test::EpsilonTimeout());
-  EXPECT_LT(elapsed, (2 + 1) * test::EpsilonTimeout());
+            w.Wait(2 * test::EpsilonDeadline(), nullptr));
+  MojoDeadline elapsed = stopwatch.Elapsed();
+  EXPECT_GT(elapsed, (2 - 1) * test::EpsilonDeadline());
+  EXPECT_LT(elapsed, (2 + 1) * test::EpsilonDeadline());
   hss = HandleSignalsState();
-  d->RemoveWaiter(&w, &hss);
+  d->RemoveAwakable(&w, &hss);
   EXPECT_EQ(MOJO_HANDLE_SIGNAL_READABLE, hss.satisfied_signals);
   EXPECT_EQ(MOJO_HANDLE_SIGNAL_READABLE | MOJO_HANDLE_SIGNAL_WRITABLE,
             hss.satisfiable_signals);
@@ -199,7 +198,7 @@ TEST(SimpleDispatcherTest, BasicUnsatisfiable) {
 
   scoped_refptr<MockSimpleDispatcher> d(new MockSimpleDispatcher());
   Waiter w;
-  uint32_t context = 0;
+  uintptr_t context = 0;
   HandleSignalsState hss;
 
   // Try adding a writable waiter when it can never be writable.
@@ -208,7 +207,7 @@ TEST(SimpleDispatcherTest, BasicUnsatisfiable) {
   d->SetSatisfiedSignals(0);
   hss = HandleSignalsState();
   EXPECT_EQ(MOJO_RESULT_FAILED_PRECONDITION,
-            d->AddWaiter(&w, MOJO_HANDLE_SIGNAL_WRITABLE, 1, &hss));
+            d->AddAwakable(&w, MOJO_HANDLE_SIGNAL_WRITABLE, 1, &hss));
   EXPECT_EQ(0u, hss.satisfied_signals);
   EXPECT_EQ(MOJO_HANDLE_SIGNAL_READABLE, hss.satisfiable_signals);
   // Shouldn't need to remove the waiter (it was not added).
@@ -218,15 +217,15 @@ TEST(SimpleDispatcherTest, BasicUnsatisfiable) {
   d->SetSatisfiableSignals(MOJO_HANDLE_SIGNAL_READABLE |
                            MOJO_HANDLE_SIGNAL_WRITABLE);
   ASSERT_EQ(MOJO_RESULT_OK,
-            d->AddWaiter(&w, MOJO_HANDLE_SIGNAL_WRITABLE, 2, nullptr));
+            d->AddAwakable(&w, MOJO_HANDLE_SIGNAL_WRITABLE, 2, nullptr));
   d->SetSatisfiableSignals(MOJO_HANDLE_SIGNAL_READABLE);
   stopwatch.Start();
   EXPECT_EQ(MOJO_RESULT_FAILED_PRECONDITION,
             w.Wait(MOJO_DEADLINE_INDEFINITE, &context));
-  EXPECT_LT(stopwatch.Elapsed(), test::EpsilonTimeout());
+  EXPECT_LT(stopwatch.Elapsed(), test::EpsilonDeadline());
   EXPECT_EQ(2u, context);
   hss = HandleSignalsState();
-  d->RemoveWaiter(&w, &hss);
+  d->RemoveAwakable(&w, &hss);
   EXPECT_EQ(0u, hss.satisfied_signals);
   EXPECT_EQ(MOJO_HANDLE_SIGNAL_READABLE, hss.satisfiable_signals);
 
@@ -235,14 +234,14 @@ TEST(SimpleDispatcherTest, BasicUnsatisfiable) {
   d->SetSatisfiableSignals(MOJO_HANDLE_SIGNAL_READABLE |
                            MOJO_HANDLE_SIGNAL_WRITABLE);
   ASSERT_EQ(MOJO_RESULT_OK,
-            d->AddWaiter(&w, MOJO_HANDLE_SIGNAL_WRITABLE, 3, nullptr));
+            d->AddAwakable(&w, MOJO_HANDLE_SIGNAL_WRITABLE, 3, nullptr));
   d->SetSatisfiableSignals(MOJO_HANDLE_SIGNAL_READABLE);
   stopwatch.Start();
   EXPECT_EQ(MOJO_RESULT_FAILED_PRECONDITION, w.Wait(0, &context));
-  EXPECT_LT(stopwatch.Elapsed(), test::EpsilonTimeout());
+  EXPECT_LT(stopwatch.Elapsed(), test::EpsilonDeadline());
   EXPECT_EQ(3u, context);
   hss = HandleSignalsState();
-  d->RemoveWaiter(&w, &hss);
+  d->RemoveAwakable(&w, &hss);
   EXPECT_EQ(0u, hss.satisfied_signals);
   EXPECT_EQ(MOJO_HANDLE_SIGNAL_READABLE, hss.satisfiable_signals);
 
@@ -252,15 +251,15 @@ TEST(SimpleDispatcherTest, BasicUnsatisfiable) {
   d->SetSatisfiableSignals(MOJO_HANDLE_SIGNAL_READABLE |
                            MOJO_HANDLE_SIGNAL_WRITABLE);
   ASSERT_EQ(MOJO_RESULT_OK,
-            d->AddWaiter(&w, MOJO_HANDLE_SIGNAL_WRITABLE, 4, nullptr));
+            d->AddAwakable(&w, MOJO_HANDLE_SIGNAL_WRITABLE, 4, nullptr));
   d->SetSatisfiableSignals(MOJO_HANDLE_SIGNAL_READABLE);
   stopwatch.Start();
   EXPECT_EQ(MOJO_RESULT_FAILED_PRECONDITION,
-            w.Wait(2 * test::EpsilonTimeout().InMicroseconds(), &context));
-  EXPECT_LT(stopwatch.Elapsed(), test::EpsilonTimeout());
+            w.Wait(2 * test::EpsilonDeadline(), &context));
+  EXPECT_LT(stopwatch.Elapsed(), test::EpsilonDeadline());
   EXPECT_EQ(4u, context);
   hss = HandleSignalsState();
-  d->RemoveWaiter(&w, &hss);
+  d->RemoveAwakable(&w, &hss);
   EXPECT_EQ(0u, hss.satisfied_signals);
   EXPECT_EQ(MOJO_HANDLE_SIGNAL_READABLE, hss.satisfiable_signals);
 
@@ -272,7 +271,7 @@ TEST(SimpleDispatcherTest, BasicClosed) {
 
   scoped_refptr<MockSimpleDispatcher> d;
   Waiter w;
-  uint32_t context = 0;
+  uintptr_t context = 0;
   HandleSignalsState hss;
 
   // Try adding a writable waiter when the dispatcher has been closed.
@@ -281,7 +280,7 @@ TEST(SimpleDispatcherTest, BasicClosed) {
   EXPECT_EQ(MOJO_RESULT_OK, d->Close());
   hss = HandleSignalsState();
   EXPECT_EQ(MOJO_RESULT_INVALID_ARGUMENT,
-            d->AddWaiter(&w, MOJO_HANDLE_SIGNAL_WRITABLE, 1, &hss));
+            d->AddAwakable(&w, MOJO_HANDLE_SIGNAL_WRITABLE, 1, &hss));
   EXPECT_EQ(0u, hss.satisfied_signals);
   EXPECT_EQ(0u, hss.satisfiable_signals);
   // Shouldn't need to remove the waiter (it was not added).
@@ -290,11 +289,11 @@ TEST(SimpleDispatcherTest, BasicClosed) {
   d = new MockSimpleDispatcher();
   w.Init();
   ASSERT_EQ(MOJO_RESULT_OK,
-            d->AddWaiter(&w, MOJO_HANDLE_SIGNAL_WRITABLE, 2, nullptr));
+            d->AddAwakable(&w, MOJO_HANDLE_SIGNAL_WRITABLE, 2, nullptr));
   EXPECT_EQ(MOJO_RESULT_OK, d->Close());
   stopwatch.Start();
   EXPECT_EQ(MOJO_RESULT_CANCELLED, w.Wait(MOJO_DEADLINE_INDEFINITE, &context));
-  EXPECT_LT(stopwatch.Elapsed(), test::EpsilonTimeout());
+  EXPECT_LT(stopwatch.Elapsed(), test::EpsilonDeadline());
   EXPECT_EQ(2u, context);
   // Don't need to remove waiters from closed dispatchers.
 
@@ -302,11 +301,11 @@ TEST(SimpleDispatcherTest, BasicClosed) {
   d = new MockSimpleDispatcher();
   w.Init();
   ASSERT_EQ(MOJO_RESULT_OK,
-            d->AddWaiter(&w, MOJO_HANDLE_SIGNAL_WRITABLE, 3, nullptr));
+            d->AddAwakable(&w, MOJO_HANDLE_SIGNAL_WRITABLE, 3, nullptr));
   EXPECT_EQ(MOJO_RESULT_OK, d->Close());
   stopwatch.Start();
   EXPECT_EQ(MOJO_RESULT_CANCELLED, w.Wait(0, &context));
-  EXPECT_LT(stopwatch.Elapsed(), test::EpsilonTimeout());
+  EXPECT_LT(stopwatch.Elapsed(), test::EpsilonDeadline());
   EXPECT_EQ(3u, context);
   // Don't need to remove waiters from closed dispatchers.
 
@@ -315,12 +314,12 @@ TEST(SimpleDispatcherTest, BasicClosed) {
   d = new MockSimpleDispatcher();
   w.Init();
   ASSERT_EQ(MOJO_RESULT_OK,
-            d->AddWaiter(&w, MOJO_HANDLE_SIGNAL_WRITABLE, 4, nullptr));
+            d->AddAwakable(&w, MOJO_HANDLE_SIGNAL_WRITABLE, 4, nullptr));
   EXPECT_EQ(MOJO_RESULT_OK, d->Close());
   stopwatch.Start();
   EXPECT_EQ(MOJO_RESULT_CANCELLED,
-            w.Wait(2 * test::EpsilonTimeout().InMicroseconds(), &context));
-  EXPECT_LT(stopwatch.Elapsed(), test::EpsilonTimeout());
+            w.Wait(2 * test::EpsilonDeadline(), &context));
+  EXPECT_LT(stopwatch.Elapsed(), test::EpsilonDeadline());
   EXPECT_EQ(4u, context);
   // Don't need to remove waiters from closed dispatchers.
 }
@@ -335,7 +334,7 @@ TEST(SimpleDispatcherTest, MAYBE_BasicThreaded) {
   test::Stopwatch stopwatch;
   bool did_wait;
   MojoResult result;
-  uint32_t context;
+  uintptr_t context;
   HandleSignalsState hss;
 
   // Wait for readable (already readable).
@@ -343,21 +342,16 @@ TEST(SimpleDispatcherTest, MAYBE_BasicThreaded) {
     scoped_refptr<MockSimpleDispatcher> d(new MockSimpleDispatcher());
     {
       d->SetSatisfiedSignals(MOJO_HANDLE_SIGNAL_READABLE);
-      test::WaiterThread thread(d,
-                                MOJO_HANDLE_SIGNAL_READABLE,
-                                MOJO_DEADLINE_INDEFINITE,
-                                1,
-                                &did_wait,
-                                &result,
-                                &context,
-                                &hss);
+      test::WaiterThread thread(d, MOJO_HANDLE_SIGNAL_READABLE,
+                                MOJO_DEADLINE_INDEFINITE, 1, &did_wait, &result,
+                                &context, &hss);
       stopwatch.Start();
       thread.Start();
     }  // Joins the thread.
     // If we closed earlier, then probably we'd get a |MOJO_RESULT_CANCELLED|.
     EXPECT_EQ(MOJO_RESULT_OK, d->Close());
   }
-  EXPECT_LT(stopwatch.Elapsed(), test::EpsilonTimeout());
+  EXPECT_LT(stopwatch.Elapsed(), test::EpsilonDeadline());
   EXPECT_FALSE(did_wait);
   EXPECT_EQ(MOJO_RESULT_ALREADY_EXISTS, result);
   EXPECT_EQ(MOJO_HANDLE_SIGNAL_READABLE, hss.satisfied_signals);
@@ -368,24 +362,19 @@ TEST(SimpleDispatcherTest, MAYBE_BasicThreaded) {
   {
     scoped_refptr<MockSimpleDispatcher> d(new MockSimpleDispatcher());
     {
-      test::WaiterThread thread(d,
-                                MOJO_HANDLE_SIGNAL_READABLE,
-                                MOJO_DEADLINE_INDEFINITE,
-                                2,
-                                &did_wait,
-                                &result,
-                                &context,
-                                &hss);
+      test::WaiterThread thread(d, MOJO_HANDLE_SIGNAL_READABLE,
+                                MOJO_DEADLINE_INDEFINITE, 2, &did_wait, &result,
+                                &context, &hss);
       stopwatch.Start();
       thread.Start();
-      base::PlatformThread::Sleep(2 * test::EpsilonTimeout());
+      test::Sleep(2 * test::EpsilonDeadline());
       d->SetSatisfiedSignals(MOJO_HANDLE_SIGNAL_READABLE);
     }  // Joins the thread.
     EXPECT_EQ(MOJO_RESULT_OK, d->Close());
   }
-  base::TimeDelta elapsed = stopwatch.Elapsed();
-  EXPECT_GT(elapsed, (2 - 1) * test::EpsilonTimeout());
-  EXPECT_LT(elapsed, (2 + 1) * test::EpsilonTimeout());
+  MojoDeadline elapsed = stopwatch.Elapsed();
+  EXPECT_GT(elapsed, (2 - 1) * test::EpsilonDeadline());
+  EXPECT_LT(elapsed, (2 + 1) * test::EpsilonDeadline());
   EXPECT_TRUE(did_wait);
   EXPECT_EQ(MOJO_RESULT_OK, result);
   EXPECT_EQ(2u, context);
@@ -397,24 +386,19 @@ TEST(SimpleDispatcherTest, MAYBE_BasicThreaded) {
   {
     scoped_refptr<MockSimpleDispatcher> d(new MockSimpleDispatcher());
     {
-      test::WaiterThread thread(d,
-                                MOJO_HANDLE_SIGNAL_READABLE,
-                                MOJO_DEADLINE_INDEFINITE,
-                                3,
-                                &did_wait,
-                                &result,
-                                &context,
-                                &hss);
+      test::WaiterThread thread(d, MOJO_HANDLE_SIGNAL_READABLE,
+                                MOJO_DEADLINE_INDEFINITE, 3, &did_wait, &result,
+                                &context, &hss);
       stopwatch.Start();
       thread.Start();
-      base::PlatformThread::Sleep(2 * test::EpsilonTimeout());
+      test::Sleep(2 * test::EpsilonDeadline());
       d->SetSatisfiableSignals(MOJO_HANDLE_SIGNAL_NONE);
     }  // Joins the thread.
     EXPECT_EQ(MOJO_RESULT_OK, d->Close());
   }
   elapsed = stopwatch.Elapsed();
-  EXPECT_GT(elapsed, (2 - 1) * test::EpsilonTimeout());
-  EXPECT_LT(elapsed, (2 + 1) * test::EpsilonTimeout());
+  EXPECT_GT(elapsed, (2 - 1) * test::EpsilonDeadline());
+  EXPECT_LT(elapsed, (2 + 1) * test::EpsilonDeadline());
   EXPECT_TRUE(did_wait);
   EXPECT_EQ(MOJO_RESULT_FAILED_PRECONDITION, result);
   EXPECT_EQ(3u, context);
@@ -424,22 +408,17 @@ TEST(SimpleDispatcherTest, MAYBE_BasicThreaded) {
   // Wait for readable and dispatcher gets closed.
   {
     scoped_refptr<MockSimpleDispatcher> d(new MockSimpleDispatcher());
-    test::WaiterThread thread(d,
-                              MOJO_HANDLE_SIGNAL_READABLE,
-                              MOJO_DEADLINE_INDEFINITE,
-                              4,
-                              &did_wait,
-                              &result,
-                              &context,
-                              &hss);
+    test::WaiterThread thread(d, MOJO_HANDLE_SIGNAL_READABLE,
+                              MOJO_DEADLINE_INDEFINITE, 4, &did_wait, &result,
+                              &context, &hss);
     stopwatch.Start();
     thread.Start();
-    base::PlatformThread::Sleep(2 * test::EpsilonTimeout());
+    test::Sleep(2 * test::EpsilonDeadline());
     EXPECT_EQ(MOJO_RESULT_OK, d->Close());
   }  // Joins the thread.
   elapsed = stopwatch.Elapsed();
-  EXPECT_GT(elapsed, (2 - 1) * test::EpsilonTimeout());
-  EXPECT_LT(elapsed, (2 + 1) * test::EpsilonTimeout());
+  EXPECT_GT(elapsed, (2 - 1) * test::EpsilonDeadline());
+  EXPECT_LT(elapsed, (2 + 1) * test::EpsilonDeadline());
   EXPECT_TRUE(did_wait);
   EXPECT_EQ(MOJO_RESULT_CANCELLED, result);
   EXPECT_EQ(4u, context);
@@ -450,17 +429,12 @@ TEST(SimpleDispatcherTest, MAYBE_BasicThreaded) {
   {
     scoped_refptr<MockSimpleDispatcher> d(new MockSimpleDispatcher());
     {
-      test::WaiterThread thread(d,
-                                MOJO_HANDLE_SIGNAL_READABLE,
-                                2 * test::EpsilonTimeout().InMicroseconds(),
-                                5,
-                                &did_wait,
-                                &result,
-                                &context,
-                                &hss);
+      test::WaiterThread thread(d, MOJO_HANDLE_SIGNAL_READABLE,
+                                2 * test::EpsilonDeadline(), 5, &did_wait,
+                                &result, &context, &hss);
       stopwatch.Start();
       thread.Start();
-      base::PlatformThread::Sleep(1 * test::EpsilonTimeout());
+      test::Sleep(1 * test::EpsilonDeadline());
       // Not what we're waiting for.
       d->SetSatisfiedSignals(MOJO_HANDLE_SIGNAL_WRITABLE);
     }  // Joins the thread (after its wait times out).
@@ -468,8 +442,8 @@ TEST(SimpleDispatcherTest, MAYBE_BasicThreaded) {
     EXPECT_EQ(MOJO_RESULT_OK, d->Close());
   }
   elapsed = stopwatch.Elapsed();
-  EXPECT_GT(elapsed, (2 - 1) * test::EpsilonTimeout());
-  EXPECT_LT(elapsed, (2 + 1) * test::EpsilonTimeout());
+  EXPECT_GT(elapsed, (2 - 1) * test::EpsilonDeadline());
+  EXPECT_LT(elapsed, (2 + 1) * test::EpsilonDeadline());
   EXPECT_TRUE(did_wait);
   EXPECT_EQ(MOJO_RESULT_DEADLINE_EXCEEDED, result);
   EXPECT_EQ(MOJO_HANDLE_SIGNAL_WRITABLE, hss.satisfied_signals);
@@ -488,7 +462,7 @@ TEST(SimpleDispatcherTest, MAYBE_MultipleWaiters) {
 
   bool did_wait[kNumWaiters];
   MojoResult result[kNumWaiters];
-  uint32_t context[kNumWaiters];
+  uintptr_t context[kNumWaiters];
   HandleSignalsState hss[kNumWaiters];
 
   // All wait for readable and becomes readable after some time.
@@ -496,17 +470,12 @@ TEST(SimpleDispatcherTest, MAYBE_MultipleWaiters) {
     scoped_refptr<MockSimpleDispatcher> d(new MockSimpleDispatcher());
     ScopedVector<test::WaiterThread> threads;
     for (uint32_t i = 0; i < kNumWaiters; i++) {
-      threads.push_back(new test::WaiterThread(d,
-                                               MOJO_HANDLE_SIGNAL_READABLE,
-                                               MOJO_DEADLINE_INDEFINITE,
-                                               i,
-                                               &did_wait[i],
-                                               &result[i],
-                                               &context[i],
-                                               &hss[i]));
+      threads.push_back(new test::WaiterThread(
+          d, MOJO_HANDLE_SIGNAL_READABLE, MOJO_DEADLINE_INDEFINITE, i,
+          &did_wait[i], &result[i], &context[i], &hss[i]));
       threads.back()->Start();
     }
-    base::PlatformThread::Sleep(2 * test::EpsilonTimeout());
+    test::Sleep(2 * test::EpsilonDeadline());
     d->SetSatisfiedSignals(MOJO_HANDLE_SIGNAL_READABLE);
     EXPECT_EQ(MOJO_RESULT_OK, d->Close());
   }  // Joins the threads.
@@ -524,28 +493,18 @@ TEST(SimpleDispatcherTest, MAYBE_MultipleWaiters) {
     scoped_refptr<MockSimpleDispatcher> d(new MockSimpleDispatcher());
     ScopedVector<test::WaiterThread> threads;
     for (uint32_t i = 0; i < kNumWaiters / 2; i++) {
-      threads.push_back(new test::WaiterThread(d,
-                                               MOJO_HANDLE_SIGNAL_READABLE,
-                                               MOJO_DEADLINE_INDEFINITE,
-                                               i,
-                                               &did_wait[i],
-                                               &result[i],
-                                               &context[i],
-                                               &hss[i]));
+      threads.push_back(new test::WaiterThread(
+          d, MOJO_HANDLE_SIGNAL_READABLE, MOJO_DEADLINE_INDEFINITE, i,
+          &did_wait[i], &result[i], &context[i], &hss[i]));
       threads.back()->Start();
     }
     for (uint32_t i = kNumWaiters / 2; i < kNumWaiters; i++) {
-      threads.push_back(new test::WaiterThread(d,
-                                               MOJO_HANDLE_SIGNAL_WRITABLE,
-                                               MOJO_DEADLINE_INDEFINITE,
-                                               i,
-                                               &did_wait[i],
-                                               &result[i],
-                                               &context[i],
-                                               &hss[i]));
+      threads.push_back(new test::WaiterThread(
+          d, MOJO_HANDLE_SIGNAL_WRITABLE, MOJO_DEADLINE_INDEFINITE, i,
+          &did_wait[i], &result[i], &context[i], &hss[i]));
       threads.back()->Start();
     }
-    base::PlatformThread::Sleep(2 * test::EpsilonTimeout());
+    test::Sleep(2 * test::EpsilonDeadline());
     d->SetSatisfiedSignals(MOJO_HANDLE_SIGNAL_READABLE);
     // This will wake up the ones waiting to write.
     EXPECT_EQ(MOJO_RESULT_OK, d->Close());
@@ -571,30 +530,20 @@ TEST(SimpleDispatcherTest, MAYBE_MultipleWaiters) {
     scoped_refptr<MockSimpleDispatcher> d(new MockSimpleDispatcher());
     ScopedVector<test::WaiterThread> threads;
     for (uint32_t i = 0; i < kNumWaiters / 2; i++) {
-      threads.push_back(new test::WaiterThread(d,
-                                               MOJO_HANDLE_SIGNAL_READABLE,
-                                               MOJO_DEADLINE_INDEFINITE,
-                                               i,
-                                               &did_wait[i],
-                                               &result[i],
-                                               &context[i],
-                                               &hss[i]));
+      threads.push_back(new test::WaiterThread(
+          d, MOJO_HANDLE_SIGNAL_READABLE, MOJO_DEADLINE_INDEFINITE, i,
+          &did_wait[i], &result[i], &context[i], &hss[i]));
       threads.back()->Start();
     }
     for (uint32_t i = kNumWaiters / 2; i < kNumWaiters; i++) {
-      threads.push_back(new test::WaiterThread(d,
-                                               MOJO_HANDLE_SIGNAL_WRITABLE,
-                                               MOJO_DEADLINE_INDEFINITE,
-                                               i,
-                                               &did_wait[i],
-                                               &result[i],
-                                               &context[i],
-                                               &hss[i]));
+      threads.push_back(new test::WaiterThread(
+          d, MOJO_HANDLE_SIGNAL_WRITABLE, MOJO_DEADLINE_INDEFINITE, i,
+          &did_wait[i], &result[i], &context[i], &hss[i]));
       threads.back()->Start();
     }
-    base::PlatformThread::Sleep(1 * test::EpsilonTimeout());
+    test::Sleep(1 * test::EpsilonDeadline());
     d->SetSatisfiableSignals(MOJO_HANDLE_SIGNAL_READABLE);
-    base::PlatformThread::Sleep(1 * test::EpsilonTimeout());
+    test::Sleep(1 * test::EpsilonDeadline());
     d->SetSatisfiedSignals(MOJO_HANDLE_SIGNAL_READABLE);
     EXPECT_EQ(MOJO_RESULT_OK, d->Close());
   }  // Joins the threads.
@@ -619,30 +568,18 @@ TEST(SimpleDispatcherTest, MAYBE_MultipleWaiters) {
     scoped_refptr<MockSimpleDispatcher> d(new MockSimpleDispatcher());
     ScopedVector<test::WaiterThread> threads;
     for (uint32_t i = 0; i < kNumWaiters / 2; i++) {
-      threads.push_back(
-          new test::WaiterThread(d,
-                                 MOJO_HANDLE_SIGNAL_READABLE,
-                                 3 * test::EpsilonTimeout().InMicroseconds(),
-                                 i,
-                                 &did_wait[i],
-                                 &result[i],
-                                 &context[i],
-                                 &hss[i]));
+      threads.push_back(new test::WaiterThread(
+          d, MOJO_HANDLE_SIGNAL_READABLE, 3 * test::EpsilonDeadline(), i,
+          &did_wait[i], &result[i], &context[i], &hss[i]));
       threads.back()->Start();
     }
     for (uint32_t i = kNumWaiters / 2; i < kNumWaiters; i++) {
-      threads.push_back(
-          new test::WaiterThread(d,
-                                 MOJO_HANDLE_SIGNAL_WRITABLE,
-                                 1 * test::EpsilonTimeout().InMicroseconds(),
-                                 i,
-                                 &did_wait[i],
-                                 &result[i],
-                                 &context[i],
-                                 &hss[i]));
+      threads.push_back(new test::WaiterThread(
+          d, MOJO_HANDLE_SIGNAL_WRITABLE, 1 * test::EpsilonDeadline(), i,
+          &did_wait[i], &result[i], &context[i], &hss[i]));
       threads.back()->Start();
     }
-    base::PlatformThread::Sleep(2 * test::EpsilonTimeout());
+    test::Sleep(2 * test::EpsilonDeadline());
     d->SetSatisfiedSignals(MOJO_HANDLE_SIGNAL_READABLE);
     // All those waiting for writable should have timed out.
     EXPECT_EQ(MOJO_RESULT_OK, d->Close());
@@ -665,5 +602,5 @@ TEST(SimpleDispatcherTest, MAYBE_MultipleWaiters) {
 // TODO(vtl): Stress test?
 
 }  // namespace
-}  // namespace system
+}  // namespace edk
 }  // namespace mojo
