@@ -2,7 +2,7 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
-#include "third_party/mojo/src/mojo/public/cpp/environment/environment.h"
+#include "mojo/public/cpp/environment/environment.h"
 
 #include "mojo/environment/default_async_waiter_impl.h"
 #include "mojo/environment/default_logger_impl.h"
