@@ -4,27 +4,21 @@
 
 #include "mojo/services/tracing/trace_data_sink.h"
 
+#include <utility>
+
 #include "base/logging.h"
 #include "mojo/common/data_pipe_utils.h"
 
 using mojo::common::BlockingCopyFromString;
 
 namespace tracing {
-namespace {
-
-const char kStart[] = "{\"traceEvents\":[";
-const char kEnd[] = "]}";
-
-}  // namespace
 
 TraceDataSink::TraceDataSink(mojo::ScopedDataPipeProducerHandle pipe)
-    : pipe_(pipe.Pass()), empty_(true) {
-  BlockingCopyFromString(kStart, pipe_);
-}
+    : pipe_(std::move(pipe)), empty_(true) {}
 
 TraceDataSink::~TraceDataSink() {
   if (pipe_.is_valid())
-    Flush();
+    pipe_.reset();
   DCHECK(!pipe_.is_valid());
 }
 
@@ -35,9 +29,4 @@ void TraceDataSink::AddChunk(const std::string& json) {
   BlockingCopyFromString(json, pipe_);
 }
 
-void TraceDataSink::Flush() {
-  BlockingCopyFromString(kEnd, pipe_);
-  pipe_.reset();
-}
-
 }  // namespace tracing
