@@ -4,23 +4,26 @@
 
 #include "net/base/file_stream.h"
 
+#include <utility>
+
 #include "base/bind.h"
 #include "base/callback.h"
 #include "base/files/file.h"
 #include "base/files/file_util.h"
+#include "base/macros.h"
 #include "base/message_loop/message_loop.h"
-#include "base/message_loop/message_loop_proxy.h"
 #include "base/path_service.h"
 #include "base/run_loop.h"
 #include "base/strings/string_util.h"
 #include "base/synchronization/waitable_event.h"
+#include "base/test/sequenced_worker_pool_owner.h"
 #include "base/test/test_timeouts.h"
-#include "base/threading/sequenced_worker_pool.h"
+#include "base/thread_task_runner_handle.h"
 #include "base/threading/thread_restrictions.h"
-#include "net/base/capturing_net_log.h"
 #include "net/base/io_buffer.h"
 #include "net/base/net_errors.h"
 #include "net/base/test_completion_callback.h"
+#include "net/log/test_net_log.h"
 #include "testing/gtest/include/gtest/gtest.h"
 #include "testing/platform_test.h"
 
@@ -71,7 +74,7 @@ namespace {
 
 TEST_F(FileStreamTest, OpenExplicitClose) {
   TestCompletionCallback callback;
-  FileStream stream(base::MessageLoopProxy::current());
+  FileStream stream(base::ThreadTaskRunnerHandle::Get());
   int flags = base::File::FLAG_OPEN |
               base::File::FLAG_READ |
               base::File::FLAG_ASYNC;
@@ -79,24 +82,21 @@ TEST_F(FileStreamTest, OpenExplicitClose) {
   EXPECT_EQ(ERR_IO_PENDING, rv);
   EXPECT_EQ(OK, callback.WaitForResult());
   EXPECT_TRUE(stream.IsOpen());
-  EXPECT_TRUE(stream.GetFileForTesting().IsValid());
   EXPECT_EQ(ERR_IO_PENDING, stream.Close(callback.callback()));
   EXPECT_EQ(OK, callback.WaitForResult());
   EXPECT_FALSE(stream.IsOpen());
-  EXPECT_FALSE(stream.GetFileForTesting().IsValid());
 }
 
 TEST_F(FileStreamTest, OpenExplicitCloseOrphaned) {
   TestCompletionCallback callback;
-  scoped_ptr<FileStream> stream(new FileStream(
-      base::MessageLoopProxy::current()));
+  scoped_ptr<FileStream> stream(
+      new FileStream(base::ThreadTaskRunnerHandle::Get()));
   int flags = base::File::FLAG_OPEN | base::File::FLAG_READ |
               base::File::FLAG_ASYNC;
   int rv = stream->Open(temp_file_path(), flags, callback.callback());
   EXPECT_EQ(ERR_IO_PENDING, rv);
   EXPECT_EQ(OK, callback.WaitForResult());
   EXPECT_TRUE(stream->IsOpen());
-  EXPECT_TRUE(stream->GetFileForTesting().IsValid());
   EXPECT_EQ(ERR_IO_PENDING, stream->Close(callback.callback()));
   stream.reset();
   // File isn't actually closed yet.
@@ -119,10 +119,8 @@ TEST_F(FileStreamTest, UseFileHandle) {
 
   // Seek to the beginning of the file and read.
   scoped_ptr<FileStream> read_stream(
-      new FileStream(file.Pass(), base::MessageLoopProxy::current()));
-  ASSERT_EQ(ERR_IO_PENDING,
-            read_stream->Seek(base::File::FROM_BEGIN, 0,
-                              callback64.callback()));
+      new FileStream(std::move(file), base::ThreadTaskRunnerHandle::Get()));
+  ASSERT_EQ(ERR_IO_PENDING, read_stream->Seek(0, callback64.callback()));
   ASSERT_EQ(0, callback64.WaitForResult());
   // Read into buffer and compare.
   scoped_refptr<IOBufferWithSize> read_buffer =
@@ -139,10 +137,8 @@ TEST_F(FileStreamTest, UseFileHandle) {
   file.Initialize(temp_file_path(), flags);
 
   scoped_ptr<FileStream> write_stream(
-      new FileStream(file.Pass(), base::MessageLoopProxy::current()));
-  ASSERT_EQ(ERR_IO_PENDING,
-            write_stream->Seek(base::File::FROM_BEGIN, 0,
-                               callback64.callback()));
+      new FileStream(std::move(file), base::ThreadTaskRunnerHandle::Get()));
+  ASSERT_EQ(ERR_IO_PENDING, write_stream->Seek(0, callback64.callback()));
   ASSERT_EQ(0, callback64.WaitForResult());
   scoped_refptr<IOBufferWithSize> write_buffer = CreateTestDataBuffer();
   rv = write_stream->Write(write_buffer.get(), kTestDataSize,
@@ -162,12 +158,12 @@ TEST_F(FileStreamTest, UseClosedStream) {
   TestCompletionCallback callback;
   TestInt64CompletionCallback callback64;
 
-  FileStream stream(base::MessageLoopProxy::current());
+  FileStream stream(base::ThreadTaskRunnerHandle::Get());
 
   EXPECT_FALSE(stream.IsOpen());
 
   // Try seeking...
-  rv = stream.Seek(base::File::FROM_BEGIN, 5, callback64.callback());
+  rv = stream.Seek(5, callback64.callback());
   EXPECT_EQ(ERR_UNEXPECTED, callback64.GetResult(rv));
 
   // Try reading...
@@ -177,10 +173,10 @@ TEST_F(FileStreamTest, UseClosedStream) {
 }
 
 TEST_F(FileStreamTest, Read) {
-  int64 file_size;
+  int64_t file_size;
   EXPECT_TRUE(base::GetFileSize(temp_file_path(), &file_size));
 
-  FileStream stream(base::MessageLoopProxy::current());
+  FileStream stream(base::ThreadTaskRunnerHandle::Get());
   int flags = base::File::FLAG_OPEN | base::File::FLAG_READ |
               base::File::FLAG_ASYNC;
   TestCompletionCallback callback;
@@ -205,11 +201,11 @@ TEST_F(FileStreamTest, Read) {
 }
 
 TEST_F(FileStreamTest, Read_EarlyDelete) {
-  int64 file_size;
+  int64_t file_size;
   EXPECT_TRUE(base::GetFileSize(temp_file_path(), &file_size));
 
   scoped_ptr<FileStream> stream(
-      new FileStream(base::MessageLoopProxy::current()));
+      new FileStream(base::ThreadTaskRunnerHandle::Get()));
   int flags = base::File::FLAG_OPEN | base::File::FLAG_READ |
               base::File::FLAG_ASYNC;
   TestCompletionCallback callback;
@@ -231,10 +227,10 @@ TEST_F(FileStreamTest, Read_EarlyDelete) {
 }
 
 TEST_F(FileStreamTest, Read_FromOffset) {
-  int64 file_size;
+  int64_t file_size;
   EXPECT_TRUE(base::GetFileSize(temp_file_path(), &file_size));
 
-  FileStream stream(base::MessageLoopProxy::current());
+  FileStream stream(base::ThreadTaskRunnerHandle::Get());
   int flags = base::File::FLAG_OPEN | base::File::FLAG_READ |
               base::File::FLAG_ASYNC;
   TestCompletionCallback callback;
@@ -243,10 +239,10 @@ TEST_F(FileStreamTest, Read_FromOffset) {
   EXPECT_EQ(OK, callback.WaitForResult());
 
   TestInt64CompletionCallback callback64;
-  const int64 kOffset = 3;
-  rv = stream.Seek(base::File::FROM_BEGIN, kOffset, callback64.callback());
+  const int64_t kOffset = 3;
+  rv = stream.Seek(kOffset, callback64.callback());
   ASSERT_EQ(ERR_IO_PENDING, rv);
-  int64 new_offset = callback64.WaitForResult();
+  int64_t new_offset = callback64.WaitForResult();
   EXPECT_EQ(kOffset, new_offset);
 
   int total_bytes_read = 0;
@@ -267,50 +263,15 @@ TEST_F(FileStreamTest, Read_FromOffset) {
   EXPECT_EQ(kTestData + kOffset, data_read);
 }
 
-TEST_F(FileStreamTest, SeekAround) {
-  FileStream stream(base::MessageLoopProxy::current());
-  int flags = base::File::FLAG_OPEN | base::File::FLAG_ASYNC |
-              base::File::FLAG_READ;
-  TestCompletionCallback callback;
-  int rv = stream.Open(temp_file_path(), flags, callback.callback());
-  EXPECT_EQ(ERR_IO_PENDING, rv);
-  EXPECT_EQ(OK, callback.WaitForResult());
-
-  TestInt64CompletionCallback callback64;
-
-  const int64 kOffset = 3;
-  rv = stream.Seek(base::File::FROM_BEGIN, kOffset, callback64.callback());
-  ASSERT_EQ(ERR_IO_PENDING, rv);
-  int64 new_offset = callback64.WaitForResult();
-  EXPECT_EQ(kOffset, new_offset);
-
-  rv = stream.Seek(base::File::FROM_CURRENT, kOffset, callback64.callback());
-  ASSERT_EQ(ERR_IO_PENDING, rv);
-  new_offset = callback64.WaitForResult();
-  EXPECT_EQ(2 * kOffset, new_offset);
-
-  rv = stream.Seek(base::File::FROM_CURRENT, -kOffset, callback64.callback());
-  ASSERT_EQ(ERR_IO_PENDING, rv);
-  new_offset = callback64.WaitForResult();
-  EXPECT_EQ(kOffset, new_offset);
-
-  const int kTestDataLen = arraysize(kTestData) - 1;
-
-  rv = stream.Seek(base::File::FROM_END, -kTestDataLen, callback64.callback());
-  ASSERT_EQ(ERR_IO_PENDING, rv);
-  new_offset = callback64.WaitForResult();
-  EXPECT_EQ(0, new_offset);
-}
-
 TEST_F(FileStreamTest, Write) {
-  FileStream stream(base::MessageLoopProxy::current());
+  FileStream stream(base::ThreadTaskRunnerHandle::Get());
   int flags = base::File::FLAG_CREATE_ALWAYS | base::File::FLAG_WRITE |
               base::File::FLAG_ASYNC;
   TestCompletionCallback callback;
   int rv = stream.Open(temp_file_path(), flags, callback.callback());
   EXPECT_EQ(OK, callback.GetResult(rv));
 
-  int64 file_size;
+  int64_t file_size;
   EXPECT_TRUE(base::GetFileSize(temp_file_path(), &file_size));
   EXPECT_EQ(0, file_size);
 
@@ -329,7 +290,7 @@ TEST_F(FileStreamTest, Write) {
 
 TEST_F(FileStreamTest, Write_EarlyDelete) {
   scoped_ptr<FileStream> stream(
-      new FileStream(base::MessageLoopProxy::current()));
+      new FileStream(base::ThreadTaskRunnerHandle::Get()));
   int flags = base::File::FLAG_CREATE_ALWAYS | base::File::FLAG_WRITE |
               base::File::FLAG_ASYNC;
   TestCompletionCallback callback;
@@ -337,7 +298,7 @@ TEST_F(FileStreamTest, Write_EarlyDelete) {
   EXPECT_EQ(ERR_IO_PENDING, rv);
   EXPECT_EQ(OK, callback.WaitForResult());
 
-  int64 file_size;
+  int64_t file_size;
   EXPECT_TRUE(base::GetFileSize(temp_file_path(), &file_size));
   EXPECT_EQ(0, file_size);
 
@@ -356,10 +317,10 @@ TEST_F(FileStreamTest, Write_EarlyDelete) {
 }
 
 TEST_F(FileStreamTest, Write_FromOffset) {
-  int64 file_size;
+  int64_t file_size;
   EXPECT_TRUE(base::GetFileSize(temp_file_path(), &file_size));
 
-  FileStream stream(base::MessageLoopProxy::current());
+  FileStream stream(base::ThreadTaskRunnerHandle::Get());
   int flags = base::File::FLAG_OPEN | base::File::FLAG_WRITE |
               base::File::FLAG_ASYNC;
   TestCompletionCallback callback;
@@ -368,10 +329,10 @@ TEST_F(FileStreamTest, Write_FromOffset) {
   EXPECT_EQ(OK, callback.WaitForResult());
 
   TestInt64CompletionCallback callback64;
-  const int64 kOffset = 0;
-  rv = stream.Seek(base::File::FROM_END, kOffset, callback64.callback());
+  const int64_t kOffset = kTestDataSize;
+  rv = stream.Seek(kOffset, callback64.callback());
   ASSERT_EQ(ERR_IO_PENDING, rv);
-  int64 new_offset = callback64.WaitForResult();
+  int64_t new_offset = callback64.WaitForResult();
   EXPECT_EQ(kTestDataSize, new_offset);
 
   int total_bytes_written = 0;
@@ -395,11 +356,11 @@ TEST_F(FileStreamTest, Write_FromOffset) {
 }
 
 TEST_F(FileStreamTest, BasicReadWrite) {
-  int64 file_size;
+  int64_t file_size;
   EXPECT_TRUE(base::GetFileSize(temp_file_path(), &file_size));
 
   scoped_ptr<FileStream> stream(
-      new FileStream(base::MessageLoopProxy::current()));
+      new FileStream(base::ThreadTaskRunnerHandle::Get()));
   int flags = base::File::FLAG_OPEN | base::File::FLAG_READ |
               base::File::FLAG_WRITE | base::File::FLAG_ASYNC;
   TestCompletionCallback callback;
@@ -407,7 +368,7 @@ TEST_F(FileStreamTest, BasicReadWrite) {
   EXPECT_EQ(ERR_IO_PENDING, rv);
   EXPECT_EQ(OK, callback.WaitForResult());
 
-  int64 total_bytes_read = 0;
+  int64_t total_bytes_read = 0;
 
   std::string data_read;
   for (;;) {
@@ -448,11 +409,11 @@ TEST_F(FileStreamTest, BasicReadWrite) {
 }
 
 TEST_F(FileStreamTest, BasicWriteRead) {
-  int64 file_size;
+  int64_t file_size;
   EXPECT_TRUE(base::GetFileSize(temp_file_path(), &file_size));
 
   scoped_ptr<FileStream> stream(
-      new FileStream(base::MessageLoopProxy::current()));
+      new FileStream(base::ThreadTaskRunnerHandle::Get()));
   int flags = base::File::FLAG_OPEN | base::File::FLAG_READ |
               base::File::FLAG_WRITE | base::File::FLAG_ASYNC;
   TestCompletionCallback callback;
@@ -461,9 +422,9 @@ TEST_F(FileStreamTest, BasicWriteRead) {
   EXPECT_EQ(OK, callback.WaitForResult());
 
   TestInt64CompletionCallback callback64;
-  rv = stream->Seek(base::File::FROM_END, 0, callback64.callback());
+  rv = stream->Seek(file_size, callback64.callback());
   ASSERT_EQ(ERR_IO_PENDING, rv);
-  int64 offset = callback64.WaitForResult();
+  int64_t offset = callback64.WaitForResult();
   EXPECT_EQ(offset, file_size);
 
   int total_bytes_written = 0;
@@ -485,7 +446,7 @@ TEST_F(FileStreamTest, BasicWriteRead) {
 
   EXPECT_EQ(kTestDataSize, total_bytes_written);
 
-  rv = stream->Seek(base::File::FROM_BEGIN, 0, callback64.callback());
+  rv = stream->Seek(0, callback64.callback());
   ASSERT_EQ(ERR_IO_PENDING, rv);
   offset = callback64.WaitForResult();
   EXPECT_EQ(0, offset);
@@ -546,6 +507,25 @@ class TestWriteReadCompletionCallback {
 
   const CompletionCallback& callback() const { return callback_; }
 
+  void ValidateWrittenData() {
+    TestCompletionCallback callback;
+    int rv = 0;
+    for (;;) {
+      scoped_refptr<IOBufferWithSize> buf = new IOBufferWithSize(4);
+      rv = stream_->Read(buf.get(), buf->size(), callback.callback());
+      if (rv == ERR_IO_PENDING) {
+        base::MessageLoop::ScopedNestableTaskAllower allow(
+            base::MessageLoop::current());
+        rv = callback.WaitForResult();
+      }
+      EXPECT_LE(0, rv);
+      if (rv <= 0)
+        break;
+      *total_bytes_read_ += rv;
+      data_read_->append(buf->data(), rv);
+    }
+  }
+
  private:
   void OnComplete(int result) {
     DCHECK_LT(0, result);
@@ -569,36 +549,18 @@ class TestWriteReadCompletionCallback {
       *data_read_ += data_read;
     } else {  // We're done writing all data.  Start reading the data.
       TestInt64CompletionCallback callback64;
-      EXPECT_EQ(ERR_IO_PENDING,
-                stream_->Seek(base::File::FROM_BEGIN, 0,
-                              callback64.callback()));
+      EXPECT_EQ(ERR_IO_PENDING, stream_->Seek(0, callback64.callback()));
       {
         base::MessageLoop::ScopedNestableTaskAllower allow(
             base::MessageLoop::current());
         EXPECT_LE(0, callback64.WaitForResult());
       }
-
-      TestCompletionCallback callback;
-      for (;;) {
-        scoped_refptr<IOBufferWithSize> buf = new IOBufferWithSize(4);
-        rv = stream_->Read(buf.get(), buf->size(), callback.callback());
-        if (rv == ERR_IO_PENDING) {
-          base::MessageLoop::ScopedNestableTaskAllower allow(
-              base::MessageLoop::current());
-          rv = callback.WaitForResult();
-        }
-        EXPECT_LE(0, rv);
-        if (rv <= 0)
-          break;
-        *total_bytes_read_ += rv;
-        data_read_->append(buf->data(), rv);
-      }
     }
 
     result_ = *total_bytes_written_;
     have_result_ = true;
     if (waiting_for_result_)
-      base::MessageLoop::current()->Quit();
+      base::MessageLoop::current()->QuitWhenIdle();
   }
 
   int result_;
@@ -616,11 +578,11 @@ class TestWriteReadCompletionCallback {
 };
 
 TEST_F(FileStreamTest, WriteRead) {
-  int64 file_size;
+  int64_t file_size;
   EXPECT_TRUE(base::GetFileSize(temp_file_path(), &file_size));
 
   scoped_ptr<FileStream> stream(
-      new FileStream(base::MessageLoopProxy::current()));
+      new FileStream(base::ThreadTaskRunnerHandle::Get()));
   int flags = base::File::FLAG_OPEN | base::File::FLAG_READ |
               base::File::FLAG_WRITE | base::File::FLAG_ASYNC;
   TestCompletionCallback open_callback;
@@ -629,8 +591,7 @@ TEST_F(FileStreamTest, WriteRead) {
   EXPECT_EQ(OK, open_callback.WaitForResult());
 
   TestInt64CompletionCallback callback64;
-  EXPECT_EQ(ERR_IO_PENDING,
-            stream->Seek(base::File::FROM_END, 0, callback64.callback()));
+  EXPECT_EQ(ERR_IO_PENDING, stream->Seek(file_size, callback64.callback()));
   EXPECT_EQ(file_size, callback64.WaitForResult());
 
   int total_bytes_written = 0;
@@ -646,6 +607,8 @@ TEST_F(FileStreamTest, WriteRead) {
   EXPECT_LT(0, rv);
   EXPECT_EQ(kTestDataSize, total_bytes_written);
 
+  callback.ValidateWrittenData();
+
   stream.reset();
 
   EXPECT_TRUE(base::GetFileSize(temp_file_path(), &file_size));
@@ -705,7 +668,7 @@ class TestWriteCloseCompletionCallback {
     result_ = *total_bytes_written_;
     have_result_ = true;
     if (waiting_for_result_)
-      base::MessageLoop::current()->Quit();
+      base::MessageLoop::current()->QuitWhenIdle();
   }
 
   int result_;
@@ -721,11 +684,11 @@ class TestWriteCloseCompletionCallback {
 };
 
 TEST_F(FileStreamTest, WriteClose) {
-  int64 file_size;
+  int64_t file_size;
   EXPECT_TRUE(base::GetFileSize(temp_file_path(), &file_size));
 
   scoped_ptr<FileStream> stream(
-      new FileStream(base::MessageLoopProxy::current()));
+      new FileStream(base::ThreadTaskRunnerHandle::Get()));
   int flags = base::File::FLAG_OPEN | base::File::FLAG_READ |
               base::File::FLAG_WRITE | base::File::FLAG_ASYNC;
   TestCompletionCallback open_callback;
@@ -734,8 +697,7 @@ TEST_F(FileStreamTest, WriteClose) {
   EXPECT_EQ(OK, open_callback.WaitForResult());
 
   TestInt64CompletionCallback callback64;
-  EXPECT_EQ(ERR_IO_PENDING,
-            stream->Seek(base::File::FROM_END, 0, callback64.callback()));
+  EXPECT_EQ(ERR_IO_PENDING, stream->Seek(file_size, callback64.callback()));
   EXPECT_EQ(file_size, callback64.WaitForResult());
 
   int total_bytes_written = 0;
@@ -755,11 +717,10 @@ TEST_F(FileStreamTest, WriteClose) {
 }
 
 TEST_F(FileStreamTest, OpenAndDelete) {
-  scoped_refptr<base::SequencedWorkerPool> pool(
-      new base::SequencedWorkerPool(1, "StreamTest"));
+  base::SequencedWorkerPoolOwner pool_owner(1, "StreamTest");
 
   bool prev = base::ThreadRestrictions::SetIOAllowed(false);
-  scoped_ptr<FileStream> stream(new FileStream(pool.get()));
+  scoped_ptr<FileStream> stream(new FileStream(pool_owner.pool()));
   int flags = base::File::FLAG_OPEN | base::File::FLAG_WRITE |
               base::File::FLAG_ASYNC;
   TestCompletionCallback open_callback;
@@ -771,14 +732,12 @@ TEST_F(FileStreamTest, OpenAndDelete) {
   stream.reset();
 
   // Force an operation through the pool.
-  scoped_ptr<FileStream> stream2(new FileStream(pool.get()));
+  scoped_ptr<FileStream> stream2(new FileStream(pool_owner.pool()));
   TestCompletionCallback open_callback2;
   rv = stream2->Open(temp_file_path(), flags, open_callback2.callback());
   EXPECT_EQ(OK, open_callback2.GetResult(rv));
   stream2.reset();
 
-  pool->Shutdown();
-
   // open_callback won't be called.
   base::RunLoop().RunUntilIdle();
   EXPECT_FALSE(open_callback.have_result());
@@ -788,14 +747,14 @@ TEST_F(FileStreamTest, OpenAndDelete) {
 // Verify that Write() errors are mapped correctly.
 TEST_F(FileStreamTest, WriteError) {
   // Try opening file as read-only and then writing to it using FileStream.
-  uint32 flags = base::File::FLAG_OPEN | base::File::FLAG_READ |
-                 base::File::FLAG_ASYNC;
+  uint32_t flags =
+      base::File::FLAG_OPEN | base::File::FLAG_READ | base::File::FLAG_ASYNC;
 
   base::File file(temp_file_path(), flags);
   ASSERT_TRUE(file.IsValid());
 
   scoped_ptr<FileStream> stream(
-  new FileStream(file.Pass(), base::MessageLoopProxy::current()));
+      new FileStream(std::move(file), base::ThreadTaskRunnerHandle::Get()));
 
   scoped_refptr<IOBuffer> buf = new IOBuffer(1);
   buf->data()[0] = 0;
@@ -813,14 +772,14 @@ TEST_F(FileStreamTest, WriteError) {
 // Verify that Read() errors are mapped correctly.
 TEST_F(FileStreamTest, ReadError) {
   // Try opening file for write and then reading from it using FileStream.
-  uint32 flags = base::File::FLAG_OPEN | base::File::FLAG_WRITE |
-                 base::File::FLAG_ASYNC;
+  uint32_t flags =
+      base::File::FLAG_OPEN | base::File::FLAG_WRITE | base::File::FLAG_ASYNC;
 
   base::File file(temp_file_path(), flags);
   ASSERT_TRUE(file.IsValid());
 
   scoped_ptr<FileStream> stream(
-  new FileStream(file.Pass(), base::MessageLoopProxy::current()));
+      new FileStream(std::move(file), base::ThreadTaskRunnerHandle::Get()));
 
   scoped_refptr<IOBuffer> buf = new IOBuffer(1);
   TestCompletionCallback callback;
@@ -848,11 +807,11 @@ TEST_F(FileStreamTest, ContentUriRead) {
   base::FilePath path = base::InsertImageIntoMediaStore(image_file);
   EXPECT_TRUE(path.IsContentUri());
   EXPECT_TRUE(base::PathExists(path));
-  int64 file_size;
+  int64_t file_size;
   EXPECT_TRUE(base::GetFileSize(path, &file_size));
   EXPECT_LT(0, file_size);
 
-  FileStream stream(base::MessageLoopProxy::current());
+  FileStream stream(base::ThreadTaskRunnerHandle::Get());
   int flags = base::File::FLAG_OPEN | base::File::FLAG_READ |
               base::File::FLAG_ASYNC;
   TestCompletionCallback callback;
