@@ -15,6 +15,8 @@
 #include <sslerr.h>
 
 #include "base/logging.h"
+#include "base/macros.h"
+#include "build/build_config.h"
 #include "crypto/nss_util.h"
 #include "crypto/scoped_nss_types.h"
 #include "crypto/sha2.h"
@@ -35,6 +37,8 @@
 
 #if defined(USE_NSS_CERTS)
 #include <dlfcn.h>
+#else
+#include <ocsp.h>
 #endif
 
 namespace net {
@@ -162,7 +166,7 @@ void GetCertChainInfo(CERTCertList* cert_list,
 
   CERTCertificate* verified_cert = NULL;
   std::vector<CERTCertificate*> verified_chain;
-  int i = 0;
+  size_t i = 0;
   for (CERTCertListNode* node = CERT_LIST_HEAD(cert_list);
        !CERT_LIST_END(node, cert_list);
        node = CERT_LIST_NEXT(node), ++i) {
@@ -212,6 +216,8 @@ void GetCertChainInfo(CERTCertList* cert_list,
       case SEC_OID_ANSIX9_DSA_SIGNATURE_WITH_SHA1_DIGEST:
       case SEC_OID_ANSIX962_ECDSA_SHA1_SIGNATURE:
         verify_result->has_sha1 = true;
+        if (i == 0)
+          verify_result->has_sha1_leaf = true;
         break;
       default:
         break;
@@ -765,6 +771,9 @@ CertVerifyProcNSS::CertVerifyProcNSS()
     : cache_ocsp_response_from_side_channel_(
           reinterpret_cast<CacheOCSPResponseFromSideChannelFunction>(
               dlsym(RTLD_DEFAULT, "CERT_CacheOCSPResponseFromSideChannel")))
+#else
+    : cache_ocsp_response_from_side_channel_(
+          &CERT_CacheOCSPResponseFromSideChannel)
 #endif
 {
 }
@@ -776,12 +785,7 @@ bool CertVerifyProcNSS::SupportsAdditionalTrustAnchors() const {
 }
 
 bool CertVerifyProcNSS::SupportsOCSPStapling() const {
-#if defined(USE_NSS_CERTS)
   return cache_ocsp_response_from_side_channel_;
-#else
-  // TODO(davidben): Support OCSP stapling on iOS.
-  return false;
-#endif
 }
 
 int CertVerifyProcNSS::VerifyInternalImpl(
@@ -802,7 +806,6 @@ int CertVerifyProcNSS::VerifyInternalImpl(
   CERTCertificate* cert_handle = cert->os_cert_handle();
 #endif  // defined(OS_IOS)
 
-#if defined(USE_NSS_CERTS)
   if (!ocsp_response.empty() && cache_ocsp_response_from_side_channel_) {
     // Note: NSS uses a thread-safe global hash table, so this call will
     // affect any concurrent verification operations on |cert| or copies of
@@ -813,9 +816,9 @@ int CertVerifyProcNSS::VerifyInternalImpl(
         const_cast<char*>(ocsp_response.data()));
     ocsp_response_item.len = ocsp_response.size();
     cache_ocsp_response_from_side_channel_(CERT_GetDefaultCertDB(), cert_handle,
-                                           PR_Now(), &ocsp_response_item, NULL);
+                                           PR_Now(), &ocsp_response_item,
+                                           nullptr);
   }
-#endif  // defined(USE_NSS_CERTS)
 
   if (!cert->VerifyNameMatch(hostname,
                              &verify_result->common_name_fallback_used)) {
