@@ -5,6 +5,7 @@
 #include "net/http/http_network_transaction.h"
 
 #include <set>
+#include <utility>
 #include <vector>
 
 #include "base/bind.h"
@@ -60,6 +61,7 @@
 #include "net/spdy/spdy_session_pool.h"
 #include "net/ssl/ssl_cert_request_info.h"
 #include "net/ssl/ssl_connection_status_flags.h"
+#include "net/ssl/ssl_private_key.h"
 #include "url/gurl.h"
 #include "url/url_canon.h"
 
@@ -67,10 +69,21 @@ namespace net {
 
 namespace {
 
-void ProcessAlternateProtocol(
-    HttpNetworkSession* session,
-    const HttpResponseHeaders& headers,
-    const HostPortPair& http_host_port_pair) {
+void ProcessAlternativeServices(HttpNetworkSession* session,
+                                const HttpResponseHeaders& headers,
+                                const HostPortPair& http_host_port_pair) {
+  if (session->params().use_alternative_services &&
+      headers.HasHeader(kAlternativeServiceHeader)) {
+    std::string alternative_service_str;
+    headers.GetNormalizedHeader(kAlternativeServiceHeader,
+                                &alternative_service_str);
+    session->http_stream_factory()->ProcessAlternativeService(
+        session->http_server_properties(), alternative_service_str,
+        http_host_port_pair, *session);
+    // If there is an "Alt-Svc" header, then ignore "Alternate-Protocol".
+    return;
+  }
+
   if (!headers.HasHeader(kAlternateProtocolHeader))
     return;
 
@@ -97,8 +110,8 @@ scoped_ptr<base::Value> NetLogSSLVersionFallbackCallback(
     const GURL* url,
     int net_error,
     SSLFailureState ssl_failure_state,
-    uint16 version_before,
-    uint16 version_after,
+    uint16_t version_before,
+    uint16_t version_after,
     NetLogCaptureMode /* capture_mode */) {
   scoped_ptr<base::DictionaryValue> dict(new base::DictionaryValue());
   dict->SetString("host_and_port", GetHostAndPort(*url));
@@ -106,7 +119,7 @@ scoped_ptr<base::Value> NetLogSSLVersionFallbackCallback(
   dict->SetInteger("ssl_failure_state", ssl_failure_state);
   dict->SetInteger("version_before", version_before);
   dict->SetInteger("version_after", version_after);
-  return dict.Pass();
+  return std::move(dict);
 }
 
 scoped_ptr<base::Value> NetLogSSLCipherFallbackCallback(
@@ -116,7 +129,7 @@ scoped_ptr<base::Value> NetLogSSLCipherFallbackCallback(
   scoped_ptr<base::DictionaryValue> dict(new base::DictionaryValue());
   dict->SetString("host_and_port", GetHostAndPort(*url));
   dict->SetInteger("net_error", net_error);
-  return dict.Pass();
+  return std::move(dict);
 }
 
 }  // namespace
@@ -138,38 +151,31 @@ HttpNetworkTransaction::HttpNetworkTransaction(RequestPriority priority,
       request_headers_(),
       read_buf_len_(0),
       total_received_bytes_(0),
+      total_sent_bytes_(0),
       next_state_(STATE_NONE),
       establishing_tunnel_(false),
-      websocket_handshake_stream_base_create_helper_(NULL) {
+      websocket_handshake_stream_base_create_helper_(NULL),
+      net_error_details_() {
   session->ssl_config_service()->GetSSLConfig(&server_ssl_config_);
-  session->GetNextProtos(&server_ssl_config_.next_protos);
+  session->GetAlpnProtos(&server_ssl_config_.alpn_protos);
+  session->GetNpnProtos(&server_ssl_config_.npn_protos);
   proxy_ssl_config_ = server_ssl_config_;
 }
 
 HttpNetworkTransaction::~HttpNetworkTransaction() {
   if (stream_.get()) {
-    HttpResponseHeaders* headers = GetResponseHeaders();
     // TODO(mbelshe): The stream_ should be able to compute whether or not the
     //                stream should be kept alive.  No reason to compute here
     //                and pass it in.
-    bool try_to_keep_alive =
-        next_state_ == STATE_NONE &&
-        stream_->CanFindEndOfResponse() &&
-        (!headers || headers->IsKeepAlive());
-    if (!try_to_keep_alive) {
+    if (!stream_->CanReuseConnection() || next_state_ != STATE_NONE) {
       stream_->Close(true /* not reusable */);
+    } else if (stream_->IsResponseBodyComplete()) {
+      // If the response body is complete, we can just reuse the socket.
+      stream_->Close(false /* reusable */);
     } else {
-      if (stream_->IsResponseBodyComplete()) {
-        // If the response body is complete, we can just reuse the socket.
-        stream_->Close(false /* reusable */);
-      } else if (stream_->IsSpdyHttpStream()) {
-        // Doesn't really matter for SpdyHttpStream. Just close it.
-        stream_->Close(true /* not reusable */);
-      } else {
-        // Otherwise, we try to drain the response body.
-        HttpStream* stream = stream_.release();
-        stream->Drain(session_);
-      }
+      // Otherwise, we try to drain the response body.
+      HttpStream* stream = stream_.release();
+      stream->Drain(session_);
     }
   }
 
@@ -195,12 +201,6 @@ int HttpNetworkTransaction::Start(const HttpRequestInfo* request_info,
   if (request_->privacy_mode == PRIVACY_MODE_ENABLED)
     server_ssl_config_.channel_id_enabled = false;
 
-  if (server_ssl_config_.fastradio_padding_enabled) {
-    server_ssl_config_.fastradio_padding_eligible =
-        session_->ssl_config_service()->SupportsFastradioPadding(
-            request_info->url);
-  }
-
   next_state_ = STATE_NOTIFY_BEFORE_CREATE_STREAM;
   int rv = DoLoop(OK);
   if (rv == ERR_IO_PENDING)
@@ -223,7 +223,9 @@ int HttpNetworkTransaction::RestartIgnoringLastError(
 }
 
 int HttpNetworkTransaction::RestartWithCertificate(
-    X509Certificate* client_cert, const CompletionCallback& callback) {
+    X509Certificate* client_cert,
+    SSLPrivateKey* client_private_key,
+    const CompletionCallback& callback) {
   // In HandleCertificateRequest(), we always tear down existing stream
   // requests to force a new connection.  So we shouldn't have one here.
   DCHECK(!stream_request_.get());
@@ -234,8 +236,10 @@ int HttpNetworkTransaction::RestartWithCertificate(
       &proxy_ssl_config_ : &server_ssl_config_;
   ssl_config->send_client_cert = true;
   ssl_config->client_cert = client_cert;
+  ssl_config->client_private_key = client_private_key;
   session_->ssl_client_auth_cache()->Add(
-      response_.cert_request_info->host_and_port, client_cert);
+      response_.cert_request_info->host_and_port, client_cert,
+      client_private_key);
   // Reset the other member variables.
   // Note: this is necessary only with SSL renegotiation.
   ResetStateForRestart();
@@ -288,8 +292,7 @@ void HttpNetworkTransaction::PrepareForAuthRestart(HttpAuth::Target target) {
   bool keep_alive = false;
   // Even if the server says the connection is keep-alive, we have to be
   // able to find the end of each response in order to reuse the connection.
-  if (GetResponseHeaders()->IsKeepAlive() &&
-      stream_->CanFindEndOfResponse()) {
+  if (stream_->CanReuseConnection()) {
     // If the response body hasn't been completely read, we need to drain
     // it first.
     if (!stream_->IsResponseBodyComplete()) {
@@ -311,8 +314,9 @@ void HttpNetworkTransaction::DidDrainBodyForAuthRestart(bool keep_alive) {
 
   if (stream_.get()) {
     total_received_bytes_ += stream_->GetTotalReceivedBytes();
+    total_sent_bytes_ += stream_->GetTotalSentBytes();
     HttpStream* new_stream = NULL;
-    if (keep_alive && stream_->IsConnectionReusable()) {
+    if (keep_alive && stream_->CanReuseConnection()) {
       // We should call connection_->set_idle_time(), but this doesn't occur
       // often enough to be worth the trouble.
       stream_->SetConnectionReused();
@@ -326,8 +330,9 @@ void HttpNetworkTransaction::DidDrainBodyForAuthRestart(bool keep_alive) {
       stream_->Close(true);
       next_state_ = STATE_CREATE_STREAM;
     } else {
-      // Renewed streams shouldn't carry over received bytes.
+      // Renewed streams shouldn't carry over sent or received bytes.
       DCHECK_EQ(0, new_stream->GetTotalReceivedBytes());
+      DCHECK_EQ(0, new_stream->GetTotalSentBytes());
       next_state_ = STATE_INIT_STREAM;
     }
     stream_.reset(new_stream);
@@ -390,13 +395,20 @@ bool HttpNetworkTransaction::GetFullRequestHeaders(
   return true;
 }
 
-int64 HttpNetworkTransaction::GetTotalReceivedBytes() const {
-  int64 total_received_bytes = total_received_bytes_;
+int64_t HttpNetworkTransaction::GetTotalReceivedBytes() const {
+  int64_t total_received_bytes = total_received_bytes_;
   if (stream_)
     total_received_bytes += stream_->GetTotalReceivedBytes();
   return total_received_bytes;
 }
 
+int64_t HttpNetworkTransaction::GetTotalSentBytes() const {
+  int64_t total_sent_bytes = total_sent_bytes_;
+  if (stream_)
+    total_sent_bytes += stream_->GetTotalSentBytes();
+  return total_sent_bytes;
+}
+
 void HttpNetworkTransaction::DoneReading() {}
 
 const HttpResponseInfo* HttpNetworkTransaction::GetResponseInfo() const {
@@ -447,6 +459,21 @@ bool HttpNetworkTransaction::GetLoadTimingInfo(
   return true;
 }
 
+bool HttpNetworkTransaction::GetRemoteEndpoint(IPEndPoint* endpoint) const {
+  if (!remote_endpoint_.address().size())
+    return false;
+
+  *endpoint = remote_endpoint_;
+  return true;
+}
+
+void HttpNetworkTransaction::PopulateNetErrorDetails(
+    NetErrorDetails* details) const {
+  *details = net_error_details_;
+  if (stream_)
+    stream_->PopulateNetErrorDetails(details);
+}
+
 void HttpNetworkTransaction::SetPriority(RequestPriority priority) {
   priority_ = priority;
   if (stream_request_)
@@ -481,8 +508,10 @@ void HttpNetworkTransaction::OnStreamReady(const SSLConfig& used_ssl_config,
   DCHECK_EQ(STATE_CREATE_STREAM_COMPLETE, next_state_);
   DCHECK(stream_request_.get());
 
-  if (stream_)
+  if (stream_) {
     total_received_bytes_ += stream_->GetTotalReceivedBytes();
+    total_sent_bytes_ += stream_->GetTotalSentBytes();
+  }
   stream_.reset(stream);
   server_ssl_config_ = used_ssl_config;
   proxy_info_ = used_proxy_info;
@@ -496,6 +525,13 @@ void HttpNetworkTransaction::OnStreamReady(const SSLConfig& used_ssl_config,
   OnIOComplete(OK);
 }
 
+void HttpNetworkTransaction::OnBidirectionalStreamJobReady(
+    const SSLConfig& used_ssl_config,
+    const ProxyInfo& used_proxy_info,
+    BidirectionalStreamJob* stream_job) {
+  NOTREACHED();
+}
+
 void HttpNetworkTransaction::OnWebSocketHandshakeStreamReady(
     const SSLConfig& used_ssl_config,
     const ProxyInfo& used_proxy_info,
@@ -581,13 +617,19 @@ void HttpNetworkTransaction::OnHttpsProxyTunnelResponse(
   response_ = response_info;
   server_ssl_config_ = used_ssl_config;
   proxy_info_ = used_proxy_info;
-  if (stream_)
+  if (stream_) {
     total_received_bytes_ += stream_->GetTotalReceivedBytes();
+    total_sent_bytes_ += stream_->GetTotalSentBytes();
+  }
   stream_.reset(stream);
   stream_request_.reset();  // we're done with the stream request
   OnIOComplete(ERR_HTTPS_PROXY_TUNNEL_RESPONSE);
 }
 
+void HttpNetworkTransaction::OnQuicBroken() {
+  net_error_details_.quic_broken = true;
+}
+
 void HttpNetworkTransaction::GetConnectionAttempts(
     ConnectionAttempts* out) const {
   *out = connection_attempts_;
@@ -805,6 +847,9 @@ int HttpNetworkTransaction::DoCreateStreamComplete(int result) {
 int HttpNetworkTransaction::DoInitStream() {
   DCHECK(stream_.get());
   next_state_ = STATE_INIT_STREAM_COMPLETE;
+
+  stream_->GetRemoteEndpoint(&remote_endpoint_);
+
   return stream_->InitializeStream(request_, priority_, net_log_, io_callback_);
 }
 
@@ -816,9 +861,11 @@ int HttpNetworkTransaction::DoInitStreamComplete(int result) {
       result = HandleIOError(result);
 
     // The stream initialization failed, so this stream will never be useful.
-    if (stream_)
-        total_received_bytes_ += stream_->GetTotalReceivedBytes();
-    stream_.reset();
+    if (stream_) {
+      total_received_bytes_ += stream_->GetTotalReceivedBytes();
+      total_sent_bytes_ += stream_->GetTotalSentBytes();
+    }
+    CacheNetErrorDetailsAndResetStream();
   }
 
   return result;
@@ -896,12 +943,13 @@ void HttpNetworkTransaction::BuildRequestHeaders(
           HttpRequestHeaders::kContentLength,
           base::Uint64ToString(request_->upload_data_stream->size()));
     }
-  } else if (request_->method == "POST" || request_->method == "PUT" ||
-             request_->method == "HEAD") {
+  } else if (request_->method == "POST" || request_->method == "PUT") {
     // An empty POST/PUT request still needs a content length.  As for HEAD,
     // IE and Safari also add a content length header.  Presumably it is to
     // support sending a HEAD request to an URL that only expects to be sent a
     // POST or some other method that normally would have a message body.
+    // Firefox (40.0) does not send the header, and RFC 7230 & 7231
+    // specify that it should not be sent due to undefined behavior.
     request_headers_.SetHeader(HttpRequestHeaders::kContentLength, "0");
   }
 
@@ -1056,7 +1104,7 @@ int HttpNetworkTransaction::DoReadHeadersComplete(int result) {
       NetLog::TYPE_HTTP_TRANSACTION_READ_RESPONSE_HEADERS,
       base::Bind(&HttpResponseHeaders::NetLogCallback, response_.headers));
 
-  if (response_.headers->GetParsedHttpVersion() < HttpVersion(1, 0)) {
+  if (response_.headers->GetHttpVersion() < HttpVersion(1, 0)) {
     // HTTP/0.9 doesn't support the PUT method, so lack of response headers
     // indicates a buggy server.  See:
     // https://bugzilla.mozilla.org/show_bug.cgi?id=193921
@@ -1077,8 +1125,8 @@ int HttpNetworkTransaction::DoReadHeadersComplete(int result) {
     return OK;
   }
 
-  ProcessAlternateProtocol(session_, *response_.headers.get(),
-                           HostPortPair::FromURL(request_->url));
+  ProcessAlternativeServices(session_, *response_.headers.get(),
+                             HostPortPair::FromURL(request_->url));
 
   int rv = HandleAuthChallenge();
   if (rv != OK)
@@ -1109,26 +1157,18 @@ int HttpNetworkTransaction::DoReadBodyComplete(int result) {
     done = true;
   }
 
-  bool keep_alive = false;
-  if (stream_->IsResponseBodyComplete()) {
+  // Clean up connection if we are done.
+  if (done) {
     // Note: Just because IsResponseBodyComplete is true, we're not
     // necessarily "done".  We're only "done" when it is the last
     // read on this HttpNetworkTransaction, which will be signified
     // by a zero-length read.
-    // TODO(mbelshe): The keepalive property is really a property of
+    // TODO(mbelshe): The keep-alive property is really a property of
     //    the stream.  No need to compute it here just to pass back
     //    to the stream's Close function.
-    // TODO(rtenneti): CanFindEndOfResponse should return false if there are no
-    // ResponseHeaders.
-    if (stream_->CanFindEndOfResponse()) {
-      HttpResponseHeaders* headers = GetResponseHeaders();
-      if (headers)
-        keep_alive = headers->IsKeepAlive();
-    }
-  }
+    bool keep_alive =
+        stream_->IsResponseBodyComplete() && stream_->CanReuseConnection();
 
-  // Clean up connection if we are done.
-  if (done) {
     stream_->Close(!keep_alive);
     // Note: we don't reset the stream here.  We've closed it, but we still
     // need it around so that callers can call methods such as
@@ -1198,8 +1238,9 @@ int HttpNetworkTransaction::HandleCertificateRequest(int error) {
     // renegotiation.
     DCHECK(!stream_request_.get());
     total_received_bytes_ += stream_->GetTotalReceivedBytes();
+    total_sent_bytes_ += stream_->GetTotalSentBytes();
     stream_->Close(true);
-    stream_.reset();
+    CacheNetErrorDetailsAndResetStream();
   }
 
   // The server is asking for a client certificate during the initial
@@ -1210,8 +1251,10 @@ int HttpNetworkTransaction::HandleCertificateRequest(int error) {
   // to provide one for this server before, use the past decision
   // automatically.
   scoped_refptr<X509Certificate> client_cert;
+  scoped_refptr<SSLPrivateKey> client_private_key;
   bool found_cached_cert = session_->ssl_client_auth_cache()->Lookup(
-      response_.cert_request_info->host_and_port, &client_cert);
+      response_.cert_request_info->host_and_port, &client_cert,
+      &client_private_key);
   if (!found_cached_cert)
     return error;
 
@@ -1235,6 +1278,7 @@ int HttpNetworkTransaction::HandleCertificateRequest(int error) {
       &proxy_ssl_config_ : &server_ssl_config_;
   ssl_config->send_client_cert = true;
   ssl_config->client_cert = client_cert;
+  ssl_config->client_private_key = client_private_key;
   next_state_ = STATE_CREATE_STREAM;
   // Reset the other member variables.
   // Note: this is necessary only with SSL renegotiation.
@@ -1275,19 +1319,19 @@ int HttpNetworkTransaction::HandleSSLHandshakeError(int error) {
   // reflect servers require a deprecated cipher rather than merely prefer
   // it. This, however, has no security benefit until the ciphers are actually
   // removed.
-  if (!server_ssl_config_.enable_deprecated_cipher_suites &&
+  if (!server_ssl_config_.deprecated_cipher_suites_enabled &&
       (error == ERR_SSL_VERSION_OR_CIPHER_MISMATCH ||
        error == ERR_CONNECTION_CLOSED || error == ERR_CONNECTION_RESET)) {
     net_log_.AddEvent(
         NetLog::TYPE_SSL_CIPHER_FALLBACK,
         base::Bind(&NetLogSSLCipherFallbackCallback, &request_->url, error));
-    server_ssl_config_.enable_deprecated_cipher_suites = true;
+    server_ssl_config_.deprecated_cipher_suites_enabled = true;
     ResetConnectionAndRequestForResend();
     return OK;
   }
 
   bool should_fallback = false;
-  uint16 version_max = server_ssl_config_.version_max;
+  uint16_t version_max = server_ssl_config_.version_max;
 
   switch (error) {
     case ERR_CONNECTION_CLOSED:
@@ -1421,9 +1465,11 @@ int HttpNetworkTransaction::HandleIOError(int error) {
 
 void HttpNetworkTransaction::ResetStateForRestart() {
   ResetStateForAuthRestart();
-  if (stream_)
+  if (stream_) {
     total_received_bytes_ += stream_->GetTotalReceivedBytes();
-  stream_.reset();
+    total_sent_bytes_ += stream_->GetTotalSentBytes();
+  }
+  CacheNetErrorDetailsAndResetStream();
 }
 
 void HttpNetworkTransaction::ResetStateForAuthRestart() {
@@ -1437,6 +1483,15 @@ void HttpNetworkTransaction::ResetStateForAuthRestart() {
   request_headers_.Clear();
   response_ = HttpResponseInfo();
   establishing_tunnel_ = false;
+  remote_endpoint_ = IPEndPoint();
+  net_error_details_.quic_broken = false;
+  net_error_details_.quic_connection_error = QUIC_NO_ERROR;
+}
+
+void HttpNetworkTransaction::CacheNetErrorDetailsAndResetStream() {
+  if (stream_)
+    stream_->PopulateNetErrorDetails(&net_error_details_);
+  stream_.reset();
 }
 
 void HttpNetworkTransaction::RecordSSLFallbackMetrics(int result) {
@@ -1444,7 +1499,8 @@ void HttpNetworkTransaction::RecordSSLFallbackMetrics(int result) {
     return;
 
   const std::string& host = request_->url.host();
-  bool is_google = base::EndsWith(host, "google.com", true) &&
+  bool is_google = base::EndsWith(host, "google.com",
+                                  base::CompareCase::SENSITIVE) &&
                    (host.size() == 10 || host[host.size() - 11] == '.');
   if (is_google) {
     // Some fraction of successful connections use the fallback, but only due to
@@ -1494,7 +1550,7 @@ void HttpNetworkTransaction::RecordSSLFallbackMetrics(int result) {
   }
 
   UMA_HISTOGRAM_BOOLEAN("Net.ConnectionUsedSSLDeprecatedCipherFallback2",
-                        server_ssl_config_.enable_deprecated_cipher_suites);
+                        server_ssl_config_.deprecated_cipher_suites_enabled);
 
   if (server_ssl_config_.version_fallback) {
     // Record the error code which triggered the fallback and the state the
@@ -1525,7 +1581,7 @@ bool HttpNetworkTransaction::ShouldResendRequest() const {
 void HttpNetworkTransaction::ResetConnectionAndRequestForResend() {
   if (stream_.get()) {
     stream_->Close(true);
-    stream_.reset();
+    CacheNetErrorDetailsAndResetStream();
   }
 
   // We need to clear request_headers_ because it contains the real request
