@@ -17,11 +17,10 @@ PrrSender::PrrSender()
     : bytes_sent_since_loss_(0),
       bytes_delivered_since_loss_(0),
       ack_count_since_loss_(0),
-      bytes_in_flight_before_loss_(0) {
-}
+      bytes_in_flight_before_loss_(0) {}
 
 void PrrSender::OnPacketSent(QuicByteCount sent_bytes) {
-    bytes_sent_since_loss_ += sent_bytes;
+  bytes_sent_since_loss_ += sent_bytes;
 }
 
 void PrrSender::OnPacketLost(QuicByteCount bytes_in_flight) {
@@ -40,6 +39,9 @@ QuicTime::Delta PrrSender::TimeUntilSend(
     QuicByteCount congestion_window,
     QuicByteCount bytes_in_flight,
     QuicByteCount slowstart_threshold) const {
+  //   if (FLAGS_?? && bytes_in_flight < congestion_window) {
+  //     return QuicTime::Delta::Zero();
+  //   }
   // Return QuicTime::Zero In order to ensure limited transmit always works.
   if (bytes_sent_since_loss_ == 0 || bytes_in_flight < kMaxSegmentSize) {
     return QuicTime::Delta::Zero();
