@@ -15,7 +15,7 @@ HttpServerRequestInfo::~HttpServerRequestInfo() {}
 
 std::string HttpServerRequestInfo::GetHeaderValue(
     const std::string& header_name) const {
-  DCHECK_EQ(base::StringToLowerASCII(header_name), header_name);
+  DCHECK_EQ(base::ToLowerASCII(header_name), header_name);
   HttpServerRequestInfo::HeadersMap::const_iterator it =
       headers.find(header_name);
   if (it != headers.end())
@@ -26,9 +26,8 @@ std::string HttpServerRequestInfo::GetHeaderValue(
 bool HttpServerRequestInfo::HasHeaderValue(
     const std::string& header_name,
     const std::string& header_value) const {
-  DCHECK_EQ(base::StringToLowerASCII(header_value), header_value);
-  std::string complete_value = GetHeaderValue(header_name);
-  base::StringToLowerASCII(&complete_value);
+  DCHECK_EQ(base::ToLowerASCII(header_value), header_value);
+  std::string complete_value = base::ToLowerASCII(GetHeaderValue(header_name));
 
   for (const base::StringPiece& cur :
        base::SplitString(complete_value, ",", base::KEEP_WHITESPACE,
