@@ -8,32 +8,30 @@ namespace net {
 
 NextProtoVector NextProtosDefaults() {
   NextProtoVector next_protos;
-  next_protos.push_back(kProtoHTTP11);
-  next_protos.push_back(kProtoSPDY31);
-  next_protos.push_back(kProtoHTTP2_14);
   next_protos.push_back(kProtoHTTP2);
+  next_protos.push_back(kProtoSPDY31);
+  next_protos.push_back(kProtoHTTP11);
   return next_protos;
 }
 
 NextProtoVector NextProtosWithSpdyAndQuic(bool spdy_enabled,
                                           bool quic_enabled) {
   NextProtoVector next_protos;
-  next_protos.push_back(kProtoHTTP11);
   if (quic_enabled)
     next_protos.push_back(kProtoQUIC1SPDY3);
   if (spdy_enabled) {
-    next_protos.push_back(kProtoSPDY31);
-    next_protos.push_back(kProtoHTTP2_14);
     next_protos.push_back(kProtoHTTP2);
+    next_protos.push_back(kProtoSPDY31);
   }
+  next_protos.push_back(kProtoHTTP11);
   return next_protos;
 }
 
 NextProtoVector NextProtosSpdy31() {
   NextProtoVector next_protos;
-  next_protos.push_back(kProtoHTTP11);
   next_protos.push_back(kProtoQUIC1SPDY3);
   next_protos.push_back(kProtoSPDY31);
+  next_protos.push_back(kProtoHTTP11);
   return next_protos;
 }
 
@@ -42,4 +40,15 @@ bool NextProtoIsSPDY(NextProto next_proto) {
          next_proto <= kProtoSPDYMaximumVersion;
 }
 
+void DisableHTTP2(NextProtoVector* next_protos) {
+  for (NextProtoVector::iterator it = next_protos->begin();
+       it != next_protos->end();) {
+    if (*it == kProtoHTTP2) {
+      it = next_protos->erase(it);
+      continue;
+    }
+    ++it;
+  }
+}
+
 }  // namespace net
