@@ -5,10 +5,13 @@
 #include "net/socket/tcp_socket.h"
 #include "net/socket/tcp_socket_win.h"
 
+#include <errno.h>
 #include <mstcpip.h>
 
 #include "base/callback_helpers.h"
+#include "base/files/file_util.h"
 #include "base/logging.h"
+#include "base/macros.h"
 #include "base/profiler/scoped_tracker.h"
 #include "base/win/windows_version.h"
 #include "net/base/address_list.h"
@@ -16,9 +19,9 @@
 #include "net/base/io_buffer.h"
 #include "net/base/ip_endpoint.h"
 #include "net/base/net_errors.h"
-#include "net/base/net_util.h"
 #include "net/base/network_activity_monitor.h"
 #include "net/base/network_change_notifier.h"
+#include "net/base/sockaddr_storage.h"
 #include "net/base/winsock_init.h"
 #include "net/base/winsock_util.h"
 #include "net/socket/socket_descriptor.h"
@@ -30,7 +33,7 @@ namespace {
 
 const int kTCPKeepAliveSeconds = 45;
 
-int SetSocketReceiveBufferSize(SOCKET socket, int32 size) {
+int SetSocketReceiveBufferSize(SOCKET socket, int32_t size) {
   int rv = setsockopt(socket, SOL_SOCKET, SO_RCVBUF,
                       reinterpret_cast<const char*>(&size), sizeof(size));
   int net_error = (rv == 0) ? OK : MapSystemError(WSAGetLastError());
@@ -38,7 +41,7 @@ int SetSocketReceiveBufferSize(SOCKET socket, int32 size) {
   return net_error;
 }
 
-int SetSocketSendBufferSize(SOCKET socket, int32 size) {
+int SetSocketSendBufferSize(SOCKET socket, int32_t size) {
   int rv = setsockopt(socket, SOL_SOCKET, SO_SNDBUF,
                       reinterpret_cast<const char*>(&size), sizeof(size));
   int net_error = (rv == 0) ? OK : MapSystemError(WSAGetLastError());
@@ -235,14 +238,14 @@ void TCPSocketWin::Core::WatchForRead() {
   // We grab an extra reference because there is an IO operation in progress.
   // Balanced in ReadDelegate::OnObjectSignaled().
   AddRef();
-  read_watcher_.StartWatching(read_overlapped_.hEvent, &reader_);
+  read_watcher_.StartWatchingOnce(read_overlapped_.hEvent, &reader_);
 }
 
 void TCPSocketWin::Core::WatchForWrite() {
   // We grab an extra reference because there is an IO operation in progress.
   // Balanced in WriteDelegate::OnObjectSignaled().
   AddRef();
-  write_watcher_.StartWatching(write_overlapped_.hEvent, &writer_);
+  write_watcher_.StartWatchingOnce(write_overlapped_.hEvent, &writer_);
 }
 
 void TCPSocketWin::Core::ReadDelegate::OnObjectSignaled(HANDLE object) {
@@ -301,7 +304,7 @@ int TCPSocketWin::Open(AddressFamily family) {
     return MapSystemError(WSAGetLastError());
   }
 
-  if (SetNonBlocking(socket_)) {
+  if (!base::SetNonBlocking(socket_)) {
     int result = MapSystemError(WSAGetLastError());
     Close();
     return result;
@@ -318,7 +321,7 @@ int TCPSocketWin::AdoptConnectedSocket(SOCKET socket,
 
   socket_ = socket;
 
-  if (SetNonBlocking(socket_)) {
+  if (!base::SetNonBlocking(socket_)) {
     int result = MapSystemError(WSAGetLastError());
     Close();
     return result;
@@ -336,7 +339,7 @@ int TCPSocketWin::AdoptListenSocket(SOCKET socket) {
 
   socket_ = socket;
 
-  if (SetNonBlocking(socket_)) {
+  if (!base::SetNonBlocking(socket_)) {
     int result = MapSystemError(WSAGetLastError());
     Close();
     return result;
@@ -402,7 +405,7 @@ int TCPSocketWin::Accept(scoped_ptr<TCPSocketWin>* socket,
   if (result == ERR_IO_PENDING) {
     // Start watching.
     WSAEventSelect(socket_, accept_event_, FD_ACCEPT);
-    accept_watcher_.StartWatching(accept_event_, this);
+    accept_watcher_.StartWatchingOnce(accept_event_, this);
 
     accept_socket_ = socket;
     accept_address_ = address;
@@ -425,7 +428,7 @@ int TCPSocketWin::Connect(const IPEndPoint& address,
   // completed and failed. Although it is allowed to connect the same |socket_|
   // again after a connection attempt failed on Windows, it results in
   // unspecified behavior according to POSIX. Therefore, we make it behave in
-  // the same way as TCPSocketLibevent.
+  // the same way as TCPSocketPosix.
   DCHECK(!peer_address_ && !core_.get());
 
   if (!logging_multiple_connect_attempts_)
@@ -588,7 +591,7 @@ void TCPSocketWin::SetDefaultOptionsForClient() {
   // Since Vista's auto-tune is better than any static value we can could set,
   // only change these on pre-vista machines.
   if (base::win::GetVersion() < base::win::VERSION_VISTA) {
-    const int32 kSocketBufferSize = 64 * 1024;
+    const int32_t kSocketBufferSize = 64 * 1024;
     SetSocketReceiveBufferSize(socket_, kSocketBufferSize);
     SetSocketSendBufferSize(socket_, kSocketBufferSize);
   }
@@ -623,12 +626,12 @@ int TCPSocketWin::SetExclusiveAddrUse() {
   return OK;
 }
 
-int TCPSocketWin::SetReceiveBufferSize(int32 size) {
+int TCPSocketWin::SetReceiveBufferSize(int32_t size) {
   DCHECK(CalledOnValidThread());
   return SetSocketReceiveBufferSize(socket_, size);
 }
 
-int TCPSocketWin::SetSendBufferSize(int32 size) {
+int TCPSocketWin::SetSendBufferSize(int32_t size) {
   DCHECK(CalledOnValidThread());
   return SetSocketSendBufferSize(socket_, size);
 }
@@ -698,6 +701,10 @@ void TCPSocketWin::Close() {
   connect_os_error_ = 0;
 }
 
+void TCPSocketWin::DetachFromThread() {
+  base::NonThreadSafe::DetachFromThread();
+}
+
 void TCPSocketWin::StartLoggingMultipleConnectAttempts(
     const AddressList& addresses) {
   if (!logging_multiple_connect_attempts_) {
@@ -772,7 +779,7 @@ void TCPSocketWin::OnObjectSignaled(HANDLE object) {
 
     // Start watching the next FD_ACCEPT event.
     WSAEventSelect(socket_, accept_event_, FD_ACCEPT);
-    accept_watcher_.StartWatching(accept_event_, this);
+    accept_watcher_.StartWatchingOnce(accept_event_, this);
   }
 }
 
@@ -841,7 +848,7 @@ void TCPSocketWin::DoConnectComplete(int result) {
   connect_os_error_ = 0;
   if (result != OK) {
     net_log_.EndEvent(NetLog::TYPE_TCP_CONNECT_ATTEMPT,
-                      NetLog::IntegerCallback("os_error", os_error));
+                      NetLog::IntCallback("os_error", os_error));
   } else {
     net_log_.EndEvent(NetLog::TYPE_TCP_CONNECT_ATTEMPT);
   }
