@@ -5,7 +5,9 @@
 #ifndef NET_TOOLS_QUIC_QUIC_DEFAULT_PACKET_WRITER_H_
 #define NET_TOOLS_QUIC_QUIC_DEFAULT_PACKET_WRITER_H_
 
-#include "base/basictypes.h"
+#include <stddef.h>
+
+#include "base/macros.h"
 #include "net/base/ip_endpoint.h"
 #include "net/quic/quic_packet_writer.h"
 
@@ -29,13 +31,12 @@ class QuicDefaultPacketWriter : public QuicPacketWriter {
   bool IsWriteBlockedDataBuffered() const override;
   bool IsWriteBlocked() const override;
   void SetWritable() override;
+  QuicByteCount GetMaxPacketSize(const IPEndPoint& peer_address) const override;
 
   void set_fd(int fd) { fd_ = fd; }
 
  protected:
-  void set_write_blocked(bool is_blocked) {
-    write_blocked_ = is_blocked;
-  }
+  void set_write_blocked(bool is_blocked) { write_blocked_ = is_blocked; }
   int fd() { return fd_; }
 
  private:
