@@ -4,8 +4,13 @@
 
 #include "ppapi/proxy/nacl_message_scanner.h"
 
+#include <stddef.h>
+
+#include <utility>
 #include <vector>
+
 #include "base/bind.h"
+#include "build/build_config.h"
 #include "ipc/ipc_message.h"
 #include "ipc/ipc_message_macros.h"
 #include "ppapi/proxy/ppapi_messages.h"
@@ -57,6 +62,7 @@ void WriteHandle(int handle_index,
     // Now write the handle itself in POSIX style.
     // See ParamTraits<FileDescriptor>::Read for where these values are read.
     msg->WriteBool(true);  // valid == true
+    msg->WriteBool(false);  // brokerable == false
     msg->WriteInt(handle_index);
   }
 }
@@ -137,6 +143,15 @@ void ScanParam(const IPC::Message& param, ScanningResults* results) {
     IPC::WriteParam(results->new_msg.get(), param);
 }
 
+template <class T>
+void ScanParam(const std::vector<T>& vec, ScanningResults* results) {
+  if (results->new_msg)
+    IPC::WriteParam(results->new_msg.get(), static_cast<int>(vec.size()));
+  for (const T& element : vec) {
+    ScanParam(element, results);
+  }
+}
+
 // Overload to match all other types. If we need to rewrite the message, write
 // the parameter.
 template <class T>
@@ -149,26 +164,26 @@ void ScanParam(const T& param, ScanningResults* results) {
 // The idea is to scan elements in the tuple which require special handling,
 // and write them into the |results| struct.
 template <class A>
-void ScanTuple(const Tuple1<A>& t1, ScanningResults* results) {
-  ScanParam(t1.a, results);
+void ScanTuple(const base::Tuple<A>& t1, ScanningResults* results) {
+  ScanParam(base::get<0>(t1), results);
 }
 template <class A, class B>
-void ScanTuple(const Tuple2<A, B>& t1, ScanningResults* results) {
-  ScanParam(t1.a, results);
-  ScanParam(t1.b, results);
+void ScanTuple(const base::Tuple<A, B>& t1, ScanningResults* results) {
+  ScanParam(base::get<0>(t1), results);
+  ScanParam(base::get<1>(t1), results);
 }
 template <class A, class B, class C>
-void ScanTuple(const Tuple3<A, B, C>& t1, ScanningResults* results) {
-  ScanParam(t1.a, results);
-  ScanParam(t1.b, results);
-  ScanParam(t1.c, results);
+void ScanTuple(const base::Tuple<A, B, C>& t1, ScanningResults* results) {
+  ScanParam(base::get<0>(t1), results);
+  ScanParam(base::get<1>(t1), results);
+  ScanParam(base::get<2>(t1), results);
 }
 template <class A, class B, class C, class D>
-void ScanTuple(const Tuple4<A, B, C, D>& t1, ScanningResults* results) {
-  ScanParam(t1.a, results);
-  ScanParam(t1.b, results);
-  ScanParam(t1.c, results);
-  ScanParam(t1.d, results);
+void ScanTuple(const base::Tuple<A, B, C, D>& t1, ScanningResults* results) {
+  ScanParam(base::get<0>(t1), results);
+  ScanParam(base::get<1>(t1), results);
+  ScanParam(base::get<2>(t1), results);
+  ScanParam(base::get<3>(t1), results);
 }
 
 template <class MessageType>
@@ -178,16 +193,32 @@ class MessageScannerImpl {
       : msg_(static_cast<const MessageType*>(msg)) {
   }
   bool ScanMessage(ScanningResults* results) {
-    typename TupleTypes<typename MessageType::Schema::Param>::ValueTuple params;
+    typename base::TupleTypes<typename MessageType::Schema::Param>::ValueTuple
+        params;
     if (!MessageType::Read(msg_, &params))
       return false;
     ScanTuple(params, results);
     return true;
   }
 
+  bool ScanSyncMessage(ScanningResults* results) {
+    typename base::TupleTypes<typename MessageType::Schema::SendParam>
+        ::ValueTuple params;
+    if (!MessageType::ReadSendParam(msg_, &params))
+      return false;
+    // If we need to rewrite the message, write the message id first.
+    if (results->new_msg) {
+      results->new_msg->set_sync();
+      int id = IPC::SyncMessage::GetMessageId(*msg_);
+      results->new_msg->WriteInt(id);
+    }
+    ScanTuple(params, results);
+    return true;
+  }
+
   bool ScanReply(ScanningResults* results) {
-    typename TupleTypes<typename MessageType::Schema::ReplyParam>::ValueTuple
-        params;
+    typename base::TupleTypes<typename MessageType::Schema::ReplyParam>
+        ::ValueTuple params;
     if (!MessageType::ReadReplyParam(msg_, &params))
       return false;
     // If we need to rewrite the message, write the message id first.
@@ -199,8 +230,6 @@ class MessageScannerImpl {
     ScanTuple(params, results);
     return true;
   }
-  // TODO(dmichael): Add ScanSyncMessage for outgoing sync messages, if we ever
-  //                 need to scan those.
 
  private:
   const MessageType* msg_;
@@ -219,6 +248,17 @@ class MessageScannerImpl {
           return false; \
         break; \
       }
+#define CASE_FOR_SYNC_MESSAGE(MESSAGE_TYPE) \
+      case MESSAGE_TYPE::ID: { \
+        MessageScannerImpl<MESSAGE_TYPE> scanner(&msg); \
+        if (rewrite_msg) \
+          results.new_msg.reset( \
+              new IPC::Message(msg.routing_id(), msg.type(), \
+                               IPC::Message::PRIORITY_NORMAL)); \
+        if (!scanner.ScanSyncMessage(&results)) \
+          return false; \
+        break; \
+      }
 #define CASE_FOR_REPLY(MESSAGE_TYPE) \
       case MESSAGE_TYPE::ID: { \
         MessageScannerImpl<MESSAGE_TYPE> scanner(&msg); \
@@ -292,6 +332,9 @@ NaClMessageScanner::~NaClMessageScanner() {
 // message body, rather than passed in a separate FileDescriptorSet. Therefore,
 // on Windows, any message containing handles must be rewritten in the POSIX
 // format before we can send it to the NaCl plugin.
+// On Mac, base::SharedMemoryHandle has a different serialization than
+// base::FileDescriptor (which base::SharedMemoryHandle is typedef-ed to in
+// OS_NACL).
 bool NaClMessageScanner::ScanMessage(
     const IPC::Message& msg,
     uint32_t type,
@@ -303,7 +346,7 @@ bool NaClMessageScanner::ScanMessage(
   DCHECK(!new_msg_ptr->get());
 
   bool rewrite_msg =
-#if defined(OS_WIN)
+#if defined(OS_WIN) || defined(OS_MACOSX)
       true;
 #else
       false;
@@ -321,6 +364,9 @@ bool NaClMessageScanner::ScanMessage(
     CASE_FOR_MESSAGE(PpapiMsg_PPBAudio_NotifyAudioStreamCreated)
     CASE_FOR_MESSAGE(PpapiMsg_PPPMessaging_HandleMessage)
     CASE_FOR_MESSAGE(PpapiPluginMsg_ResourceReply)
+    CASE_FOR_SYNC_MESSAGE(PpapiMsg_PPPMessageHandler_HandleBlockingMessage)
+    CASE_FOR_SYNC_MESSAGE(PpapiMsg_PnaclTranslatorCompileInit)
+    CASE_FOR_SYNC_MESSAGE(PpapiMsg_PnaclTranslatorLink)
     CASE_FOR_REPLY(PpapiHostMsg_OpenResource)
     CASE_FOR_REPLY(PpapiHostMsg_PPBGraphics3D_Create)
     CASE_FOR_REPLY(PpapiHostMsg_PPBGraphics3D_CreateTransferBuffer)
@@ -337,7 +383,7 @@ bool NaClMessageScanner::ScanMessage(
   // we ever add new param types that also require rewriting.
   if (!results.handles.empty()) {
     handles->swap(results.handles);
-    *new_msg_ptr = results.new_msg.Pass();
+    *new_msg_ptr = std::move(results.new_msg);
   }
   return true;
 }
