@@ -4,12 +4,16 @@
 
 #include "ppapi/proxy/ppapi_param_traits.h"
 
+#include <stddef.h>
+#include <stdint.h>
 #include <string.h>  // For memcpy
 
+#include "base/macros.h"
+#include "build/build_config.h"
 #include "ppapi/c/pp_resource.h"
 #include "ppapi/proxy/ppapi_messages.h"
-#include "ppapi/proxy/serialized_var.h"
 #include "ppapi/proxy/serialized_flash_menu.h"
+#include "ppapi/proxy/serialized_var.h"
 #include "ppapi/shared_impl/host_resource.h"
 #include "ppapi/shared_impl/private/ppb_x509_certificate_private_shared.h"
 
@@ -36,14 +40,14 @@ namespace {
 //
 // The solution is to make a new object for each deserialized item, and then
 // add it to the vector one at a time.
-template<typename T>
+template <typename T>
 bool ReadVectorWithoutCopy(const Message* m,
-                           PickleIterator* iter,
+                           base::PickleIterator* iter,
                            std::vector<T>* output) {
   // This part is just a copy of the the default ParamTraits vector Read().
   int size;
   // ReadLength() checks for < 0 itself.
-  if (!m->ReadLength(iter, &size))
+  if (!iter->ReadLength(&size))
     return false;
   // Resizing beforehand is not safe, see BUG 1006367 for details.
   if (INT_MAX / sizeof(T) <= static_cast<size_t>(size))
@@ -76,18 +80,18 @@ void WriteVectorWithoutCopy(Message* m, const std::vector<T>& p) {
 
 // static
 void ParamTraits<PP_Bool>::Write(Message* m, const param_type& p) {
-  ParamTraits<bool>::Write(m, PP_ToBool(p));
+  WriteParam(m, PP_ToBool(p));
 }
 
 // static
 bool ParamTraits<PP_Bool>::Read(const Message* m,
-                                PickleIterator* iter,
+                                base::PickleIterator* iter,
                                 param_type* r) {
   // We specifically want to be strict here about what types of input we accept,
   // which ParamTraits<bool> does for us. We don't want to deserialize "2" into
   // a PP_Bool, for example.
   bool result = false;
-  if (!ParamTraits<bool>::Read(m, iter, &result))
+  if (!ReadParam(m, iter, &result))
     return false;
   *r = PP_FromBool(result);
   return true;
@@ -97,6 +101,52 @@ bool ParamTraits<PP_Bool>::Read(const Message* m,
 void ParamTraits<PP_Bool>::Log(const param_type& p, std::string* l) {
 }
 
+// PP_KeyInformation -------------------------------------------------------
+
+// static
+void ParamTraits<PP_KeyInformation>::Write(Message* m, const param_type& p) {
+  WriteParam(m, p.key_id_size);
+  m->WriteBytes(p.key_id, static_cast<int>(p.key_id_size));
+  WriteParam(m, p.key_status);
+  WriteParam(m, p.system_code);
+}
+
+// static
+bool ParamTraits<PP_KeyInformation>::Read(const Message* m,
+                                          base::PickleIterator* iter,
+                                          param_type* p) {
+  uint32_t size;
+  if (!ReadParam(m, iter, &size))
+    return false;
+  if (size > sizeof(p->key_id))
+    return false;
+  p->key_id_size = size;
+
+  const char* data;
+  if (!iter->ReadBytes(&data, size))
+    return false;
+  memcpy(p->key_id, data, size);
+
+  PP_CdmKeyStatus key_status;
+  if (!ReadParam(m, iter, &key_status))
+    return false;
+  p->key_status = key_status;
+
+  uint32_t system_code;
+  if (!ReadParam(m, iter, &system_code))
+    return false;
+  p->system_code = system_code;
+
+  return true;
+}
+
+// static
+void ParamTraits<PP_KeyInformation>::Log(const param_type& p, std::string* l) {
+  l->append("<PP_KeyInformation (");
+  LogParam(p.key_id_size, l);
+  l->append(" bytes)>");
+}
+
 // PP_NetAddress_Private -------------------------------------------------------
 
 // static
@@ -108,9 +158,9 @@ void ParamTraits<PP_NetAddress_Private>::Write(Message* m,
 
 // static
 bool ParamTraits<PP_NetAddress_Private>::Read(const Message* m,
-                                              PickleIterator* iter,
+                                              base::PickleIterator* iter,
                                               param_type* p) {
-  uint16 size;
+  uint16_t size;
   if (!ReadParam(m, iter, &size))
     return false;
   if (size > sizeof(p->data))
@@ -118,7 +168,7 @@ bool ParamTraits<PP_NetAddress_Private>::Read(const Message* m,
   p->size = size;
 
   const char* data;
-  if (!m->ReadBytes(iter, &data, size))
+  if (!iter->ReadBytes(&data, size))
     return false;
   memcpy(p->data, data, size);
   return true;
@@ -137,18 +187,17 @@ void ParamTraits<PP_NetAddress_Private>::Log(const param_type& p,
 // static
 void ParamTraits<ppapi::HostResource>::Write(Message* m,
                                              const param_type& p) {
-  ParamTraits<PP_Instance>::Write(m, p.instance());
-  ParamTraits<PP_Resource>::Write(m, p.host_resource());
+  WriteParam(m, p.instance());
+  WriteParam(m, p.host_resource());
 }
 
 // static
 bool ParamTraits<ppapi::HostResource>::Read(const Message* m,
-                                            PickleIterator* iter,
+                                            base::PickleIterator* iter,
                                             param_type* r) {
   PP_Instance instance;
   PP_Resource resource;
-  if (!ParamTraits<PP_Instance>::Read(m, iter, &instance) ||
-      !ParamTraits<PP_Resource>::Read(m, iter, &resource))
+  if (!ReadParam(m, iter, &instance) || !ReadParam(m, iter, &resource))
     return false;
   r->SetHostResource(instance, resource);
   return true;
@@ -169,7 +218,7 @@ void ParamTraits<ppapi::proxy::SerializedVar>::Write(Message* m,
 
 // static
 bool ParamTraits<ppapi::proxy::SerializedVar>::Read(const Message* m,
-                                                    PickleIterator* iter,
+                                                    base::PickleIterator* iter,
                                                     param_type* r) {
   return r->ReadFromMessage(m, iter);
 }
@@ -188,9 +237,9 @@ void ParamTraits< std::vector<ppapi::proxy::SerializedVar> >::Write(
 }
 
 // static
-bool ParamTraits< std::vector<ppapi::proxy::SerializedVar> >::Read(
+bool ParamTraits<std::vector<ppapi::proxy::SerializedVar>>::Read(
     const Message* m,
-    PickleIterator* iter,
+    base::PickleIterator* iter,
     param_type* r) {
   return ReadVectorWithoutCopy(m, iter, r);
 }
@@ -205,15 +254,15 @@ void ParamTraits< std::vector<ppapi::proxy::SerializedVar> >::Log(
 
 void ParamTraits<ppapi::PpapiPermissions>::Write(Message* m,
                                                  const param_type& p) {
-  ParamTraits<uint32_t>::Write(m, p.GetBits());
+  WriteParam(m, p.GetBits());
 }
 
 // static
 bool ParamTraits<ppapi::PpapiPermissions>::Read(const Message* m,
-                                                PickleIterator* iter,
+                                                base::PickleIterator* iter,
                                                 param_type* r) {
   uint32_t bits;
-  if (!ParamTraits<uint32_t>::Read(m, iter, &bits))
+  if (!ReadParam(m, iter, &bits))
     return false;
   *r = ppapi::PpapiPermissions(bits);
   return true;
@@ -232,11 +281,11 @@ void ParamTraits<ppapi::proxy::SerializedHandle>::Write(Message* m,
   ppapi::proxy::SerializedHandle::WriteHeader(p.header(), m);
   switch (p.type()) {
     case ppapi::proxy::SerializedHandle::SHARED_MEMORY:
-      ParamTraits<base::SharedMemoryHandle>::Write(m, p.shmem());
+      WriteParam(m, p.shmem());
       break;
     case ppapi::proxy::SerializedHandle::SOCKET:
     case ppapi::proxy::SerializedHandle::FILE:
-      ParamTraits<IPC::PlatformFileForTransit>::Write(m, p.descriptor());
+      WriteParam(m, p.descriptor());
       break;
     case ppapi::proxy::SerializedHandle::INVALID:
       break;
@@ -245,16 +294,17 @@ void ParamTraits<ppapi::proxy::SerializedHandle>::Write(Message* m,
 }
 
 // static
-bool ParamTraits<ppapi::proxy::SerializedHandle>::Read(const Message* m,
-                                                       PickleIterator* iter,
-                                                       param_type* r) {
+bool ParamTraits<ppapi::proxy::SerializedHandle>::Read(
+    const Message* m,
+    base::PickleIterator* iter,
+    param_type* r) {
   ppapi::proxy::SerializedHandle::Header header;
   if (!ppapi::proxy::SerializedHandle::ReadHeader(iter, &header))
     return false;
   switch (header.type) {
     case ppapi::proxy::SerializedHandle::SHARED_MEMORY: {
       base::SharedMemoryHandle handle;
-      if (ParamTraits<base::SharedMemoryHandle>::Read(m, iter, &handle)) {
+      if (ReadParam(m, iter, &handle)) {
         r->set_shmem(handle, header.size);
         return true;
       }
@@ -262,7 +312,7 @@ bool ParamTraits<ppapi::proxy::SerializedHandle>::Read(const Message* m,
     }
     case ppapi::proxy::SerializedHandle::SOCKET: {
       IPC::PlatformFileForTransit socket;
-      if (ParamTraits<IPC::PlatformFileForTransit>::Read(m, iter, &socket)) {
+      if (ReadParam(m, iter, &socket)) {
         r->set_socket(socket);
         return true;
       }
@@ -270,7 +320,7 @@ bool ParamTraits<ppapi::proxy::SerializedHandle>::Read(const Message* m,
     }
     case ppapi::proxy::SerializedHandle::FILE: {
       IPC::PlatformFileForTransit desc;
-      if (ParamTraits<IPC::PlatformFileForTransit>::Read(m, iter, &desc)) {
+      if (ReadParam(m, iter, &desc)) {
         r->set_file_handle(desc, header.open_flags, header.file_io);
         return true;
       }
@@ -294,26 +344,24 @@ void ParamTraits<ppapi::proxy::SerializedHandle>::Log(const param_type& p,
 void ParamTraits<ppapi::proxy::PPBURLLoader_UpdateProgress_Params>::Write(
     Message* m,
     const param_type& p) {
-  ParamTraits<PP_Instance>::Write(m, p.instance);
-  ParamTraits<ppapi::HostResource>::Write(m, p.resource);
-  ParamTraits<int64_t>::Write(m, p.bytes_sent);
-  ParamTraits<int64_t>::Write(m, p.total_bytes_to_be_sent);
-  ParamTraits<int64_t>::Write(m, p.bytes_received);
-  ParamTraits<int64_t>::Write(m, p.total_bytes_to_be_received);
+  WriteParam(m, p.instance);
+  WriteParam(m, p.resource);
+  WriteParam(m, p.bytes_sent);
+  WriteParam(m, p.total_bytes_to_be_sent);
+  WriteParam(m, p.bytes_received);
+  WriteParam(m, p.total_bytes_to_be_received);
 }
 
 // static
 bool ParamTraits<ppapi::proxy::PPBURLLoader_UpdateProgress_Params>::Read(
     const Message* m,
-    PickleIterator* iter,
+    base::PickleIterator* iter,
     param_type* r) {
-  return
-      ParamTraits<PP_Instance>::Read(m, iter, &r->instance) &&
-      ParamTraits<ppapi::HostResource>::Read(m, iter, &r->resource) &&
-      ParamTraits<int64_t>::Read(m, iter, &r->bytes_sent) &&
-      ParamTraits<int64_t>::Read(m, iter, &r->total_bytes_to_be_sent) &&
-      ParamTraits<int64_t>::Read(m, iter, &r->bytes_received) &&
-      ParamTraits<int64_t>::Read(m, iter, &r->total_bytes_to_be_received);
+  return ReadParam(m, iter, &r->instance) && ReadParam(m, iter, &r->resource) &&
+         ReadParam(m, iter, &r->bytes_sent) &&
+         ReadParam(m, iter, &r->total_bytes_to_be_sent) &&
+         ReadParam(m, iter, &r->bytes_received) &&
+         ReadParam(m, iter, &r->total_bytes_to_be_received);
 }
 
 // static
@@ -328,52 +376,48 @@ void ParamTraits<ppapi::proxy::PPBURLLoader_UpdateProgress_Params>::Log(
 void ParamTraits<ppapi::proxy::PPBFlash_DrawGlyphs_Params>::Write(
     Message* m,
     const param_type& p) {
-  ParamTraits<PP_Instance>::Write(m, p.instance);
-  ParamTraits<ppapi::HostResource>::Write(m, p.image_data);
-  ParamTraits<ppapi::proxy::SerializedFontDescription>::Write(m, p.font_desc);
-  ParamTraits<uint32_t>::Write(m, p.color);
-  ParamTraits<PP_Point>::Write(m, p.position);
-  ParamTraits<PP_Rect>::Write(m, p.clip);
-  ParamTraits<float>::Write(m, p.transformation[0][0]);
-  ParamTraits<float>::Write(m, p.transformation[0][1]);
-  ParamTraits<float>::Write(m, p.transformation[0][2]);
-  ParamTraits<float>::Write(m, p.transformation[1][0]);
-  ParamTraits<float>::Write(m, p.transformation[1][1]);
-  ParamTraits<float>::Write(m, p.transformation[1][2]);
-  ParamTraits<float>::Write(m, p.transformation[2][0]);
-  ParamTraits<float>::Write(m, p.transformation[2][1]);
-  ParamTraits<float>::Write(m, p.transformation[2][2]);
-  ParamTraits<PP_Bool>::Write(m, p.allow_subpixel_aa);
-  ParamTraits<std::vector<uint16_t> >::Write(m, p.glyph_indices);
-  ParamTraits<std::vector<PP_Point> >::Write(m, p.glyph_advances);
+  WriteParam(m, p.instance);
+  WriteParam(m, p.image_data);
+  WriteParam(m, p.font_desc);
+  WriteParam(m, p.color);
+  WriteParam(m, p.position);
+  WriteParam(m, p.clip);
+  WriteParam(m, p.transformation[0][0]);
+  WriteParam(m, p.transformation[0][1]);
+  WriteParam(m, p.transformation[0][2]);
+  WriteParam(m, p.transformation[1][0]);
+  WriteParam(m, p.transformation[1][1]);
+  WriteParam(m, p.transformation[1][2]);
+  WriteParam(m, p.transformation[2][0]);
+  WriteParam(m, p.transformation[2][1]);
+  WriteParam(m, p.transformation[2][2]);
+  WriteParam(m, p.allow_subpixel_aa);
+  WriteParam(m, p.glyph_indices);
+  WriteParam(m, p.glyph_advances);
 }
 
 // static
 bool ParamTraits<ppapi::proxy::PPBFlash_DrawGlyphs_Params>::Read(
     const Message* m,
-    PickleIterator* iter,
+    base::PickleIterator* iter,
     param_type* r) {
-  return
-      ParamTraits<PP_Instance>::Read(m, iter, &r->instance) &&
-      ParamTraits<ppapi::HostResource>::Read(m, iter, &r->image_data) &&
-      ParamTraits<ppapi::proxy::SerializedFontDescription>::Read(m, iter,
-                                                              &r->font_desc) &&
-      ParamTraits<uint32_t>::Read(m, iter, &r->color) &&
-      ParamTraits<PP_Point>::Read(m, iter, &r->position) &&
-      ParamTraits<PP_Rect>::Read(m, iter, &r->clip) &&
-      ParamTraits<float>::Read(m, iter, &r->transformation[0][0]) &&
-      ParamTraits<float>::Read(m, iter, &r->transformation[0][1]) &&
-      ParamTraits<float>::Read(m, iter, &r->transformation[0][2]) &&
-      ParamTraits<float>::Read(m, iter, &r->transformation[1][0]) &&
-      ParamTraits<float>::Read(m, iter, &r->transformation[1][1]) &&
-      ParamTraits<float>::Read(m, iter, &r->transformation[1][2]) &&
-      ParamTraits<float>::Read(m, iter, &r->transformation[2][0]) &&
-      ParamTraits<float>::Read(m, iter, &r->transformation[2][1]) &&
-      ParamTraits<float>::Read(m, iter, &r->transformation[2][2]) &&
-      ParamTraits<PP_Bool>::Read(m, iter, &r->allow_subpixel_aa) &&
-      ParamTraits<std::vector<uint16_t> >::Read(m, iter, &r->glyph_indices) &&
-      ParamTraits<std::vector<PP_Point> >::Read(m, iter, &r->glyph_advances) &&
-      r->glyph_indices.size() == r->glyph_advances.size();
+  return ReadParam(m, iter, &r->instance) &&
+         ReadParam(m, iter, &r->image_data) &&
+         ReadParam(m, iter, &r->font_desc) && ReadParam(m, iter, &r->color) &&
+         ReadParam(m, iter, &r->position) && ReadParam(m, iter, &r->clip) &&
+         ReadParam(m, iter, &r->transformation[0][0]) &&
+         ReadParam(m, iter, &r->transformation[0][1]) &&
+         ReadParam(m, iter, &r->transformation[0][2]) &&
+         ReadParam(m, iter, &r->transformation[1][0]) &&
+         ReadParam(m, iter, &r->transformation[1][1]) &&
+         ReadParam(m, iter, &r->transformation[1][2]) &&
+         ReadParam(m, iter, &r->transformation[2][0]) &&
+         ReadParam(m, iter, &r->transformation[2][1]) &&
+         ReadParam(m, iter, &r->transformation[2][2]) &&
+         ReadParam(m, iter, &r->allow_subpixel_aa) &&
+         ReadParam(m, iter, &r->glyph_indices) &&
+         ReadParam(m, iter, &r->glyph_advances) &&
+         r->glyph_indices.size() == r->glyph_advances.size();
 }
 
 // static
@@ -387,16 +431,16 @@ void ParamTraits<ppapi::proxy::PPBFlash_DrawGlyphs_Params>::Log(
 // static
 void ParamTraits<ppapi::proxy::SerializedDirEntry>::Write(Message* m,
                                                           const param_type& p) {
-  ParamTraits<std::string>::Write(m, p.name);
-  ParamTraits<bool>::Write(m, p.is_dir);
+  WriteParam(m, p.name);
+  WriteParam(m, p.is_dir);
 }
 
 // static
-bool ParamTraits<ppapi::proxy::SerializedDirEntry>::Read(const Message* m,
-                                                         PickleIterator* iter,
-                                                         param_type* r) {
-  return ParamTraits<std::string>::Read(m, iter, &r->name) &&
-         ParamTraits<bool>::Read(m, iter, &r->is_dir);
+bool ParamTraits<ppapi::proxy::SerializedDirEntry>::Read(
+    const Message* m,
+    base::PickleIterator* iter,
+    param_type* r) {
+  return ReadParam(m, iter, &r->name) && ReadParam(m, iter, &r->is_dir);
 }
 
 // static
@@ -410,30 +454,26 @@ void ParamTraits<ppapi::proxy::SerializedDirEntry>::Log(const param_type& p,
 void ParamTraits<ppapi::proxy::SerializedFontDescription>::Write(
     Message* m,
     const param_type& p) {
-  ParamTraits<std::string>::Write(m, p.face);
-  ParamTraits<int32_t>::Write(m, p.family);
-  ParamTraits<uint32_t>::Write(m, p.size);
-  ParamTraits<int32_t>::Write(m, p.weight);
-  ParamTraits<PP_Bool>::Write(m, p.italic);
-  ParamTraits<PP_Bool>::Write(m, p.small_caps);
-  ParamTraits<int32_t>::Write(m, p.letter_spacing);
-  ParamTraits<int32_t>::Write(m, p.word_spacing);
+  WriteParam(m, p.face);
+  WriteParam(m, p.family);
+  WriteParam(m, p.size);
+  WriteParam(m, p.weight);
+  WriteParam(m, p.italic);
+  WriteParam(m, p.small_caps);
+  WriteParam(m, p.letter_spacing);
+  WriteParam(m, p.word_spacing);
 }
 
 // static
 bool ParamTraits<ppapi::proxy::SerializedFontDescription>::Read(
     const Message* m,
-    PickleIterator* iter,
+    base::PickleIterator* iter,
     param_type* r) {
-  return
-      ParamTraits<std::string>::Read(m, iter, &r->face) &&
-      ParamTraits<int32_t>::Read(m, iter, &r->family) &&
-      ParamTraits<uint32_t>::Read(m, iter, &r->size) &&
-      ParamTraits<int32_t>::Read(m, iter, &r->weight) &&
-      ParamTraits<PP_Bool>::Read(m, iter, &r->italic) &&
-      ParamTraits<PP_Bool>::Read(m, iter, &r->small_caps) &&
-      ParamTraits<int32_t>::Read(m, iter, &r->letter_spacing) &&
-      ParamTraits<int32_t>::Read(m, iter, &r->word_spacing);
+  return ReadParam(m, iter, &r->face) && ReadParam(m, iter, &r->family) &&
+         ReadParam(m, iter, &r->size) && ReadParam(m, iter, &r->weight) &&
+         ReadParam(m, iter, &r->italic) && ReadParam(m, iter, &r->small_caps) &&
+         ReadParam(m, iter, &r->letter_spacing) &&
+         ReadParam(m, iter, &r->word_spacing);
 }
 
 // static
@@ -449,27 +489,23 @@ void ParamTraits<ppapi::proxy::SerializedFontDescription>::Log(
 void ParamTraits<ppapi::proxy::SerializedTrueTypeFontDesc>::Write(
     Message* m,
     const param_type& p) {
-  ParamTraits<std::string>::Write(m, p.family);
-  ParamTraits<PP_TrueTypeFontFamily_Dev>::Write(m, p.generic_family);
-  ParamTraits<PP_TrueTypeFontStyle_Dev>::Write(m, p.style);
-  ParamTraits<PP_TrueTypeFontWeight_Dev>::Write(m, p.weight);
-  ParamTraits<PP_TrueTypeFontWidth_Dev>::Write(m, p.width);
-  ParamTraits<PP_TrueTypeFontCharset_Dev>::Write(m, p.charset);
+  WriteParam(m, p.family);
+  WriteParam(m, p.generic_family);
+  WriteParam(m, p.style);
+  WriteParam(m, p.weight);
+  WriteParam(m, p.width);
+  WriteParam(m, p.charset);
 }
 
 // static
 bool ParamTraits<ppapi::proxy::SerializedTrueTypeFontDesc>::Read(
     const Message* m,
-    PickleIterator* iter,
+    base::PickleIterator* iter,
     param_type* r) {
-  return
-      ParamTraits<std::string>::Read(m, iter, &r->family) &&
-      ParamTraits<PP_TrueTypeFontFamily_Dev>::Read(m, iter,
-                                                   &r->generic_family) &&
-      ParamTraits<PP_TrueTypeFontStyle_Dev>::Read(m, iter, &r->style) &&
-      ParamTraits<PP_TrueTypeFontWeight_Dev>::Read(m, iter, &r->weight) &&
-      ParamTraits<PP_TrueTypeFontWidth_Dev>::Read(m, iter, &r->width) &&
-      ParamTraits<PP_TrueTypeFontCharset_Dev>::Read(m, iter, &r->charset);
+  return ReadParam(m, iter, &r->family) &&
+         ReadParam(m, iter, &r->generic_family) &&
+         ReadParam(m, iter, &r->style) && ReadParam(m, iter, &r->weight) &&
+         ReadParam(m, iter, &r->width) && ReadParam(m, iter, &r->charset);
 }
 
 // static
@@ -490,7 +526,7 @@ void ParamTraits<ppapi::PepperFilePath>::Write(Message* m,
 
 // static
 bool ParamTraits<ppapi::PepperFilePath>::Read(const Message* m,
-                                              PickleIterator* iter,
+                                              base::PickleIterator* iter,
                                               param_type* p) {
   unsigned domain;
   base::FilePath path;
@@ -524,9 +560,10 @@ void ParamTraits<ppapi::proxy::SerializedFlashMenu>::Write(
 }
 
 // static
-bool ParamTraits<ppapi::proxy::SerializedFlashMenu>::Read(const Message* m,
-                                                          PickleIterator* iter,
-                                                          param_type* r) {
+bool ParamTraits<ppapi::proxy::SerializedFlashMenu>::Read(
+    const Message* m,
+    base::PickleIterator* iter,
+    param_type* r) {
   return r->ReadFromMessage(m, iter);
 }
 
@@ -542,14 +579,15 @@ void ParamTraits<ppapi::proxy::SerializedFlashMenu>::Log(const param_type& p,
 void ParamTraits<ppapi::PPB_X509Certificate_Fields>::Write(
     Message* m,
     const param_type& p) {
-  ParamTraits<base::ListValue>::Write(m, p.values_);
+  WriteParam(m, p.values_);
 }
 
 // static
-bool ParamTraits<ppapi::PPB_X509Certificate_Fields>::Read(const Message* m,
-                                                          PickleIterator* iter,
-                                                          param_type* r) {
-  return ParamTraits<base::ListValue>::Read(m, iter, &(r->values_));
+bool ParamTraits<ppapi::PPB_X509Certificate_Fields>::Read(
+    const Message* m,
+    base::PickleIterator* iter,
+    param_type* r) {
+  return ReadParam(m, iter, &(r->values_));
 }
 
 // static
@@ -563,7 +601,7 @@ void ParamTraits<ppapi::PPB_X509Certificate_Fields>::Log(const param_type& p,
 void ParamTraits<ppapi::SocketOptionData>::Write(Message* m,
                                                  const param_type& p) {
   ppapi::SocketOptionData::Type type = p.GetType();
-  ParamTraits<int32_t>::Write(m, static_cast<int32_t>(type));
+  WriteParam(m, static_cast<int32_t>(type));
   switch (type) {
     case ppapi::SocketOptionData::TYPE_INVALID: {
       break;
@@ -575,7 +613,7 @@ void ParamTraits<ppapi::SocketOptionData>::Write(Message* m,
       static_cast<void>(result);
       DCHECK(result);
 
-      ParamTraits<bool>::Write(m, out_value);
+      WriteParam(m, out_value);
       break;
     }
     case ppapi::SocketOptionData::TYPE_INT32: {
@@ -585,7 +623,7 @@ void ParamTraits<ppapi::SocketOptionData>::Write(Message* m,
       static_cast<void>(result);
       DCHECK(result);
 
-      ParamTraits<int32_t>::Write(m, out_value);
+      WriteParam(m, out_value);
       break;
     }
     // No default so the compiler will warn on new types.
@@ -594,11 +632,11 @@ void ParamTraits<ppapi::SocketOptionData>::Write(Message* m,
 
 // static
 bool ParamTraits<ppapi::SocketOptionData>::Read(const Message* m,
-                                                PickleIterator* iter,
+                                                base::PickleIterator* iter,
                                                 param_type* r) {
   *r = ppapi::SocketOptionData();
   int32_t type = 0;
-  if (!ParamTraits<int32_t>::Read(m, iter, &type))
+  if (!ReadParam(m, iter, &type))
     return false;
   if (type != ppapi::SocketOptionData::TYPE_INVALID &&
       type != ppapi::SocketOptionData::TYPE_BOOL &&
@@ -611,14 +649,14 @@ bool ParamTraits<ppapi::SocketOptionData>::Read(const Message* m,
     }
     case ppapi::SocketOptionData::TYPE_BOOL: {
       bool value = false;
-      if (!ParamTraits<bool>::Read(m, iter, &value))
+      if (!ReadParam(m, iter, &value))
         return false;
       r->SetBool(value);
       return true;
     }
     case ppapi::SocketOptionData::TYPE_INT32: {
       int32_t value = 0;
-      if (!ParamTraits<int32_t>::Read(m, iter, &value))
+      if (!ReadParam(m, iter, &value))
         return false;
       r->SetInt32(value);
       return true;
@@ -640,16 +678,16 @@ void ParamTraits<ppapi::CompositorLayerData::Transform>::Write(
     Message* m,
     const param_type& p) {
   for (size_t i = 0; i < arraysize(p.matrix); i++)
-    ParamTraits<float>::Write(m, p.matrix[i]);
+    WriteParam(m, p.matrix[i]);
 }
 
 // static
 bool ParamTraits<ppapi::CompositorLayerData::Transform>::Read(
     const Message* m,
-    PickleIterator* iter,
+    base::PickleIterator* iter,
     param_type* r) {
   for (size_t i = 0; i < arraysize(r->matrix);i++) {
-    if (!ParamTraits<float>::Read(m, iter, &r->matrix[i]))
+    if (!ReadParam(m, iter, &r->matrix[i]))
       return false;
   }
   return true;
