@@ -28,8 +28,8 @@ namespace {
 // When sending array buffers, if the size is over 256K, we use shared
 // memory instead of sending the data over IPC. Light testing suggests
 // shared memory is much faster for 256K and larger messages.
-static const uint32 kMinimumArrayBufferSizeForShmem = 256 * 1024;
-static uint32 g_minimum_array_buffer_size_for_shmem =
+static const uint32_t kMinimumArrayBufferSizeForShmem = 256 * 1024;
+static uint32_t g_minimum_array_buffer_size_for_shmem =
     kMinimumArrayBufferSizeForShmem;
 
 struct StackEntry {
@@ -156,7 +156,7 @@ scoped_ptr<RawVarDataGraph> RawVarDataGraph::Create(const PP_Var& var,
       }
     }
   }
-  return graph.Pass();
+  return graph;
 }
 
 PP_Var RawVarDataGraph::CreatePPVar(PP_Instance instance) {
@@ -185,21 +185,21 @@ void RawVarDataGraph::Write(IPC::Message* m,
 
 // static
 scoped_ptr<RawVarDataGraph> RawVarDataGraph::Read(const IPC::Message* m,
-                                                  PickleIterator* iter) {
+                                                  base::PickleIterator* iter) {
   scoped_ptr<RawVarDataGraph> result(new RawVarDataGraph);
   uint32_t size = 0;
-  if (!m->ReadUInt32(iter, &size))
+  if (!iter->ReadUInt32(&size))
     return scoped_ptr<RawVarDataGraph>();
   for (uint32_t i = 0; i < size; ++i) {
     int32_t type;
-    if (!m->ReadInt(iter, &type))
+    if (!iter->ReadInt(&type))
       return scoped_ptr<RawVarDataGraph>();
     PP_VarType var_type = static_cast<PP_VarType>(type);
     result->data_.push_back(RawVarData::Create(var_type));
     if (!result->data_.back()->Read(var_type, m, iter))
       return scoped_ptr<RawVarDataGraph>();
   }
-  return result.Pass();
+  return result;
 }
 
 std::vector<SerializedHandle*> RawVarDataGraph::GetHandles() {
@@ -214,7 +214,7 @@ std::vector<SerializedHandle*> RawVarDataGraph::GetHandles() {
 
 // static
 void RawVarDataGraph::SetMinimumArrayBufferSizeForShmemForTest(
-    uint32 threshold) {
+    uint32_t threshold) {
   if (threshold == 0)
     g_minimum_array_buffer_size_for_shmem = kMinimumArrayBufferSizeForShmem;
   else
@@ -299,7 +299,7 @@ void BasicRawVarData::Write(
       m->WriteInt(var_.value.as_int);
       break;
     case PP_VARTYPE_DOUBLE:
-      IPC::ParamTraits<double>::Write(m, var_.value.as_double);
+      IPC::WriteParam(m, var_.value.as_double);
       break;
     case PP_VARTYPE_OBJECT:
       m->WriteInt64(var_.value.as_id);
@@ -312,7 +312,7 @@ void BasicRawVarData::Write(
 
 bool BasicRawVarData::Read(PP_VarType type,
                            const IPC::Message* m,
-                           PickleIterator* iter) {
+                           base::PickleIterator* iter) {
   PP_Var result;
   result.type = type;
   switch (type) {
@@ -323,21 +323,21 @@ bool BasicRawVarData::Read(PP_VarType type,
       break;
     case PP_VARTYPE_BOOL: {
       bool bool_value;
-      if (!m->ReadBool(iter, &bool_value))
+      if (!iter->ReadBool(&bool_value))
         return false;
       result.value.as_bool = PP_FromBool(bool_value);
       break;
     }
     case PP_VARTYPE_INT32:
-      if (!m->ReadInt(iter, &result.value.as_int))
+      if (!iter->ReadInt(&result.value.as_int))
         return false;
       break;
     case PP_VARTYPE_DOUBLE:
-      if (!IPC::ParamTraits<double>::Read(m, iter, &result.value.as_double))
+      if (!IPC::ReadParam(m, iter, &result.value.as_double))
         return false;
       break;
     case PP_VARTYPE_OBJECT:
-      if (!m->ReadInt64(iter, &result.value.as_id))
+      if (!iter->ReadInt64(&result.value.as_id))
         return false;
       break;
     default:
@@ -384,8 +384,8 @@ void StringRawVarData::Write(IPC::Message* m,
 
 bool StringRawVarData::Read(PP_VarType type,
                             const IPC::Message* m,
-                            PickleIterator* iter) {
-  if (!m->ReadString(iter, &data_))
+                            base::PickleIterator* iter) {
+  if (!iter->ReadString(&data_))
     return false;
   return true;
 }
@@ -444,7 +444,7 @@ PP_Var ArrayBufferRawVarData::CreatePPVar(PP_Instance instance) {
   switch (type_) {
     case ARRAY_BUFFER_SHMEM_HOST: {
       base::SharedMemoryHandle host_handle;
-      uint32 size_in_bytes;
+      uint32_t size_in_bytes;
       bool ok = PpapiGlobals::Get()->GetVarTracker()->
           StopTrackingSharedMemoryHandle(host_shm_handle_id_,
                                          instance,
@@ -467,7 +467,7 @@ PP_Var ArrayBufferRawVarData::CreatePPVar(PP_Instance instance) {
     }
     case ARRAY_BUFFER_NO_SHMEM: {
       result = PpapiGlobals::Get()->GetVarTracker()->MakeArrayBufferPPVar(
-          static_cast<uint32>(data_.size()), data_.data());
+          static_cast<uint32_t>(data_.size()), data_.data());
       break;
     }
     default:
@@ -501,24 +501,23 @@ void ArrayBufferRawVarData::Write(
 
 bool ArrayBufferRawVarData::Read(PP_VarType type,
                                  const IPC::Message* m,
-                                 PickleIterator* iter) {
+                                 base::PickleIterator* iter) {
   int shmem_type;
-  if (!m->ReadInt(iter, &shmem_type))
+  if (!iter->ReadInt(&shmem_type))
     return false;
   type_ = static_cast<ShmemType>(shmem_type);
   switch (type_) {
     case ARRAY_BUFFER_SHMEM_HOST:
-      if (!m->ReadInt(iter, &host_shm_handle_id_))
+      if (!iter->ReadInt(&host_shm_handle_id_))
         return false;
       break;
     case ARRAY_BUFFER_SHMEM_PLUGIN:
-      if (!IPC::ParamTraits<SerializedHandle>::Read(
-              m, iter, &plugin_shm_handle_)) {
+      if (!IPC::ReadParam(m, iter, &plugin_shm_handle_)) {
         return false;
       }
       break;
     case ARRAY_BUFFER_NO_SHMEM:
-      if (!m->ReadString(iter, &data_))
+      if (!iter->ReadString(&data_))
         return false;
       break;
     default:
@@ -582,13 +581,13 @@ void ArrayRawVarData::Write(IPC::Message* m,
 
 bool ArrayRawVarData::Read(PP_VarType type,
                            const IPC::Message* m,
-                           PickleIterator* iter) {
+                           base::PickleIterator* iter) {
   uint32_t size;
-  if (!m->ReadUInt32(iter, &size))
+  if (!iter->ReadUInt32(&size))
     return false;
   for (uint32_t i = 0; i < size; ++i) {
     uint32_t index;
-    if (!m->ReadUInt32(iter, &index))
+    if (!iter->ReadUInt32(&index))
       return false;
     children_.push_back(index);
   }
@@ -648,16 +647,16 @@ void DictionaryRawVarData::Write(
 
 bool DictionaryRawVarData::Read(PP_VarType type,
                                 const IPC::Message* m,
-                                PickleIterator* iter) {
+                                base::PickleIterator* iter) {
   uint32_t size;
-  if (!m->ReadUInt32(iter, &size))
+  if (!iter->ReadUInt32(&size))
     return false;
   for (uint32_t i = 0; i < size; ++i) {
     std::string key;
     uint32_t value;
-    if (!m->ReadString(iter, &key))
+    if (!iter->ReadString(&key))
       return false;
-    if (!m->ReadUInt32(iter, &value))
+    if (!iter->ReadUInt32(&value))
       return false;
     children_.push_back(make_pair(key, value));
   }
@@ -720,26 +719,26 @@ void ResourceRawVarData::Write(IPC::Message* m,
   m->WriteInt(pending_browser_host_id_);
   m->WriteBool(creation_message_);
   if (creation_message_)
-    IPC::ParamTraits<IPC::Message>::Write(m, *creation_message_);
+    IPC::WriteParam(m, *creation_message_);
 }
 
 bool ResourceRawVarData::Read(PP_VarType type,
                               const IPC::Message* m,
-                              PickleIterator* iter) {
+                              base::PickleIterator* iter) {
   int value;
-  if (!m->ReadInt(iter, &value))
+  if (!iter->ReadInt(&value))
     return false;
   pp_resource_ = static_cast<PP_Resource>(value);
-  if (!m->ReadInt(iter, &pending_renderer_host_id_))
+  if (!iter->ReadInt(&pending_renderer_host_id_))
     return false;
-  if (!m->ReadInt(iter, &pending_browser_host_id_))
+  if (!iter->ReadInt(&pending_browser_host_id_))
     return false;
   bool has_creation_message;
-  if (!m->ReadBool(iter, &has_creation_message))
+  if (!iter->ReadBool(&has_creation_message))
     return false;
   if (has_creation_message) {
     creation_message_.reset(new IPC::Message());
-    if (!IPC::ParamTraits<IPC::Message>::Read(m, iter, creation_message_.get()))
+    if (!IPC::ReadParam(m, iter, creation_message_.get()))
       return false;
   } else {
     creation_message_.reset();
