@@ -4,7 +4,6 @@
 
 #include "ppapi/shared_impl/ppb_trace_event_impl.h"
 
-#include "base/basictypes.h"
 #include "base/threading/platform_thread.h"
 #include "base/trace_event/trace_event.h"
 #include "ppapi/thunk/thunk.h"
@@ -78,8 +77,9 @@ void TraceEventImpl::AddTraceEventWithThreadIdAndTimestamp(
           static_cast<const unsigned char*>(category_enabled),
           name,
           id,
+          trace_event_internal::kNoId,
           thread_id,
-          base::TraceTicks::FromInternalValue(timestamp),
+          base::TimeTicks::FromInternalValue(timestamp),
           num_args,
           arg_names,
           arg_types,
@@ -94,7 +94,7 @@ void TraceEventImpl::AddTraceEventWithThreadIdAndTimestamp(
 
 // static
 int64_t TraceEventImpl::Now() {
-  return base::TraceTicks::Now().ToInternalValue();
+  return base::TimeTicks::Now().ToInternalValue();
 }
 
 // static
