@@ -5,6 +5,7 @@
 #ifndef STORAGE_BROWSER_FILEAPI_DRAGGED_FILE_UTIL_H_
 #define STORAGE_BROWSER_FILEAPI_DRAGGED_FILE_UTIL_H_
 
+#include "base/macros.h"
 #include "base/memory/scoped_ptr.h"
 #include "storage/browser/fileapi/local_file_util.h"
 #include "storage/browser/storage_browser_export.h"
@@ -16,8 +17,7 @@ class FileSystemOperationContext;
 // Dragged file system is a specialized LocalFileUtil where read access to
 // the virtual root directory (i.e. empty cracked path case) is allowed
 // and single isolated context may be associated with multiple file paths.
-class STORAGE_EXPORT_PRIVATE DraggedFileUtil
-    : public LocalFileUtil {
+class STORAGE_EXPORT DraggedFileUtil : public LocalFileUtil {
  public:
   DraggedFileUtil();
   ~DraggedFileUtil() override {}
