@@ -19,17 +19,18 @@ RecordInfo::RecordInfo(CXXRecordDecl* record, RecordCache* cache)
       is_non_newable_(kNotComputed),
       is_only_placement_newable_(kNotComputed),
       does_need_finalization_(kNotComputed),
+      has_gc_mixin_methods_(kNotComputed),
+      is_declaring_local_trace_(kNotComputed),
+      is_eagerly_finalized_(kNotComputed),
       determined_trace_methods_(false),
       trace_method_(0),
       trace_dispatch_method_(0),
       finalize_dispatch_method_(0),
-      is_gc_derived_(false),
-      base_paths_(0) {}
+      is_gc_derived_(false) {}
 
 RecordInfo::~RecordInfo() {
   delete fields_;
   delete bases_;
-  delete base_paths_;
 }
 
 // Get |count| number of template arguments. Returns false if there
@@ -74,22 +75,12 @@ bool RecordInfo::IsHeapAllocatedCollection() {
   return Config::IsGCCollection(name_);
 }
 
-static bool IsGCBaseCallback(const CXXBaseSpecifier* specifier,
-                             CXXBasePath& path,
-                             void* data) {
-  if (CXXRecordDecl* record = specifier->getType()->getAsCXXRecordDecl())
-    return Config::IsGCBase(record->getName());
-  return false;
-}
-
 // Test if a record is derived from a garbage collected base.
 bool RecordInfo::IsGCDerived() {
   // If already computed, return the known result.
-  if (base_paths_)
+  if (gc_base_names_.size())
     return is_gc_derived_;
 
-  base_paths_ = new CXXBasePaths(true, true, false);
-
   if (!record_->hasDefinition())
     return false;
 
@@ -98,19 +89,62 @@ bool RecordInfo::IsGCDerived() {
     return false;
 
   // Walk the inheritance tree to find GC base classes.
-  is_gc_derived_ = record_->lookupInBases(IsGCBaseCallback, 0, *base_paths_);
+  walkBases();
   return is_gc_derived_;
 }
 
+CXXRecordDecl* RecordInfo::GetDependentTemplatedDecl(const Type& type) {
+  const TemplateSpecializationType* tmpl_type =
+      type.getAs<TemplateSpecializationType>();
+  if (!tmpl_type)
+    return 0;
+
+  TemplateDecl* tmpl_decl = tmpl_type->getTemplateName().getAsTemplateDecl();
+  if (!tmpl_decl)
+    return 0;
+
+  return dyn_cast_or_null<CXXRecordDecl>(tmpl_decl->getTemplatedDecl());
+}
+
+void RecordInfo::walkBases() {
+  // This traversal is akin to CXXRecordDecl::forallBases()'s,
+  // but without stepping over dependent bases -- these might also
+  // have a "GC base name", so are to be included and considered.
+  SmallVector<const CXXRecordDecl*, 8> queue;
+
+  const CXXRecordDecl *base_record = record();
+  while (true) {
+    for (const auto& it : base_record->bases()) {
+      const RecordType *type = it.getType()->getAs<RecordType>();
+      CXXRecordDecl* base;
+      if (!type)
+        base = GetDependentTemplatedDecl(*it.getType());
+      else {
+        base = cast_or_null<CXXRecordDecl>(type->getDecl()->getDefinition());
+        if (base)
+          queue.push_back(base);
+      }
+      if (!base)
+        continue;
+
+      const std::string& name = base->getName();
+      if (Config::IsGCBase(name)) {
+        gc_base_names_.push_back(name);
+        is_gc_derived_ = true;
+      }
+    }
+
+    if (queue.empty())
+      break;
+    base_record = queue.pop_back_val(); // not actually a queue.
+  }
+}
+
 bool RecordInfo::IsGCFinalized() {
   if (!IsGCDerived())
     return false;
-  for (CXXBasePaths::paths_iterator it = base_paths_->begin();
-       it != base_paths_->end();
-       ++it) {
-    const CXXBasePathElement& elem = (*it)[it->size() - 1];
-    CXXRecordDecl* base = elem.Base->getType()->getAsCXXRecordDecl();
-    if (Config::IsGCFinalizedBase(base->getName()))
+  for (const auto& gc_base : gc_base_names_) {
+    if (Config::IsGCFinalizedBase(gc_base))
       return true;
   }
   return false;
@@ -119,16 +153,11 @@ bool RecordInfo::IsGCFinalized() {
 // A GC mixin is a class that inherits from a GC mixin base and has
 // not yet been "mixed in" with another GC base class.
 bool RecordInfo::IsGCMixin() {
-  if (!IsGCDerived() || base_paths_->begin() == base_paths_->end())
+  if (!IsGCDerived() || !gc_base_names_.size())
     return false;
-  for (CXXBasePaths::paths_iterator it = base_paths_->begin();
-       it != base_paths_->end();
-       ++it) {
-      // Get the last element of the path.
-      const CXXBasePathElement& elem = (*it)[it->size() - 1];
-      CXXRecordDecl* base = elem.Base->getType()->getAsCXXRecordDecl();
+  for (const auto& gc_base : gc_base_names_) {
       // If it is not a mixin base we are done.
-      if (!Config::IsGCMixinBase(base->getName()))
+      if (!Config::IsGCMixinBase(gc_base))
           return false;
   }
   // This is a mixin if all GC bases are mixins.
@@ -140,6 +169,37 @@ bool RecordInfo::IsGCAllocated() {
   return IsGCDerived() || IsHeapAllocatedCollection();
 }
 
+bool RecordInfo::IsEagerlyFinalized() {
+  if (is_eagerly_finalized_ == kNotComputed) {
+    is_eagerly_finalized_ = kFalse;
+    if (IsGCFinalized()) {
+      for (Decl* decl : record_->decls()) {
+        if (TypedefDecl* typedef_decl = dyn_cast<TypedefDecl>(decl)) {
+          if (typedef_decl->getNameAsString() == kIsEagerlyFinalizedName) {
+            is_eagerly_finalized_ = kTrue;
+            break;
+          }
+        }
+      }
+    }
+  }
+  return is_eagerly_finalized_;
+}
+
+bool RecordInfo::IsGCRefCounted() {
+  if (!IsGCDerived())
+    return false;
+  for (const auto& gc_base : gc_base_names_) {
+    if (Config::IsGCRefCountedBase(gc_base))
+      return true;
+  }
+  return false;
+}
+
+bool RecordInfo::HasDefinition() {
+  return record_->hasDefinition();
+}
+
 RecordInfo* RecordCache::Lookup(CXXRecordDecl* record) {
   // Ignore classes annotated with the GC_PLUGIN_IGNORE macro.
   if (!record || Config::IsIgnoreAnnotated(record))
@@ -283,6 +343,28 @@ CXXMethodDecl* RecordInfo::InheritsNonVirtualTrace() {
   return 0;
 }
 
+bool RecordInfo::DeclaresGCMixinMethods() {
+  DetermineTracingMethods();
+  return has_gc_mixin_methods_;
+}
+
+bool RecordInfo::DeclaresLocalTraceMethod() {
+  if (is_declaring_local_trace_ != kNotComputed)
+    return is_declaring_local_trace_;
+  DetermineTracingMethods();
+  is_declaring_local_trace_ = trace_method_ ? kTrue : kFalse;
+  if (is_declaring_local_trace_) {
+    for (auto it = record_->method_begin();
+         it != record_->method_end(); ++it) {
+      if (*it == trace_method_) {
+        is_declaring_local_trace_ = kTrue;
+        break;
+      }
+    }
+  }
+  return is_declaring_local_trace_;
+}
+
 // A (non-virtual) class is considered abstract in Blink if it has
 // no public constructors and no create methods.
 bool RecordInfo::IsConsideredAbstract() {
@@ -356,30 +438,56 @@ void RecordInfo::DetermineTracingMethods() {
   determined_trace_methods_ = true;
   if (Config::IsGCBase(name_))
     return;
-  CXXMethodDecl* trace = 0;
-  CXXMethodDecl* traceAfterDispatch = 0;
-  bool isTraceAfterDispatch;
-  for (CXXRecordDecl::method_iterator it = record_->method_begin();
-       it != record_->method_end();
-       ++it) {
-    if (Config::IsTraceMethod(*it, &isTraceAfterDispatch)) {
-      if (isTraceAfterDispatch) {
-        traceAfterDispatch = *it;
-      } else {
-        trace = *it;
-      }
-    } else if (it->getNameAsString() == kFinalizeName) {
-      finalize_dispatch_method_ = *it;
+  CXXMethodDecl* trace = nullptr;
+  CXXMethodDecl* trace_impl = nullptr;
+  CXXMethodDecl* trace_after_dispatch = nullptr;
+  bool has_adjust_and_mark = false;
+  bool has_is_heap_object_alive = false;
+  for (Decl* decl : record_->decls()) {
+    CXXMethodDecl* method = dyn_cast<CXXMethodDecl>(decl);
+    if (!method) {
+      if (FunctionTemplateDecl* func_template =
+          dyn_cast<FunctionTemplateDecl>(decl))
+        method = dyn_cast<CXXMethodDecl>(func_template->getTemplatedDecl());
+    }
+    if (!method)
+      continue;
+
+    switch (Config::GetTraceMethodType(method)) {
+      case Config::TRACE_METHOD:
+        trace = method;
+        break;
+      case Config::TRACE_AFTER_DISPATCH_METHOD:
+        trace_after_dispatch = method;
+        break;
+      case Config::TRACE_IMPL_METHOD:
+        trace_impl = method;
+        break;
+      case Config::TRACE_AFTER_DISPATCH_IMPL_METHOD:
+        break;
+      case Config::NOT_TRACE_METHOD:
+        if (method->getNameAsString() == kFinalizeName) {
+          finalize_dispatch_method_ = method;
+        } else if (method->getNameAsString() == kAdjustAndMarkName) {
+          has_adjust_and_mark = true;
+        } else if (method->getNameAsString() == kIsHeapObjectAliveName) {
+          has_is_heap_object_alive = true;
+        }
+        break;
     }
   }
-  if (traceAfterDispatch) {
-    trace_method_ = traceAfterDispatch;
-    trace_dispatch_method_ = trace;
+
+  // Record if class defines the two GCMixin methods.
+  has_gc_mixin_methods_ =
+      has_adjust_and_mark && has_is_heap_object_alive ? kTrue : kFalse;
+  if (trace_after_dispatch) {
+    trace_method_ = trace_after_dispatch;
+    trace_dispatch_method_ = trace_impl ? trace_impl : trace;
   } else {
     // TODO: Can we never have a dispatch method called trace without the same
     // class defining a traceAfterDispatch method?
     trace_method_ = trace;
-    trace_dispatch_method_ = 0;
+    trace_dispatch_method_ = nullptr;
   }
   if (trace_dispatch_method_ && finalize_dispatch_method_)
     return;
@@ -474,9 +582,9 @@ Edge* RecordInfo::CreateEdge(const Type* type) {
     return 0;
   }
 
-  if (type->isPointerType()) {
+  if (type->isPointerType() || type->isReferenceType()) {
     if (Edge* ptr = CreateEdge(type->getPointeeType().getTypePtrOrNull()))
-      return new RawPtr(ptr, false);
+      return new RawPtr(ptr, false, type->isReferenceType());
     return 0;
   }
 
@@ -491,7 +599,7 @@ Edge* RecordInfo::CreateEdge(const Type* type) {
 
   if (Config::IsRawPtr(info->name()) && info->GetTemplateArgs(1, &args)) {
     if (Edge* ptr = CreateEdge(args[0]))
-      return new RawPtr(ptr, true);
+      return new RawPtr(ptr, true, false);
     return 0;
   }
 
