@@ -15,18 +15,27 @@ CLANG_LIB_PATH = os.path.normpath(os.path.join(
 
 FLAGS = '-Xclang -add-plugin -Xclang blink-gc-plugin'
 PREFIX= ' -Xclang -plugin-arg-blink-gc-plugin -Xclang '
+
+warn_raw_pointers = None
 for arg in sys.argv[1:]:
   if arg == 'enable-oilpan=1':
     FLAGS += PREFIX + 'enable-oilpan'
+    if warn_raw_pointers is None:
+      warn_raw_pointers = True
   elif arg == 'dump-graph=1':
     FLAGS += PREFIX + 'dump-graph'
   elif arg == 'warn-raw-ptr=1':
-    FLAGS += PREFIX + 'warn-raw-ptr'
+    warn_raw_pointers = True
+  elif arg == 'warn-raw-ptr=0':
+    warn_raw_pointers = False
   elif arg == 'warn-unneeded-finalizer=1':
     FLAGS += PREFIX + 'warn-unneeded-finalizer'
   elif arg.startswith('custom_clang_lib_path='):
     CLANG_LIB_PATH = arg[len('custom_clang_lib_path='):]
 
+if warn_raw_pointers is True:
+  FLAGS += PREFIX + 'warn-raw-ptr'
+
 if not sys.platform in ['win32', 'cygwin']:
   LIBSUFFIX = 'dylib' if sys.platform == 'darwin' else 'so'
   FLAGS = ('-Xclang -load -Xclang "%s/libBlinkGCPlugin.%s" ' + FLAGS) % \
