@@ -45,7 +45,7 @@ class MacTool(object):
     """Transforms a tool name like copy-info-plist to CopyInfoPlist"""
     return name_string.title().replace('-', '')
 
-  def ExecCopyBundleResource(self, source, dest):
+  def ExecCopyBundleResource(self, source, dest, convert_to_binary):
     """Copies a resource file to the bundle/Resources directory, performing any
     necessary compilation on each resource."""
     extension = os.path.splitext(source)[1].lower()
@@ -62,7 +62,7 @@ class MacTool(object):
     elif extension == '.storyboard':
       return self._CopyXIBFile(source, dest)
     elif extension == '.strings':
-      self._CopyStringsFile(source, dest)
+      self._CopyStringsFile(source, dest, convert_to_binary)
     else:
       shutil.copy(source, dest)
 
@@ -92,7 +92,11 @@ class MacTool(object):
         sys.stdout.write(line)
     return ibtoolout.returncode
 
-  def _CopyStringsFile(self, source, dest):
+  def _ConvertToBinary(self, dest):
+    subprocess.check_call([
+        'xcrun', 'plutil', '-convert', 'binary1', '-o', dest, dest])
+
+  def _CopyStringsFile(self, source, dest, convert_to_binary):
     """Copies a .strings file using iconv to reconvert the input into UTF-16."""
     input_code = self._DetectInputEncoding(source) or "UTF-8"
 
@@ -112,6 +116,9 @@ class MacTool(object):
     fp.write(s.decode(input_code).encode('UTF-16'))
     fp.close()
 
+    if convert_to_binary == 'True':
+      self._ConvertToBinary(dest)
+
   def _DetectInputEncoding(self, file_name):
     """Reads the first few bytes from file_name and tries to guess the text
     encoding. Returns None as a guess if it can't detect it."""
@@ -131,7 +138,7 @@ class MacTool(object):
     else:
       return None
 
-  def ExecCopyInfoPlist(self, source, dest, *keys):
+  def ExecCopyInfoPlist(self, source, dest, convert_to_binary, *keys):
     """Copies the |source| Info.plist to the destination directory |dest|."""
     # Read the source Info.plist into memory.
     fd = open(source, 'r')
@@ -146,7 +153,7 @@ class MacTool(object):
 
     # Go through all the environment variables and replace them as variables in
     # the file.
-    IDENT_RE = re.compile('[/\s]')
+    IDENT_RE = re.compile(r'[/\s]')
     for key in os.environ:
       if key.startswith('_'):
         continue
@@ -185,6 +192,9 @@ class MacTool(object):
     # "compiled".
     self._WritePkgInfo(dest)
 
+    if convert_to_binary == 'True':
+      self._ConvertToBinary(dest)
+
   def _WritePkgInfo(self, info_plist):
     """This writes the PkgInfo file from the data stored in Info.plist."""
     plist = plistlib.readPlist(info_plist)
@@ -218,7 +228,8 @@ class MacTool(object):
   def ExecFilterLibtool(self, *cmd_list):
     """Calls libtool and filters out '/path/to/libtool: file: foo.o has no
     symbols'."""
-    libtool_re = re.compile(r'^.*libtool: file: .* has no symbols$')
+    libtool_re = re.compile(r'^.*libtool: (?:for architecture: \S* )?'
+                            r'file: .* has no symbols$')
     libtool_re5 = re.compile(
         r'^.*libtool: warning for library: ' +
         r'.* the table of contents is empty ' +
@@ -227,21 +238,20 @@ class MacTool(object):
     # Ref:
     # http://www.opensource.apple.com/source/cctools/cctools-809/misc/libtool.c
     # The problem with this flag is that it resets the file mtime on the file to
-    # epoch=0, e.g. 1970-1-1 or 1969-12-31 depending on daylight saving.
+    # epoch=0, e.g. 1970-1-1 or 1969-12-31 depending on timezone.
     env['ZERO_AR_DATE'] = '1'
     libtoolout = subprocess.Popen(cmd_list, stderr=subprocess.PIPE, env=env)
     _, err = libtoolout.communicate()
     for line in err.splitlines():
       if not libtool_re.match(line) and not libtool_re5.match(line):
         print >>sys.stderr, line
-    # Unconditionally touch any file .a file on the command line if present if
-    # succeeded. A bit hacky.
+    # Unconditionally touch the output .a file on the command line if present
+    # and the command succeeded. A bit hacky.
     if not libtoolout.returncode:
-      archives = [
-        cmd for cmd in cmd_list if cmd.endswith('.a') and os.path.isfile(cmd)
-      ]
-      if len(archives) == 1:
-        os.utime(archives[0], None)
+      for i in range(len(cmd_list) - 1):
+        if cmd_list[i] == "-o" and cmd_list[i+1].endswith('.a'):
+          os.utime(cmd_list[i+1], None)
+          break
     return libtoolout.returncode
 
   def ExecPackageFramework(self, framework, version):
