@@ -5,8 +5,8 @@
 #ifndef UI_AURA_CLIENT_DEFAULT_CAPTURE_CLIENT_H_
 #define UI_AURA_CLIENT_DEFAULT_CAPTURE_CLIENT_H_
 
-#include "base/basictypes.h"
 #include "base/compiler_specific.h"
+#include "base/macros.h"
 #include "ui/aura/aura_export.h"
 #include "ui/aura/client/capture_client.h"
 
