@@ -7,8 +7,9 @@
 
 #include <vector>
 
-#include "base/basictypes.h"
 #include "base/compiler_specific.h"
+#include "base/macros.h"
+#include "build/build_config.h"
 #include "ui/base/theme_provider.h"
 #include "ui/base/ui_base_export.h"
 
@@ -24,7 +25,6 @@ class UI_BASE_EXPORT DefaultThemeProvider : public ThemeProvider {
   ~DefaultThemeProvider() override;
 
   // Overridden from ui::ThemeProvider:
-  bool UsingSystemTheme() const override;
   gfx::ImageSkia* GetImageSkiaNamed(int id) const override;
   SkColor GetColor(int id) const override;
   int GetDisplayProperty(int id) const override;
@@ -34,6 +34,7 @@ class UI_BASE_EXPORT DefaultThemeProvider : public ThemeProvider {
       const override;
 
 #if defined(OS_MACOSX)
+  bool UsingSystemTheme() const override;
   NSImage* GetNSImageNamed(int id) const override;
   NSColor* GetNSImageColorNamed(int id) const override;
   NSColor* GetNSColor(int id) const override;
