@@ -7,9 +7,9 @@
 #include "base/logging.h"
 #include "ui/base/dragdrop/os_exchange_data.h"
 #include "ui/gfx/canvas.h"
+#include "ui/gfx/geometry/size.h"
+#include "ui/gfx/geometry/vector2d.h"
 #include "ui/gfx/image/image_skia.h"
-#include "ui/gfx/size.h"
-#include "ui/gfx/vector2d.h"
 
 namespace drag_utils {
 
