@@ -15,10 +15,13 @@ const char kEnableHardwareOverlays[] = "enable-hardware-overlays";
 // Forces tests to produce pixel output when they normally wouldn't.
 const char kEnablePixelOutputInTests[] = "enable-pixel-output-in-tests";
 
-const char kUIDisableThreadedCompositing[] = "ui-disable-threaded-compositing";
+const char kUIDisableCompositorAnimationTimelines[] =
+    "ui-disable-compositor-animation-timelines";
 
-const char kUIEnableCompositorAnimationTimelines[] =
-    "ui-enable-compositor-animation-timelines";
+// Disable partial swap which is needed for some OpenGL drivers / emulators.
+const char kUIDisablePartialSwap[] = "ui-disable-partial-swap";
+
+const char kUIEnableRGBA4444Textures[] = "ui-enable-rgba-4444-textures";
 
 const char kUIEnableZeroCopy[] = "ui-enable-zero-copy";
 
@@ -34,9 +37,4 @@ bool IsUIZeroCopyEnabled() {
   return command_line.HasSwitch(switches::kUIEnableZeroCopy);
 }
 
-bool IsUIOneCopyEnabled() {
-  // One-copy is on by default unless zero copy is enabled.
-  return !IsUIZeroCopyEnabled();
-}
-
 }  // namespace ui
