@@ -4,6 +4,8 @@
 
 #include "ui/events/ozone/device/device_manager.h"
 
+#include "base/trace_event/trace_event.h"
+
 #if defined(USE_UDEV)
 #include "ui/events/ozone/device/udev/device_manager_udev.h"
 #else
@@ -13,6 +15,7 @@
 namespace ui {
 
 scoped_ptr<DeviceManager> CreateDeviceManager() {
+TRACE_EVENT0("ozone", "CreateDeviceManager");
 #if defined(USE_UDEV)
   return make_scoped_ptr(new DeviceManagerUdev());
 #else
