@@ -5,6 +5,8 @@
 #ifndef UI_GFX_WIN_DIRECT_WRITE_H_
 #define UI_GFX_WIN_DIRECT_WRITE_H_
 
+#include <dwrite.h>
+
 #include "ui/gfx/gfx_export.h"
 
 namespace gfx {
@@ -13,6 +15,14 @@ namespace win {
 // Returns whether DirectWrite font rendering should be used.
 GFX_EXPORT bool ShouldUseDirectWrite();
 
+GFX_EXPORT void MaybeInitializeDirectWrite();
+
+// Returns true if we are using DirectWrite for font metrics and rendering.
+GFX_EXPORT bool IsDirectWriteEnabled();
+
+// Creates a DirectWrite factory, if using DirectWrite.
+GFX_EXPORT void CreateDWriteFactory(IDWriteFactory** factory);
+
 }  // namespace win
 }  // namespace gfx
 
