@@ -5,7 +5,8 @@
 #ifndef UI_GFX_X_X11_UTIL_H_
 #define UI_GFX_X_X11_UTIL_H_
 
-#include "base/basictypes.h"
+#include <stdint.h>
+
 #include "base/memory/scoped_ptr.h"
 #include "ui/gfx/gfx_export.h"
 
@@ -14,6 +15,9 @@ typedef unsigned long XID;
 typedef struct _XImage XImage;
 typedef struct _XGC *GC;
 typedef struct _XDisplay XDisplay;
+typedef struct __GLXFBConfigRec *GLXFBConfig;
+typedef XID GLXWindow;
+typedef XID GLXDrawable;
 
 extern "C" {
 int XFree(void*);
@@ -45,23 +49,32 @@ GFX_EXPORT int BitsPerPixelForPixmapDepth(XDisplay* display, int depth);
 // dimensions as |data| or larger.  |data| is also assumed to be in row order
 // with each line being exactly |width| * 4 bytes long.
 GFX_EXPORT void PutARGBImage(XDisplay* display,
-                             void* visual, int depth,
-                             XID pixmap, void* pixmap_gc,
-                             const uint8* data,
-                             int width, int height);
+                             void* visual,
+                             int depth,
+                             XID pixmap,
+                             void* pixmap_gc,
+                             const uint8_t* data,
+                             int width,
+                             int height);
 
 // Same as above only more general:
 // - |data_width| and |data_height| refer to the data image
 // - |src_x|, |src_y|, |copy_width| and |copy_height| define source region
 // - |dst_x|, |dst_y|, |copy_width| and |copy_height| define destination region
 GFX_EXPORT void PutARGBImage(XDisplay* display,
-                             void* visual, int depth,
-                             XID pixmap, void* pixmap_gc,
-                             const uint8* data,
-                             int data_width, int data_height,
-                             int src_x, int src_y,
-                             int dst_x, int dst_y,
-                             int copy_width, int copy_height);
+                             void* visual,
+                             int depth,
+                             XID pixmap,
+                             void* pixmap_gc,
+                             const uint8_t* data,
+                             int data_width,
+                             int data_height,
+                             int src_x,
+                             int src_y,
+                             int dst_x,
+                             int dst_y,
+                             int copy_width,
+                             int copy_height);
 
 }  // namespace gfx
 
