@@ -8,6 +8,7 @@
 #include <string>
 
 #include "base/compiler_specific.h"
+#include "base/macros.h"
 #include "ui/gfx/x/x11_types.h"
 #include "ui/gl/gl_context.h"
 #include "ui/gl/gl_export.h"
@@ -26,20 +27,20 @@ class GL_EXPORT GLContextGLX : public GLContextReal {
   // Implement GLContext.
   bool Initialize(GLSurface* compatible_surface,
                   GpuPreference gpu_preference) override;
-  void Destroy() override;
   bool MakeCurrent(GLSurface* surface) override;
   void ReleaseCurrent(GLSurface* surface) override;
   bool IsCurrent(GLSurface* surface) override;
   void* GetHandle() override;
   void OnSetSwapInterval(int interval) override;
   std::string GetExtensions() override;
-  bool GetTotalGpuMemory(size_t* bytes) override;
   bool WasAllocatedUsingRobustnessExtension() override;
 
  protected:
   ~GLContextGLX() override;
 
  private:
+  void Destroy();
+
   void* context_;
   XDisplay* display_;
 
