@@ -5,9 +5,11 @@
 #ifndef UI_GL_GL_IMAGE_STUB_H_
 #define UI_GL_GL_IMAGE_STUB_H_
 
+#include <stdint.h>
+
 #include "ui/gl/gl_image.h"
 
-namespace gfx {
+namespace gl {
 
 // A GLImage that does nothing for unit tests.
 class GL_EXPORT GLImageStub : public GLImage {
@@ -20,23 +22,23 @@ class GL_EXPORT GLImageStub : public GLImage {
   unsigned GetInternalFormat() override;
   bool BindTexImage(unsigned target) override;
   void ReleaseTexImage(unsigned target) override {}
+  bool CopyTexImage(unsigned target) override;
   bool CopyTexSubImage(unsigned target,
-                       const Point& offset,
-                       const Rect& rect) override;
-  void WillUseTexImage() override {}
-  void DidUseTexImage() override {}
-  void WillModifyTexImage() override {}
-  void DidModifyTexImage() override {}
+                       const gfx::Point& offset,
+                       const gfx::Rect& rect) override;
   bool ScheduleOverlayPlane(gfx::AcceleratedWidget widget,
                             int z_order,
-                            OverlayTransform transform,
-                            const Rect& bounds_rect,
-                            const RectF& crop_rect) override;
+                            gfx::OverlayTransform transform,
+                            const gfx::Rect& bounds_rect,
+                            const gfx::RectF& crop_rect) override;
+  void OnMemoryDump(base::trace_event::ProcessMemoryDump* pmd,
+                    uint64_t process_tracing_id,
+                    const std::string& dump_name) override {}
 
  protected:
   ~GLImageStub() override;
 };
 
-}  // namespace gfx
+}  // namespace gl
 
 #endif  // UI_GL_GL_IMAGE_STUB_H_
