@@ -4,14 +4,15 @@
 
 #import "ui/message_center/cocoa/notification_controller.h"
 
+#include <stddef.h>
+
 #include <algorithm>
 
 #include "base/mac/foundation_util.h"
 #include "base/strings/string_util.h"
 #include "base/strings/sys_string_conversions.h"
 #include "base/strings/utf_string_conversions.h"
-#include "grit/ui_resources.h"
-#include "grit/ui_strings.h"
+#include "components/url_formatter/elide_url.h"
 #include "skia/ext/skia_utils_mac.h"
 #import "ui/base/cocoa/hover_image_button.h"
 #include "ui/base/l10n/l10n_util_mac.h"
@@ -22,7 +23,9 @@
 #include "ui/message_center/message_center.h"
 #include "ui/message_center/message_center_style.h"
 #include "ui/message_center/notification.h"
-
+#include "ui/resources/grit/ui_resources.h"
+#include "ui/strings/grit/ui_strings.h"
+#include "url/gurl.h"
 
 @interface MCNotificationProgressBar : NSProgressIndicator
 @end
@@ -38,7 +41,7 @@
   NSBezierPath* path = [NSBezierPath bezierPathWithRoundedRect:dirtyRect
       xRadius:message_center::kProgressBarCornerRadius
       yRadius:message_center::kProgressBarCornerRadius];
-  [gfx::SkColorToCalibratedNSColor(message_center::kProgressBarBackgroundColor)
+  [skia::SkColorToCalibratedNSColor(message_center::kProgressBarBackgroundColor)
       set];
   [path fill];
 
@@ -48,7 +51,8 @@
   path = [NSBezierPath bezierPathWithRoundedRect:sliceRect
       xRadius:message_center::kProgressBarCornerRadius
       yRadius:message_center::kProgressBarCornerRadius];
-  [gfx::SkColorToCalibratedNSColor(message_center::kProgressBarSliceColor) set];
+  [skia::SkColorToCalibratedNSColor(message_center::kProgressBarSliceColor)
+      set];
   [path fill];
 }
 
@@ -76,7 +80,7 @@
 // drawRect: needs to fill the button with a background, otherwise we don't get
 // subpixel antialiasing.
 - (void)drawRect:(NSRect)dirtyRect {
-  NSColor* color = gfx::SkColorToCalibratedNSColor(
+  NSColor* color = skia::SkColorToCalibratedNSColor(
       message_center::kNotificationBackgroundColor);
   [color set];
   NSRectFill(dirtyRect);
@@ -102,7 +106,7 @@
 
   if (!hovered_)
     return;
-  [gfx::SkColorToCalibratedNSColor(
+  [skia::SkColorToCalibratedNSColor(
       message_center::kHoveredButtonBackgroundColor) set];
   NSRectFill(frame);
 }
@@ -139,7 +143,7 @@
     NSFontAttributeName :
         [title attribute:NSFontAttributeName atIndex:0 effectiveRange:NULL],
     NSForegroundColorAttributeName :
-        gfx::SkColorToCalibratedNSColor(message_center::kRegularTextColor),
+        skia::SkColorToCalibratedNSColor(message_center::kRegularTextColor),
   };
   [[title string] drawWithRect:frame
                        options:(NSStringDrawingUsesLineFragmentOrigin |
@@ -255,8 +259,11 @@
 // Initializes the closeButton_ ivar with the configured button.
 - (void)configureCloseButtonInFrame:(NSRect)rootFrame;
 
-// Initializes the smallImage_ ivar with the appropriate frame.
-- (void)configureSmallImageInFrame:(NSRect)rootFrame;
+// Initializes the settingsButton_ ivar with the configured button.
+- (void)configureSettingsButtonInFrame:(NSRect)rootFrame;
+
+// Creates the smallImage_ ivar with the appropriate frame.
+- (NSView*)createSmallImageInFrame:(NSRect)rootFrame;
 
 // Initializes title_ in the given frame.
 - (void)configureTitleInFrame:(NSRect)rootFrame;
@@ -315,7 +322,7 @@
   base::scoped_nsobject<MCNotificationView> rootView(
       [[MCNotificationView alloc] initWithController:self frame:rootFrame]);
   [self configureCustomBox:rootView];
-  [rootView setFillColor:gfx::SkColorToCalibratedNSColor(
+  [rootView setFillColor:skia::SkColorToCalibratedNSColor(
       message_center::kNotificationBackgroundColor)];
   [self setView:rootView];
 
@@ -328,6 +335,13 @@
   // Create the small image.
   [rootView addSubview:[self createSmallImageInFrame:rootFrame]];
 
+  // Create the settings button.
+  if (notification_->delegate() &&
+      notification_->delegate()->ShouldDisplaySettingsButton()) {
+    [self configureSettingsButtonInFrame:rootFrame];
+    [rootView addSubview:settingsButton_];
+  }
+
   NSRect contentFrame = [self currentContentRect];
 
   // Create the title.
@@ -350,9 +364,10 @@
   DCHECK_EQ(notification->id(), notificationID_);
   notification_ = notification;
 
-  NSRect rootFrame = NSMakeRect(0, 0,
-      message_center::kNotificationPreferredImageWidth,
-      message_center::kNotificationIconSize);
+  message_center::NotificationLayoutParams layoutParams;
+  layoutParams.rootFrame =
+      NSMakeRect(0, 0, message_center::kNotificationPreferredImageWidth,
+                 message_center::kNotificationIconSize);
 
   [smallImage_ setImage:notification_->small_image().AsNSImage()];
 
@@ -380,14 +395,16 @@
 
   // Set the title and recalculate the frame.
   size_t actualTitleLines = 0;
-  [title_ setString:base::SysUTF16ToNSString(
-      [self wrapText:notification_->title()
-                forFont:[title_ font]
-       maxNumberOfLines:message_center::kMaxTitleLines
-            actualLines:&actualTitleLines])];
+  [title_ setString:base::SysUTF16ToNSString([self
+                                wrapText:notification_->title()
+                                 forFont:[title_ font]
+                        maxNumberOfLines:message_center::kMaxTitleLines
+                             actualLines:&actualTitleLines])];
   [title_ sizeToFit];
-  NSRect titleFrame = [title_ frame];
-  titleFrame.origin.y = NSMaxY(rootFrame) - titlePadding - NSHeight(titleFrame);
+  layoutParams.titleFrame = [title_ frame];
+  layoutParams.titleFrame.origin.y = NSMaxY(layoutParams.rootFrame) -
+                                     titlePadding -
+                                     NSHeight(layoutParams.titleFrame);
 
   // The number of message lines depends on the number of context message lines
   // and the lines within the title, and whether an image exists.
@@ -396,7 +413,9 @@
     messageLineLimit -= (actualTitleLines - 1) * 2;
   if (!notification_->image().IsEmpty()) {
     messageLineLimit /= 2;
-    if (!notification_->context_message().empty())
+
+    if (!notification_->context_message().empty() &&
+        !notification_->UseOriginAsContextMessage())
       messageLineLimit -= message_center::kContextMessageLineLimit;
   }
   if (messageLineLimit < 0)
@@ -408,7 +427,7 @@
              forFont:[message_ font]
       maxNumberOfLines:messageLineLimit])];
   [message_ sizeToFit];
-  NSRect messageFrame = [message_ frame];
+  layoutParams.messageFrame = [message_ frame];
 
   // If there are list items, then the message_ view should not be displayed.
   const std::vector<message_center::NotificationItem>& items =
@@ -418,44 +437,74 @@
   // space (size to fit leaves it 15 px tall.
   if (items.size() > 0 || notification_->message().empty()) {
     [message_ setHidden:YES];
-    messageFrame.origin.y = titleFrame.origin.y;
-    messageFrame.size.height = 0;
+    layoutParams.messageFrame.origin.y = layoutParams.titleFrame.origin.y;
+    layoutParams.messageFrame.size.height = 0;
   } else {
     [message_ setHidden:NO];
-    messageFrame.origin.y =
-        NSMinY(titleFrame) - messagePadding - NSHeight(messageFrame);
-    messageFrame.size.height = NSHeight([message_ frame]);
+    layoutParams.messageFrame.origin.y = NSMinY(layoutParams.titleFrame) -
+                                         messagePadding -
+                                         NSHeight(layoutParams.messageFrame);
+    layoutParams.messageFrame.size.height = NSHeight([message_ frame]);
   }
 
   // Set the context message and recalculate the frame.
-  [contextMessage_ setString:base::SysUTF16ToNSString(
-      [self wrapText:notification_->context_message()
-             forFont:[contextMessage_ font]
-       maxNumberOfLines:message_center::kContextMessageLineLimit])];
-  [contextMessage_ sizeToFit];
-  NSRect contextMessageFrame = [contextMessage_ frame];
+  base::string16 message;
+  if (notification->UseOriginAsContextMessage()) {
+    gfx::FontList font_list((gfx::Font([message_ font])));
+    message =
+        url_formatter::ElideHost(notification->origin_url(), font_list,
+                                 message_center::kContextMessageViewWidth);
+  } else {
+    message = notification->context_message();
+  }
 
-  if (notification_->context_message().empty()) {
+  base::string16 elided =
+      [self wrapText:message
+             forFont:[contextMessage_ font]
+          maxNumberOfLines:message_center::kContextMessageLineLimit];
+  [contextMessage_ setString:base::SysUTF16ToNSString(elided)];
+  [contextMessage_ sizeToFit];
+
+  layoutParams.contextMessageFrame = [contextMessage_ frame];
+
+  if (notification->context_message().empty() &&
+      !notification->UseOriginAsContextMessage()) {
     [contextMessage_ setHidden:YES];
-    contextMessageFrame.origin.y = messageFrame.origin.y;
-    contextMessageFrame.size.height = 0;
+    layoutParams.contextMessageFrame.origin.y =
+        layoutParams.messageFrame.origin.y;
+    layoutParams.contextMessageFrame.size.height = 0;
   } else {
     [contextMessage_ setHidden:NO];
-    contextMessageFrame.origin.y =
-        NSMinY(messageFrame) -
-        contextMessagePadding -
-        NSHeight(contextMessageFrame);
-    contextMessageFrame.size.height = NSHeight([contextMessage_ frame]);
+
+    // If the context message is used as a domain make sure it's placed at the
+    // bottom of the top section.
+    CGFloat contextMessageY = NSMinY(layoutParams.messageFrame) -
+                              contextMessagePadding -
+                              NSHeight(layoutParams.contextMessageFrame);
+    layoutParams.contextMessageFrame.origin.y =
+        notification->UseOriginAsContextMessage()
+            ? std::min(NSMinY([icon_ frame]) + contextMessagePadding,
+                       contextMessageY)
+            : contextMessageY;
+    layoutParams.contextMessageFrame.size.height =
+        NSHeight([contextMessage_ frame]);
   }
 
+  // Calculate the settings button position. It is dependent on whether the
+  // context message aligns or not with the icon.
+  layoutParams.settingsButtonFrame = [settingsButton_ frame];
+  layoutParams.settingsButtonFrame.origin.y =
+      MIN(NSMinY([icon_ frame]) + message_center::kSmallImagePadding,
+          NSMinY(layoutParams.contextMessageFrame));
+
   // Create the list item views (up to a maximum).
   [listView_ removeFromSuperview];
-  NSRect listFrame = NSZeroRect;
+  layoutParams.listFrame = NSZeroRect;
   if (items.size() > 0) {
-    listFrame = [self currentContentRect];
-    listFrame.origin.y = 0;
-    listFrame.size.height = 0;
-    listView_.reset([[NSView alloc] initWithFrame:listFrame]);
+    layoutParams.listFrame = [self currentContentRect];
+    layoutParams.listFrame.origin.y = 0;
+    layoutParams.listFrame.size.height = 0;
+    listView_.reset([[NSView alloc] initWithFrame:layoutParams.listFrame]);
     [listView_ accessibilitySetOverrideValue:NSAccessibilityListRole
                                     forAttribute:NSAccessibilityRoleAttribute];
     [listView_
@@ -469,8 +518,9 @@
     const int kNumNotifications =
         std::min(items.size(), message_center::kNotificationMaximumItems);
     for (int i = kNumNotifications - 1; i >= 0; --i) {
-      NSTextView* itemView = [self newLabelWithFrame:
-          NSMakeRect(0, y, NSWidth(listFrame), lineHeight)];
+      NSTextView* itemView = [self
+          newLabelWithFrame:NSMakeRect(0, y, NSWidth(layoutParams.listFrame),
+                                       lineHeight)];
       [itemView setFont:font];
 
       // Disable the word-wrap in order to show the text in single line.
@@ -486,7 +536,7 @@
 
       // Use dim color for the title part.
       NSColor* titleColor =
-          gfx::SkColorToCalibratedNSColor(message_center::kRegularTextColor);
+          skia::SkColorToCalibratedNSColor(message_center::kRegularTextColor);
       NSRange titleRange = NSMakeRange(
           0,
           std::min(ellidedText.size(), items[i].title.size()));
@@ -495,7 +545,7 @@
       // Use dim color for the message part if it has not been truncated.
       if (ellidedText.size() > items[i].title.size() + 1) {
         NSColor* messageColor =
-            gfx::SkColorToCalibratedNSColor(message_center::kDimTextColor);
+            skia::SkColorToCalibratedNSColor(message_center::kDimTextColor);
         NSRange messageRange = NSMakeRange(
             items[i].title.size() + 1,
             ellidedText.size() - items[i].title.size() - 1);
@@ -508,24 +558,27 @@
     // TODO(thakis): The spacing is not completely right.
     CGFloat listTopPadding =
         message_center::kTextTopPadding - contextMessageTopGap;
-    listFrame.size.height = y;
-    listFrame.origin.y =
-        NSMinY(contextMessageFrame) - listTopPadding - NSHeight(listFrame);
-    [listView_ setFrame:listFrame];
+    layoutParams.listFrame.size.height = y;
+    layoutParams.listFrame.origin.y = NSMinY(layoutParams.contextMessageFrame) -
+                                      listTopPadding -
+                                      NSHeight(layoutParams.listFrame);
+    [listView_ setFrame:layoutParams.listFrame];
     [[self view] addSubview:listView_];
   }
 
   // Create the progress bar view if needed.
   [progressBarView_ removeFromSuperview];
-  NSRect progressBarFrame = NSZeroRect;
+  layoutParams.progressBarFrame = NSZeroRect;
   if (notification->type() == message_center::NOTIFICATION_TYPE_PROGRESS) {
-    progressBarFrame = [self currentContentRect];
-    progressBarFrame.origin.y = NSMinY(contextMessageFrame) -
+    layoutParams.progressBarFrame = [self currentContentRect];
+    layoutParams.progressBarFrame.origin.y =
+        NSMinY(layoutParams.contextMessageFrame) -
         message_center::kProgressBarTopPadding -
         message_center::kProgressBarThickness;
-    progressBarFrame.size.height = message_center::kProgressBarThickness;
-    progressBarView_.reset(
-        [[MCNotificationProgressBar alloc] initWithFrame:progressBarFrame]);
+    layoutParams.progressBarFrame.size.height =
+        message_center::kProgressBarThickness;
+    progressBarView_.reset([[MCNotificationProgressBar alloc]
+        initWithFrame:layoutParams.progressBarFrame]);
     // Setting indeterminate to NO does not work with custom drawRect.
     [progressBarView_ setIndeterminate:YES];
     [progressBarView_ setStyle:NSProgressIndicatorBarStyle];
@@ -535,23 +588,18 @@
 
   // If the bottom-most element so far is out of the rootView's bounds, resize
   // the view.
-  CGFloat minY = NSMinY(contextMessageFrame);
-  if (listView_ && NSMinY(listFrame) < minY)
-    minY = NSMinY(listFrame);
-  if (progressBarView_ && NSMinY(progressBarFrame) < minY)
-    minY = NSMinY(progressBarFrame);
+  CGFloat minY = NSMinY(layoutParams.contextMessageFrame);
+  if (listView_ && NSMinY(layoutParams.listFrame) < minY)
+    minY = NSMinY(layoutParams.listFrame);
+  if (progressBarView_ && NSMinY(layoutParams.progressBarFrame) < minY)
+    minY = NSMinY(layoutParams.progressBarFrame);
   if (minY < messagePadding) {
     CGFloat delta = messagePadding - minY;
-    rootFrame.size.height += delta;
-    titleFrame.origin.y += delta;
-    messageFrame.origin.y += delta;
-    contextMessageFrame.origin.y += delta;
-    listFrame.origin.y += delta;
-    progressBarFrame.origin.y += delta;
+    [self adjustFrameHeight:&layoutParams delta:delta];
   }
 
   // Add the bottom container view.
-  NSRect frame = rootFrame;
+  NSRect frame = layoutParams.rootFrame;
   frame.size.height = 0;
   [bottomView_ removeFromSuperview];
   bottomView_.reset([[NSView alloc] initWithFrame:frame]);
@@ -587,7 +635,7 @@
     base::scoped_nsobject<NSBox> separator(
         [[AccessibilityIgnoredBox alloc] initWithFrame:separatorFrame]);
     [self configureCustomBox:separator];
-    [separator setFillColor:gfx::SkColorToCalibratedNSColor(
+    [separator setFillColor:skia::SkColorToCalibratedNSColor(
         message_center::kButtonSeparatorColor)];
     y += NSHeight(separatorFrame);
     frame.size.height += NSHeight(separatorFrame);
@@ -611,35 +659,26 @@
 
   [bottomView_ setFrame:frame];
   [[self view] addSubview:bottomView_];
-
-  rootFrame.size.height += NSHeight(frame);
-  titleFrame.origin.y += NSHeight(frame);
-  messageFrame.origin.y += NSHeight(frame);
-  contextMessageFrame.origin.y += NSHeight(frame);
-  listFrame.origin.y += NSHeight(frame);
-  progressBarFrame.origin.y += NSHeight(frame);
+  [self adjustFrameHeight:&layoutParams delta:NSHeight(frame)];
 
   // Make sure that there is a minimum amount of spacing below the icon and
   // the edge of the frame.
-  CGFloat bottomDelta = NSHeight(rootFrame) - NSHeight([icon_ frame]);
+  CGFloat bottomDelta =
+      NSHeight(layoutParams.rootFrame) - NSHeight([icon_ frame]);
   if (bottomDelta > 0 && bottomDelta < message_center::kIconBottomPadding) {
     CGFloat bottomAdjust = message_center::kIconBottomPadding - bottomDelta;
-    rootFrame.size.height += bottomAdjust;
-    titleFrame.origin.y += bottomAdjust;
-    messageFrame.origin.y += bottomAdjust;
-    contextMessageFrame.origin.y += bottomAdjust;
-    listFrame.origin.y += bottomAdjust;
-    progressBarFrame.origin.y += bottomAdjust;
+    [self adjustFrameHeight:&layoutParams delta:bottomAdjust];
   }
 
-  [[self view] setFrame:rootFrame];
-  [title_ setFrame:titleFrame];
-  [message_ setFrame:messageFrame];
-  [contextMessage_ setFrame:contextMessageFrame];
-  [listView_ setFrame:listFrame];
-  [progressBarView_ setFrame:progressBarFrame];
+  [[self view] setFrame:layoutParams.rootFrame];
+  [title_ setFrame:layoutParams.titleFrame];
+  [message_ setFrame:layoutParams.messageFrame];
+  [contextMessage_ setFrame:layoutParams.contextMessageFrame];
+  [settingsButton_ setFrame:layoutParams.settingsButtonFrame];
+  [listView_ setFrame:layoutParams.listFrame];
+  [progressBarView_ setFrame:layoutParams.progressBarFrame];
 
-  return rootFrame;
+  return layoutParams.rootFrame;
 }
 
 - (void)close:(id)sender {
@@ -647,6 +686,10 @@
   messageCenter_->RemoveNotification([self notificationID], /*by_user=*/true);
 }
 
+- (void)settingsClicked:(id)sender {
+  messageCenter_->ClickOnSettingsButton([self notificationID]);
+}
+
 - (void)buttonClicked:(id)button {
   messageCenter_->ClickOnNotificationButton([self notificationID],
                                             [button tag]);
@@ -664,6 +707,17 @@
   messageCenter_->ClickOnNotification([self notificationID]);
 }
 
+- (void)adjustFrameHeight:(message_center::NotificationLayoutParams*)frames
+                    delta:(CGFloat)delta {
+  frames->rootFrame.size.height += delta;
+  frames->titleFrame.origin.y += delta;
+  frames->messageFrame.origin.y += delta;
+  frames->contextMessageFrame.origin.y += delta;
+  frames->settingsButtonFrame.origin.y += delta;
+  frames->listFrame.origin.y += delta;
+  frames->progressBarFrame.origin.y += delta;
+}
+
 // Private /////////////////////////////////////////////////////////////////////
 
 - (void)configureCustomBox:(NSBox*)box {
@@ -682,7 +736,7 @@
   base::scoped_nsobject<NSBox> imageBox(
       [[AccessibilityIgnoredBox alloc] initWithFrame:imageFrame]);
   [self configureCustomBox:imageBox];
-  [imageBox setFillColor:gfx::SkColorToCalibratedNSColor(
+  [imageBox setFillColor:skia::SkColorToCalibratedNSColor(
       message_center::kIconBackgroundColor)];
   [imageBox setAutoresizingMask:NSViewMinYMargin];
 
@@ -704,7 +758,7 @@
   base::scoped_nsobject<NSBox> imageBox(
       [[AccessibilityIgnoredBox alloc] initWithFrame:imageFrame]);
   [self configureCustomBox:imageBox];
-  [imageBox setFillColor:gfx::SkColorToCalibratedNSColor(
+  [imageBox setFillColor:skia::SkColorToCalibratedNSColor(
       message_center::kImageBackgroundColor)];
 
   // Images with non-preferred aspect ratios get a border on all sides.
@@ -751,6 +805,7 @@
   [closeButton_ setAutoresizingMask:NSViewMinYMargin];
   [closeButton_ setTarget:self];
   [closeButton_ setAction:@selector(close:)];
+  [closeButton_ setDisableActivationOnClick:YES];
   [[closeButton_ cell]
       accessibilitySetOverrideValue:NSAccessibilityCloseButtonSubrole
                        forAttribute:NSAccessibilitySubroleAttribute];
@@ -760,6 +815,42 @@
                        forAttribute:NSAccessibilityTitleAttribute];
 }
 
+- (void)configureSettingsButtonInFrame:(NSRect)rootFrame {
+  // The settings button is configured to be the same size as the small image.
+  int settingsButtonOriginOffset =
+      message_center::kSmallImageSize + message_center::kSmallImagePadding;
+  NSRect settingsButtonFrame = NSMakeRect(
+      NSMaxX(rootFrame) - settingsButtonOriginOffset,
+      message_center::kSmallImagePadding, message_center::kSmallImageSize,
+      message_center::kSmallImageSize);
+
+  settingsButton_.reset(
+      [[HoverImageButton alloc] initWithFrame:settingsButtonFrame]);
+  ui::ResourceBundle& rb = ui::ResourceBundle::GetSharedInstance();
+  [settingsButton_ setDefaultImage:rb.GetNativeImageNamed(
+                                         IDR_NOTIFICATION_SETTINGS_BUTTON_ICON)
+                                       .ToNSImage()];
+  [settingsButton_
+      setHoverImage:rb.GetNativeImageNamed(
+                          IDR_NOTIFICATION_SETTINGS_BUTTON_ICON_HOVER)
+                        .ToNSImage()];
+  [settingsButton_
+      setPressedImage:rb.GetNativeImageNamed(
+                            IDR_NOTIFICATION_SETTINGS_BUTTON_ICON_PRESSED)
+                          .ToNSImage()];
+  [[settingsButton_ cell] setHighlightsBy:NSOnState];
+  [settingsButton_ setTrackingEnabled:YES];
+  [settingsButton_ setBordered:NO];
+  [settingsButton_ setAutoresizingMask:NSViewMinYMargin];
+  [settingsButton_ setTarget:self];
+  [settingsButton_ setAction:@selector(settingsClicked:)];
+  [[settingsButton_ cell]
+      accessibilitySetOverrideValue:
+          l10n_util::GetNSString(
+              IDS_MESSAGE_NOTIFICATION_SETTINGS_BUTTON_ACCESSIBLE_NAME)
+                       forAttribute:NSAccessibilityTitleAttribute];
+}
+
 - (NSView*)createSmallImageInFrame:(NSRect)rootFrame {
   int smallImageXOffset =
       message_center::kSmallImagePadding + message_center::kSmallImageSize;
@@ -793,7 +884,7 @@
   contentFrame.size.height = 0;
   title_.reset([self newLabelWithFrame:contentFrame]);
   [title_ setAutoresizingMask:NSViewMinYMargin];
-  [title_ setTextColor:gfx::SkColorToCalibratedNSColor(
+  [title_ setTextColor:skia::SkColorToCalibratedNSColor(
       message_center::kRegularTextColor)];
   [title_ setFont:[NSFont messageFontOfSize:message_center::kTitleFontSize]];
 }
@@ -802,7 +893,7 @@
   contentFrame.size.height = 0;
   message_.reset([self newLabelWithFrame:contentFrame]);
   [message_ setAutoresizingMask:NSViewMinYMargin];
-  [message_ setTextColor:gfx::SkColorToCalibratedNSColor(
+  [message_ setTextColor:skia::SkColorToCalibratedNSColor(
       message_center::kRegularTextColor)];
   [message_ setFont:
       [NSFont messageFontOfSize:message_center::kMessageFontSize]];
@@ -812,7 +903,7 @@
   contentFrame.size.height = 0;
   contextMessage_.reset([self newLabelWithFrame:contentFrame]);
   [contextMessage_ setAutoresizingMask:NSViewMinYMargin];
-  [contextMessage_ setTextColor:gfx::SkColorToCalibratedNSColor(
+  [contextMessage_ setTextColor:skia::SkColorToCalibratedNSColor(
       message_center::kDimTextColor)];
   [contextMessage_ setFont:
       [NSFont messageFontOfSize:message_center::kMessageFontSize]];
@@ -824,7 +915,7 @@
   // The labels MUST draw their background so that subpixel antialiasing can
   // happen on the text.
   [label setDrawsBackground:YES];
-  [label setBackgroundColor:gfx::SkColorToCalibratedNSColor(
+  [label setBackgroundColor:skia::SkColorToCalibratedNSColor(
       message_center::kNotificationBackgroundColor)];
 
   [label setEditable:NO];
@@ -883,7 +974,8 @@
   }
 
   *actualLines = wrapped.size();
-  return lines == 1 ? wrapped[0] : JoinString(wrapped, '\n');
+  return lines == 1 ? wrapped[0]
+                    : base::JoinString(wrapped, base::ASCIIToUTF16("\n"));
 }
 
 - (base::string16)wrapText:(const base::string16&)text
