@@ -4,8 +4,12 @@
 
 #include "ui/message_center/notification_list.h"
 
-#include "base/basictypes.h"
+#include <stddef.h>
+
+#include <utility>
+
 #include "base/i18n/time_formatting.h"
+#include "base/macros.h"
 #include "base/strings/stringprintf.h"
 #include "base/strings/utf_string_conversions.h"
 #include "base/values.h"
@@ -38,7 +42,7 @@ class NotificationListTest : public testing::Test {
     std::string new_id;
     scoped_ptr<Notification> notification(
         MakeNotification(optional_fields, &new_id));
-    notification_list_->AddNotification(notification.Pass());
+    notification_list_->AddNotification(std::move(notification));
     counter_++;
     return new_id;
   }
@@ -53,16 +57,13 @@ class NotificationListTest : public testing::Test {
       std::string* id_out) {
     *id_out = base::StringPrintf(kIdFormat, counter_);
     scoped_ptr<Notification> notification(new Notification(
-        message_center::NOTIFICATION_TYPE_SIMPLE,
-        *id_out,
+        message_center::NOTIFICATION_TYPE_SIMPLE, *id_out,
         UTF8ToUTF16(base::StringPrintf(kTitleFormat, counter_)),
-        UTF8ToUTF16(base::StringPrintf(kMessageFormat, counter_)),
-        gfx::Image(),
-        UTF8ToUTF16(kDisplaySource),
-        NotifierId(NotifierId::APPLICATION, kExtensionId),
-        optional_fields,
+        UTF8ToUTF16(base::StringPrintf(kMessageFormat, counter_)), gfx::Image(),
+        UTF8ToUTF16(kDisplaySource), GURL(),
+        NotifierId(NotifierId::APPLICATION, kExtensionId), optional_fields,
         NULL));
-    return notification.Pass();
+    return notification;
   }
 
   scoped_ptr<Notification> MakeNotification(std::string* id_out) {
@@ -186,16 +187,12 @@ TEST_F(NotificationListTest, UpdateNotification) {
   std::string replaced = id0 + "_replaced";
   EXPECT_EQ(1u, notification_list()->NotificationCount(blockers()));
   scoped_ptr<Notification> notification(
-      new Notification(message_center::NOTIFICATION_TYPE_SIMPLE,
-                       replaced,
-                       UTF8ToUTF16("newtitle"),
-                       UTF8ToUTF16("newbody"),
-                       gfx::Image(),
-                       UTF8ToUTF16(kDisplaySource),
+      new Notification(message_center::NOTIFICATION_TYPE_SIMPLE, replaced,
+                       UTF8ToUTF16("newtitle"), UTF8ToUTF16("newbody"),
+                       gfx::Image(), UTF8ToUTF16(kDisplaySource), GURL(),
                        NotifierId(NotifierId::APPLICATION, kExtensionId),
-                       message_center::RichNotificationData(),
-                       NULL));
-  notification_list()->UpdateNotificationMessage(id0, notification.Pass());
+                       message_center::RichNotificationData(), NULL));
+  notification_list()->UpdateNotificationMessage(id0, std::move(notification));
   EXPECT_EQ(1u, notification_list()->NotificationCount(blockers()));
   const NotificationList::Notifications notifications =
       notification_list()->GetVisibleNotifications(blockers());
@@ -209,67 +206,36 @@ TEST_F(NotificationListTest, GetNotificationsByNotifierId) {
   NotifierId id1(NotifierId::APPLICATION, "ext1");
   NotifierId id2(GURL("http://example.com"));
   NotifierId id3(NotifierId::SYSTEM_COMPONENT, "system-notifier");
-  scoped_ptr<Notification> notification(
-      new Notification(message_center::NOTIFICATION_TYPE_SIMPLE,
-                       "id0",
-                       UTF8ToUTF16("title0"),
-                       UTF8ToUTF16("message0"),
-                       gfx::Image(),
-                       UTF8ToUTF16("source0"),
-                       id0,
-                       message_center::RichNotificationData(),
-                       NULL));
-  notification_list()->AddNotification(notification.Pass());
-  notification.reset(new Notification(message_center::NOTIFICATION_TYPE_SIMPLE,
-                                      "id1",
-                                      UTF8ToUTF16("title1"),
-                                      UTF8ToUTF16("message1"),
-                                      gfx::Image(),
-                                      UTF8ToUTF16("source0"),
-                                      id0,
-                                      message_center::RichNotificationData(),
-                                      NULL));
-  notification_list()->AddNotification(notification.Pass());
-  notification.reset(new Notification(message_center::NOTIFICATION_TYPE_SIMPLE,
-                                      "id2",
-                                      UTF8ToUTF16("title1"),
-                                      UTF8ToUTF16("message1"),
-                                      gfx::Image(),
-                                      UTF8ToUTF16("source1"),
-                                      id0,
-                                      message_center::RichNotificationData(),
-                                      NULL));
-  notification_list()->AddNotification(notification.Pass());
-  notification.reset(new Notification(message_center::NOTIFICATION_TYPE_SIMPLE,
-                                      "id3",
-                                      UTF8ToUTF16("title1"),
-                                      UTF8ToUTF16("message1"),
-                                      gfx::Image(),
-                                      UTF8ToUTF16("source2"),
-                                      id1,
-                                      message_center::RichNotificationData(),
-                                      NULL));
-  notification_list()->AddNotification(notification.Pass());
-  notification.reset(new Notification(message_center::NOTIFICATION_TYPE_SIMPLE,
-                                      "id4",
-                                      UTF8ToUTF16("title1"),
-                                      UTF8ToUTF16("message1"),
-                                      gfx::Image(),
-                                      UTF8ToUTF16("source2"),
-                                      id2,
-                                      message_center::RichNotificationData(),
-                                      NULL));
-  notification_list()->AddNotification(notification.Pass());
-  notification.reset(new Notification(message_center::NOTIFICATION_TYPE_SIMPLE,
-                                      "id5",
-                                      UTF8ToUTF16("title1"),
-                                      UTF8ToUTF16("message1"),
-                                      gfx::Image(),
-                                      UTF8ToUTF16("source2"),
-                                      id3,
-                                      message_center::RichNotificationData(),
-                                      NULL));
-  notification_list()->AddNotification(notification.Pass());
+  scoped_ptr<Notification> notification(new Notification(
+      message_center::NOTIFICATION_TYPE_SIMPLE, "id0", UTF8ToUTF16("title0"),
+      UTF8ToUTF16("message0"), gfx::Image(), UTF8ToUTF16("source0"), GURL(),
+      id0, message_center::RichNotificationData(), NULL));
+  notification_list()->AddNotification(std::move(notification));
+  notification.reset(new Notification(
+      message_center::NOTIFICATION_TYPE_SIMPLE, "id1", UTF8ToUTF16("title1"),
+      UTF8ToUTF16("message1"), gfx::Image(), UTF8ToUTF16("source0"), GURL(),
+      id0, message_center::RichNotificationData(), NULL));
+  notification_list()->AddNotification(std::move(notification));
+  notification.reset(new Notification(
+      message_center::NOTIFICATION_TYPE_SIMPLE, "id2", UTF8ToUTF16("title1"),
+      UTF8ToUTF16("message1"), gfx::Image(), UTF8ToUTF16("source1"), GURL(),
+      id0, message_center::RichNotificationData(), NULL));
+  notification_list()->AddNotification(std::move(notification));
+  notification.reset(new Notification(
+      message_center::NOTIFICATION_TYPE_SIMPLE, "id3", UTF8ToUTF16("title1"),
+      UTF8ToUTF16("message1"), gfx::Image(), UTF8ToUTF16("source2"), GURL(),
+      id1, message_center::RichNotificationData(), NULL));
+  notification_list()->AddNotification(std::move(notification));
+  notification.reset(new Notification(
+      message_center::NOTIFICATION_TYPE_SIMPLE, "id4", UTF8ToUTF16("title1"),
+      UTF8ToUTF16("message1"), gfx::Image(), UTF8ToUTF16("source2"), GURL(),
+      id2, message_center::RichNotificationData(), NULL));
+  notification_list()->AddNotification(std::move(notification));
+  notification.reset(new Notification(
+      message_center::NOTIFICATION_TYPE_SIMPLE, "id5", UTF8ToUTF16("title1"),
+      UTF8ToUTF16("message1"), gfx::Image(), UTF8ToUTF16("source2"), GURL(),
+      id3, message_center::RichNotificationData(), NULL));
+  notification_list()->AddNotification(std::move(notification));
 
   NotificationList::Notifications by_notifier_id =
       notification_list()->GetNotificationsByNotifierId(id0);
@@ -418,17 +384,12 @@ TEST_F(NotificationListTest, PriorityPromotion) {
   EXPECT_EQ(0u, GetPopupCounts());
   message_center::RichNotificationData optional;
   optional.priority = 1;
-  scoped_ptr<Notification> notification(
-      new Notification(message_center::NOTIFICATION_TYPE_SIMPLE,
-                       replaced,
-                       UTF8ToUTF16("newtitle"),
-                       UTF8ToUTF16("newbody"),
-                       gfx::Image(),
-                       UTF8ToUTF16(kDisplaySource),
-                       NotifierId(NotifierId::APPLICATION, kExtensionId),
-                       optional,
-                       NULL));
-  notification_list()->UpdateNotificationMessage(id0, notification.Pass());
+  scoped_ptr<Notification> notification(new Notification(
+      message_center::NOTIFICATION_TYPE_SIMPLE, replaced,
+      UTF8ToUTF16("newtitle"), UTF8ToUTF16("newbody"), gfx::Image(),
+      UTF8ToUTF16(kDisplaySource), GURL(),
+      NotifierId(NotifierId::APPLICATION, kExtensionId), optional, NULL));
+  notification_list()->UpdateNotificationMessage(id0, std::move(notification));
   EXPECT_EQ(1u, notification_list()->NotificationCount(blockers()));
   EXPECT_EQ(1u, GetPopupCounts());
   const NotificationList::Notifications notifications =
@@ -449,120 +410,86 @@ TEST_F(NotificationListTest, PriorityPromotionWithPopups) {
   // id0 promoted to LOW->DEFAULT, it'll appear as toast (popup).
   message_center::RichNotificationData priority;
   priority.priority = DEFAULT_PRIORITY;
-  scoped_ptr<Notification> notification(
-      new Notification(message_center::NOTIFICATION_TYPE_SIMPLE,
-                       id0,
-                       UTF8ToUTF16("newtitle"),
-                       UTF8ToUTF16("newbody"),
-                       gfx::Image(),
-                       UTF8ToUTF16(kDisplaySource),
-                       NotifierId(NotifierId::APPLICATION, kExtensionId),
-                       priority,
-                       NULL));
-  notification_list()->UpdateNotificationMessage(id0, notification.Pass());
+  scoped_ptr<Notification> notification(new Notification(
+      message_center::NOTIFICATION_TYPE_SIMPLE, id0, UTF8ToUTF16("newtitle"),
+      UTF8ToUTF16("newbody"), gfx::Image(), UTF8ToUTF16(kDisplaySource), GURL(),
+      NotifierId(NotifierId::APPLICATION, kExtensionId), priority, NULL));
+  notification_list()->UpdateNotificationMessage(id0, std::move(notification));
   EXPECT_EQ(1u, GetPopupCounts());
   notification_list()->MarkSinglePopupAsShown(id0, true);
   EXPECT_EQ(0u, GetPopupCounts());
 
   // update with no promotion change for id0, it won't appear as a toast.
-  notification.reset(new Notification(message_center::NOTIFICATION_TYPE_SIMPLE,
-                                      id0,
-                                      UTF8ToUTF16("newtitle2"),
-                                      UTF8ToUTF16("newbody2"),
-                                      gfx::Image(),
-                                      UTF8ToUTF16(kDisplaySource),
-                                      NotifierId(NotifierId::APPLICATION,
-                                                 kExtensionId),
-                                      priority,
-                                      NULL));
-  notification_list()->UpdateNotificationMessage(id0, notification.Pass());
+  notification.reset(new Notification(
+      message_center::NOTIFICATION_TYPE_SIMPLE, id0, UTF8ToUTF16("newtitle2"),
+      UTF8ToUTF16("newbody2"), gfx::Image(), UTF8ToUTF16(kDisplaySource),
+      GURL(), NotifierId(NotifierId::APPLICATION, kExtensionId), priority,
+      NULL));
+  notification_list()->UpdateNotificationMessage(id0, std::move(notification));
   EXPECT_EQ(0u, GetPopupCounts());
 
   // id1 promoted to DEFAULT->HIGH, it'll appear as toast (popup).
   priority.priority = HIGH_PRIORITY;
-  notification.reset(new Notification(message_center::NOTIFICATION_TYPE_SIMPLE,
-                                      id1,
-                                      UTF8ToUTF16("newtitle"),
-                                      UTF8ToUTF16("newbody"),
-                                      gfx::Image(),
-                                      UTF8ToUTF16(kDisplaySource),
-                                      NotifierId(NotifierId::APPLICATION,
-                                                 kExtensionId),
-                                      priority,
-                                      NULL));
-  notification_list()->UpdateNotificationMessage(id1, notification.Pass());
+  notification.reset(new Notification(
+      message_center::NOTIFICATION_TYPE_SIMPLE, id1, UTF8ToUTF16("newtitle"),
+      UTF8ToUTF16("newbody"), gfx::Image(), UTF8ToUTF16(kDisplaySource), GURL(),
+      NotifierId(NotifierId::APPLICATION, kExtensionId), priority, NULL));
+  notification_list()->UpdateNotificationMessage(id1, std::move(notification));
   EXPECT_EQ(1u, GetPopupCounts());
   notification_list()->MarkSinglePopupAsShown(id1, true);
   EXPECT_EQ(0u, GetPopupCounts());
 
   // id1 promoted to HIGH->MAX, it'll appear as toast again.
   priority.priority = MAX_PRIORITY;
-  notification.reset(new Notification(message_center::NOTIFICATION_TYPE_SIMPLE,
-                                      id1,
-                                      UTF8ToUTF16("newtitle2"),
-                                      UTF8ToUTF16("newbody2"),
-                                      gfx::Image(),
-                                      UTF8ToUTF16(kDisplaySource),
-                                      NotifierId(NotifierId::APPLICATION,
-                                                 kExtensionId),
-                                      priority,
-                                      NULL));
-  notification_list()->UpdateNotificationMessage(id1, notification.Pass());
+  notification.reset(new Notification(
+      message_center::NOTIFICATION_TYPE_SIMPLE, id1, UTF8ToUTF16("newtitle2"),
+      UTF8ToUTF16("newbody2"), gfx::Image(), UTF8ToUTF16(kDisplaySource),
+      GURL(), NotifierId(NotifierId::APPLICATION, kExtensionId), priority,
+      NULL));
+  notification_list()->UpdateNotificationMessage(id1, std::move(notification));
   EXPECT_EQ(1u, GetPopupCounts());
   notification_list()->MarkSinglePopupAsShown(id1, true);
   EXPECT_EQ(0u, GetPopupCounts());
 
   // id1 demoted to MAX->DEFAULT, no appearing as toast.
   priority.priority = DEFAULT_PRIORITY;
-  notification.reset(new Notification(message_center::NOTIFICATION_TYPE_SIMPLE,
-                                      id1,
-                                      UTF8ToUTF16("newtitle3"),
-                                      UTF8ToUTF16("newbody3"),
-                                      gfx::Image(),
-                                      UTF8ToUTF16(kDisplaySource),
-                                      NotifierId(NotifierId::APPLICATION,
-                                                 kExtensionId),
-                                      priority,
-                                      NULL));
-  notification_list()->UpdateNotificationMessage(id1, notification.Pass());
+  notification.reset(new Notification(
+      message_center::NOTIFICATION_TYPE_SIMPLE, id1, UTF8ToUTF16("newtitle3"),
+      UTF8ToUTF16("newbody3"), gfx::Image(), UTF8ToUTF16(kDisplaySource),
+      GURL(), NotifierId(NotifierId::APPLICATION, kExtensionId), priority,
+      NULL));
+  notification_list()->UpdateNotificationMessage(id1, std::move(notification));
   EXPECT_EQ(0u, GetPopupCounts());
 }
 
 TEST_F(NotificationListTest, WebNotificationUpdatePromotion) {
   std::string notification_id = "replaced-web-notification";
-  scoped_ptr<Notification> original_notification(
-      new Notification(message_center::NOTIFICATION_TYPE_SIMPLE,
-                       notification_id,
-                       UTF8ToUTF16("Web Notification"),
-                       UTF8ToUTF16("Notification contents"),
-                       gfx::Image(),
-                       UTF8ToUTF16(kDisplaySource),
-                       NotifierId(GURL("https://example.com/")),
-                       message_center::RichNotificationData(),
-                       NULL));
+  scoped_ptr<Notification> original_notification(new Notification(
+      message_center::NOTIFICATION_TYPE_SIMPLE, notification_id,
+      UTF8ToUTF16("Web Notification"), UTF8ToUTF16("Notification contents"),
+      gfx::Image(), UTF8ToUTF16(kDisplaySource), GURL(),
+      NotifierId(GURL("https://example.com/")),
+      message_center::RichNotificationData(), NULL));
 
   EXPECT_EQ(0u, GetPopupCounts());
-  notification_list()->AddNotification(original_notification.Pass());
+  notification_list()->AddNotification(std::move(original_notification));
   EXPECT_EQ(1u, GetPopupCounts());
 
   notification_list()->MarkSinglePopupAsShown(notification_id, true);
   EXPECT_EQ(0u, GetPopupCounts());
 
-  scoped_ptr<Notification> replaced_notification(
-      new Notification(message_center::NOTIFICATION_TYPE_SIMPLE,
-                       notification_id,
-                       UTF8ToUTF16("Web Notification Replacement"),
-                       UTF8ToUTF16("New notification contents"),
-                       gfx::Image(),
-                       UTF8ToUTF16(kDisplaySource),
-                       NotifierId(GURL("https://example.com/")),
-                       message_center::RichNotificationData(),
-                       NULL));
+  scoped_ptr<Notification> replaced_notification(new Notification(
+      message_center::NOTIFICATION_TYPE_SIMPLE, notification_id,
+      UTF8ToUTF16("Web Notification Replacement"),
+      UTF8ToUTF16("New notification contents"), gfx::Image(),
+      UTF8ToUTF16(kDisplaySource), GURL(),
+      NotifierId(GURL("https://example.com/")),
+      message_center::RichNotificationData(), NULL));
 
   // Web Notifications will be re-shown as popups even if their priority didn't
   // change, to match the behavior of the Web Notification API.
-  notification_list()->UpdateNotificationMessage(notification_id,
-                                                 replaced_notification.Pass());
+  notification_list()->UpdateNotificationMessage(
+      notification_id, std::move(replaced_notification));
   EXPECT_EQ(1u, GetPopupCounts());
 }
 
@@ -659,16 +586,12 @@ TEST_F(NotificationListTest, UpdateAfterMarkedAsShown) {
 
   const std::string replaced("test-replaced-id");
   scoped_ptr<Notification> notification(
-      new Notification(message_center::NOTIFICATION_TYPE_SIMPLE,
-                       replaced,
-                       UTF8ToUTF16("newtitle"),
-                       UTF8ToUTF16("newbody"),
-                       gfx::Image(),
-                       UTF8ToUTF16(kDisplaySource),
+      new Notification(message_center::NOTIFICATION_TYPE_SIMPLE, replaced,
+                       UTF8ToUTF16("newtitle"), UTF8ToUTF16("newbody"),
+                       gfx::Image(), UTF8ToUTF16(kDisplaySource), GURL(),
                        NotifierId(NotifierId::APPLICATION, kExtensionId),
-                       message_center::RichNotificationData(),
-                       NULL));
-  notification_list()->UpdateNotificationMessage(id1, notification.Pass());
+                       message_center::RichNotificationData(), NULL));
+  notification_list()->UpdateNotificationMessage(id1, std::move(notification));
   n1 = GetNotification(id1);
   EXPECT_TRUE(n1 == NULL);
   const Notification* nr = GetNotification(replaced);
@@ -705,16 +628,10 @@ TEST_F(NotificationListTest, UnreadCountNoNegative) {
 
   // Updates the notification  and verifies unread_count doesn't change.
   scoped_ptr<Notification> updated_notification(new Notification(
-      message_center::NOTIFICATION_TYPE_SIMPLE,
-      id,
-      UTF8ToUTF16("updated"),
-      UTF8ToUTF16("updated"),
-      gfx::Image(),
-      base::string16(),
-      NotifierId(),
-      RichNotificationData(),
-      NULL));
-  notification_list()->AddNotification(updated_notification.Pass());
+      message_center::NOTIFICATION_TYPE_SIMPLE, id, UTF8ToUTF16("updated"),
+      UTF8ToUTF16("updated"), gfx::Image(), base::string16(), GURL(),
+      NotifierId(), RichNotificationData(), NULL));
+  notification_list()->AddNotification(std::move(updated_notification));
   EXPECT_EQ(1u, notification_list()->UnreadCount(blockers()));
 }
 
@@ -725,7 +642,7 @@ TEST_F(NotificationListTest, TestPushingShownNotification) {
   notification->set_shown_as_popup(true);
 
   // Call PushNotification on this notification.
-  notification_list()->PushNotification(notification.Pass());
+  notification_list()->PushNotification(std::move(notification));
 
   // Ensure it is still marked as shown.
   EXPECT_TRUE(GetNotification(id1)->shown_as_popup());
@@ -740,16 +657,10 @@ TEST_F(NotificationListTest, TestHasNotificationOfType) {
       id, message_center::NOTIFICATION_TYPE_PROGRESS));
 
   scoped_ptr<Notification> updated_notification(new Notification(
-      message_center::NOTIFICATION_TYPE_PROGRESS,
-      id,
-      UTF8ToUTF16("updated"),
-      UTF8ToUTF16("updated"),
-      gfx::Image(),
-      base::string16(),
-      NotifierId(),
-      RichNotificationData(),
-      NULL));
-  notification_list()->AddNotification(updated_notification.Pass());
+      message_center::NOTIFICATION_TYPE_PROGRESS, id, UTF8ToUTF16("updated"),
+      UTF8ToUTF16("updated"), gfx::Image(), base::string16(), GURL(),
+      NotifierId(), RichNotificationData(), NULL));
+  notification_list()->AddNotification(std::move(updated_notification));
 
   EXPECT_FALSE(notification_list()->HasNotificationOfType(
       id, message_center::NOTIFICATION_TYPE_SIMPLE));
