@@ -5,15 +5,18 @@
 #ifndef UI_OZONE_COMMON_EGL_UTIL_H_
 #define UI_OZONE_COMMON_EGL_UTIL_H_
 
+#include "ui/ozone/ozone_base_export.h"
 #include "ui/ozone/public/surface_factory_ozone.h"
 
 namespace ui {
 
+OZONE_BASE_EXPORT
 bool LoadDefaultEGLGLES2Bindings(
     SurfaceFactoryOzone::AddGLLibraryCallback add_gl_library,
     SurfaceFactoryOzone::SetGLGetProcAddressProcCallback
         set_gl_get_proc_address);
 
+OZONE_BASE_EXPORT
 bool LoadEGLGLES2Bindings(
     SurfaceFactoryOzone::AddGLLibraryCallback add_gl_library,
     SurfaceFactoryOzone::SetGLGetProcAddressProcCallback
