@@ -5,8 +5,12 @@
 #ifndef UI_OZONE_PLATFORM_CACA_CACA_WINDOW_MANAGER_H_
 #define UI_OZONE_PLATFORM_CACA_CACA_WINDOW_MANAGER_H_
 
+#include <stdint.h>
+
 #include "base/id_map.h"
+#include "base/macros.h"
 #include "base/memory/scoped_ptr.h"
+#include "base/threading/thread_checker.h"
 #include "ui/ozone/public/surface_factory_ozone.h"
 
 namespace gfx {
@@ -37,6 +41,7 @@ class CacaWindowManager : public SurfaceFactoryOzone {
 
  private:
   IDMap<CacaWindow> windows_;
+  base::ThreadChecker thread_checker_;
 
   DISALLOW_COPY_AND_ASSIGN(CacaWindowManager);
 };
