@@ -24,4 +24,11 @@ void PlatformWindowCast::SetBounds(const gfx::Rect& bounds) {
   delegate_->OnBoundsChanged(bounds);
 }
 
+void PlatformWindowCast::SetTitle(const base::string16& title) {
+}
+
+PlatformImeController* PlatformWindowCast::GetPlatformImeController() {
+  return nullptr;
+}
+
 }  // namespace ui
