@@ -5,6 +5,9 @@
 #ifndef UI_OZONE_PLATFORM_DRM_GPU_HARDWARE_DISPLAY_PLANE_MANAGER_LEGACY_H_
 #define UI_OZONE_PLATFORM_DRM_GPU_HARDWARE_DISPLAY_PLANE_MANAGER_LEGACY_H_
 
+#include <stdint.h>
+
+#include "base/macros.h"
 #include "ui/ozone/ozone_export.h"
 #include "ui/ozone/platform/drm/gpu/hardware_display_plane_manager.h"
 
@@ -18,10 +21,9 @@ class OZONE_EXPORT HardwareDisplayPlaneManagerLegacy
 
   // HardwareDisplayPlaneManager:
   bool Commit(HardwareDisplayPlaneList* plane_list,
-              bool is_sync,
               bool test_only) override;
 
- private:
+ protected:
   bool SetPlaneData(HardwareDisplayPlaneList* plane_list,
                     HardwareDisplayPlane* hw_plane,
                     const OverlayPlane& overlay,
@@ -29,6 +31,7 @@ class OZONE_EXPORT HardwareDisplayPlaneManagerLegacy
                     const gfx::Rect& src_rect,
                     CrtcController* crtc) override;
 
+ private:
   DISALLOW_COPY_AND_ASSIGN(HardwareDisplayPlaneManagerLegacy);
 };
 
