@@ -36,6 +36,8 @@ gfx::Rect StubWindow::GetBounds() {
   return bounds_;
 }
 
+void StubWindow::SetTitle(const base::string16& title) {}
+
 void StubWindow::SetCapture() {
 }
 
@@ -63,4 +65,8 @@ void StubWindow::MoveCursorTo(const gfx::Point& location) {
 void StubWindow::ConfineCursorToBounds(const gfx::Rect& bounds) {
 }
 
+PlatformImeController* StubWindow::GetPlatformImeController() {
+  return nullptr;
+}
+
 }  // namespace ui
