@@ -4,13 +4,14 @@
 
 #include "ui/views/controls/image_view.h"
 
+#include <utility>
+
 #include "base/logging.h"
 #include "base/strings/utf_string_conversions.h"
 #include "third_party/skia/include/core/SkPaint.h"
 #include "ui/accessibility/ax_view_state.h"
 #include "ui/gfx/canvas.h"
 #include "ui/gfx/geometry/insets.h"
-#include "ui/gfx/paint_vector_icon.h"
 #include "ui/views/painter.h"
 
 namespace views {
@@ -32,8 +33,6 @@ const char ImageView::kViewClassName[] = "ImageView";
 
 ImageView::ImageView()
     : image_size_set_(false),
-      vector_id_(gfx::VectorIconId::VECTOR_ICON_NONE),
-      vector_color_(SK_ColorGREEN),
       horiz_alignment_(CENTER),
       vert_alignment_(CENTER),
       interactive_(true),
@@ -70,14 +69,6 @@ const gfx::ImageSkia& ImageView::GetImage() {
   return image_;
 }
 
-void ImageView::SetVectorIcon(gfx::VectorIconId id,
-                              SkColor color,
-                              const gfx::Size& image_size) {
-  SetImageSize(image_size);
-  vector_id_ = id;
-  vector_color_ = color;
-}
-
 void ImageView::SetImageSize(const gfx::Size& image_size) {
   image_size_set_ = true;
   image_size_ = image_size;
@@ -85,8 +76,7 @@ void ImageView::SetImageSize(const gfx::Size& image_size) {
 }
 
 gfx::Rect ImageView::GetImageBounds() const {
-  gfx::Size image_size(image_size_set_ ?
-    image_size_ : gfx::Size(image_.width(), image_.height()));
+  gfx::Size image_size = GetImageSize();
   return gfx::Rect(ComputeImageOrigin(image_size), image_size);
 }
 
@@ -95,18 +85,13 @@ void ImageView::ResetImageSize() {
 }
 
 void ImageView::SetFocusPainter(scoped_ptr<Painter> focus_painter) {
-  focus_painter_ = focus_painter.Pass();
+  focus_painter_ = std::move(focus_painter);
 }
 
 gfx::Size ImageView::GetPreferredSize() const {
-  gfx::Insets insets = GetInsets();
-  if (image_size_set_) {
-    gfx::Size image_size = image_size_;
-    image_size.Enlarge(insets.width(), insets.height());
-    return image_size;
-  }
-  return gfx::Size(image_.width() + insets.width(),
-                   image_.height() + insets.height());
+  gfx::Size size = GetImageSize();
+  size.Enlarge(GetInsets().width(), GetInsets().height());
+  return size;
 }
 
 bool ImageView::IsImageEqual(const gfx::ImageSkia& img) const {
@@ -120,6 +105,10 @@ bool ImageView::IsImageEqual(const gfx::ImageSkia& img) const {
       last_painted_bitmap_pixels_ == GetBitmapPixels(img, last_paint_scale_);
 }
 
+gfx::Size ImageView::GetImageSize() const {
+  return image_size_set_ ? image_size_ : image_.size();
+}
+
 gfx::Point ImageView::ComputeImageOrigin(const gfx::Size& image_size) const {
   gfx::Insets insets = GetInsets();
 
@@ -164,7 +153,6 @@ void ImageView::OnBlur() {
 void ImageView::OnPaint(gfx::Canvas* canvas) {
   View::OnPaint(canvas);
   OnPaintImage(canvas);
-  OnPaintVectorIcon(canvas);
   Painter::PaintFocusPainter(this, canvas, focus_painter_.get());
 }
 
@@ -244,13 +232,4 @@ void ImageView::OnPaintImage(gfx::Canvas* canvas) {
   last_painted_bitmap_pixels_ = GetBitmapPixels(image_, last_paint_scale_);
 }
 
-void ImageView::OnPaintVectorIcon(gfx::Canvas* canvas) {
-  if (vector_id_ == gfx::VectorIconId::VECTOR_ICON_NONE)
-    return;
-
-  DCHECK(image_size_set_);
-  canvas->Translate(ComputeImageOrigin(image_size_).OffsetFromOrigin());
-  gfx::PaintVectorIcon(canvas, vector_id_, image_size_.width(), vector_color_);
-}
-
 }  // namespace views
