@@ -6,6 +6,7 @@
 #define UI_VIEWS_CONTROLS_MENU_MENU_SEPARATOR_H_
 
 #include "base/compiler_specific.h"
+#include "base/macros.h"
 #include "ui/base/models/menu_separator_types.h"
 #include "ui/views/view.h"
 
@@ -15,9 +16,7 @@ class MenuItemView;
 
 class MenuSeparator : public View {
  public:
-  MenuSeparator(MenuItemView* parent, ui::MenuSeparatorType type)
-      : type_(type),
-        parent_menu_item_(parent) {}
+  explicit MenuSeparator(ui::MenuSeparatorType type) : type_(type) {}
 
   // View overrides.
   void OnPaint(gfx::Canvas* canvas) override;
@@ -32,9 +31,6 @@ class MenuSeparator : public View {
   // The type of the separator.
   const ui::MenuSeparatorType type_;
 
-  // Our parent.
-  MenuItemView* parent_menu_item_;
-
   DISALLOW_COPY_AND_ASSIGN(MenuSeparator);
 };
 
