@@ -4,6 +4,8 @@
 
 #include "ui/views/examples/examples_window_with_content.h"
 
+#include <utility>
+
 #include "content/public/browser/browser_context.h"
 #include "ui/views/examples/webview_example.h"
 
@@ -16,7 +18,7 @@ void ShowExamplesWindowWithContent(Operation operation,
   scoped_ptr<ScopedVector<ExampleBase> > extra_examples(
       new ScopedVector<ExampleBase>);
   extra_examples->push_back(new WebViewExample(browser_context));
-  ShowExamplesWindow(operation, window_context, extra_examples.Pass());
+  ShowExamplesWindow(operation, window_context, std::move(extra_examples));
 }
 
 }  // namespace examples
