@@ -2,13 +2,16 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
-#include "base/basictypes.h"
+#include <stddef.h>
+
 #include "base/bind.h"
 #include "base/command_line.h"
+#include "base/macros.h"
 #include "base/path_service.h"
 #include "base/run_loop.h"
 #include "base/strings/stringprintf.h"
 #include "base/strings/utf_string_conversions.h"
+#include "build/build_config.h"
 #include "ui/base/ime/input_method.h"
 #include "ui/base/ime/text_input_client.h"
 #include "ui/base/resource/resource_bundle.h"
@@ -18,7 +21,7 @@
 #include "ui/events/event_utils.h"
 #include "ui/events/test/event_generator.h"
 #include "ui/gfx/native_widget_types.h"
-#include "ui/gl/gl_surface.h"
+#include "ui/gl/test/gl_surface_test_support.h"
 #include "ui/views/controls/textfield/textfield.h"
 #include "ui/views/controls/textfield/textfield_test_api.h"
 #include "ui/views/focus/focus_manager.h"
@@ -274,7 +277,7 @@ class WidgetTestInteractive : public WidgetTest {
   ~WidgetTestInteractive() override {}
 
   void SetUp() override {
-    gfx::GLSurface::InitializeOneOffForTests();
+    gfx::GLSurfaceTestSupport::InitializeOneOff();
     ui::RegisterPathProvider();
     base::FilePath ui_test_pak_path;
     ASSERT_TRUE(PathService::Get(ui::UI_TEST_PAK, &ui_test_pak_path));
@@ -847,7 +850,7 @@ TEST_F(WidgetTestInteractive, MAYBE_SystemModalWindowReleasesCapture) {
 
   ASSERT_FALSE(focus_listener.focus_changes().empty());
   EXPECT_EQ(top_level_widget.GetNativeView(),
-            focus_listener.focus_changes().back());;
+            focus_listener.focus_changes().back());
 
   EXPECT_FALSE(top_level_widget.HasCapture());
   top_level_widget.SetCapture(NULL);
@@ -890,8 +893,6 @@ TEST_F(WidgetTestInteractive, CanActivateFlagIsHonored) {
 #if defined(USE_AURA)
 // Test that touch selection quick menu is not activated when opened.
 TEST_F(WidgetTestInteractive, TouchSelectionQuickMenuIsNotActivated) {
-  base::CommandLine::ForCurrentProcess()->AppendSwitch(
-      switches::kEnableTouchEditing);
 #if defined(OS_WIN)
   views_delegate()->set_use_desktop_native_widgets(true);
 #endif  // !defined(OS_WIN)
@@ -1142,7 +1143,7 @@ class WidgetCaptureTest : public ViewsTestBase {
   ~WidgetCaptureTest() override {}
 
   void SetUp() override {
-    gfx::GLSurface::InitializeOneOffForTests();
+    gfx::GLSurfaceTestSupport::InitializeOneOff();
     ui::RegisterPathProvider();
     base::FilePath ui_test_pak_path;
     ASSERT_TRUE(PathService::Get(ui::UI_TEST_PAK, &ui_test_pak_path));
@@ -1592,5 +1593,33 @@ TEST_F(WidgetInputMethodInteractiveTest, TextField) {
   widget->CloseNow();
 }
 
+// Test input method should not work for accelerator.
+TEST_F(WidgetInputMethodInteractiveTest, AcceleratorInTextfield) {
+  Widget* widget = CreateWidget();
+  Textfield* textfield = new Textfield;
+  widget->GetRootView()->AddChildView(textfield);
+  ShowSync(widget);
+  textfield->SetTextInputType(ui::TEXT_INPUT_TYPE_TEXT);
+  textfield->RequestFocus();
+
+  ui::KeyEvent key_event(ui::ET_KEY_PRESSED,
+                         ui::VKEY_F, ui::EF_ALT_DOWN);
+  ui::Accelerator accelerator(key_event);
+  widget->GetFocusManager()->RegisterAccelerator(
+      accelerator, ui::AcceleratorManager::kNormalPriority,
+      textfield);
+
+  widget->OnKeyEvent(&key_event);
+  EXPECT_TRUE(key_event.stopped_propagation());
+
+  widget->GetFocusManager()->UnregisterAccelerators(textfield);
+
+  ui::KeyEvent key_event2(key_event);
+  widget->OnKeyEvent(&key_event2);
+  EXPECT_FALSE(key_event2.stopped_propagation());
+
+  widget->CloseNow();
+}
+
 }  // namespace test
 }  // namespace views
