@@ -223,11 +223,6 @@ void LookupIterator::PrepareTransitionToDataProperty(
     Handle<Object> value, PropertyAttributes attributes,
     Object::StoreFromKeyed store_mode) {
   if (state_ == TRANSITION) return;
-
-  if (!IsElement() && name()->IsPrivate()) {
-    attributes = static_cast<PropertyAttributes>(attributes | DONT_ENUM);
-  }
-
   DCHECK(state_ != LookupIterator::ACCESSOR ||
          (GetAccessors()->IsAccessorInfo() &&
           AccessorInfo::cast(*GetAccessors())->is_special_data_property()));
@@ -308,9 +303,6 @@ void LookupIterator::TransitionToAccessorProperty(
   // handled via a trap. Adding properties to primitive values is not
   // observable.
   Handle<JSObject> receiver = GetStoreTarget();
-  if (!IsElement() && name()->IsPrivate()) {
-    attributes = static_cast<PropertyAttributes>(attributes | DONT_ENUM);
-  }
 
   if (!IsElement() && !receiver->map()->is_dictionary_map()) {
     holder_ = receiver;
