@@ -95,11 +95,8 @@ void WebContentsViewQt::RenderViewCreated(content::RenderViewHost* host)
 {
     // The render process is done creating the RenderView and it's ready to be routed
     // messages at this point.
-    if (m_client) {
-        content::RenderWidgetHostView* rwhv = m_webContents->GetRenderWidgetHostView();
-        if (rwhv)
-            rwhv->SetBackgroundColor(toSk(m_client->backgroundColor()));
-    }
+    if (m_client)
+        m_webContents->GetRenderWidgetHostView()->SetBackgroundColor(toSk(m_client->backgroundColor()));
 }
 
 void WebContentsViewQt::CreateView(const gfx::Size& initial_size, gfx::NativeView context)
