@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtWebEngine module of the Qt Toolkit.
 **
@@ -11,24 +11,27 @@
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
 ** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPLv3 included in the
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
 ** packaging of this file. Please review the following information to
 ** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl.html.
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or later as published by the Free
-** Software Foundation and appearing in the file LICENSE.GPL included in
-** the packaging of this file. Please review the following information to
-** ensure the GNU General Public License version 2.0 requirements will be
-** met: http://www.gnu.org/licenses/gpl-2.0.html.
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -45,12 +48,17 @@
 #include <QQmlEngine>
 
 #include "browser_context_adapter.h"
+#include <qtwebenginecoreglobal.h>
 #include "web_engine_settings.h"
 
 using QtWebEngineCore::BrowserContextAdapter;
 
 QT_BEGIN_NAMESPACE
 
+ASSERT_ENUMS_MATCH(QQuickWebEngineDownloadItem::UnknownSaveFormat, QtWebEngineCore::BrowserContextAdapterClient::UnknownSavePageFormat)
+ASSERT_ENUMS_MATCH(QQuickWebEngineDownloadItem::SingleHtmlSaveFormat, QtWebEngineCore::BrowserContextAdapterClient::SingleHtmlSaveFormat)
+ASSERT_ENUMS_MATCH(QQuickWebEngineDownloadItem::CompleteHtmlSaveFormat, QtWebEngineCore::BrowserContextAdapterClient::CompleteHtmlSaveFormat)
+ASSERT_ENUMS_MATCH(QQuickWebEngineDownloadItem::MimeHtmlSaveFormat, QtWebEngineCore::BrowserContextAdapterClient::MimeHtmlSaveFormat)
 
 /*!
     \class QQuickWebEngineProfile
@@ -87,6 +95,7 @@ QT_BEGIN_NAMESPACE
     \value MemoryHttpCache Use an in-memory cache. This is the only setting possible if
     \c off-the-record is set or no cache path is available.
     \value DiskHttpCache Use a disk cache. This is the default.
+    \value NoCache Disable both in-memory and disk caching. (Added in Qt 5.7)
 */
 
 /*!
@@ -169,6 +178,8 @@ void QQuickWebEngineProfilePrivate::downloadRequested(DownloadItemInfo &info)
     itemPrivate->totalBytes = info.totalBytes;
     itemPrivate->mimeType = info.mimeType;
     itemPrivate->downloadPath = info.path;
+    itemPrivate->savePageFormat = static_cast<QQuickWebEngineDownloadItem::SavePageFormat>(
+                info.savePageFormat);
 
     QQuickWebEngineDownloadItem *download = new QQuickWebEngineDownloadItem(itemPrivate, q);
 
@@ -179,8 +190,16 @@ void QQuickWebEngineProfilePrivate::downloadRequested(DownloadItemInfo &info)
 
     QQuickWebEngineDownloadItem::DownloadState state = download->state();
     info.path = download->path();
+    info.savePageFormat = itemPrivate->savePageFormat;
     info.accepted = state != QQuickWebEngineDownloadItem::DownloadCancelled
                       && state != QQuickWebEngineDownloadItem::DownloadRequested;
+
+    if (state == QQuickWebEngineDownloadItem::DownloadRequested) {
+        // Delete unaccepted downloads.
+        info.accepted = false;
+        m_ongoingDownloads.remove(info.id);
+        delete download;
+    }
 }
 
 void QQuickWebEngineProfilePrivate::downloadUpdated(const DownloadItemInfo &info)
@@ -264,7 +283,7 @@ QQuickWebEngineProfile::~QQuickWebEngineProfile()
 }
 
 /*!
-    \qmlproperty QString WebEngineProfile::storageName
+    \qmlproperty string WebEngineProfile::storageName
 
     The storage name that is used to create separate subdirectories for each profile that uses
     the disk for storing persistent data and cache.
@@ -343,7 +362,7 @@ void QQuickWebEngineProfile::setOffTheRecord(bool offTheRecord)
 }
 
 /*!
-    \qmlproperty QString WebEngineProfile::persistentStoragePath
+    \qmlproperty string WebEngineProfile::persistentStoragePath
 
     The path to the location where the persistent data for the browser and web content are
     stored. Persistent data includes persistent cookies, HTML5 local storage, and visited links.
@@ -380,7 +399,7 @@ void QQuickWebEngineProfile::setPersistentStoragePath(const QString &path)
 }
 
 /*!
-    \qmlproperty QString WebEngineProfile::cachePath
+    \qmlproperty string WebEngineProfile::cachePath
 
     The path to the location where the profile's caches are stored, in particular the HTTP cache.
 
@@ -415,7 +434,7 @@ void QQuickWebEngineProfile::setCachePath(const QString &path)
 }
 
 /*!
-    \qmlproperty QString WebEngineProfile::httpUserAgent
+    \qmlproperty string WebEngineProfile::httpUserAgent
 
     The user-agent string sent with HTTP to identify the browser.
 */
@@ -452,6 +471,8 @@ void QQuickWebEngineProfile::setHttpUserAgent(const QString &userAgent)
             no persistentStoragePath is available.
     \value  WebEngineProfile.DiskHttpCache
             Uses a disk cache. This is the default value.
+    \value  WebEngineProfile.NoCache
+            Disables caching. (Added in 5.7)
 */
 
 /*!
@@ -548,7 +569,7 @@ void QQuickWebEngineProfile::setHttpCacheMaximumSize(int maximumSize)
 }
 
 /*!
-    \qmlproperty QString WebEngineProfile::httpAcceptLanguage
+    \qmlproperty string WebEngineProfile::httpAcceptLanguage
 
     The value of the Accept-Language HTTP request-header field.
 
@@ -600,6 +621,29 @@ QWebEngineCookieStore *QQuickWebEngineProfile::cookieStore() const
     return d->browserContext()->cookieStore();
 }
 
+/*!
+    \qmlmethod void WebEngineProfile::clearHttpCache()
+    \since QtWebEngine 1.3
+
+    Removes the profile's cache entries.
+
+    \sa WebEngineProfile::cachePath
+*/
+
+/*!
+    \since 5.7
+
+    Removes the profile's cache entries.
+
+    \sa WebEngineProfile::clearHttpCache
+*/
+void QQuickWebEngineProfile::clearHttpCache()
+{
+    Q_D(QQuickWebEngineProfile);
+    d->browserContext()->clearHttpCache();
+}
+
+
 /*!
     Registers a request interceptor singleton \a interceptor to intercept URL requests.
 
