@@ -237,6 +237,72 @@ bool QQuickWebEngineSettings::fullScreenSupportEnabled() const
     return d_ptr->testAttribute(WebEngineSettings::FullScreenSupportEnabled);
 }
 
+/*!
+    \qmlproperty bool WebEngineSettings::screenCaptureEnabled
+    \since QtWebEngine 1.3
+
+    Tells the web engine whether screen capture is supported in this application or not.
+
+    Disabled by default.
+*/
+bool QQuickWebEngineSettings::screenCaptureEnabled() const
+{
+    return d_ptr->testAttribute(WebEngineSettings::ScreenCaptureEnabled);
+}
+
+/*!
+    \qmlproperty bool WebEngineSettings::webGLEnabled
+    \since QtWebEngine 1.3
+
+    Enables support for HTML 5 WebGL.
+
+    Enabled by default if available.
+*/
+bool QQuickWebEngineSettings::webGLEnabled() const
+{
+    return d_ptr->testAttribute(WebEngineSettings::WebGLEnabled);
+}
+
+/*!
+    \qmlproperty bool WebEngineSettings::accelerated2dCanvasEnabled
+    \since QtWebEngine 1.3
+
+    Specifies whether the HTML 5 2D canvas should be a OpenGL framebuffer.
+    This makes many painting operations faster, but slows down pixel access.
+
+    Enabled by default if available.
+*/
+bool QQuickWebEngineSettings::accelerated2dCanvasEnabled() const
+{
+    return d_ptr->testAttribute(WebEngineSettings::Accelerated2dCanvasEnabled);
+}
+
+/*!
+  \qmlproperty bool WebEngineSettings::autoLoadIconsForPage
+  \since QtWebEngine 1.3
+
+  Automatically downloads icons for web pages.
+
+  Enabled by default.
+*/
+bool QQuickWebEngineSettings::autoLoadIconsForPage() const
+{
+    return d_ptr->testAttribute(WebEngineSettings::AutoLoadIconsForPage);
+}
+
+/*!
+  \qmlproperty bool WebEngineSettings::touchIconsEnabled
+  \since QtWebEngine 1.3
+
+  Enables support for touch icons and precomposed touch icons.
+
+  Disabled by default.
+*/
+bool QQuickWebEngineSettings::touchIconsEnabled() const
+{
+    return d_ptr->testAttribute(WebEngineSettings::TouchIconsEnabled);
+}
+
 /*!
     \qmlproperty string WebEngineSettings::defaultTextEncoding
     \since QtWebEngine 1.2
@@ -358,6 +424,46 @@ void QQuickWebEngineSettings::setFullScreenSupportEnabled(bool on)
         Q_EMIT fullScreenSupportEnabledChanged();
 }
 
+void QQuickWebEngineSettings::setScreenCaptureEnabled(bool on)
+{
+    bool wasOn = d_ptr->testAttribute(WebEngineSettings::ScreenCaptureEnabled);
+    d_ptr->setAttribute(WebEngineSettings::ScreenCaptureEnabled, on);
+    if (wasOn != on)
+        Q_EMIT screenCaptureEnabledChanged();
+}
+
+void QQuickWebEngineSettings::setWebGLEnabled(bool on)
+{
+    bool wasOn = d_ptr->testAttribute(WebEngineSettings::WebGLEnabled);
+    d_ptr->setAttribute(WebEngineSettings::WebGLEnabled, on);
+    if (wasOn != on)
+        Q_EMIT webGLEnabledChanged();
+}
+
+void QQuickWebEngineSettings::setAccelerated2dCanvasEnabled(bool on)
+{
+    bool wasOn = d_ptr->testAttribute(WebEngineSettings::Accelerated2dCanvasEnabled);
+    d_ptr->setAttribute(WebEngineSettings::Accelerated2dCanvasEnabled, on);
+    if (wasOn != on)
+        Q_EMIT accelerated2dCanvasEnabledChanged();
+}
+
+void QQuickWebEngineSettings::setAutoLoadIconsForPage(bool on)
+{
+    bool wasOn = d_ptr->testAttribute(WebEngineSettings::AutoLoadIconsForPage);
+    d_ptr->setAttribute(WebEngineSettings::AutoLoadIconsForPage, on);
+    if (wasOn != on)
+        Q_EMIT autoLoadIconsForPageChanged();
+}
+
+void QQuickWebEngineSettings::setTouchIconsEnabled(bool on)
+{
+    bool wasOn = d_ptr->testAttribute(WebEngineSettings::TouchIconsEnabled);
+    d_ptr->setAttribute(WebEngineSettings::TouchIconsEnabled, on);
+    if (wasOn != on)
+        Q_EMIT touchIconsEnabledChanged();
+}
+
 void QQuickWebEngineSettings::setDefaultTextEncoding(QString encoding)
 {
     const QString oldDefaultTextEncoding = d_ptr->defaultTextEncoding();
