@@ -41,6 +41,7 @@
 
 #include "qquickwebengineview_p.h"
 #include "qquickwebengineview_p_p.h"
+#include "qquickwebenginecontextmenudata_p.h"
 
 QT_BEGIN_NAMESPACE
 
@@ -69,6 +70,8 @@ public:
             tr("Cannot create a separate instance of WebEngineViewExperimental"));
         qmlRegisterUncreatableType<QQuickWebEngineViewport>(uri, 1, 0, "WebEngineViewport",
             tr("Cannot create a separate instance of WebEngineViewport"));
+        qmlRegisterUncreatableType<const QQuickWebEngineContextMenuData>(uri, 1, 0, "WebEngineContextMenuData",
+            tr("Cannot create a separate instance of WebEngineContextMenuData"));
 
         // Use the latest revision of QQuickWebEngineView when importing QtWebEngine.experimental 1.0
         qmlRegisterRevision<QQuickWebEngineView, LATEST_WEBENGINEVIEW_REVISION>(uri, 1, 0);
