@@ -71,6 +71,7 @@ private Q_SLOTS:
     void inputMethodHints();
     void basicRenderingSanity();
     void setZoomFactor();
+    void printToPdf();
     void stopSettingFocusWhenDisabled();
     void stopSettingFocusWhenDisabled_data();
     void inputEventForwardingDisabledWhenActiveFocusOnPressDisabled();
@@ -493,6 +494,27 @@ void tst_QQuickWebEngineView::setZoomFactor()
     QVERIFY(qFuzzyCompare(view->zoomFactor(), 2.5));
 }
 
+void tst_QQuickWebEngineView::printToPdf()
+{
+    QTemporaryDir tempDir(QDir::tempPath() + "/tst_qwebengineview-XXXXXX");
+    QVERIFY(tempDir.isValid());
+    QQuickWebEngineView *view = webEngineView();
+    view->setUrl(urlFromTestPath("html/basic_page.html"));
+    QVERIFY(waitForLoadSucceeded(view));
+
+    QString path = tempDir.path() + "/print_success.pdf";
+    view->printToPdf(path, QQuickWebEngineView::A4, QQuickWebEngineView::Portrait);
+    QTRY_VERIFY(QFile::exists(path));
+
+#if !defined(Q_OS_WIN)
+    path = tempDir.path() + "/print_//fail.pdf";
+#else
+    path = tempDir.path() + "/print_|fail.pdf";
+#endif // #if !defined(Q_OS_WIN)
+    view->printToPdf(path, QQuickWebEngineView::A4, QQuickWebEngineView::Portrait);
+    QTRY_VERIFY(!QFile::exists(path));
+}
+
 void tst_QQuickWebEngineView::stopSettingFocusWhenDisabled()
 {
     QFETCH(bool, viewEnabled);
